@@ -41,6 +41,7 @@
 #include "gretl_string_table.h"
 #include "dbread.h"
 #include "boxplots.h"
+#include "gretl_www.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -67,6 +68,7 @@ PRN *cmdprn;
 FILE *fb;
 int batch;
 int runit;
+int makepkg;
 int data_status;
 char linebak[MAXLINE];      /* for storing comments */
 char *line_read;
@@ -86,6 +88,7 @@ static void usage (int err)
 	     "Options:\n"
 	     " -b or --batch     Process a command script and exit.\n"
 	     " -r or --run       Run a script then hand control to command line.\n"
+	     " -m or --makepkg   Run a script and create a package from it.\n"
 	     " -h or --help      Print this info and exit.\n"
 	     " -v or --version   Print version info and exit.\n"
 	     " -e or --english   Force use of English rather than translation.\n"
@@ -155,12 +158,9 @@ static void nls_init (void)
 {
 #ifdef ENABLE_NLS
 # ifdef WIN32
-    char gretldir[MAXLEN], LOCALEDIR[MAXLEN];
+    char LOCALEDIR[MAXLEN];
 
-    if (read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", gretldir)) {
-        return;
-    }
-    sprintf(LOCALEDIR, "%s\\locale", gretldir);
+    build_path(LOCALEDIR, gretl_home(), "locale", NULL);
 # endif /* WIN32 */
 
     setlocale (LC_ALL, "");
@@ -342,6 +342,10 @@ int main (int argc, char *argv[])
     PRN *prn;
     int err = 0;
 
+#ifdef G_OS_WIN32
+    win32_set_gretldir(callname);
+#endif
+
     nls_init();
 
 #ifdef HAVE_READLINE
@@ -378,7 +382,7 @@ int main (int argc, char *argv[])
 	    }
 	}
 	    
-	if (opt & (OPT_BATCH | OPT_RUNIT)) {
+	if (opt & (OPT_BATCH | OPT_RUNIT | OPT_MAKEPKG)) {
 	    if (*filearg == '\0') {
 		/* we're missing a filename argument */
 		usage(1);
@@ -388,6 +392,9 @@ int main (int argc, char *argv[])
 		load_datafile = 0;
 		if (opt & OPT_BATCH) {
 		    batch = 1;
+		} else if (opt & OPT_MAKEPKG) {
+		    batch = 1;
+		    makepkg = 1;
 		} else {
 		    runit = 1;
 		}
@@ -412,6 +419,9 @@ int main (int argc, char *argv[])
 #endif
 
     libgretl_init();
+#if 0
+    gretl_www_init(NULL, NULL, 0);
+#endif
 
     logo(quiet); 
     session_time(NULL);
@@ -553,7 +563,7 @@ int main (int argc, char *argv[])
 	/* re-initialize: will be incremented by "run" cmd */
 	runit = 0;
 	sprintf(line, "run %s\n", runfile);
-	exec_line(&state, &Z, datainfo);
+	err = exec_line(&state, &Z, datainfo);
     }
 
     /* main command loop */
@@ -596,6 +606,12 @@ int main (int argc, char *argv[])
 	}
     }
 
+    if (makepkg && !err) {
+	switch_ext(filearg, runfile, "gfn");
+	sprintf(line, "makepkg %s\n", filearg);
+	exec_line(&state, &Z, datainfo);
+    }
+
     /* leak check -- try explicitly freeing all memory allocated */
 
     destroy_working_models(models, 2);
@@ -648,17 +664,42 @@ static int cli_renumber_series (const char *s, double **Z,
     return err;
 }
 
+static int cli_try_http (const char *s, char *fname, int *http)
+{
+    int err = 0;
+
+    /* skip past command word */
+    s += strcspn(s, " ");
+    s += strspn(s, " ");
+
+    if (strncmp(s, "http://", 7) == 0) {
+	err = retrieve_public_file(s, fname);
+	if (!err) {
+	    *http = 1;
+	} 
+    }
+
+    return err;
+}
+
 static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 			    DATAINFO *pdinfo, MODEL **models,
 			    PRN *prn)
 {
     char newfile[MAXLEN] = {0};
     char response[3];
-    int ftype, dbdata = 0;
+    int http = 0, dbdata = 0;
+    int ftype;
     int err = 0;
 
-    if (!(cmd->opt & OPT_O)) {
-	/* not using ODBC */
+    err = cli_try_http(line, newfile, &http);
+    if (err) {
+	errmsg(err, prn);
+	return err;
+    }
+
+    if (!http && !(cmd->opt & OPT_O)) {
+	/* not using http or ODBC */
 	err = getopenfile(line, newfile, (cmd->opt & OPT_W)?
 			  OPT_W : OPT_NONE);
 	if (err) {
@@ -666,7 +707,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 	    return err;
 	}
     }
-
+    
     if (cmd->opt & OPT_W) {
 	ftype = GRETL_NATIVE_DB_WWW;
     } else if (cmd->opt & OPT_O) {
@@ -716,7 +757,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 	return err;
     }
 
-    if (!dbdata && cmd->ci != APPEND) {
+    if (!dbdata && !http && cmd->ci != APPEND) {
 	strncpy(datafile, newfile, MAXLEN - 1);
     }
 
@@ -726,6 +767,10 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 	varlist(pdinfo, prn);
     }
 
+    if (http) {
+	remove(newfile);
+    }
+
     return err;
 }
 
