@@ -425,7 +425,6 @@ void selector_callback (GtkAction *action, gpointer data)
 {
     const gchar *s = gtk_action_get_name(action);
     windata_t *vwin = (windata_t *) data;
-    char title[64];
     int ci;
 
     ci = selector_callback_code(s);
@@ -434,8 +433,6 @@ void selector_callback (GtkAction *action, gpointer data)
 	set_window_busy(vwin);
     }
 
-    strcpy(title, "gretl: ");
-
     if (ci == COINT || ci == COINT2) {
 	selection_dialog(_("gretl: cointegration test"), do_coint, ci);
     } else if (ci == VAR || ci == VECM) {
@@ -482,6 +479,9 @@ void selector_callback (GtkAction *action, gpointer data)
     } else if (ci == TSPLOTS) {
 	simple_selection(_("gretl: define graph"), do_scatters, ci, vwin);
     } else if (ci == SPEARMAN) {
+	char title[64];
+	
+	strcpy(title, "gretl: ");
 	strcat(title, _("rank correlation"));
 	simple_selection(title, do_rankcorr, ci, vwin);
     } else {
@@ -519,7 +519,7 @@ static int gretl_callback_code (const gchar *s)
 }
 
 /* callback for menu items, where we want to prepare an "edit dialog"
-   for handle the request */
+   to handle the request */
 
 void gretl_callback (GtkAction *action, gpointer data)
 {
@@ -536,8 +536,12 @@ void gretl_callback (GtkAction *action, gpointer data)
     switch (cmd) {
     case GENR:
 	title = N_("gretl: add var");
-	query = N_("Enter formula for new variable\n"
-		   "(or just a name, to enter data manually)");
+	if (datainfo->n > 5000) {
+	    query = N_("Enter formula for new variable");
+	} else {
+	    query = N_("Enter formula for new variable\n"
+		       "(or just a name, to enter data manually)");
+	}
 	okfunc = do_genr;
 	varclick = VARCLICK_INSERT_NAME;
 	break;
@@ -664,9 +668,16 @@ void revise_system_model (void *ptr)
 
 void genr_callback (void)
 {
-    edit_dialog(_("gretl: add var"), 
-		_("Enter formula for new variable\n"
-		  "(or just a name, to enter data manually)"),
+    const char *msg;
+
+    if (datainfo->n > 5000) {
+	msg = N_("Enter formula for new variable");
+    } else {
+	msg = N_("Enter formula for new variable\n"
+		 "(or just a name, to enter data manually)");
+    }
+
+    edit_dialog(_("gretl: add var"), _(msg),
 		NULL, do_genr, NULL, 
 		GENR, VARCLICK_INSERT_NAME, NULL);   
 }
@@ -689,7 +700,7 @@ void newdata_callback (void)
 
     resp = spin_dialog(_("gretl: create data set"), NULL, &n, 
 		       _("Number of observations:"), 
-		       2, 100000, 0);
+		       2, 1000000, 0);
 
     if (resp < 0) {
 	/* canceled */
@@ -757,8 +768,66 @@ void do_nistcheck (GtkAction *action)
 
 #if defined (ENABLE_MAILER) && !defined(G_OS_WIN32)
 
+#if 0
+
+static gchar *my_url_encode (gchar const *text, int type)
+{
+    char const *good;
+    static char const hex[16] = "0123456789ABCDEF";
+    GString* result;
+
+    g_return_val_if_fail(text != NULL, NULL);
+    g_return_val_if_fail(*text != '\0', NULL);
+
+    switch (type) {
+    case 0: /* mailto: */
+	good = ".-_@";
+	break;
+    case 1: /* file: or http: */
+	good = "!$&'()*+,-./:=@_";
+	break;
+    default:
+	return NULL;
+    }
+
+    result = g_string_new(NULL);
+
+    while (*text) {
+	unsigned char c = *text++;
+	if (g_ascii_isalnum(c) || strchr(good, c))
+	    g_string_append_c(result, c);
+	else {
+	    g_string_append_c(result, '%');
+	    g_string_append_c(result, hex[c >> 4]);
+	    g_string_append_c(result, hex[c & 0xf]);
+	}
+    }
+
+    return g_string_free(result, FALSE);
+}
+
+#endif
+
 void send_file (char *fullname)
 {
+#if 0 /* if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 14 */
+    GError *error = NULL;
+    gchar *uri, *tmp;
+
+    tmp = my_url_encode(fullname, 0);
+    /* this idea is from gnumeric, but it doesn't work with thunderbird */
+    uri = g_strdup_printf("mailto:?subject=gretl data file&"
+			  "body=Attached&attachment=%s", tmp);
+    fprintf(stderr, "uri: '%s'\n", uri);
+    g_free(tmp);
+
+    gtk_show_uri(NULL, uri, GDK_CURRENT_TIME, &error);
+    if (error != NULL) {
+        errbox(error->message);
+        g_error_free(error);
+    }
+    g_free(uri);
+# else
     int (*email_file) (const char *, const char *);
     void *handle;
 
@@ -769,6 +838,7 @@ void send_file (char *fullname)
     
     email_file(fullname, gretl_dotdir());
     close_plugin(handle);
+# endif
 }
 
 #endif
