@@ -29,6 +29,7 @@
 #include "treeutils.h"
 #include "winstack.h"
 #include "toolbar.h"
+#include "dlgutils.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -202,14 +203,42 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbinfo)
     free(list);
 }
 
-static gchar *expand_warning (int mult)
+static int expand_data_dialog (int src_pd, int targ_pd, int *interpol)
 {
-    return g_strdup_printf(_("Do you really want to add a lower frequency series\n"
-			     "to a higher frequency dataset?\n\n"
-			     "If you say 'yes' I will expand the source data by\n"
-			     "repeating each value %d times.  In general, this is\n"
-			     "not a valid thing to do."),
-			   mult);
+    int mult = targ_pd / src_pd;
+    int resp;
+
+    if ((targ_pd == 4 && src_pd == 1) ||
+	(targ_pd == 12 && src_pd == 4)) {
+	/* interpolation is an option */
+	const char *opts[] = {
+	    N_("Interpolate higher frequency values"),
+	    N_("Repeat the lower frequency values")
+	};
+
+	resp = radio_dialog("gretl", _("Adding a lower frequency series to a\n"
+				       "higher frequency dataset"),
+			    opts, 2, 0, 0);
+	if (resp == 0) {
+	    *interpol = 1;
+	    resp = GRETL_YES;
+	} else if (resp == 1) {
+	    resp = GRETL_YES;
+	}
+    } else {
+	/* can only do expansion via replication */
+	gchar *msg = 
+	    g_strdup_printf(_("Do you really want to add a lower frequency series\n"
+			      "to a higher frequency dataset?\n\n"
+			      "If you say 'yes' I will expand the source data by\n"
+			      "repeating each value %d times.  In general, this is\n"
+			      "not a valid thing to do."),
+			    mult);
+	resp = yes_no_dialog("gretl", msg, 0);
+	g_free(msg);
+    }
+
+    return resp;
 }
 
 static int obs_overlap_check (SERIESINFO *sinfo)
@@ -245,11 +274,54 @@ static int pd_convert_check (SERIESINFO *sinfo)
     return err;
 }
 
+static const char *compact_method_string (CompactMethod method)
+{
+    if (method == COMPACT_SUM) {
+	return "sum";
+    } else if (method == COMPACT_SOP) {
+	return "first";
+    } else if (method == COMPACT_EOP) {
+	return "last";
+    } else {
+	return NULL;
+    }
+}
+
+static void record_db_open_command (dbwrapper *dw)
+{
+    if (dw->fname != NULL) {
+	gchar *cmdline;
+
+	if (dw->dbtype == GRETL_PCGIVE_DB) {
+	    cmdline = g_strdup_printf("open %s.bn7", dw->fname);
+	} else if (dw->dbtype == GRETL_NATIVE_DB) {
+	    const char *tmp = gretl_binbase();
+	    int n = strlen(tmp);
+
+	    if (!strncmp(dw->fname, tmp, n)) {
+		tmp = dw->fname + n;
+		if (*tmp == SLASH) tmp++;
+		cmdline = g_strdup_printf("open %s.bin", tmp);
+	    } else {
+		cmdline = g_strdup_printf("open %s.bin", dw->fname);
+	    }
+	} else if (dw->dbtype == GRETL_NATIVE_DB_WWW) {
+	    cmdline = g_strdup_printf("open %s --www", dw->fname);
+	} else {
+	    cmdline = g_strdup_printf("open %s", dw->fname);
+	}
+
+	record_command_verbatim(cmdline);
+	g_free(cmdline);
+    }
+}
+
 static int 
 add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw) 
 {
     SERIESINFO *sinfo;
     CompactMethod method = COMPACT_AVG;
+    gchar *cmdline = NULL;
     int resp, warned = 0, chosen = 0;
     int i, t, err = 0;
 
@@ -257,11 +329,16 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	return 1;
     }
 
+    record_db_open_command(dw);
+
     for (i=0; i<dw->nv && !err; i++) {
 	int overwrite = 0;
 	double x, *xvec = NULL;
+	const char *cstr = NULL;
 	int v, dbv, start, stop;
-	int pad1 = 0, pad2 = 0;	
+	int pad1 = 0, pad2 = 0;
+	int expand = 0, compact = 0;
+	int interpol = 0;
 
 	sinfo = &dw->sinfo[i];
 	v = sinfo->v;
@@ -271,10 +348,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	if (sinfo->pd < datainfo->pd && !warned) {
-	    gchar *msg = expand_warning(datainfo->pd / sinfo->pd);
-
-	    resp = yes_no_dialog("gretl", msg, 0);
-	    g_free(msg);
+	    resp = expand_data_dialog(sinfo->pd, datainfo->pd, &interpol);
 	    if (resp != GRETL_YES) {
 		return 0;
 	    }
@@ -305,8 +379,12 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	if (sinfo->pd < datainfo->pd) {
-	    xvec = expand_db_series(dbZ[v], sinfo, datainfo->pd);
+	    /* the series needs to be expanded */
+	    expand = 1;
+	    xvec = expand_db_series(dbZ[v], sinfo, datainfo->pd, interpol);
 	} else if (sinfo->pd > datainfo->pd) {
+	    /* the series needs to be compacted */
+	    compact = 1;
 	    if (!chosen) {
 		data_compact_dialog(vwin->main, sinfo->pd, &datainfo->pd, NULL, 
 				    &method, NULL);
@@ -321,7 +399,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    xvec = compact_db_series(dbZ[v], sinfo, datainfo->pd, 
 				     method);
 	} else {  
-	    /* series does not need compacting */
+	    /* the frequency does not need adjustment */
 	    xvec = mymalloc(sinfo->nobs * sizeof *xvec);
 	    if (xvec != NULL) {
 		for (t=0; t<sinfo->nobs; t++) {
@@ -338,6 +416,18 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    return 1;
 	}
 
+	/* record successful importation in command log */
+	if (compact && (cstr = compact_method_string(method)) != NULL) {
+	    cmdline = g_strdup_printf("data (compact=%s) %s", cstr,
+				      sinfo->varname);
+	} else {
+	    /* FIXME: handle expand/interpolate option */
+	    cmdline = g_strdup_printf("data %s", sinfo->varname);
+	}
+
+	record_command_verbatim(cmdline);
+	g_free(cmdline);
+
 	/* common stuff for adding a var */
 	strcpy(datainfo->varname[dbv], sinfo->varname);
 	strcpy(VARLABEL(datainfo, dbv), sinfo->descrip);
@@ -390,12 +480,16 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
 	add_db_series_to_dataset(vwin, dbZ, dw);
     } else {  
 	/* no data open: start new data set from db */
+	gchar *cmdline;
+
 	destroy_dataset(Z, datainfo);
 
 	Z = dbZ;
 	datainfo = dbinfo;
 	*freeit = 0;
 
+	record_db_open_command(dw);
+
 	for (i=1; i<=dw->nv && !err; i++) {
 	    sinfo = &dw->sinfo[i-1];
 
@@ -405,6 +499,10 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
 	    
 	    strcpy(datainfo->varname[i], sinfo->varname);
 	    strcpy(VARLABEL(datainfo, i), sinfo->descrip);
+	    
+	    cmdline = g_strdup_printf("data %s", sinfo->varname);
+	    record_command_verbatim(cmdline);
+	    g_free(cmdline);
 	}
 	
 	data_status |= (GUI_DATA | MODIFIED_DATA);
@@ -808,6 +906,7 @@ static void
 maybe_adjust_descrip_column (windata_t *vwin)
 {
     GtkTreeViewColumn *col;
+    GdkWindow *window;
     gint w0, w1, lw, w1max;
 
     col = gtk_tree_view_get_column(GTK_TREE_VIEW(vwin->listbox), 0);
@@ -816,7 +915,8 @@ maybe_adjust_descrip_column (windata_t *vwin)
     col = gtk_tree_view_get_column(GTK_TREE_VIEW(vwin->listbox), 1);
     w1 = gtk_tree_view_column_get_width(col);
 
-    gdk_drawable_get_size(vwin->listbox->window, &lw, NULL);
+    window = gtk_widget_get_window(vwin->listbox);
+    gdk_drawable_get_size(window, &lw, NULL);
 
     w1max = lw - w0 - 140;
 
@@ -1230,7 +1330,7 @@ static int add_rats_db_series_list (windata_t *vwin)
     }
 
     /* extract catalog from RATS file */
-    dw = read_rats_db(fp);
+    dw = read_rats_db(vwin->fname, fp);
     fclose(fp);
 
     if (dw == NULL) {
@@ -1262,7 +1362,7 @@ static int add_pcgive_db_series_list (windata_t *vwin)
     }
 
     /* extract catalog from PcGive file */
-    dw = read_pcgive_db(fp);
+    dw = read_pcgive_db(vwin->fname, fp);
     fclose(fp);
 
     if (dw == NULL) {
@@ -1351,6 +1451,19 @@ static int *db_get_selection_list (windata_t *vwin)
     return list;
 }
 
+static int db_role_to_dbtype (int role)
+{
+    if (role == PCGIVE_SERIES) { 
+	return GRETL_PCGIVE_DB;
+    } else if (role == REMOTE_SERIES) {
+	return GRETL_NATIVE_DB_WWW;
+    } else if (role == RATS_SERIES) {
+	return GRETL_RATS_DB;
+    } else {
+	return GRETL_NATIVE_DB;
+    }
+}
+
 static dbwrapper *get_series_info (windata_t *vwin, int action)
 {
     GtkTreeView *view = GTK_TREE_VIEW(vwin->listbox);
@@ -1368,7 +1481,7 @@ static dbwrapper *get_series_info (windata_t *vwin, int action)
 
     sc = rowlist[0];
 
-    dw = dbwrapper_new(sc);
+    dw = dbwrapper_new(sc, vwin->fname, db_role_to_dbtype(vwin->role));
     if (dw == NULL) {
 	free(rowlist);
 	return NULL;
@@ -1483,7 +1596,8 @@ void open_named_db_index (char *dbname)
 
     fp = gretl_fopen(dbname, "rb");
 
-    if (fp == NULL && action == NATIVE_SERIES) {
+    if (fp == NULL && action == NATIVE_SERIES &&
+	!has_suffix(dbname, ".bin")) {
 	strcat(dbname, ".bin");
 	fp = gretl_fopen(dbname, "rb");
     }
@@ -2152,17 +2266,16 @@ static void get_local_object_status (const char *fname, int role,
 	    if (err == -1) {
 		if (role == REMOTE_FUNC_FILES || role == REMOTE_DATA_PKGS) {
 		    /* try default working dir */
-		    char *tmp = gretl_default_workdir();
+		    const char *dw = gretl_default_workdir();
 
-		    if (tmp != NULL) {
+		    if (dw != NULL) {
 			if (role == REMOTE_FUNC_FILES) {
-			    build_path(filedir, tmp, "functions", NULL);
+			    build_path(filedir, dw, "functions", NULL);
 			    build_path(fullname, filedir, fname, NULL);
 			} else {
-			    build_path(fullname, tmp, fname, NULL);
+			    build_path(fullname, dw, fname, NULL);
 			}
 			err = stat(fullname, &fbuf);
-			free(tmp);
 		    }
 		} 
 		if (err == -1) {
@@ -2758,18 +2871,23 @@ void do_compact_data_set (void)
 
 void do_expand_data_set (void)
 {
-    int err, newpd = 0;
+    int err, newpd, interpol = 1;
 
     if (maybe_restore_full_data(EXPAND)) {
 	return;
     }
 
-    data_expand_dialog(mdata->main, datainfo->pd, &newpd);
-    if (newpd < 0) {
+    /* supported: annual to quarterly or quarterly to monthly */
+    newpd = (datainfo->pd == 1)? 4 : 12;
+
+    data_expand_dialog(mdata->main, datainfo->pd, &interpol);
+
+    if (interpol < 0) {
+	/* canceled */
 	return;
     }
 
-    err = expand_data_set(&Z, datainfo, newpd);
+    err = expand_data_set(&Z, datainfo, newpd, interpol);
 
     if (err) {
 	gui_errmsg(err);
