@@ -454,11 +454,11 @@ static int seek_file_collections (int location)
     int i = 0, err = 0;
 
     if (location == DATA_SEARCH) {
-	tmp = g_strdup_printf("%sdata", gretl_home());
+	tmp = gretl_strdup_printf("%sdata", gretl_home());
     } else if (location == SCRIPT_SEARCH) {
-	tmp = g_strdup_printf("%sscripts", gretl_home());
+	tmp = gretl_strdup_printf("%sscripts", gretl_home());
     } else if (location == USER_SEARCH) {
-	tmp = g_strdup(gretl_workdir());
+	tmp = gretl_strdup(gretl_workdir());
 	trim_slash(tmp);
     } else {
 	return 1;
@@ -468,7 +468,7 @@ static int seek_file_collections (int location)
 
     dir = opendir(tmp);
     if (dir == NULL) {
-	g_free(tmp);
+	free(tmp);
 	return 1;
     }
 
@@ -498,10 +498,10 @@ static int seek_file_collections (int location)
     }
 
     closedir(dir);
-    g_free(tmp);
+    free(tmp);
 
     if (location == USER_SEARCH && i++ == 0) {
-	tmp = gretl_default_workdir();
+	tmp = gretl_strdup(gretl_default_workdir());
 	if (tmp != NULL) {
 	    goto user_search_2;
 	}
@@ -757,8 +757,8 @@ static int gui_delete_fn_pkg (const char *fname, windata_t *vwin)
     int active[] = {1, 1};
     int resp, err = 0;
 
-    resp = checks_only_dialog ("gretl", msg,
-			       opts, 2, active, 0);
+    resp = checks_only_dialog("gretl", msg,
+			      opts, 2, active, 0);
     g_free(msg);
 
     if (resp < 0 || (active[0] == 0 && active[1] == 0)) {
@@ -856,16 +856,33 @@ void set_funcs_dir_callback (windata_t *vwin, char *path)
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
 
-	resp = radio_dialog("gretl", NULL, opts, 2, 0, 0);
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	read_fn_files_in_dir(dir, path, store, NULL, &nfn, &maxlen);
+
+	if (nfn == nfn0) {
+	    warnbox(_("No additional function files were found"));
+	    closedir(dir);
+	    return;
+	} 
+
+	resp = radio_dialog("gretl", "function packages", opts, 2, 0, 0);
+
 	if (resp < 0) {
+	    /* canceled */
+	    closedir(dir);
 	    return;
 	} else if (resp > 0) {
 	    gtk_list_store_clear(store);
 	    nfn = nfn0 = 0;
+	} else {
+	    nfn = nfn0;
 	}
 
+	rewinddir(dir);
+
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 	read_fn_files_in_dir(dir, path, store, &iter, &nfn, &maxlen);
+
 	if (nfn > nfn0) {
 	    infobox(_("Found %d function file(s)"), nfn - nfn0);
 	    g_object_set_data(G_OBJECT(vwin->listbox), "nfn", GINT_TO_POINTER(nfn));
@@ -1349,9 +1366,14 @@ db_window_handle_drag  (GtkWidget *widget,
 			gpointer p)
 {
     /* handle drag of pointer from remote database window */
-    if (info == GRETL_REMOTE_DB_PTR && data != NULL && 
-	data->type == GDK_SELECTION_TYPE_INTEGER) {
-	install_file_from_server(NULL, *(void **) data->data);
+    if (info == GRETL_REMOTE_DB_PTR && data != NULL) {
+	GdkAtom type = gtk_selection_data_get_data_type(data);
+	
+	if (type == GDK_SELECTION_TYPE_INTEGER) {
+	    const guchar *seldata = gtk_selection_data_get_data(data);
+
+	    install_file_from_server(NULL, *(void **) seldata);
+	}
     }
 }
 
@@ -1366,9 +1388,14 @@ pkg_window_handle_drag  (GtkWidget *widget,
 			 gpointer p)
 {
     /* handle drag of pointer from remote function package window */
-    if (info == GRETL_REMOTE_FNPKG_PTR && data != NULL && 
-	data->type == GDK_SELECTION_TYPE_INTEGER) {
-	install_file_from_server(NULL, *(void **) data->data);
+    if (info == GRETL_REMOTE_FNPKG_PTR && data != NULL) {
+	GdkAtom type = gtk_selection_data_get_data_type(data);
+
+	if (type == GDK_SELECTION_TYPE_INTEGER) {
+	    const guchar *seldata = gtk_selection_data_get_data(data);
+
+	    install_file_from_server(NULL, *(void **) seldata);
+	}
     }
 }
 
@@ -1575,6 +1602,11 @@ void show_files (GtkAction *action, gpointer p)
     display_files(code, p);
 }
 
+void show_native_dbs (void)
+{
+    display_files(NATIVE_DB, NULL);
+}
+
 static int get_func_info (const char *path, char **pdesc, 
 			  char **pver)
 {
@@ -1625,7 +1657,10 @@ static int fn_file_is_duplicate (const char *fname,
     return ret;
 }
 
-#define pkg_is_loaded(n) (get_function_package_by_filename(n) != NULL)
+/* Read (or just count) the .gfn files in a given directory.
+   The signal to count rather than read is that the @iter
+   argument is NULL.
+*/
 
 static void
 read_fn_files_in_dir (DIR *dir, const char *path, 
@@ -1634,7 +1669,6 @@ read_fn_files_in_dir (DIR *dir, const char *path,
 {
     struct dirent *dirent;
     char fullname[MAXLEN];
-    gchar *fname;
     int n, imax = *nfn;
     int err;
 
@@ -1643,6 +1677,7 @@ read_fn_files_in_dir (DIR *dir, const char *path,
 	    !strcmp(dirent->d_name, "..")) {
 	    continue;
 	}
+
 	if (has_suffix(dirent->d_name, ".gfn")) {
 	    char *descrip = NULL, *version = NULL;
 
@@ -1650,21 +1685,24 @@ read_fn_files_in_dir (DIR *dir, const char *path,
 	    err = get_func_info(fullname, &descrip, &version);
 
 	    if (!err && !fn_file_is_duplicate(dirent->d_name, version, store, imax)) {
-		gtk_list_store_append(store, iter);
-		fname = g_strdup(dirent->d_name);
-		n = strlen(fname) - 4;
-		fname[n] = '\0';
-		if (n > *maxlen) {
-		    *maxlen = n;
+		if (iter != NULL) {
+		    gchar *fname = g_strdup(dirent->d_name);
+
+		    gtk_list_store_append(store, iter);
+		    n = strlen(fname) - 4;
+		    fname[n] = '\0';
+		    if (n > *maxlen) {
+			*maxlen = n;
+		    }
+		    gtk_list_store_set(store, iter, 
+				       0, fname, 
+				       1, version,
+				       2, descrip, 
+				       3, function_package_is_loaded(fullname), 
+				       4, path, -1);
+		    g_free(fname);
 		}
-		gtk_list_store_set(store, iter, 
-				   0, fname, 
-				   1, version,
-				   2, descrip, 
-				   3, pkg_is_loaded(fullname), 
-				   4, path, -1);
 		*nfn += 1;
-		g_free(fname);
 	    }
 	    free(descrip);
 	    free(version);
@@ -1784,11 +1822,10 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 		build_path(fndir, gretl_dotdir(), "functions", NULL);
 	    } else if (i == 4) {
 		/* plus any in the default working dir, if not already searched */
-		char *tmp = gretl_default_workdir();
+		const char *wdir = gretl_default_workdir();
 
-		if (tmp != NULL) {
-		    build_path(fndir, tmp, "functions", NULL);
-		    g_free(tmp);
+		if (wdir != NULL) {
+		    build_path(fndir, wdir, "functions", NULL);
 		} 
 	    }
 
@@ -1826,6 +1863,10 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     return 0;
 }
 
+#ifndef NAME_MAX
+# define NAME_MAX 255
+#endif
+
 /* Given the name of a package (its internal name, not a path, e.g.
    "gig"), search some plausible directories and see if we can
    find the corresponding gfn file. If so, return an allocated copy 
@@ -1850,18 +1891,17 @@ gchar *gretl_function_package_get_path (const char *name)
 
 	if ((dir = opendir(dirname)) != NULL) {
 	    struct dirent *dirent;
-	    char mypath[FILENAME_MAX];
-	    char *myname;
+	    char *p, test[NAME_MAX+1];
 
 	    while ((dirent = readdir(dir)) != NULL && !found) {
 		if (has_suffix(dirent->d_name, ".gfn")) {
-		    build_path(mypath, dirname, dirent->d_name, NULL);
-		    myname = get_function_package_name(mypath);
-		    if (!strcmp(myname, name)) {
-			path = g_strdup(mypath);
+		    strcpy(test, dirent->d_name);
+		    p = strrchr(test, '.');
+		    *p = '\0';
+		    if (!strcmp(test, name)) {
+			path = g_strdup_printf("%s%c%s", dirname, SLASH, dirent->d_name);
 			found = 1;
 		    }
-		    free(myname);
 		}
 	    }
 	    closedir(dir);
@@ -1894,7 +1934,7 @@ static void revise_loaded_status (GtkWidget *lbox)
 	g_free(fname);
 	g_free(fdir);
 	gtk_list_store_set(store, &iter, 
-			   3, pkg_is_loaded(fullname), 
+			   3, function_package_is_loaded(fullname), 
 			   -1);
 	if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
 	    break;
