@@ -79,13 +79,18 @@ void sample_range_dialog (GtkAction *action, gpointer p);
 
 void sample_restrict_dialog (GtkAction *action, gpointer p);
 
+int panel_units_selector (int *t1, int *t2, int nmax);
+
 void panel_structure_dialog (DATAINFO *pdinfo);
 
 void data_compact_dialog (GtkWidget *w, int spd, int *target_pd, 
 			  int *mon_start, CompactMethod *method,
 			  int *repday);
 
-void data_expand_dialog (GtkWidget *w, int spd, int *target_pd);
+void data_expand_dialog (GtkWidget *w, int spd, int *interpol);
+
+void pergm_dialog (gretlopt *opt, int *spinval, int spinmin, int spinmax,
+		   int *cancel);
 
 int density_dialog (int vnum, double *bw);
 
@@ -148,7 +153,7 @@ int freq_dialog (const char *title, const char *blurb,
 		 double xmin, double xmax, int *dist, int *plot);
 
 int model_table_dialog (int *colhead_opt, int *se_opt, int *pv_opt,
-			int *ast_opt, int *figs);
+			int *ast_opt, int *figs, char *fmt);
 
 void bootstrap_dialog (windata_t *vwin, int *pp, int *pB,
 		       gretlopt *popt, int *cancelled);
@@ -162,6 +167,10 @@ void iter_control_dialog (int *optim, int *pmaxit, double *ptol,
 
 void tex_format_dialog (void);
 
+void object_name_entry_dialog (char *name, GretlType type,
+			       const char *labeltxt,
+			       int *cancel);
+
 int gretl_all_done (void);
 
 #endif /* DIALOGS_H */
