@@ -42,18 +42,7 @@ struct combo_opts_ {
 
 #define button_is_active(b) gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b))
 #define widget_get_int(w,s) GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), s))
-
-/* remedial macros for old GTK installations */
-
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)
-# define gtk_dialog_get_content_area(d) (d->vbox)
-# define gtk_dialog_get_action_area(d) (d->action_area)
-#endif
-
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 18)
-# define gtk_widget_is_sensitive(w) GTK_WIDGET_IS_SENSITIVE(w)
-# define gtk_widget_has_focus(w) GTK_WIDGET_HAS_FOCUS(w)
-#endif
+#define widget_set_int(w,s,i) g_object_set_data(G_OBJECT(w), s, GINT_TO_POINTER(i))
 
 /* and a remedial function */
 
