@@ -24,13 +24,17 @@
 #include "filters.h"
 #include "libset.h"
 #include "plotspec.h"
+#include "usermat.h"
 #include "cmdstack.h"
+#include "ssheet.h"
 
 enum {
     FILTER_SMA = 1,
     FILTER_EMA,
     FILTER_HP,
     FILTER_BK,
+    FILTER_BW,
+    FILTER_POLY,
     FILTER_FD
 };
 
@@ -41,6 +45,7 @@ enum {
 #define FILTER_GRAPH_NONE 0
 #define FILTER_GRAPH_TREND OPT_A
 #define FILTER_GRAPH_CYCLE OPT_B
+#define FILTER_GRAPH_RESP OPT_C
 
 typedef struct filter_info_ filter_info;
 
@@ -56,6 +61,10 @@ struct filter_info_ {
     int bkk;              /* Baxter-King parameter */
     int bkl;              /* Baxter-King lower value */
     int bku;              /* Baxter-King upper value */
+    int order;            /* Butterworth or polynomial order */
+    int cutoff;           /* Butterworth cut-off */
+    double midfrac;       /* for weighted polynomial fit */
+    gretlopt wopt;        /* weighting-type option */
     gretlopt graph_opt;
     gretlopt save_opt;
     char save_t[VNAMELEN];
@@ -66,8 +75,16 @@ struct filter_info_ {
     GtkWidget *entry1;
     GtkWidget *entry2;
     GtkWidget *kspin;
+    GtkWidget *spin1;
+    GtkWidget *spin2;
+    GtkWidget *button;
+    GtkWidget *label;
 };
 
+static int calculate_filter (filter_info *finfo);
+static void weights_shape_graph (GtkWidget *button, filter_info *finfo);
+static void butterworth_poles_graph (GtkWidget *button, filter_info *finfo);
+
 static const char *filter_get_title (int ftype)
 {
     if (ftype == FILTER_SMA) {
@@ -78,6 +95,10 @@ static const char *filter_get_title (int ftype)
 	return N_("Hodrick-Prescott filter");
     } else if (ftype == FILTER_BK) {
 	return N_("Baxter-King Band-pass filter");
+    } else if (ftype == FILTER_BW) {
+	return N_("Butterworth filter");
+    } else if (ftype == FILTER_POLY) {
+	return N_("Polynomial trend");
     } else if (ftype == FILTER_FD) {
 	return N_("Fractional difference");
     } else {
@@ -119,6 +140,10 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
     finfo->entry1 = NULL;
     finfo->entry2 = NULL;
     finfo->kspin = NULL;
+    finfo->spin1 = NULL;
+    finfo->spin2 = NULL;
+    finfo->button = NULL;
+    finfo->label = NULL;
 
     if (ftype == FILTER_SMA) {
 	finfo->center = 0;
@@ -130,6 +155,14 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
     } else if (ftype == FILTER_BK) {
 	finfo->bkk = get_bkbp_k(datainfo);
 	get_bkbp_periods(datainfo, &finfo->bkl, &finfo->bku);
+    } else if (ftype == FILTER_BW) {
+	finfo->order = 8;
+	finfo->cutoff = 67;
+    } else if (ftype == FILTER_POLY) {
+	finfo->order = 3;
+	finfo->wopt = OPT_NONE;
+	finfo->lambda = 1.0;
+	finfo->midfrac = 0.0;
     } else if (ftype == FILTER_FD) {
 	finfo->lambda = 0.5;
     }
@@ -148,6 +181,10 @@ static void filter_make_savename (filter_info *finfo, int i)
 	strcpy(targ, (i == 0)? "hpt_" : "hp_");
     } else if (finfo->ftype == FILTER_BK) {
 	strcpy(targ, "bk_");
+    } else if (finfo->ftype == FILTER_BW) {
+	strcpy(targ, (i == 0)? "bt_" : "bc_");
+    } else if (finfo->ftype == FILTER_POLY) {
+	strcpy(targ, (i == 0)? "pt_" : "pc_");
     } else if (finfo->ftype == FILTER_FD) {
 	strcpy(targ, "fd_");
     }
@@ -193,6 +230,22 @@ static void filter_make_varlabel (filter_info *finfo, int v, int i)
     } else if (finfo->ftype == FILTER_BK) {
 	sprintf(s, _("Filtered %s: Baxter-King, frequency %d to %d"), 
 		finfo->vname, finfo->bkl, finfo->bku);
+    } else if (finfo->ftype == FILTER_BW) {
+	if (i == FILTER_SAVE_TREND) {
+	    sprintf(s, _("Filtered %s: Butterworth low-pass (n=%d, cutoff=%d)"), 
+		    finfo->vname, finfo->order, finfo->cutoff);
+	} else {
+	    sprintf(s, _("Filtered %s: Butterworth high-pass (n=%d, cutoff=%d)"), 
+		    finfo->vname, finfo->order, finfo->cutoff);
+	}
+    } else if (finfo->ftype == FILTER_POLY) {
+	if (i == FILTER_SAVE_TREND) {
+	    sprintf(s, _("Filtered %s: polynomial of order %d"), 
+		    finfo->vname, finfo->order);
+	} else {
+	    sprintf(s, _("Filtered %s: residual from polynomial of order %d"), 
+		    finfo->vname, finfo->order);
+	}	
     } else if (finfo->ftype == FILTER_FD) {
 	sprintf(s, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
     }
@@ -248,34 +301,6 @@ static int varname_error (filter_info *finfo, int i)
     return 0;
 }
 
-static void filter_dialog_ok (GtkWidget *w, filter_info *finfo)
-{
-    if (finfo->save_opt & FILTER_SAVE_TREND) {
-	strcpy(finfo->save_t, gtk_entry_get_text(GTK_ENTRY(finfo->entry1)));
-	if (varname_error(finfo, 1)) {
-	    return;
-	}
-    }
-
-    if (finfo->save_opt & FILTER_SAVE_CYCLE) {
-	strcpy(finfo->save_c, gtk_entry_get_text(GTK_ENTRY(finfo->entry2)));
-	if (varname_error(finfo, 2)) {
-	    return;
-	} 
-    } 
-
-    if (finfo->kspin != NULL) {
-	if (gtk_widget_is_sensitive(finfo->kspin)) {
-	    finfo->k = (int) 
-		gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->kspin));
-	} else {
-	    finfo->k = 0;
-	}
-    }
-
-    gtk_widget_destroy(finfo->dlg);
-}
-
 static void filter_dialog_hsep (GtkWidget *dlg)
 {
     GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
@@ -427,18 +452,186 @@ static void ema_obs_radios (GtkWidget *dlg, filter_info *finfo)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
-static int filter_dialog (filter_info *finfo)
+static void weight_scheme_changed (GtkComboBox *combo, filter_info *finfo)
+{
+    int i = gtk_combo_box_get_active(combo);
+
+    gtk_widget_set_sensitive(finfo->spin1, (i > 0));
+    gtk_widget_set_sensitive(finfo->spin2, (i > 0));
+    gtk_widget_set_sensitive(finfo->button, (i > 0));
+
+    if (i == 0) {
+	finfo->wopt = OPT_NONE;
+    } else if (i == 1) {
+	finfo->wopt = OPT_Q;
+    } else if (i == 2) {
+	finfo->wopt = OPT_B;
+    } else {
+	finfo->wopt = OPT_C;
+    }	
+}
+
+static void add_poly_weights_options (GtkWidget *vbox, filter_info *finfo)
+{
+    GtkWidget *hbox, *label, *combo;
+    const char *wstrs[] = {
+	N_("uniform"),
+	N_("quadratic"),
+	N_("cosine-bell"),
+	N_("steps")
+    };
+    int i;
+
+    /* first row: weighting scheme */
+    hbox = gtk_hbox_new(FALSE, 5);
+    label = gtk_label_new(_("Weights:"));
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+    combo = gtk_combo_box_new_text();
+    for (i=0; i<4; i++) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _(wstrs[i]));
+    }
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+    g_signal_connect(GTK_COMBO_BOX(combo), "changed",
+		     G_CALLBACK(weight_scheme_changed), finfo);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
+    label = gtk_label_new(_("with maximum:"));
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+    finfo->spin1 = gtk_spin_button_new_with_range(1.0, 10.0, 0.1);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(finfo->spin1), 2.0);
+    gtk_widget_set_sensitive(finfo->spin1, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), finfo->spin1, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+    /* second row: size of central section plus show button */
+    hbox = gtk_hbox_new(FALSE, 5);
+    label = gtk_label_new(_("Central fraction:"));
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+    finfo->spin2 = gtk_spin_button_new_with_range(0.0, 1.0, 0.1);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(finfo->spin2), 0.4);
+    gtk_widget_set_sensitive(finfo->spin2, FALSE);
+    gtk_box_pack_start(GTK_BOX(hbox), finfo->spin2, FALSE, FALSE, 0);
+    finfo->button = gtk_button_new_with_label(_("Show weights"));
+    gtk_widget_set_sensitive(finfo->button, FALSE);
+    g_signal_connect(GTK_BUTTON(finfo->button), "clicked",
+		     G_CALLBACK(weights_shape_graph), finfo);
+    gtk_box_pack_end(GTK_BOX(hbox), finfo->button, FALSE, FALSE, 15);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+}
+
+static void filter_dialog_ok (GtkWidget *w, filter_info *finfo)
+{
+    if (finfo->save_opt & FILTER_SAVE_TREND) {
+	strcpy(finfo->save_t, gtk_entry_get_text(GTK_ENTRY(finfo->entry1)));
+	if (varname_error(finfo, 1)) {
+	    return;
+	}
+    }
+
+    if (finfo->save_opt & FILTER_SAVE_CYCLE) {
+	strcpy(finfo->save_c, gtk_entry_get_text(GTK_ENTRY(finfo->entry2)));
+	if (varname_error(finfo, 2)) {
+	    return;
+	} 
+    } 
+
+    if (finfo->kspin != NULL) {
+	if (gtk_widget_is_sensitive(finfo->kspin)) {
+	    finfo->k = (int) 
+		gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->kspin));
+	} else {
+	    finfo->k = 0;
+	}
+    }
+
+    if (finfo->spin1 != NULL && finfo->spin2 != NULL) {
+	if (gtk_widget_is_sensitive(finfo->spin1)) {
+	    finfo->lambda = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin1));
+	    finfo->midfrac = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin2));
+	}
+    }    
+
+    if (finfo->graph_opt != FILTER_GRAPH_NONE ||
+	finfo->save_opt != FILTER_SAVE_NONE) {
+	int err = calculate_filter(finfo);
+
+	if (err) {
+	    gui_errmsg(err);
+	} 
+    } 
+}
+
+#if 0
+static gboolean check_bw_feasibility (GtkSpinButton *b, 
+				      filter_info *finfo)
+{
+    if (finfo->spin1 != NULL && finfo->spin2 != NULL) {
+	const double b0 = 0.131981;
+	const double b1 = 0.0126241;
+	const double b2 = 0.0658865;
+	const double b3 = 0.883492;
+	double s, lrc;
+	int c, n;
+
+	n = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin1));
+	c = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin2));
+
+	s = log(sin(c*M_PI/360));
+	lrc = b0 + b1*n + b2*s + b3*n*s;
+
+	fprintf(stderr, "lrc = %g\n", lrc);
+
+	gtk_widget_set_sensitive(finfo->label, lrc < -12);
+    }
+
+    return FALSE;
+}
+#else
+static gboolean check_bw_feasibility (GtkSpinButton *b, 
+				      filter_info *finfo)
+{
+    if (finfo->spin1 != NULL && finfo->spin2 != NULL) {
+	double b0 = -8.56644923648263;
+	double b1 = -0.66992007228155;
+	double b2 =  2.82851981714539;
+	double x;
+	int c, n;
+
+	n = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin1));
+	c = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin2));
+
+	/* let's be conservative here */
+	c--;
+	n++;
+
+	x = b0 + b1*c + b2*n;
+	x = 1/(1+exp(-x));
+
+	gtk_widget_set_sensitive(finfo->label, x > 0.47971759156793);
+    }
+
+    return FALSE;
+}
+#endif
+
+static void filter_dialog_quit (GtkWidget *w, gpointer data)
+{
+    set_dataset_locked(FALSE);
+}
+
+static void filter_dialog (filter_info *finfo)
 {
     GtkWidget *dlg;
     GtkWidget *vbox;
     GtkWidget *hbox;
     GtkWidget *w;
-    int ret = 0;
 
     dlg = gretl_dialog_new(_("gretl: time-series filter"), mdata->main,
 			   GRETL_DLG_BLOCK);
     finfo->dlg = dlg;
 
+    g_signal_connect(G_OBJECT(dlg), "destroy",
+		     G_CALLBACK(filter_dialog_quit), NULL);
+
     /* box title */
     hbox = gtk_hbox_new(FALSE, 5);
     w = gtk_label_new(_(filter_get_title(finfo->ftype)));
@@ -509,6 +702,54 @@ static int filter_dialog (filter_info *finfo)
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	/* set periods */
 	bkbp_frequencies_table(dlg, finfo);
+    } else if (finfo->ftype == FILTER_BW) {
+	/* set Butterworth order */
+	hbox = gtk_hbox_new(FALSE, 5);
+	w = gtk_label_new(_("n (higher values -> better approximation):"));
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+	finfo->spin1 = w = gtk_spin_button_new_with_range(1.0, 16, 1);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), finfo->order);
+	g_signal_connect(G_OBJECT(w), "value-changed",
+			 G_CALLBACK(set_int_from_spinner), &finfo->order);
+	g_signal_connect(G_OBJECT(w), "value-changed",
+			 G_CALLBACK(check_bw_feasibility), finfo);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);    
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	/* set cutoff */
+	hbox = gtk_hbox_new(FALSE, 5);
+	w = gtk_label_new(_("frequency cutoff (degrees):"));
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+        finfo->spin2 = w = gtk_spin_button_new_with_range(1, 179, 1);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), finfo->cutoff);
+	g_signal_connect(G_OBJECT(w), "value-changed",
+			 G_CALLBACK(set_int_from_spinner), &finfo->cutoff);
+	g_signal_connect(G_OBJECT(w), "value-changed",
+			 G_CALLBACK(check_bw_feasibility), finfo);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);	
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new(FALSE, 5);
+	w = gtk_button_new_with_label(_("show poles"));
+	g_signal_connect(G_OBJECT(w), "clicked",
+			 G_CALLBACK(butterworth_poles_graph), finfo);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+	finfo->label = w = gtk_label_new(_("filter may be numerically unstable"));
+	gtk_widget_set_sensitive(w, FALSE);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    } else if (finfo->ftype == FILTER_POLY) {
+	/* set polynomial order */
+	hbox = gtk_hbox_new(FALSE, 5);
+	w = gtk_label_new(_("Order:"));
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+	w = gtk_spin_button_new_with_range(1.0, 15, 1);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), finfo->order);
+	g_signal_connect(G_OBJECT(w), "value-changed",
+			 G_CALLBACK(set_int_from_spinner), &finfo->order);
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);    
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	/* weights settings (optional) */
+	filter_dialog_hsep(dlg);
+	add_poly_weights_options(vbox, finfo);
     } else if (finfo->ftype == FILTER_FD) {
 	/* set "d" */
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -553,6 +794,10 @@ static int filter_dialog (filter_info *finfo)
 				  FILTER_GRAPH_TREND);
 	filter_graph_check_button(dlg, finfo, _("Graph residual or cycle series"),
 				  FILTER_GRAPH_CYCLE);
+	if (finfo->ftype == FILTER_HP || finfo->ftype == FILTER_BW) {
+	    filter_graph_check_button(dlg, finfo, _("Graph frequency response"),
+				      FILTER_GRAPH_RESP);
+	} 
 	/* save to dataset? */
 	filter_dialog_hsep(dlg);
 	filter_save_check_buttons(dlg, finfo);
@@ -561,8 +806,10 @@ static int filter_dialog (filter_info *finfo)
     /* convenience pointer for button box */
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
 
-    /* Cancel button */
-    cancel_delete_button(hbox, dlg, &ret);
+    /* Close button */
+    w = close_button(hbox);
+    g_signal_connect_swapped(G_OBJECT(w), "clicked", 
+			     G_CALLBACK(gtk_widget_destroy), dlg);
 
     /* "OK" button */
     w = ok_button(hbox);
@@ -570,9 +817,15 @@ static int filter_dialog (filter_info *finfo)
 		     G_CALLBACK(filter_dialog_ok), finfo);
     gtk_widget_grab_default(w);
 
-    gtk_widget_show_all(dlg);
+    if (finfo->ftype == FILTER_BW) {
+	context_help_button(hbox, BWFILTER);
+    } else if (finfo->ftype == FILTER_POLY) {
+	context_help_button(hbox, POLYWEIGHTS);
+    }
 
-    return ret;
+    set_dataset_locked(TRUE);
+
+    gtk_widget_show_all(dlg);
 }
 
 static void print_gp_data (filter_info *finfo, const double *obs,
@@ -604,7 +857,7 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
     char title[128];
     int v, err = 0;
 
-    obs = gretl_plotx(datainfo);
+    obs = gretl_plotx(NULL, datainfo);
     if (obs == NULL) {
 	return E_ALLOC;
     }
@@ -645,6 +898,9 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 
     gretl_push_c_numeric_locale();
 
+    fprintf(fp, "set xrange [%g:%g]\n", floor(obs[datainfo->t1]), 
+	    ceil(obs[datainfo->t2]));
+
     if (twoplot) {
 	fputs("set size 1.0,1.0\nset multiplot\nset size 1.0,0.60\n", fp);
 
@@ -714,6 +970,186 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
     return err;
 }
 
+static void butterworth_poles_graph (GtkWidget *button, filter_info *finfo)
+{
+    const char *fmt = N_("Butterworth poles (n = %d, cutoff = %d degrees)");
+    gchar *title;
+    FILE *fp;
+    double cut, theta, delta; 
+    double x, y, px, py;
+    int i, n = finfo->order;
+    int err = 0;
+
+    fp = get_plot_input_stream(PLOT_ROOTS, &err);
+    if (err) { 
+	gui_errmsg(err);
+	return;
+    }
+
+    cut = finfo->cutoff * M_PI / 180;
+    cut = tan(cut / 2);
+
+    title = g_strdup_printf(_(fmt), finfo->order, finfo->cutoff);
+    fprintf(fp, "set title \"%s\"\n", title);
+    g_free(title);
+
+    fputs("unset border\n", fp);
+    fputs("unset key\n", fp);
+    fputs("set xzeroaxis\n", fp);
+    fputs("set yzeroaxis\n", fp);
+    fputs("unset xtics\n", fp);
+    fputs("unset ytics\n", fp);
+    fputs("set size square\n", fp);
+    fputs("set polar\n", fp);
+    fputs("plot 1.0 w lines, \\\n"
+	  "'-' w points pt 7\n", fp);
+
+    gretl_push_c_numeric_locale();
+    
+    for (i=1; i<=n; i++) {
+	theta = M_PI * (2 * i - 1) / (2 * n);
+	delta = 1 + cut * (2 * sin(theta) + cut);
+	x = (1 - cut * cut) / delta;
+	y = 2 * cut * cos(theta) / delta;
+	px = atan2(y, x);
+	py = sqrt(x * x + y * y);
+	fprintf(fp, "%g %g # %.4f,%.4f\n", px, py, x, y);
+    }
+    fputs("e\n", fp);
+
+    gretl_pop_c_numeric_locale();
+
+    fclose(fp);
+
+    err = gnuplot_make_graph();
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph(NULL);
+    }
+}
+
+static void weights_shape_graph (GtkWidget *button, filter_info *finfo)
+{
+    gretl_matrix *W;
+    int T = sample_size(datainfo);
+    int err = 0;
+
+    W = gretl_matrix_alloc(T, 2);
+
+    if (W == NULL) {
+	err = E_ALLOC;
+    } else {
+	int t, list[3] = {2, 1, 2};
+
+	err = private_matrix_add(W, "$Weights");
+
+	if (!err) {
+	    err = umatrix_set_colnames_from_string(W, "weight t");
+	}
+
+	if (!err) {
+	    char rstr[48];
+
+	    for (t=0; t<T; t++) {
+		W->val[T+t] = t;
+	    }
+
+	    finfo->lambda = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin1));
+	    finfo->midfrac = gtk_spin_button_get_value(GTK_SPIN_BUTTON(finfo->spin2));
+	    poly_weights(W->val, T, finfo->lambda, finfo->midfrac, finfo->wopt);
+	    gretl_push_c_numeric_locale();
+	    sprintf(rstr, "set xrange [0:%d] ; set yrange [0.8:%g] ;", T - 1, 
+		    1.1 * finfo->lambda);
+	    gretl_pop_c_numeric_locale();
+	    err = matrix_plot(W, list, rstr, OPT_O);
+	}
+
+	destroy_private_matrices();
+    }
+
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph(NULL);
+    }
+}
+
+static int do_filter_response_graph (filter_info *finfo)
+{
+    gretl_matrix *G;
+    FILE *fp = NULL;
+    char title[128];
+    double omega_star = 0.0;
+    int i, nlit, err = 0;
+
+    if (finfo->ftype == FILTER_BW) {
+	omega_star = finfo->cutoff * M_PI / 180;
+	G = butterworth_gain(finfo->order, omega_star, 0);
+	nlit = 2;
+    } else {
+	G = hp_gain(finfo->lambda, 0);
+	nlit = 1;
+    }
+
+    if (G == NULL) {
+	gui_errmsg(E_ALLOC);
+	return E_ALLOC;
+    }
+
+    fp = get_plot_input_stream(PLOT_REGULAR, &err);
+    if (err) { 
+	gui_errmsg(err);
+	gretl_matrix_free(G);
+	return err;
+    }
+
+    fputs("set xrange [0:3.1416]\n", fp);
+    fputs("set yrange [0:1.1]\n", fp);
+
+    if (finfo->ftype == FILTER_BW) {
+	sprintf(title, "%s (n = %d, %s %.2fπ)", _("Gain for Butterworth filter"), 
+		finfo->order, _("nominal cutoff"),
+		(double) finfo->cutoff / 180);
+    } else {
+	sprintf(title, _("Gain for H-P filter (lambda = %g)"), finfo->lambda);
+    }	
+    fprintf(fp, "set title \"%s\"\n", title);
+
+    fprintf(fp, "# literal lines = %d\n", nlit);
+    fputs("set xtics (\"0\" 0, \"π/4\" pi/4, \"π/2\" pi/2, \"3π/4\" 3*pi/4, "
+	  "\"π\" pi)\n", fp);
+
+    gretl_push_c_numeric_locale();
+
+    if (finfo->ftype == FILTER_BW) {
+	fprintf(fp, "set arrow from %g,0 to %g,1.1 nohead\n", 
+		omega_star, omega_star);
+    }
+
+    fputs("plot \\\n", fp);
+    fputs("'-' using 1:2 notitle w lines\n", fp);
+    for (i=0; i<G->rows; i++) {
+	fprintf(fp, "%.5f %.5f\n", gretl_matrix_get(G, i, 0),
+		gretl_matrix_get(G, i, 1));
+    }
+    fputs("e\n", fp);
+
+    gretl_pop_c_numeric_locale();
+
+    gretl_matrix_free(G);
+    fclose(fp);
+
+    err = gnuplot_make_graph();
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph(NULL);
+    }
+
+    return err;
+}
+
 static int save_filtered_var (filter_info *finfo, double *x, int i,
 			      int *saved)
 {
@@ -774,6 +1210,11 @@ static void record_filter_command (filter_info *finfo)
     } else if (finfo->ftype == FILTER_BK) {
 	sprintf(s, "bkfilt(%s, %d, %d, %d)\n", finfo->vname,
 		finfo->bkl, finfo->bku, finfo->bkk);
+    } else if (finfo->ftype == FILTER_BW) {
+	sprintf(s, "bwfilt(%s, %d, %d)\n", finfo->vname,
+		finfo->order, finfo->cutoff);
+    } else if (finfo->ftype == FILTER_POLY) {
+	sprintf(s, "polyfit(%s, %d)\n", finfo->vname, finfo->order);
     } else if (finfo->ftype == FILTER_FD) {
 	sprintf(s, "fracdiff(%s, %g)\n", finfo->vname, finfo->lambda);
     }
@@ -843,6 +1284,17 @@ static int calculate_filter (filter_info *finfo)
     } else if (finfo->ftype == FILTER_BK) {
 	/* Baxter and King bandpass */
 	err = bkbp_filter(x, u, datainfo, finfo->bkl, finfo->bku, finfo->bkk);
+    } else if (finfo->ftype == FILTER_BW) {
+	/* Butterworth */
+	err = butterworth_filter(x, fx, datainfo, finfo->order, finfo->cutoff);
+    } else if (finfo->ftype == FILTER_POLY) {
+	if (finfo->wopt != OPT_NONE) {
+	    err = weighted_poly_trend(x, fx, datainfo, finfo->order,
+				      finfo->wopt, finfo->lambda,
+				      finfo->midfrac);
+	} else {
+	    err = poly_trend(x, fx, datainfo, finfo->order);
+	}
     } else if (finfo->ftype == FILTER_FD) {
 	/* fractional differencing */
 	err = fracdiff_series(x, fx, finfo->lambda, 1, -1, datainfo);
@@ -866,6 +1318,10 @@ static int calculate_filter (filter_info *finfo)
 	do_filter_graph(finfo, fx, u);
     }
 
+    if (finfo->graph_opt & FILTER_GRAPH_RESP) {
+	do_filter_response_graph(finfo);
+    }
+
     if (finfo->save_opt & FILTER_SAVE_TREND) {
 	err = save_filtered_var(finfo, fx, FILTER_SAVE_TREND, &saved);
     } else {
@@ -902,6 +1358,10 @@ static int filter_code (GtkAction *action)
 	return FILTER_HP;
     else if (!strcmp(s, "FilterBK")) 
 	return FILTER_BK;
+    else if (!strcmp(s, "FilterBW")) 
+	return FILTER_BW;
+    else if (!strcmp(s, "FilterPoly")) 
+	return FILTER_POLY;
     else if (!strcmp(s, "FilterFD")) 
 	return FILTER_FD;
     else
@@ -914,36 +1374,22 @@ void filter_callback (GtkAction *action)
     int v = mdata_active_var();
     int t1 = datainfo->t1;
     int t2 = datainfo->t2;
-    int code, cancel = 0;
-    int err = 0;
+    int code, err = 0;
 
     code = filter_code(action);
 
-    err = array_adjust_t1t2(Z[v], &t1, &t2);
-    if (err) {
-	gui_errmsg(E_MISSDATA);
-	return;
+    err = series_adjust_sample(Z[v], &t1, &t2);
+
+    if (!err && t2 - t1 + 1 < 4) {
+	err = E_TOOFEW;
     }
 
-    if (t2 - t1 + 1 < 4) {
-	gui_errmsg(E_MISSDATA);
+    if (err) {
+	gui_errmsg(err);
 	return;
-    }	
+    }
 
     filter_info_init(&finfo, code, v, t1, t2);
 
-    cancel = filter_dialog(&finfo);
-    if (cancel) {
-	return;
-    }
-
-    if (finfo.graph_opt == FILTER_GRAPH_NONE &&
-	finfo.save_opt == FILTER_SAVE_NONE) {
-	return;
-    }
-
-    err = calculate_filter(&finfo);
-    if (err) {
-	gui_errmsg(err);
-    }    
+    filter_dialog(&finfo);
 }
