@@ -27,6 +27,7 @@
 #include "gretl_www.h"
 #include "gretl_string_table.h"
 #include "gretl_scalar.h"
+#include "gretl_bundle.h"
 #include "database.h"
 #include "guiprint.h"
 #include "ssheet.h"
@@ -37,19 +38,20 @@
 typedef struct call_info_ call_info;
 
 struct call_info_ {
-    GtkWidget *dlg;
-    GList *lsels; /* list arg selectors */
-    GList *msels; /* matrix arg selectors */
-    GList *ssels; /* string arg selectors */
-    int *publist; /* list of public interfaces */
-    int iface;    /* selected interface */
+    GtkWidget *dlg;      /* main dialog */
+    GtkWidget *top_hbox; /* upper hbox in dialog */
+    GList *lsels;        /* list argument selectors */
+    GList *msels;        /* matrix arg selectors */
+    GList *ssels;        /* string arg selectors */
+    int *publist;        /* list of public interfaces */
+    int iface;           /* selected interface */
     int extracol;
     const ufunc *func;
+    FuncDataReq dreq;
     int n_params;
     char rettype;
-    gchar **args; /* FIXME use GLib */
+    gchar **args;
     gchar *ret;
-    int ok;
 };
 
 #define scalar_arg(t) (t == GRETL_TYPE_DOUBLE || t == GRETL_TYPE_SCALAR_REF)
@@ -57,8 +59,9 @@ struct call_info_ {
 #define matrix_arg(t) (t == GRETL_TYPE_MATRIX || t == GRETL_TYPE_MATRIX_REF)
 
 static GtkWidget *open_fncall_dlg;
+static gboolean close_on_OK = TRUE;
 
-static void fncall_OK_callback (GtkWidget *w, call_info *cinfo);
+static void fncall_exec_callback (GtkWidget *w, call_info *cinfo);
 
 static gchar **glib_str_array_new (int n)
 {
@@ -102,7 +105,7 @@ static call_info *cinfo_new (void)
     cinfo->ret = NULL;
 
     cinfo->extracol = 0;
-    cinfo->ok = 0;
+    cinfo->dreq = 0;
 
     return cinfo;
 }
@@ -152,10 +155,12 @@ static const char *arg_type_string (int t)
     if (t == GRETL_TYPE_SERIES) return "series";
     if (t == GRETL_TYPE_MATRIX) return "matrix";
     if (t == GRETL_TYPE_STRING) return "string";
+    if (t == GRETL_TYPE_BUNDLE) return "bundle";
     
     if (t == GRETL_TYPE_SCALAR_REF) return "scalar *";
     if (t == GRETL_TYPE_SERIES_REF) return "series *";
     if (t == GRETL_TYPE_MATRIX_REF) return "matrix *";
+    if (t == GRETL_TYPE_BUNDLE_REF) return "bundle *";
 
     return "";
 }
@@ -181,15 +186,11 @@ static int check_args (call_info *cinfo)
 
 static void fncall_dialog_destruction (GtkWidget *w, call_info *cinfo)
 {
-    if (!cinfo->ok) {
-	/* don't free cinfo if we're going to use it */
-	cinfo_free(cinfo);
-    }
-
+    cinfo_free(cinfo);
     open_fncall_dlg = NULL;
 }
 
-static void fncall_cancel (GtkWidget *w, call_info *cinfo)
+static void fncall_close (GtkWidget *w, call_info *cinfo)
 {
     gtk_widget_destroy(cinfo->dlg);
 }
@@ -203,11 +204,13 @@ static GtkWidget *label_hbox (GtkWidget *w, const char *txt,
     gtk_box_pack_start(GTK_BOX(w), hbox, FALSE, FALSE, vspace);
 
     label = gtk_label_new(txt);
+
     if (center) {
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
     } else {
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
     }
+
     gtk_widget_show(label);
 
     return hbox;
@@ -345,22 +348,49 @@ static GList *get_selection_list (call_info *cinfo, int i, int type,
     return list;
 }
 
+static windata_t *make_help_viewer (const char *fnname, char *buf)
+{
+    windata_t *vwin;
+    gchar *title;
+
+    title = g_strdup_printf(_("help on %s"), fnname);
+    vwin = view_formatted_text_buffer(title, buf, 70, 350);
+    g_free(title);
+
+    return vwin;
+}
+
 static void fncall_help (GtkWidget *w, call_info *cinfo)
 {
     const char *fnname = user_function_name_by_index(cinfo->iface);
+    char *pdfname = NULL;
     PRN *prn;
     int err;
 
+    /* FIXME incorpoate this into markup */
+    if (user_function_has_PDF_doc(fnname, &pdfname)) {
+	err = display_gfn_help(pdfname);
+	free(pdfname);
+	if (!err) {
+	    return;
+	}
+    }
+
     if (bufopen(&prn)) {
 	return;
     }
     
-    err = user_function_help(fnname, prn);
+    err = user_function_help(fnname, OPT_M, prn);
+
     if (err) {
 	gretl_print_destroy(prn);
-	dummy_call();
+	errbox("Couldn't find any help");
     } else {
-	view_buffer(prn, 80, 400, fnname, VIEW_PKG_INFO, NULL);
+	char *buf = gretl_print_steal_buffer(prn);
+
+	make_help_viewer(fnname, buf);
+	free(buf);
+	gretl_print_destroy(prn);
     }
 }
 
@@ -465,11 +495,19 @@ int do_make_list (selector *sr)
 {
     const char *buf = selector_list(sr);
     const char *lname = selector_entry_text(sr);
-    const char *msg;
-    PRN *prn;
-    int *list;
+    gpointer data = selector_get_data(sr);
+    call_info *cinfo = NULL;
+    const char *msg = NULL;
+    PRN *prn = NULL;
+    int *list = NULL;
     int err = 0;
 
+    if (data != NULL) {
+	GtkWidget *w = GTK_WIDGET(data);
+	
+	cinfo = g_object_get_data(G_OBJECT(w), "cinfo");
+    }
+
     if (lname == NULL || *lname == 0) {
 	errbox(_("No name was given for the list"));
 	return 1;
@@ -496,32 +534,38 @@ int do_make_list (selector *sr)
 	return err;
     }
 
-    if (bufopen(&prn)) {
-	free(list);
-	return 1;
+    if (cinfo != NULL) {
+	/* don't bother with "Added list..." message */
+	err = remember_list(list, lname, NULL);
+	if (err) {
+	    gui_errmsg(err);
+	}
+    } else {
+	if (bufopen(&prn)) {
+	    free(list);
+	    return 1;
+	}
+	err = remember_list(list, lname, prn);
+	msg = gretl_print_get_buffer(prn);
+	if (err) {
+	    errbox(msg);
+	}
     }
 
-    err = remember_list(list, lname, prn);
-    msg = gretl_print_get_buffer(prn);
-
-    if (err) {
-	errbox(msg);
-    } else {
-	gpointer data = selector_get_data(sr);
-
+    if (!err) {
 	gtk_widget_hide(selector_get_window(sr));
-	infobox(msg);
-
-	if (data != NULL) {
-	    GtkWidget *w = GTK_WIDGET(data);
-	    call_info *cinfo = g_object_get_data(G_OBJECT(w), "cinfo");
-	    
+	if (cinfo != NULL) {
 	    update_list_selectors(cinfo);
+	} else {
+	    infobox(msg);
 	}
     }
 
     free(list);
-    gretl_print_destroy(prn);
+
+    if (prn != NULL) {
+	gretl_print_destroy(prn);
+    }
 
     return err;
 } 
@@ -643,14 +687,6 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
     return combo;
 }
 
-static void add_table_hsep (GtkWidget *tbl, int cols, int r0)
-{
-    GtkWidget *hsep = gtk_hseparator_new();
-
-    gtk_table_attach(GTK_TABLE(tbl), hsep, 0, cols, r0, r0 + 1,
-		     GTK_FILL, GTK_FILL, 5, 5);
-}
-
 static void add_table_header (GtkWidget *tbl, gchar *txt,
 			      int cols, int r0)
 {
@@ -669,9 +705,18 @@ static void add_table_cell (GtkWidget *tbl, GtkWidget *w,
 		     GTK_FILL, GTK_FILL, 5, 3);
 }
 
+static void set_close_on_OK (GtkWidget *b, gpointer p)
+{
+    close_on_OK = button_is_active(b);
+}
+
 #define cinfo_has_return(c) (c->rettype != GRETL_TYPE_NONE && \
 			     c->rettype != GRETL_TYPE_VOID)
 
+#define cinfo_offer_return(c) (c->rettype != GRETL_TYPE_NONE && \
+			       c->rettype != GRETL_TYPE_VOID && \
+			       c->rettype != GRETL_TYPE_BUNDLE)
+
 static void function_call_dialog (call_info *cinfo)
 {
     GtkWidget *button, *label;
@@ -694,23 +739,26 @@ static void function_call_dialog (call_info *cinfo)
 	return;
     }
 
+    fnname = user_function_name_by_index(cinfo->iface);
+
     cinfo->dlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title(GTK_WINDOW(cinfo->dlg), ("gretl: function call"));
+    txt = g_strdup_printf("gretl: %s", fnname);
+    gtk_window_set_title(GTK_WINDOW(cinfo->dlg), txt);
+    g_free(txt);
     gretl_emulated_dialog_add_structure(cinfo->dlg, &vbox, &bbox);
     open_fncall_dlg = cinfo->dlg;
     g_signal_connect(G_OBJECT(cinfo->dlg), "destroy",
 		     G_CALLBACK(fncall_dialog_destruction), cinfo);
 
-    fnname = user_function_name_by_index(cinfo->iface);
-    hbox = label_hbox(vbox, fnname, 5, 1);
+    cinfo->top_hbox = hbox = label_hbox(vbox, fnname, 5, 0);
 
     if (cinfo->n_params > 0) {
 	tcols = (cinfo->extracol)? 3 : 2;
 	trows = cinfo->n_params + 1;
-	if (cinfo_has_return(cinfo)) { 
+	if (cinfo_offer_return(cinfo)) { 
 	    trows += 4;
 	}
-    } else if (cinfo_has_return(cinfo)) {
+    } else if (cinfo_offer_return(cinfo)) {
 	tcols = 2;
 	trows = 3;
     }
@@ -752,12 +800,16 @@ static void function_call_dialog (call_info *cinfo)
 	add_table_cell(tbl, sel, 1, 2, row);
 
 	if (ptype == GRETL_TYPE_LIST) {
+	    GtkWidget *entry = gtk_bin_get_child(GTK_BIN(sel));
+	    
 	    cinfo->lsels = g_list_append(cinfo->lsels, sel);
 	    button = gtk_button_new_with_label(_("More..."));
 	    add_table_cell(tbl, button, 2, 3, row);
+	    g_object_set_data(G_OBJECT(entry), "argnum", 
+			      GINT_TO_POINTER(i+1));
 	    g_signal_connect(G_OBJECT(button), "clicked", 
 			     G_CALLBACK(launch_list_maker),
-			     gtk_bin_get_child(GTK_BIN(sel)));
+			     entry);
 	} else if (ptype == GRETL_TYPE_MATRIX) {
 	    cinfo->msels = g_list_append(cinfo->msels, sel);
 	    button = gtk_button_new_with_label(_("New..."));
@@ -768,13 +820,12 @@ static void function_call_dialog (call_info *cinfo)
 	} 
     }
 	
-    if (cinfo_has_return(cinfo)) {
+    if (cinfo_offer_return(cinfo)) {
 	GtkWidget *child;
 	GList *list = NULL;
 
 	if (cinfo->n_params > 0) {
 	    row++;
-	    add_table_hsep(tbl, tcols, row++);
 	}	    
 
 	add_table_header(tbl, _("Assign return value (optional):"), tcols, row);
@@ -805,15 +856,25 @@ static void function_call_dialog (call_info *cinfo)
 	gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
     }
 
-    /* Cancel button */
-    button = cancel_button(bbox);
+    /* option button */
+    hbox = gtk_hbox_new(FALSE, 5);
+    button = gtk_check_button_new_with_label(_("close this dialog on \"OK\""));
+    g_signal_connect(G_OBJECT(button), "toggled",
+		     G_CALLBACK(set_close_on_OK), NULL);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), 
+				 close_on_OK);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+    /* Close button */
+    button = close_button(bbox);
     g_signal_connect(G_OBJECT (button), "clicked", 
-		     G_CALLBACK(fncall_cancel), cinfo);
+		     G_CALLBACK(fncall_close), cinfo);
 
     /* "OK" button */
     button = ok_button(bbox);
     g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(fncall_OK_callback), cinfo);
+		     G_CALLBACK(fncall_exec_callback), cinfo);
     gtk_widget_grab_default(button);
 
     /* Help button */
@@ -828,12 +889,11 @@ static int function_data_check (call_info *cinfo)
 {
     int i, err = 0;
 
-    /* FIXME provide a way for a function to signal that
-       it doesn't need data loaded? */
-
-    if (datainfo == NULL || datainfo->v == 0) {
-	warnbox(_("Please open a data file first"));
-	return 1;
+    if (cinfo->dreq != FN_NODATA_OK) {
+	if (datainfo == NULL || datainfo->v == 0) {
+	    warnbox(_("Please open a data file first"));
+	    return 1;
+	}
     }
 
     for (i=0; i<cinfo->n_params; i++) {
@@ -954,21 +1014,31 @@ static int pre_process_args (call_info *cinfo, PRN *prn)
     return err;
 }
 
+#define AUTO_BUNDLE "BUNDLE_RET__"
+
 static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 {
     ExecState state;
     char fnline[MAXLINE];
     const char *funname;
+    gretl_bundle *bundle = NULL;
+    int attach_bundle = 0;
     int orig_v = datainfo->v;
     int i, err = 0;
 
     funname = user_function_name_by_index(cinfo->iface);
     *fnline = 0;
 
+    /* compose the function command-line */
+
     if (cinfo->ret != NULL) {
 	strcat(fnline, cinfo->ret);
 	strcat(fnline, " = ");
-    }    
+    } else if (cinfo->rettype == GRETL_TYPE_BUNDLE) {
+	strcat(fnline, AUTO_BUNDLE);
+	strcat(fnline, " = ");
+	attach_bundle = 1;
+    }	
 
     strcat(fnline, funname);
     strcat(fnline, "(");
@@ -982,23 +1052,40 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	}
     }
 
-    /* destroy any "ARG" vars or matrices that were created? */
-
     strcat(fnline, ")");
-    pprintf(prn, "? %s\n", fnline);
 
-    gretl_exec_state_init(&state, SCRIPT_EXEC, NULL, get_lib_cmd(),
-			  models, prn);
+    if (!attach_bundle) {
+	pprintf(prn, "? %s\n", fnline);
+    }
 
     /* note: gretl_exec_state_init zeros the first byte of the
        supplied 'line' */
+
+    gretl_exec_state_init(&state, SCRIPT_EXEC, NULL, get_lib_cmd(),
+			  models, prn);
     state.line = fnline;
+
+#if USE_GTK_SPINNER
+    start_wait_for_output(cinfo->top_hbox, 0); 
+#endif
     err = gui_exec_line(&state, &Z, datainfo);
-    view_buffer(prn, 80, 400, funname, PRINT, NULL);
+#if USE_GTK_SPINNER
+    stop_wait_for_output(cinfo->top_hbox);
+#endif
+
+    /* destroy any "ARG" vars or matrices that were created? */
+
+    if (!err && attach_bundle) {
+	bundle = gretl_bundle_pull_from_stack(AUTO_BUNDLE, &err);
+    }
+
+    view_buffer(prn, 80, 400, funname, 
+		(bundle == NULL)? PRINT : VIEW_BUNDLE,
+		bundle);
 
     if (err) {
 	gui_errmsg(err);
-    }    
+    } 
 
     if (datainfo->v > orig_v) {
 	mark_dataset_as_modified();
@@ -1061,13 +1148,11 @@ static void select_interface (call_info *cinfo)
 
 /* Callback from "OK" button in function call GUI: if there's a
    problem with the argument selection just return so the dialog stays
-   in place and the user can correct matters; otherwise close the
-   dialog and execute the function, then clean up.  (Note that we
-   need to set cinfo->ok so that the cinfo structure won't get
-   destroyed along with the dialog.)
+   in place and the user can correct matters; otherwise really execute
+   the function.
 */
 
-static void fncall_OK_callback (GtkWidget *w, call_info *cinfo)
+static void fncall_exec_callback (GtkWidget *w, call_info *cinfo)
 {
     if (check_args(cinfo)) {
 	return;
@@ -1075,9 +1160,6 @@ static void fncall_OK_callback (GtkWidget *w, call_info *cinfo)
 	PRN *prn = NULL;
 	int err;
 
-	cinfo->ok = 1; /* flag preservation of cinfo */
-	gtk_widget_destroy(cinfo->dlg);
-
 	err = bufopen(&prn);
 
 	if (!err && cinfo->args != NULL) {
@@ -1093,7 +1175,9 @@ static void fncall_OK_callback (GtkWidget *w, call_info *cinfo)
 	    gretl_print_destroy(prn);
 	}
 
-	cinfo_free(cinfo);
+	if (close_on_OK) {
+	    gtk_widget_destroy(cinfo->dlg);
+	}
     }
 }
 
@@ -1103,29 +1187,18 @@ static void fncall_OK_callback (GtkWidget *w, call_info *cinfo)
 void call_function_package (const char *fname, GtkWidget *w,
 			    int *loaderr)
 {
-    FuncDataReq dreq = 0;
     int minver = 0;
     call_info *cinfo;
     fnpkg *pkg;
     int err = 0;
 
-    pkg = get_function_package_by_filename(fname);
+    pkg = get_function_package_by_filename(fname, &err);
 
-    if (pkg == NULL) {
-	/* not already loaded */
-	err = load_function_package_from_file(fname);
-	if (err) {
-	    file_read_errbox(fname);
-	    if (loaderr != NULL) {
-		*loaderr = 1;
-	    }
-	} else {
-	    /* should be OK now */
-	    pkg = get_function_package_by_filename(fname);
+    if (err) {
+	gui_errmsg(err); /* FIXME error not very informative? */
+	if (loaderr != NULL) {
+	    *loaderr = 1;
 	}
-    }
-
-    if (err || pkg == NULL) {
 	return;
     }
 
@@ -1138,7 +1211,7 @@ void call_function_package (const char *fname, GtkWidget *w,
 
     err = function_package_get_properties(pkg,
 					  "publist", &cinfo->publist,
-					  "data-requirement", &dreq,
+					  "data-requirement", &cinfo->dreq,
 					  "min-version", &minver,
 					  NULL);
 
@@ -1152,7 +1225,7 @@ void call_function_package (const char *fname, GtkWidget *w,
 
     if (!err) {
 	/* do we have suitable data in place? */
-	err = check_function_needs(datainfo, dreq, minver);
+	err = check_function_needs(datainfo, cinfo->dreq, minver);
 	if (err) {
 	    gui_errmsg(err);
 	}
@@ -1207,6 +1280,110 @@ void function_call_cleanup (void)
     }
 }
 
+#if 1
+
+/* This is quite complicated, because we're bypassing the "standard"
+   mechanism for a function call, and instead using output from the
+   gretl model selection dialog and constructing a "direct" call to
+   the function, as is done inside geneval.c. There's probably a
+   simpler way of doing this.
+*/
+
+static int ols_bundle_callback (selector *sr)
+{
+    const char *funname = "ols_pack";
+    const char *pkgname = "olsbundle";
+    gchar *path = NULL;
+    ufunc *uf = NULL;
+    fnargs *args = NULL;
+    int yno, err = 0;
+
+    if (selector_error(sr)) {
+	return 1;
+    }
+
+    /* get path to package; load package if not already loaded;
+       get specific function from package */
+
+    path = gretl_function_package_get_path(pkgname);
+    if (path == NULL) {
+	gretl_errmsg_sprintf("Couldn't find package '%s'", pkgname);
+	err = E_DATA;
+    } else {
+	fnpkg *pkg = get_function_package_by_filename(path, &err);
+
+	if (!err) {
+	    uf = get_function_from_package(funname, pkg);
+	}
+	if (uf == NULL) {
+	    gretl_errmsg_sprintf("Couldn't find function '%s'", funname);
+	    err = E_DATA;
+	}
+	g_free(path);
+    }
+
+    if (err) {
+	gui_errmsg(err);
+	return err;
+    } else {
+	/* args: get ID of dependent var plus X list */
+	const char *buf = selector_list(sr);
+
+	if (buf == NULL) {
+	    err = E_DATA;
+	} else {
+	    int *list = gretl_list_from_string(buf, &err);
+
+	    if (!err) {
+		yno = list[1];
+		gretl_list_delete_at_pos(list, 1);
+		remember_list(list, "arg1temp", NULL);
+		free(list);
+	    }
+	}
+    }
+
+    if (!err) {
+	/* push args to function */
+	args = fn_args_new();
+	if (args == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    err = push_fn_arg(args, GRETL_TYPE_USERIES, &yno);
+	    if (!err) {
+		err = push_fn_arg(args, GRETL_TYPE_LIST, "arg1temp");
+	    }
+	}
+    }
+
+    if (!err) {
+	/* execute function and retrieve bundle */
+	gretl_bundle *bundle = NULL;
+	PRN *prn = NULL;
+
+	err = bufopen(&prn);
+
+	if (!err) {
+	    err = gretl_function_exec(uf, args, GRETL_TYPE_BUNDLE, &Z, datainfo, 
+				      &bundle, NULL, prn);
+	}
+	if (!err) {
+	    view_buffer(prn, 80, 400, funname, VIEW_BUNDLE, bundle);
+	}
+    }
+
+    if (err) {
+	gui_errmsg(err);
+    }
+
+    fn_args_free(args);
+    delete_list_by_name("arg1temp");
+
+    return err;
+}
+
+#endif
+
 /* Callback for a menu item representing a function package, whose
    name (e.g. "gig") is attached to @action. We first see if we can
    find the full path to the corresponding gfn file; if so we
@@ -1222,13 +1399,17 @@ static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
 
     if (path == NULL) {
 	errbox("Couldn't find package %s\n", name);
+    } else if (!strcmp(name, "olsbundle")) {
+	/* see if we can get cosy with olsbundle */
+	selection_dialog(_("gretl: specify model"), 
+			 ols_bundle_callback, OLS);
     } else {
 	call_function_package(path, vwin->main, NULL);
 	g_free(path);
     }
 }
 
-/* For a "privileged" function package: given its internal
+/* For "privileged" function packages: given the internal
    package name (e.g. "gig") and a menu path where we'd like
    it to appear (e.g. "/menubar/Model/TSModels" -- see the
    ui definition file gui2/gretlmain.xml), construct the
@@ -1236,9 +1417,9 @@ static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
    which see above.
 */
 
-void maybe_add_package_to_menu (const char *pkgname, 
-				const char *menupath,
-				windata_t *vwin)
+static void maybe_add_package_to_menu (const char *pkgname, 
+				       const char *menupath,
+				       windata_t *vwin)
 {
     static GtkActionEntry pkg_item = {
 	NULL, NULL, NULL, NULL, NULL, G_CALLBACK(gfn_menu_callback)
@@ -1261,3 +1442,12 @@ void maybe_add_package_to_menu (const char *pkgname,
     g_object_unref(actions);
     g_free(label);
 }
+
+void maybe_add_packages_to_menus (windata_t *vwin)
+{
+    maybe_add_package_to_menu("gig", "/menubar/Model/TSModels", vwin);
+#if 0
+    maybe_add_package_to_menu("olsbundle", "/menubar/Model", vwin);
+#endif
+}
+
