@@ -27,8 +27,6 @@ void function_call_cleanup (void);
 
 void gui_define_list (void);
 
-void maybe_add_package_to_menu (const char *pkgname, 
-				const char *menupath,
-				windata_t *vwin);
+void maybe_add_packages_to_menus (windata_t *vwin);
 
 #endif /* FNCALL_H */
