@@ -469,8 +469,7 @@ static void edit_code_callback (GtkWidget *w, function_info *finfo)
 	return;
     }
 
-    fun = get_packaged_function_by_name(finfo->active,
-					finfo->pkg);
+    fun = get_function_from_package(finfo->active, finfo->pkg);
     if (fun == NULL) {
 	errbox(_("Can't find the function '%s'"), finfo->active);
     }
@@ -795,7 +794,8 @@ static void add_data_requirement_menu (GtkWidget *tbl, int i,
 	N_("No special requirement"),
 	N_("Time-series data"),
 	N_("Quarterly or monthly data"),
-	N_("Panel data")
+	N_("Panel data"),
+	N_("No dataset needed")
     };
     GtkWidget *datamenu, *tmp;
     int j;
@@ -806,7 +806,7 @@ static void add_data_requirement_menu (GtkWidget *tbl, int i,
     gtk_widget_show(tmp);
 
     datamenu = gtk_combo_box_new_text();
-    for (j=0; j<=FN_NEEDS_PANEL; j++) {
+    for (j=0; j<=FN_NODATA_OK; j++) {
 	gtk_combo_box_append_text(GTK_COMBO_BOX(datamenu), _(datareq[j]));
     }
     gtk_combo_box_set_active(GTK_COMBO_BOX(datamenu), finfo->dreq);
@@ -940,7 +940,7 @@ static gint today_popup (GtkWidget *entry, GdkEventButton *event,
 {
     GdkModifierType mods = parent_get_pointer_mask(entry);
 
-    if (mods & GDK_BUTTON3_MASK) {
+    if (RIGHT_CLICK(mods)) {
 	if (*popup == NULL) {
 	    GtkWidget *menu = gtk_menu_new();
 	    GtkWidget *item;
@@ -1165,7 +1165,7 @@ static void finfo_dialog (function_info *finfo)
 
     /* "Save as" button */
     button = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(finfo_save_callback), finfo);
@@ -1174,7 +1174,7 @@ static void finfo_dialog (function_info *finfo)
 
     /* Save button */
     button = gtk_button_new_from_stock(GTK_STOCK_SAVE);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(finfo_save_callback), finfo);
@@ -1182,7 +1182,7 @@ static void finfo_dialog (function_info *finfo)
 
     /* Close button */
     button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT (button), "clicked", 
 		     G_CALLBACK(delete_pkg_editor), finfo);
@@ -1270,7 +1270,7 @@ static void login_dialog (login_info *linfo)
 
     /* Website */
     button = gtk_button_new_with_label("Website");
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
 				       button, TRUE);
@@ -1389,7 +1389,7 @@ static void do_upload (const char *fname)
 	gdk_cursor_unref(cursor);
     }
 
-    err = gretl_file_get_contents(fname, &buf);
+    err = gretl_file_get_contents(fname, &buf, NULL);
 
     if (err) {
 	error_printed = 1;
@@ -1582,8 +1582,8 @@ int save_function_package_as_script (const char *fname, gpointer p)
     pprintf(prn, "# date='%s'\n", finfo->date);
 
     for (i=0; i<finfo->n_priv; i++) {
-	fun = get_packaged_function_by_name(finfo->privnames[i],
-					    finfo->pkg);
+	fun = get_function_from_package(finfo->privnames[i],
+					finfo->pkg);
 	if (fun != NULL) {
 	    pputc(prn, '\n');
 	    gretl_function_print_code(fun, prn);
@@ -1591,8 +1591,8 @@ int save_function_package_as_script (const char *fname, gpointer p)
     }
 
     for (i=0; i<finfo->n_pub; i++) {
-	fun = get_packaged_function_by_name(finfo->pubnames[i],
-					    finfo->pkg);
+	fun = get_function_from_package(finfo->pubnames[i],
+					finfo->pkg);
 	if (fun != NULL) {
 	    pputc(prn, '\n');
 	    gretl_function_print_code(fun, prn);
@@ -1725,16 +1725,10 @@ void edit_function_package (const char *fname)
     const char *p;
     int err = 0;
 
-    pkg = get_function_package_by_filename(fname);
-
-    if (pkg == NULL) {
-	err = load_function_package_from_file(fname);
-	if (err) {
-	    file_read_errbox(fname);
-	    return;
-	} else {
-	    pkg = get_function_package_by_filename(fname);
-	}
+    pkg = get_function_package_by_filename(fname, &err);
+    if (err) {
+	gui_errmsg(err);
+	return;
     }
 
     finfo = finfo_new();
