@@ -73,7 +73,6 @@ struct plot_editor_ {
     GtkWidget *dialog;
     GtkWidget *notebook;
     GPT_SPEC *spec;
-    gchar *fontname;
     gchar *barsfile;
 
     GtkWidget **linetitle;
@@ -136,11 +135,7 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins);
 static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins);
 static int add_line_widget (plot_editor *ed);
 static int add_label_widget (plot_editor *ed);
-
-static void widget_set_int (GtkWidget *w, const gchar *key, gint val)
-{
-    g_object_set_data(G_OBJECT(w), key, GINT_TO_POINTER(val));
-}
+static void plot_editor_set_fontname (plot_editor *ed, const char *name);
 
 /* graph color selection apparatus */
 
@@ -202,7 +197,7 @@ static void color_select_callback (GtkWidget *button, GtkWidget *w)
     gint i;
 
     color_button = g_object_get_data(G_OBJECT(w), "color_button");
-    csel = GTK_COLOR_SELECTION_DIALOG(w)->colorsel;
+    csel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(w));
 
     gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csel), &gcolor);
 
@@ -266,7 +261,7 @@ static void color_patch_button_reset (GtkWidget *button, int cnum)
 static void graph_color_selector (GtkWidget *w, gpointer p)
 {
     GPT_SPEC *spec;
-    GtkWidget *cdlg;
+    GtkWidget *cdlg, *csel;
     GtkWidget *button;
     gint i = GPOINTER_TO_INT(p);
     char colstr[8];
@@ -293,15 +288,14 @@ static void graph_color_selector (GtkWidget *w, gpointer p)
     widget_set_int(cdlg, "colnum", i);
     g_object_set_data(G_OBJECT(cdlg), "color_button", w);
 
-    gtk_color_selection_set_current_color(GTK_COLOR_SELECTION
-					  (GTK_COLOR_SELECTION_DIALOG(cdlg)->colorsel),
-					  &gcolor);					  
+    csel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(cdlg));
+    gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(csel), &gcolor);
 
-    button = GTK_COLOR_SELECTION_DIALOG(cdlg)->ok_button;
+    g_object_get(G_OBJECT(cdlg), "ok-button", &button, NULL);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(color_select_callback), cdlg);
 
-    button = GTK_COLOR_SELECTION_DIALOG(cdlg)->cancel_button;
+    g_object_get(G_OBJECT(cdlg), "cancel-button", &button, NULL);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(color_cancel), cdlg);
 
@@ -741,13 +735,10 @@ static void set_graph_font_from_widgets (plot_editor *ed)
 	const char *fname = get_font_filename(tmp);
 	char pngfont[128];
 
-	if (fname != NULL && ptsize > 5 && ptsize < 25) {
+	if (fname != NULL && ptsize >= 4 && ptsize < 25) {
 	    sprintf(pngfont, "%s %d", fname, ptsize);
-	} else {
-	    *pngfont = '\0';
-	}
-
-	set_gretl_png_font(pngfont);
+	    plot_editor_set_fontname(ed, pngfont);
+	} 
     }
 
     g_free(tmp);
@@ -811,7 +802,8 @@ static void try_adding_plotbars (GPT_SPEC *spec, plot_editor *ed)
     if (!err) {
 	const char *fname;
 
-	if (!button_is_active(ed->axis_range[1].isauto)) {
+	if (ed->axis_range[1].isauto != NULL &&
+	    !button_is_active(ed->axis_range[1].isauto)) {
 	    /* we have a manual y-axis setting that may override the
 	       current (ymin, ymax) as read from the plot */
 	    ymin = spec->range[1][0];
@@ -886,7 +878,9 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	}
     }
 
-    if (spec->code == PLOT_REGULAR || spec->code == PLOT_CURVE) {
+    if (spec->code == PLOT_REGULAR || 
+	spec->code == PLOT_CURVE ||
+	spec->code == PLOT_MANY_TS) {
 	k = (spec->flags & GPT_Y2AXIS)? 3 : 2;
 	for (i=0; i<k; i++) {
 	    if (ed->axis_range[i].isauto != NULL) {
@@ -985,13 +979,14 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
     }
 
     if (!err) {
-	if (ed->fontname != NULL) {
-	    set_gretl_png_font(ed->fontname);
-	} else if (ed->ttfcombo != NULL && ed->ttfspin != NULL) {
+	if (ed->ttfcombo != NULL && ed->ttfspin != NULL) {
 	    set_graph_font_from_widgets(ed);
 	}
 	if (ed->fontcheck != NULL && button_is_active(ed->fontcheck)) {
-	    update_persistent_graph_font();
+	    if (ed->spec->fontstr != NULL && *ed->spec->fontstr != '\0') {
+		set_gretl_png_font(ed->spec->fontstr);
+		update_persistent_graph_font();
+	    }
 	}
     }
 
@@ -1071,23 +1066,25 @@ static const char *get_font_filename (const char *showname)
     return NULL;
 }
 
-static void plot_editor_set_fontname (plot_editor *ed, gchar *name)
+static void plot_editor_set_fontname (plot_editor *ed, const char *name)
 {
-    g_free(ed->fontname);
-    ed->fontname = name;
+    free(ed->spec->fontstr);
+    ed->spec->fontstr = gretl_strdup(name);
 }
 
 #ifdef G_OS_WIN32
 
-static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
+static void real_graph_font_selector (GtkButton *button, gpointer p, int type,
+				      const char *default_font)
 {
-    const char *default_font;
     char fontname[128];
 
-    if (type == 1) {
-	default_font = pdf_saver_current_font(p);
-    } else {
-	default_font = gretl_png_font();
+    if (default_font == NULL) {
+	if (type == 1) {
+	    default_font = pdf_saver_current_font(p);
+	} else {
+	    default_font = gretl_png_font();
+	}
     }
 
     strcpy(fontname, default_font);
@@ -1098,10 +1095,12 @@ static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
 
 	gtk_button_set_label(button, title);
 	g_free(title);
-	if (type == 1) {
+	if (type == 0) {
+	    plot_editor_set_fontname(p, fontname);
+	} else if (type == 1) {
 	    pdf_saver_set_fontname(p, fontname);
-	} else {
-	    plot_editor_set_fontname(p, g_strdup(fontname));
+	} else if (type == 2) {
+	    activate_plot_font_choice(p, fontname);
 	}
     }
 }
@@ -1113,6 +1112,8 @@ static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
 #define gtk_font_selection_dialog_get_cancel_button(f) (f->cancel_button)
 #endif
 
+/* callback from OK button in graph font selector */
+
 static void graph_font_selection_ok (GtkWidget *w, GtkFontSelectionDialog *fs)
 {
     gchar *fontname;
@@ -1120,18 +1121,23 @@ static void graph_font_selection_ok (GtkWidget *w, GtkFontSelectionDialog *fs)
     fontname = gtk_font_selection_dialog_get_font_name(fs);
 
     if (fontname != NULL && *fontname != '\0') {
-	GtkWidget *b = g_object_get_data(G_OBJECT(fs), "launcher");
-	gpointer p = g_object_get_data(G_OBJECT(fs), "editor");
-	gint etype = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(fs), "editor-type"));
-	gchar *title = g_strdup_printf(_("font: %s"), fontname);
+	gpointer p = g_object_get_data(G_OBJECT(fs), "parent");
+	gint type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(fs), "parent-type"));
 
-	gtk_button_set_label(GTK_BUTTON(b), title);
-	g_free(title);
-	if (etype == 1) {
-	    pdf_saver_set_fontname(p, fontname);
-	} else {
+	if (type < 2) {
+	    GtkWidget *b = g_object_get_data(G_OBJECT(fs), "launcher");
+	    gchar *title = g_strdup_printf(_("font: %s"), fontname);
+
+	    gtk_button_set_label(GTK_BUTTON(b), title);
+	    g_free(title);
+	}
+
+	if (type == 0) {
 	    plot_editor_set_fontname(p, fontname);
-	    fontname = NULL;
+	} else if (type == 1) {
+	    pdf_saver_set_fontname(p, fontname);
+	} else if (type == 2) {
+	    activate_plot_font_choice(p, fontname);
 	}
     }
 
@@ -1139,10 +1145,10 @@ static void graph_font_selection_ok (GtkWidget *w, GtkFontSelectionDialog *fs)
     gtk_widget_destroy(GTK_WIDGET(fs));
 }
 
-static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
+static void real_graph_font_selector (GtkButton *button, gpointer p, int type,
+				      const char *default_font)
 {
     static GtkWidget *fontsel = NULL;
-    const char *default_font;
     GtkWidget *b;
 
     if (fontsel != NULL) {
@@ -1150,18 +1156,22 @@ static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
         return;
     }
 
-    if (type == 1) {
-	default_font = pdf_saver_current_font(p);
-    } else {
-	default_font = gretl_png_font();
+    if (default_font == NULL) {
+	if (type == 1) {
+	    default_font = pdf_saver_current_font(p);
+	} else {
+	    default_font = gretl_png_font();
+	}
     }
 
     fontsel = gtk_font_selection_dialog_new(_("Font for graphs"));
     gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fontsel), 
 					    default_font);
-    g_object_set_data(G_OBJECT(fontsel), "launcher", button);
-    g_object_set_data(G_OBJECT(fontsel), "editor", p);
-    g_object_set_data(G_OBJECT(fontsel), "editor-type", GINT_TO_POINTER(type));
+    if (button != NULL) {
+	g_object_set_data(G_OBJECT(fontsel), "launcher", button);
+    }
+    g_object_set_data(G_OBJECT(fontsel), "parent", p);
+    g_object_set_data(G_OBJECT(fontsel), "parent-type", GINT_TO_POINTER(type));
 
     gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
 
@@ -1208,12 +1218,17 @@ static int font_is_ok (const char *fname)
 
 static void graph_font_selector (GtkButton *button, gpointer p)
 {
-    real_graph_font_selector(button, p, 0);
+    real_graph_font_selector(button, p, 0, NULL);
 }
 
 void pdf_font_selector (GtkButton *button, gpointer p)
 {
-    real_graph_font_selector(button, p, 1);
+    real_graph_font_selector(button, p, 1, NULL);
+}
+
+void plot_add_font_selector (png_plot *plot, const char *oldfont) 
+{
+    real_graph_font_selector(NULL, plot, 2, oldfont);
 }
 
 static struct font_info *get_gnuplot_ttf_list (int *nf)
@@ -1461,7 +1476,7 @@ static void plot_bars_changed (GtkComboBox *box, plot_editor *ed)
 					  GTK_STOCK_CANCEL, 3,
 					  NULL);
 
-	gretl_dialog_add_message(dlg, msg);
+	gretl_dialog_add_message(dlg, _(msg));
 	gtk_dialog_set_has_separator(GTK_DIALOG(dlg), FALSE);
 	gtk_window_set_keep_above(GTK_WINDOW(dlg), TRUE);  
 	ret = gtk_dialog_run(GTK_DIALOG(dlg));
@@ -1595,6 +1610,8 @@ static int show_bars_check (GPT_SPEC *spec)
 	    spec->pd == 12);
 }
 
+#define plot_has_tics(s) (strcmp(s->xtics, "none") || strcmp(s->ytics, "none"))
+
 static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec) 
 {
     static int aa_ok = 1;
@@ -1732,8 +1749,7 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	}
     } 
 
-    if (1) {
-	/* FIXME does this need to be conditional? */
+    if (plot_has_tics(spec)) {
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	ed->grid_check = gtk_check_button_new_with_label(_("Show grid"));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
@@ -2871,7 +2887,9 @@ static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis)
 	}
     } 
 
-    if (spec->code != PLOT_REGULAR && spec->code != PLOT_CURVE) {
+    if (spec->code != PLOT_REGULAR && 
+	spec->code != PLOT_CURVE &&
+	spec->code != PLOT_MANY_TS) {
 	return;
     }
 
@@ -3048,8 +3066,6 @@ static void plot_editor_destroy (plot_editor *ed)
 	ed->spec->n_labels = ed->old_n_labels;
     }  
 
-    g_free(ed->fontname);
-
     if (ed->barsfile != NULL) {
 	remember_user_bars_file(ed->barsfile);
 	g_free(ed->barsfile);
@@ -3188,7 +3204,6 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
     }
 
     ed->spec = spec;
-    ed->fontname = NULL;
     ed->barsfile = NULL;
 
     ed->dialog = NULL;
@@ -3323,7 +3338,7 @@ GtkWidget *plot_add_editor (png_plot *plot)
 
     /* Close button (do not apply changes) */
     button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect_swapped(G_OBJECT(button), "clicked",
 			     G_CALLBACK(gtk_widget_destroy), 
