@@ -108,7 +108,7 @@ static int optdebug;
 static gchar *param_msg = 
     N_("\nYou may supply the name of a data file on the command line");
 
-#ifdef OLD_GTK
+#ifdef OLD_GTK /* handle absence of GOptionEntry */
 
 struct start_opts {
     const char *longopt;
@@ -186,7 +186,7 @@ static void old_gretl_init (int *pargc, char ***pargv, char *filearg)
     }
 }
 
-#else
+#else /* newer GLib/GTK */
 
 static GOptionEntry options[] = {
     { "run", 'r', 0, G_OPTION_ARG_FILENAME, &optrun, 
@@ -214,7 +214,7 @@ static GOptionEntry options[] = {
     { NULL, '\0', 0, 0, NULL, NULL, NULL },
 };
 
-#endif
+#endif /* old vs new GTK switch */
 
 windata_t *mdata;
 DATAINFO *datainfo;
@@ -390,14 +390,10 @@ static void real_nls_init (void)
 
 static void real_nls_init (void)
 {
-    char gretldir[MAXSTR], localedir[MAXSTR];
+    char localedir[MAXSTR];
     char *loc;
 
-    if (read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", gretldir)) {
-	return;
-    }
-
-    build_path(localedir, gretldir, "locale", NULL);
+    build_path(localedir, gretl_home(), "locale", NULL);
     loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
     bindtextdomain(PACKAGE, localedir);
@@ -431,7 +427,7 @@ static void real_nls_init (void)
     bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
 
-#else /* regular *nix treatment */
+#else /* regular *nix treatment of NLS */
 
 static void real_nls_init (void)
 {
@@ -476,6 +472,18 @@ static void record_filearg (char *targ, const char *src)
 
 #endif
 
+/* callback from within potentially lengthy libgretl
+   operations: try to avoid haveing the GUI become
+   totally unresponsive
+*/
+
+static void gui_show_activity (void)
+{
+    while (gtk_events_pending()) {
+	gtk_main_iteration();
+    }
+}
+
 static int have_data (void)
 {
     return datainfo != NULL && datainfo->v > 0;
@@ -493,6 +501,10 @@ int main (int argc, char **argv)
     GError *opterr = NULL;
 #endif
 
+#ifdef G_OS_WIN32
+    win32_set_gretldir(callname);
+#endif
+
     nls_init();
 
     *tryfile = '\0';
@@ -548,6 +560,7 @@ int main (int argc, char **argv)
     }
 
     set_workdir_callback(gui_set_working_dir);
+    set_show_activity_func(gui_show_activity);
 
     /* allocate data information struct */
     datainfo = datainfo_new();
@@ -933,18 +946,13 @@ void populate_varlist (void)
 {
     static gint check_connected;
     static gint click_connected;
-    GtkTreeView *view;
-    GtkTreeModel *model;
-    GtkTreeStore *store;
+    GtkTreeView *view = GTK_TREE_VIEW(mdata->listbox);
+    GtkTreeModel *model = gtk_tree_view_get_model(view);
+    GtkTreeStore *store = GTK_TREE_STORE(model);
     GtkTreeSelection *select;
     GtkTreeIter iter;    
     char id[8];
-    int pos = 0;
-    int i;
-
-    view = GTK_TREE_VIEW(mdata->listbox);
-    model = gtk_tree_view_get_model(view);
-    store = GTK_TREE_STORE(model);
+    int i, pos = 0;
 
     if (store != NULL) {
 	/* record line position? */
@@ -952,7 +960,6 @@ void populate_varlist (void)
     }
     
     gtk_tree_store_clear(store);
-
     gtk_tree_model_get_iter_first(model, &iter);
 
     for (i=0; i<datainfo->v; i++) {
@@ -1152,7 +1159,6 @@ static float get_gui_scale (void)
     float scale = 1.0;
 
     settings = gtk_settings_get_default();
-
     g_object_get(G_OBJECT(settings), "gtk-font-name", &fontname, NULL);
 
     if (fontname != NULL) {
@@ -1173,7 +1179,8 @@ mainwin_config (GtkWidget *w, GdkEventConfigure *event, gpointer p)
     mainwin_width = event->width;
     mainwin_height = event->height;
 
-    gdk_window_get_root_origin(mdata->main->window, &main_x, &main_y);
+    gdk_window_get_root_origin(gtk_widget_get_window(mdata->main), 
+			       &main_x, &main_y);
 
     return FALSE;
 }
@@ -1252,14 +1259,13 @@ static GtkWidget *make_main_window (void)
     gtk_window_set_title(GTK_WINDOW(mdata->main), "gretl");
     gtk_window_set_default_size(GTK_WINDOW(mdata->main), 
 				mainwin_width, mainwin_height);
+
 #ifndef G_OS_WIN32
-    g_signal_connect_after(G_OBJECT(mdata->main), "realize", 
-			   G_CALLBACK(set_wm_icon), 
-			   NULL);
+    set_wm_icon(mdata->main);
 #endif
 
     main_vbox = gtk_vbox_new(FALSE, 4);
-    gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 4); /* was 8 */
+    gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 4);
     gtk_container_add(GTK_CONTAINER(mdata->main), main_vbox);
     g_object_set_data(G_OBJECT(mdata->main), "vbox", main_vbox);
 
@@ -1436,7 +1442,7 @@ GtkActionEntry main_entries[] = {
     { "Data", NULL, N_("_Data"), NULL, NULL, NULL },
     { "DataSelectAll", NULL, N_("Select _all"), "<control>A", NULL, G_CALLBACK(mdata_select_all) },
     { "DefineList", NULL, N_("Define or edit _list..."), NULL, NULL, G_CALLBACK(gui_define_list) },
-    { "SelectList", NULL, N_("_Select listed variables..."), NULL, NULL, G_CALLBACK(mdata_select_list) },
+    { "SelectList", NULL, N_("_Set selection from list..."), NULL, NULL, G_CALLBACK(mdata_select_list) },
     { "DisplayValues", NULL, N_("_Display values"), NULL, NULL, G_CALLBACK(display_selected) },
     { "EditValues", NULL, N_("_Edit values"), NULL, NULL, G_CALLBACK(spreadsheet_edit) },
     { "AddObs", NULL, N_("_Add observations..."), NULL, NULL, G_CALLBACK(do_add_obs) },
@@ -1524,18 +1530,21 @@ GtkActionEntry main_entries[] = {
     { "Gini", NULL, N_("_Gini coefficient"), NULL, NULL, G_CALLBACK(do_gini) },
     { "rmplot", NULL, N_("_Range-mean graph"), NULL, NULL, G_CALLBACK(do_range_mean) },
     { "VarTSPlot", NULL, N_("_Time series plot"), NULL, NULL, G_CALLBACK(ts_plot_callback) },
+    { "URTests", NULL, N_("_Unit root tests"), NULL, NULL, NULL },
+    { "adf", NULL, N_("_Augmented Dickey-Fuller test"), NULL, NULL, G_CALLBACK(ur_callback) },
+    { "dfgls", NULL, N_("ADF-GLS test"), NULL, NULL, G_CALLBACK(ur_callback) },
+    { "kpss", NULL, N_("_KPSS test"), NULL, NULL, G_CALLBACK(ur_callback) },
+    { "levinlin", NULL, N_("_Levin-Lin-Chu test"), NULL, NULL, G_CALLBACK(ur_callback) },
+    { "fractint", NULL, N_("_Fractional integration"), NULL, NULL, G_CALLBACK(do_fractint) },
     { "corrgm", NULL, N_("_Correlogram"), NULL, NULL, G_CALLBACK(do_corrgm) },
-    { "Spectrum", NULL, N_("_Spectrum"), NULL, NULL, NULL },
-    { "Pergm", NULL, N_("Sample _periodogram"), NULL, NULL, G_CALLBACK(do_pergm) },
-    { "Bartlett", NULL, N_("_Bartlett lag window"), NULL, NULL, G_CALLBACK(do_pergm) },
-    { "ADF", NULL, N_("_Augmented Dickey-Fuller test"), NULL, NULL, G_CALLBACK(ur_callback) },
-    { "DFGLS", NULL, N_("ADF-GLS test"), NULL, NULL, G_CALLBACK(ur_callback) },
-    { "KPSS", NULL, N_("_KPSS test"), NULL, NULL, G_CALLBACK(ur_callback) },
+    { "pergm", NULL, N_("_Periodogram"), NULL, NULL, G_CALLBACK(do_pergm) },
     { "Filter", NULL, N_("_Filter"), NULL, NULL, NULL },
     { "FilterSMA", NULL, N_("_Simple moving average"), NULL, NULL, G_CALLBACK(filter_callback) },
     { "FilterEMA", NULL, N_("_Exponential moving average"), NULL, NULL, G_CALLBACK(filter_callback) },
     { "FilterHP", NULL, N_("_Hodrick-Prescott"), NULL, NULL, G_CALLBACK(filter_callback) },
     { "FilterBK", NULL, N_("_Baxter-King"), NULL, NULL, G_CALLBACK(filter_callback) },
+    { "FilterBW", NULL, N_("_Butterworth"), NULL, NULL, G_CALLBACK(filter_callback) },
+    { "FilterPoly", NULL, N_("_Polynomial trend"), NULL, NULL, G_CALLBACK(filter_callback) },
     { "FilterFD", NULL, N_("_Fractional difference"), NULL, NULL, G_CALLBACK(filter_callback) },
     
 #ifdef HAVE_X12A
@@ -1559,9 +1568,7 @@ GtkActionEntry main_entries[] = {
     { "LinearModels", NULL, N_("Other _linear models"), NULL, NULL, NULL },
     { "wls", NULL, N_("_Weighted Least Squares..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "hsk", NULL, N_("H_eteroskedasticity corrected..."), NULL, NULL, G_CALLBACK(model_callback) },
-#ifdef ENABLE_GMP
     { "mpols", NULL, N_("High _precision OLS..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-#endif
     { "anova", NULL, N_("ANOVA..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "TSModels", NULL, N_("_Time series"), NULL, NULL, NULL },
     { "CORC", NULL, N_("_Cochrane-Orcutt..."), NULL, NULL, G_CALLBACK(model_callback) },
@@ -1581,7 +1588,8 @@ GtkActionEntry main_entries[] = {
     { "panel", NULL, N_("_Fixed or random effects..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "PANEL_WLS", NULL, N_("_Weighted least squares..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "PANEL_B", NULL, N_("_Between model..."), NULL, NULL, G_CALLBACK(model_callback) },
-    { "arbond", NULL, N_("_Arellano-Bond..."), NULL, NULL, G_CALLBACK(model_callback) },
+    { "dpanel", NULL, N_("_Dynamic panel model..."), NULL, NULL, G_CALLBACK(model_callback) },
+
     { "NonlinearModels", NULL, N_("_Nonlinear models"), NULL, NULL, NULL },
     { "logit", NULL, N_("_Logit"), NULL, NULL, NULL }, 
     { "blogit", NULL, N_("_Binary..."), NULL, NULL, G_CALLBACK(model_callback) }, 
@@ -1590,6 +1598,7 @@ GtkActionEntry main_entries[] = {
     { "probit", NULL, N_("_Probit"), NULL, NULL, NULL }, 
     { "bprobit", NULL, N_("_Binary..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "oprobit", NULL, N_("_Ordered..."), NULL, NULL, G_CALLBACK(model_callback) }, 
+    { "biprobit", NULL, N_("Bi_variate..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "tobit", NULL, N_("To_bit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "heckit", NULL, N_("_Heckit..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "countmod", NULL, N_("_Count data..."), NULL, NULL, G_CALLBACK(model_callback) }, 
@@ -1662,9 +1671,10 @@ static void add_conditional_items (windata_t *vwin)
 			      FALSE);
     }
 
-#if 1 /* not quite yet */
-    maybe_add_package_to_menu("gig", "/menubar/Model/TSModels", vwin);
-#endif
+    /* not public yet */
+    if (getenv("GRETL_TEST_GFN") != NULL) {
+	maybe_add_packages_to_menus(vwin);
+    }
 }
 
 /* retrieve the XML description of the main window menus */
@@ -1675,7 +1685,7 @@ static gchar *get_main_ui (void)
     gchar *main_ui = NULL;
 
     sprintf(fname, "%sui%cgretlmain.xml", gretl_home(), SLASH);
-    gretl_file_get_contents(fname, &main_ui);
+    gretl_file_get_contents(fname, &main_ui, NULL);
 
     return main_ui;
 }
@@ -1683,6 +1693,7 @@ static gchar *get_main_ui (void)
 static int set_up_main_menu (void)
 {
     GtkActionGroup *actions;
+    GtkWidget *dataitem;
     gchar *main_ui = NULL;
     GError *error = NULL;
 
@@ -1713,6 +1724,10 @@ static int set_up_main_menu (void)
     g_free(main_ui);
     mdata->mbar = gtk_ui_manager_get_widget(mdata->ui, "/menubar");
 
+    dataitem = gtk_ui_manager_get_widget(mdata->ui, "/menubar/Data");
+    g_signal_connect(G_OBJECT(dataitem), "activate",
+		     G_CALLBACK(check_var_labels_state), mdata);
+
     return 0;
 }
 
@@ -2053,16 +2068,22 @@ int gretl_fork (const char *progvar, const char *arg)
 
 #ifndef G_OS_WIN32
 
-void set_wm_icon (GtkWidget *w, gpointer data)
+void set_wm_icon (GtkWidget *w)
 {
-    GdkPixmap *icon;
+    GdkPixbuf *icon = gdk_pixbuf_new_from_xpm_data(gretl_xpm);
 
-    icon = gdk_pixmap_create_from_xpm_d(w->window, NULL, NULL, gretl_xpm);
-    gdk_window_set_icon(w->window, NULL, icon, NULL);
+    gtk_window_set_icon(GTK_WINDOW(w), icon);
 }
 
 #endif
 
+static int has_db_suffix (const char *fname)
+{
+    return has_suffix(fname, ".bin") ||
+	has_suffix(fname, ".rat") ||
+	has_suffix(fname, ".bn7");
+}
+
 /* Drag 'n' drop: respond to data dropped into the main window */
 
 static void  
@@ -2075,15 +2096,21 @@ mdata_handle_drag  (GtkWidget *widget,
 		    guint time,
 		    gpointer p)
 {
+    GdkAtom type = gtk_selection_data_get_data_type(data);
+    const guchar *seldata = NULL;
     gchar *dfname;
     char tmp[MAXLEN];
     int pos, skip = 5;
     int ftype = 0;
 
+    if (data != NULL) {
+	seldata = gtk_selection_data_get_data(data);
+    }
+
     /* handle drag of pointer from database window */
     if (info == GRETL_DBSERIES_PTR && data != NULL && 
-	data->type == GDK_SELECTION_TYPE_INTEGER) {
-	import_db_series(*(void **) data->data);
+	type == GDK_SELECTION_TYPE_INTEGER) {
+	import_db_series(*(void **) seldata);
 	return;
     }
 
@@ -2092,7 +2119,7 @@ mdata_handle_drag  (GtkWidget *widget,
     }
 
     /* ignore the wrong sort of data */
-    if (data == NULL || (dfname = (gchar *) data->data) == NULL || 
+    if (data == NULL || (dfname = (gchar *) seldata) == NULL || 
 	strlen(dfname) <= 5 || strncmp(dfname, "file:", 5)) {
 	return;
     }
@@ -2120,7 +2147,9 @@ mdata_handle_drag  (GtkWidget *widget,
     strcpy(tryfile, tmp);
 #endif
 
-    if (has_suffix(tryfile, ".gretl") && gretl_is_pkzip_file(tryfile)) {
+    if (has_db_suffix(tryfile)) {
+	open_named_db_index(tryfile);
+    } else if (has_suffix(tryfile, ".gretl") && gretl_is_pkzip_file(tryfile)) {
 	verify_open_session();
     } else if ((ftype = script_type(tryfile))) {
 	do_open_script(ftype);
@@ -2178,7 +2207,7 @@ main_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 {
     GdkModifierType mods = widget_get_pointer_mask(w);
 
-    if (mods & GDK_BUTTON3_MASK) {
+    if (RIGHT_CLICK(mods)) {
 	/* ignore all but right-clicks */
 	int selcount = vwin_selection_count(mdata, NULL);
 
