@@ -44,6 +44,7 @@ enum extra_cmds {
     VIEW_MODELTABLE,
     VIEW_PKG_INFO,
     VIEW_PKG_CODE,
+    VIEW_BUNDLE,
     VIEW_FILE,
     DATA_REPORT,
     SCRIPT_OUT,
@@ -105,6 +106,8 @@ enum extra_cmds {
     IV_GMM,
     BFGS_CONFIG,
     COUNTMOD,
+    BWFILTER,
+    POLYWEIGHTS,
     GUI_CMD_MAX
 };
 
@@ -171,6 +174,7 @@ enum file_ops {
     SAVE_BOOT_DATA,
     SAVE_MARKERS,
     SAVE_LABELS,
+    END_SAVE_OTHER, /* marker for end of other user-file saving */
     EDIT_FUNCTIONS,
     SET_PROG,
     SET_DIR,
