@@ -141,6 +141,8 @@ static struct gui_help_item gui_help_items[] = {
     { SAVE_LABELS,    "save-labels" }, /* FIXME */
     { OPEN_LABELS,    "add-labels" },  /* FIXME */
     { COUNTMOD,       "count-model" },
+    { BWFILTER,       "bwfilter" },
+    { POLYWEIGHTS,    "polyweights" },
     { -1,             NULL },
 };
 
@@ -444,7 +446,7 @@ static GtkTreeStore *make_help_topics_tree (int role)
 	return NULL;
     } 
 
-    err = gretl_file_get_contents(fname, &buf);
+    err = gretl_file_get_contents(fname, &buf, NULL);
     if (err) {
 	return NULL;
     }
@@ -1877,7 +1879,7 @@ static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 
     /* find button */
     button = gtk_button_new_from_stock(GTK_STOCK_FIND);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(findfunc), find_dialog);
@@ -1895,7 +1897,14 @@ enum {
     SPANISH
 };
 
-static int get_writable_path (char *path, const char *fname)
+enum {
+    GRETL_GUIDE = 1,
+    GRETL_REF,
+    GNUPLOT_REF,
+    GFN_DOC
+};
+
+static int get_writable_path (char *path, const char *fname, int code)
 {
     static int sysdoc_writable = -1;
     static int userdoc_writable = -1;
@@ -1905,7 +1914,11 @@ static int get_writable_path (char *path, const char *fname)
     int err = 0;
 
     if (sysdoc_writable == 1) {
-	sprintf(path, "%sdoc%c%s", gretldir, SLASH, fname);
+	if (code == GFN_DOC) {
+	    sprintf(path, "%sfunctions%cdocs%c%s", gretldir, SLASH, SLASH, fname);
+	} else {
+	    sprintf(path, "%sdoc%c%s", gretldir, SLASH, fname);
+	}
 	return 0;
     } else if (userdoc_writable == 1) {
 	sprintf(path, "%sdoc%c%s", dotdir, SLASH, fname);
@@ -1914,9 +1927,14 @@ static int get_writable_path (char *path, const char *fname)
 
     if (sysdoc_writable < 0) {
 	sysdoc_writable = 0;
-	sprintf(path, "%sdoc", gretldir);
+	if (code == GFN_DOC) {
+	    sprintf(path, "%sfunctions%cdocs", gretldir, SLASH);
+	} else {
+	    sprintf(path, "%sdoc", gretldir);
+	}
 	if (gretl_mkdir(path) == 0) {
-	    sprintf(path, "%sdoc%c%s", gretldir, SLASH, fname);
+	    strcat(path, SLASHSTR);
+	    strcat(path, fname);
 	    fp = gretl_fopen(path, "w");
 	    if (fp != NULL) {
 		sysdoc_writable = 1;
@@ -1948,12 +1966,6 @@ static int get_writable_path (char *path, const char *fname)
     return err;
 }
 
-enum {
-    GRETL_GUIDE = 1,
-    GRETL_REF,
-    GNUPLOT_REF
-};
-
 static int find_or_download_pdf (int code, int i, char *fullpath)
 {
     const char *guide_files[] = {
@@ -1969,7 +1981,9 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	"gretl-ref-es.pdf"  
     };
     const char *fname;
+    char *tmp = NULL;
     FILE *fp;
+    int gotit = 0;
     int err = 0;
 
     if (i < 0 || i > 3) {
@@ -1982,44 +1996,62 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	fname = ref_files[i];
     } else if (code == GNUPLOT_REF) {
 	fname = "gnuplot.pdf";
+    } else if (code == GFN_DOC) {
+	tmp = gretl_strdup(fullpath);
+	fname = tmp;
     } else {
 	return E_DATA;
     }
 
     /* is the file available in public dir? */
-    sprintf(fullpath, "%sdoc%c%s", gretl_home(), SLASH, fname);
+    if (code == GFN_DOC) {
+	sprintf(fullpath, "%sfunctions%cdocs%c%s", gretl_home(), SLASH, 
+		SLASH, fname);
+    } else {
+	sprintf(fullpath, "%sdoc%c%s", gretl_home(), SLASH, fname);
+    }
     fp = gretl_fopen(fullpath, "r");
     if (fp != NULL) {
 	fclose(fp);
-	return 0;
+	gotit = 1;
     }
 
-    /* or maybe in user dir? */
-    sprintf(fullpath, "%sdoc%c%s", gretl_dotdir(), SLASH, fname);
-    fp = gretl_fopen(fullpath, "r");
-    if (fp != NULL) {
-	fclose(fp);
-	return 0;
-    }
-
-    /* check for download location */
-    err = get_writable_path(fullpath, fname);
-
-    /* do actual download */
-    if (!err) {
-	err = retrieve_manfile(fname, fullpath);
-    }
-
-    if (err) {
-	const char *buf = gretl_errmsg_get();
-
-	if (*buf) {
-	    errbox(buf);
-	} else {
-	    errbox(_("Failed to download file"));
+    if (!gotit) {
+	/* or maybe in user dir? */
+	sprintf(fullpath, "%sdoc%c%s", gretl_dotdir(), SLASH, fname);
+	fp = gretl_fopen(fullpath, "r");
+	if (fp != NULL) {
+	    fclose(fp);
+	    gotit = 1;
 	}
     }
 
+    if (!gotit) {
+	/* check for download location */
+	err = get_writable_path(fullpath, fname, code);
+
+	/* do actual download */
+	if (!err) {
+	    if (code == GFN_DOC) {
+		err = retrieve_gfndoc(fname, fullpath);
+	    } else {
+		err = retrieve_manfile(fname, fullpath);
+	    }
+	}
+
+	if (err) {
+	    const char *buf = gretl_errmsg_get();
+
+	    if (*buf) {
+		errbox(buf);
+	    } else {
+		errbox(_("Failed to download file"));
+	    }
+	}
+    }
+
+    free(tmp);
+
     return err;
 }
 
@@ -2039,9 +2071,11 @@ void display_pdf_help (GtkAction *action)
     char fname[FILENAME_MAX];
     int err, code = GRETL_GUIDE;
 
-    if (action != NULL && strcmp(gtk_action_get_name(action), "UserGuide")) {
-	/* PDF command ref wanted */
-	code = GRETL_REF;
+    if (action != NULL) {
+	if (strcmp(gtk_action_get_name(action), "UserGuide")) {
+	    /* PDF command ref wanted */
+	    code = GRETL_REF;
+	} 
     }
 
     err = find_or_download_pdf(code, get_manpref(), fname);
@@ -2062,3 +2096,18 @@ void display_gnuplot_help (void)
 	show_pdf(fname);
     }
 }
+
+int display_gfn_help (const char *pdfname)
+{
+    char fname[FILENAME_MAX];
+    int err;
+
+    strcpy(fname, pdfname);
+    err = find_or_download_pdf(GFN_DOC, 0, fname);
+
+    if (!err) {
+	show_pdf(fname);
+    }
+
+    return err;
+}
