@@ -59,6 +59,8 @@ int gretl_command_strcat (const char *s);
 
 int record_command_line (const char *s);
 
+int record_command_verbatim (const char *s);
+
 int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
 gint bufopen (PRN **pprn);
@@ -191,6 +193,8 @@ void do_genr (GtkWidget *w, dialog_t *dlg);
 
 void do_model_genr (GtkWidget *w, dialog_t *dlg);
 
+void do_selector_genr (GtkWidget *w, dialog_t *dlg);
+
 void do_global_setmiss (GtkWidget *w, dialog_t *dlg);
 
 void do_variable_setmiss (GtkWidget *w, dialog_t *dlg);
@@ -211,6 +215,8 @@ void residual_correlogram (GtkAction *action, gpointer p);
 
 void do_pergm (GtkAction *action);
 
+void do_fractint (GtkAction *action);
+
 void residual_periodogram (GtkAction *action, gpointer p);
 
 void residual_qq_plot (GtkAction *action, gpointer p);
@@ -243,6 +249,9 @@ void add_system_resid (GtkAction *action, gpointer p);
 
 int save_fit_resid (MODEL *pmod, int code);
 
+int save_bundled_series (const double *x, const char *key,
+			 const char *note);
+
 void add_model_stat (MODEL *pmod, int which);
 
 void resid_plot (GtkAction *action, gpointer p);
@@ -305,6 +314,14 @@ int gui_exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo);
 
 int check_and_record_command (void);
 
+#if USE_GTK_SPINNER
+
+void start_wait_for_output (GtkWidget *w, int big);
+
+void stop_wait_for_output (GtkWidget *w);
+
+#endif
+
 /* other */
 
 int menu_op_wrapper (selector *sr);
