@@ -189,14 +189,22 @@ void window_list_state (gboolean s)
 			(d)->structure == SPECIAL_TIME_SERIES || \
 			(d)->structure == STACKED_TIME_SERIES)
 
+
+
 void time_series_menu_state (gboolean s)
 {
     gboolean sx = extended_ts(datainfo);
+    gboolean panel = dataset_is_panel(datainfo);
+    gboolean ur = s;
 
     if (mdata->ui == NULL) {
 	return;
     }
 
+    if (panel) {
+	ur = datainfo->pd > 5;
+    }
+
     /* File menu */
     flip(mdata->ui, "/menubar/File/SaveDataAs/SaveAsDb", DATASET_DB_OK(datainfo));
 
@@ -206,11 +214,15 @@ void time_series_menu_state (gboolean s)
     flip(mdata->ui, "/menubar/Variable/VarTSPlot", sx);
 
     /* Variable menu */
+    flip(mdata->ui, "/menubar/Variable/URTests", ur); 
+    if (ur && !s) {
+	flip(mdata->ui, "/menubar/Variable/URTests/fractint", s);
+    }
+    if (ur && !panel) {
+	flip(mdata->ui, "/menubar/Variable/URTests/levinlin", panel);
+    }
     flip(mdata->ui, "/menubar/Variable/corrgm", s);
-    flip(mdata->ui, "/menubar/Variable/Spectrum", s);
-    flip(mdata->ui, "/menubar/Variable/ADF", s);
-    flip(mdata->ui, "/menubar/Variable/DFGLS", s);
-    flip(mdata->ui, "/menubar/Variable/KPSS", s);
+    flip(mdata->ui, "/menubar/Variable/pergm", s);
     flip(mdata->ui, "/menubar/Variable/Filter", s);
 #ifdef HAVE_X12A
     flip(mdata->ui, "/menubar/Variable/X12A", get_x12a_ok());
@@ -303,6 +315,19 @@ void main_menus_enable (gboolean s)
     }
 }
 
+void check_var_labels_state (GtkMenuItem *item, gpointer p)
+{
+    gboolean s = FALSE;
+
+    if (datainfo != NULL) {
+	if (datainfo->v > 2 || strcmp(datainfo->varname[1], "index")) {
+	    s = TRUE;
+	}
+    }
+
+    flip(mdata->ui, "/menubar/Data/VarLabels", s);
+}
+
 static gint var_popup_click (GtkWidget *w, gpointer p)
 {
     gchar *item = (gchar *) p;
@@ -322,7 +347,7 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
 	do_gini();
     else if (!strcmp(item, _("Correlogram")))
 	do_corrgm();
-    else if (!strcmp(item, _("Spectrum"))) 
+    else if (!strcmp(item, _("Periodogram"))) 
 	do_pergm(NULL);
     else if (!strcmp(item, _("ARIMA model"))) {
 	selector_set_varnum(v);
@@ -405,7 +430,7 @@ GtkWidget *build_var_popup (void)
 	N_("Frequency distribution"),
 	N_("Boxplot"),
 	N_("Correlogram"),
-	N_("Spectrum"),
+	N_("Periodogram"),
 	N_("Edit attributes"),
 	N_("Edit values"),
 	N_("Copy to clipboard"),
