@@ -63,7 +63,7 @@ int real_add_text_to_session (PRN *prn, const char *tname);
 
 void save_output_as_text_icon (windata_t *vwin);
 
-int add_graph_to_session (char *fname, char *fullname, int type);
+int gui_add_graph_to_session (char *fname, char *fullname, int type);
 
 int cli_add_graph_to_session (const char *fname, const char *gname,
 			      GretlObjType type);
