@@ -53,7 +53,7 @@ static int check_graph_file (const char *fname, int type)
    constructing the graph list de novo.)
 */
 
-static void normalize_graph_filename (char *fname, int *gnum, int *pnum)
+static void normalize_graph_filename (char *fname, int gnum)
 {
     char s[16];
     int i;
@@ -63,20 +63,10 @@ static void normalize_graph_filename (char *fname, int *gnum, int *pnum)
 	char newname[MAXLEN];
 	gchar *tmp = NULL;
 
-	if (!strcmp(s, "plot")) {
-	    if (i > *pnum) {
-		session_file_make_path(oldname, fname);
-		tmp = g_strdup_printf("%s.%d", s, *pnum);
-		session_file_make_path(newname, tmp);
-	    }
-	    *pnum += 1;
-	} else {
-	    if (i > *gnum) {
-		session_file_make_path(oldname, fname);
-		tmp = g_strdup_printf("%s.%d", s, *gnum);
-		session_file_make_path(newname, tmp);
-	    }
-	    *gnum += 1;
+	if (i != gnum && (!strcmp(s, "graph") || !strcmp(s, "plot"))) {
+	    session_file_make_path(oldname, fname);
+	    tmp = g_strdup_printf("graph.%d", gnum);
+	    session_file_make_path(newname, tmp);
 	}
 
 	if (tmp != NULL) {
@@ -90,8 +80,8 @@ static void normalize_graph_filename (char *fname, int *gnum, int *pnum)
 static int restore_session_graphs (xmlNodePtr node)
 {
     xmlNodePtr cur;
-    int gnum = 1, pnum = 1;
     int inpage = 0;
+    int gnum = 0;
     int errs = 0;
 
     /* reset prior to parsing */
@@ -124,7 +114,7 @@ static int restore_session_graphs (xmlNodePtr node)
 		fprintf(stderr, "checking '%s'\n", name);
 		err = check_graph_file((const char *) fname, type);
 		if (!err) {
-		    normalize_graph_filename((char *) fname, &gnum, &pnum);
+		    normalize_graph_filename((char *) fname, ++gnum);
 		}
 	    } 
 	}
@@ -672,11 +662,28 @@ static int write_settings_file (char *fullname)
     return libset_write_script(fullname);
 }
 
+static char *get_xmlname (char *objname, int *err)
+{
+    char *ret;
+
+    if (gretl_xml_validate(objname)) {
+	ret = objname;
+    } else {
+	ret = gretl_xml_encode(objname);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return ret;
+}
+
 static int write_session_xml (const char *datname)
 {
     MODEL *pmod;
     char fname[MAXLEN];
     char tmpname[MAXLEN];
+    char *objname, *xmlname;
     FILE *fp, *fq;
     int nmodels;
     int tabmodels;
@@ -737,16 +744,24 @@ static int write_session_xml (const char *datname)
 	    err = E_FOPEN;
 	} else {
 	    sprintf(tmpname, "model.%d", modnum++);
+	    objname = session.models[i]->name;
+	    xmlname = get_xmlname(objname, &err);
+	    if (err) {
+		break;
+	    }
 	    if (type == GRETL_OBJ_EQN) {
 		tablepos = model_table_position(ptr);
 	    }
 	    if (tablepos > 0) {
 		fprintf(fp, "  <session-model name=\"%s\" fname=\"%s\" type=\"%d\" tablepos=\"%d\"/>\n", 
-			session.models[i]->name, tmpname, type, tablepos);
+			xmlname, tmpname, type, tablepos);
 	    } else {
 		fprintf(fp, "  <session-model name=\"%s\" fname=\"%s\" type=\"%d\"/>\n", 
-			session.models[i]->name, tmpname, type);
-	    }		
+			xmlname, tmpname, type);
+	    }	
+	    if (xmlname != objname) {
+		free(xmlname);
+	    }
 	    gretl_xml_header(fq);
 	    sflags = model_save_flags(ptr, type);
 	    if (type == GRETL_OBJ_EQN) {
@@ -771,9 +786,21 @@ static int write_session_xml (const char *datname)
 	    } else {
 		int tablepos = model_table_position(pmod);
 
+		if (pmod->name == NULL) {
+		    objname = xmlname = NULL;
+		} else {
+		    objname = pmod->name;
+		    xmlname = get_xmlname(objname, &err);
+		}
+		if (err) {
+		    break;
+		}
 		sprintf(tmpname, "model.%d", modnum++);
 		fprintf(fp, "  <session-model name=\"%s\" fname=\"%s\" type=\"%d\" tablepos=\"%d\"/>\n", 
-			(pmod->name != NULL)? pmod->name : "none", tmpname, GRETL_OBJ_EQN, tablepos);
+			(xmlname != NULL)? xmlname : "none", tmpname, GRETL_OBJ_EQN, tablepos);
+		if (xmlname != NULL && xmlname != objname) {
+		    free(xmlname);
+		}
 		gretl_xml_header(fq);
 		gretl_model_serialize(pmod, model_save_flags(pmod, GRETL_OBJ_EQN), 
 				      fq);
@@ -792,10 +819,16 @@ static int write_session_xml (const char *datname)
 
     fprintf(fp, " <graphs count=\"%d\">\n", session.ngraphs);
     for (i=0; i<session.ngraphs; i++) {
-	fprintf(fp, "  <session-graph name=\"%s\" fname=\"%s\" "
-		"ID=\"%d\" type=\"%d\"", 
-		session.graphs[i]->name, session.graphs[i]->fname,
-		session.graphs[i]->ID, session.graphs[i]->type);
+	objname = session.graphs[i]->name;
+	xmlname = get_xmlname(objname, &err);
+	if (err) {
+	    break;
+	}
+	fprintf(fp, "  <session-graph name=\"%s\" fname=\"%s\" type=\"%d\"", 
+		xmlname, session.graphs[i]->fname, session.graphs[i]->type);
+	if (xmlname != objname) {
+	    free(xmlname);
+	}
 	if (in_graph_page(session.graphs[i]->fname)) {
 	    fputs(" inpage=\"1\"/>\n", fp);
 	} else {
@@ -806,7 +839,15 @@ static int write_session_xml (const char *datname)
 
     fprintf(fp, " <texts count=\"%d\">\n", session.ntexts);
     for (i=0; i<session.ntexts; i++) {
-	fprintf(fp, "  <session-text name=\"%s\">", session.texts[i]->name);
+	objname = session.texts[i]->name;
+	xmlname = get_xmlname(objname, &err);
+	if (err) {
+	    break;
+	}	
+	fprintf(fp, "  <session-text name=\"%s\">", xmlname);
+	if (xmlname != objname) {
+	    free(xmlname);
+	}	
 	gretl_xml_put_raw_string(session.texts[i]->buf, fp);
 	fputs("</session-text>\n", fp);
     }    
