@@ -62,18 +62,12 @@ static int read_gretlrc (void);
 #ifdef G_OS_WIN32
 static char fixedfontname[MAXLEN] = "Courier New 10";
 #else
-# ifdef OSX_BUILD
-static char fixedfontname[MAXLEN] = "Monospace 12";
-# else
 static char fixedfontname[MAXLEN] = "Monospace 10";
-# endif
 #endif
 
 #if defined(G_OS_WIN32)
 static char appfontname[MAXLEN] = "tahoma 8";
-#elif defined(OSX_BUILD)
-static char appfontname[MAXLEN] = "Sans 12";
-# else
+#else
 static char appfontname[MAXLEN] = "Sans 10";
 #endif
 
@@ -878,7 +872,7 @@ int options_dialog (int page, const char *varname, GtkWidget *parent)
     make_prefs_tab(notebook, TAB_VCV);
     make_prefs_tab(notebook, TAB_MAN);
 
-    hbox = GTK_DIALOG(dialog)->action_area;
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
 
     /* Apply button */
     button = apply_button(hbox);
@@ -923,7 +917,7 @@ static void flip_sensitive (GtkWidget *w, gpointer data)
 {
     GtkWidget *entry = GTK_WIDGET(data);
     
-    gtk_widget_set_sensitive(entry, GTK_TOGGLE_BUTTON(w)->active);
+    gtk_widget_set_sensitive(entry, button_is_active(w));
 }
 
 void set_path_callback (char *setvar, char *setting)
@@ -1074,7 +1068,7 @@ get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse,
 
 static void radio_change_value (GtkWidget *w, int *v)
 {
-    if (GTK_TOGGLE_BUTTON(w)->active) {
+    if (button_is_active(w)) {
 	gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
 
 	*v = i;
@@ -1190,7 +1184,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    /* special case: link between toggle and preceding entry */
 	    if (rc->len && !(rc->flags & FIXSET)) {
 		gtk_widget_set_sensitive(rc_vars[i-1].widget,
-					 GTK_TOGGLE_BUTTON(rc->widget)->active);
+					 button_is_active(rc->widget));
 		g_signal_connect(G_OBJECT(rc->widget), "clicked",
 				 G_CALLBACK(flip_sensitive),
 				 rc_vars[i-1].widget);
@@ -1906,7 +1900,8 @@ static int get_network_settings (void)
 
 /* Try reading user settings from .gretl2rc in appdata directory: if
    this succeeds it will pre-empt reading from the registry --
-   except that we'll respect the registry entry for gretldir.
+   except that we'll respect the registry entry (or perhaps
+   argv[0] at startup) for gretldir.
 */
 
 static void win32_read_gretlrc (void) 
@@ -1997,7 +1992,11 @@ int read_win32_config (int debug)
 
 	*value = '\0';
 
-	if (rcvar->flags & MACHSET) {
+	/* note: by now we have already determined gretldir as
+	   best we can; don't overwrite its setting 
+	*/
+
+	if ((rcvar->flags & MACHSET) && strcmp(rcvar->key, "gretldir")) {
 	    regerr = read_reg_val(HKEY_LOCAL_MACHINE, 
 				  get_reg_base(rcvar->key),
 				  rcvar->key, 
@@ -2296,6 +2295,7 @@ static void
 add_wdir_content (GtkWidget *dialog, struct wdir_setter *wset) 
 {
     GtkWidget *hbox, *vbox, *w = NULL;
+    GtkWidget *entry;
     GSList *group = NULL;
     GList *list = NULL;
     char tmp[MAXLEN];
@@ -2315,7 +2315,9 @@ add_wdir_content (GtkWidget *dialog, struct wdir_setter *wset)
     gtk_container_add(GTK_CONTAINER(hbox), w);
     set_combo_box_strings_from_list(GTK_COMBO_BOX(w), list);
     set_combo_box_default_text(GTK_COMBO_BOX(w), tmp);
-    gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(w))), 32);
+    entry = gtk_bin_get_child(GTK_BIN(w));
+    gtk_entry_set_width_chars(GTK_ENTRY(entry), 32);
+    gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
     wset->wdir_combo = w;
     w = gtk_button_new_with_label(_("Browse..."));
     g_signal_connect(G_OBJECT(w), "clicked",
@@ -2390,6 +2392,7 @@ apply_wdir_changes (GtkWidget *w, struct wdir_setter *wset)
     }
 
     err = set_gretl_work_dir(tmp);
+
     if (err) {
 	gui_errmsg(err);
 	delete_from_filelist(FILE_LIST_WDIR, tmp);
@@ -2401,6 +2404,10 @@ apply_wdir_changes (GtkWidget *w, struct wdir_setter *wset)
 
     usecwd = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wset->cwd_radio));
     keep_folder = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wset->keep_radio));
+
+    if (!err) {
+	gtk_widget_destroy(wset->dialog);
+    }
 }
 
 void working_dir_dialog (void) 
@@ -2435,11 +2442,9 @@ void working_dir_dialog (void)
 		     dialog);
 
     button = ok_button(hbox);
+    gtk_widget_grab_default(button);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(apply_wdir_changes), &wset);
-    g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(delete_widget), 
-		     dialog);
 
     context_help_button(hbox, WORKDIR);
 
