@@ -26,6 +26,7 @@
 #include "session.h"
 #include "winstack.h"
 #include "toolbar.h"
+#include "cmdstack.h"
 #include "usermat.h"
 #include "matrix_extra.h"
 #include "gretl_scalar.h"
@@ -599,7 +600,7 @@ spreadsheet_scroll_to_new_col (Spreadsheet *sheet, GtkTreeViewColumn *column)
     sw = gtk_widget_get_ancestor(GTK_WIDGET(view), GTK_TYPE_BIN);
     if (sw != NULL) {
 	adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(sw));
-	gtk_adjustment_set_value(adj, adj->upper);
+	gtk_adjustment_set_value(adj, gtk_adjustment_get_upper(adj));
     }
     gtk_tree_path_free(path);
     g_free(pstr);
@@ -680,7 +681,7 @@ static void add_scalar_callback (GtkWidget *w, Spreadsheet *sheet)
     sw = gtk_widget_get_ancestor(GTK_WIDGET(view), GTK_TYPE_BIN);
     if (sw != NULL) {
 	adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sw));
-	gtk_adjustment_set_value(adj, adj->upper);
+	gtk_adjustment_set_value(adj, gtk_adjustment_get_upper(adj));
     }
 }
 
@@ -701,7 +702,7 @@ spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col)
     sw = gtk_widget_get_ancestor(GTK_WIDGET(view), GTK_TYPE_BIN);
     if (sw != NULL) {
 	adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sw));
-	gtk_adjustment_set_value(adj, adj->upper);
+	gtk_adjustment_set_value(adj, gtk_adjustment_get_upper(adj));
     }
     gtk_tree_path_free(path);
     g_free(pstr);
@@ -1987,7 +1988,7 @@ static void manufacture_keystroke (GtkWidget *widget, guint uval)
 	g_free(keys);
 
 	event = gdk_event_new(GDK_KEY_PRESS);
-	event->key.window = g_object_ref(widget->window);
+	event->key.window = g_object_ref(gtk_widget_get_window(widget));
 	event->key.hardware_keycode = hardware_keycode;
 
 	event->key.keyval = gdk_unicode_to_keyval(uval);
@@ -2125,7 +2126,7 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
     mods = widget_get_pointer_mask(view);
 
     if (sheet->matrix == NULL && !editing_scalars(sheet) && 
-	(mods & GDK_BUTTON3_MASK)) {
+	(RIGHT_CLICK(mods))) {
 	GdkEventButton *bevent = (GdkEventButton *) event;
 
 	if (sheet->popup == NULL) {
@@ -2422,7 +2423,7 @@ static int sheet_list_empty (Spreadsheet *sheet)
     return ret;
 }
 
-static Spreadsheet *spreadsheet_new (SheetCmd c)
+static Spreadsheet *spreadsheet_new (SheetCmd c, int varnum)
 {
     Spreadsheet *sheet;
 
@@ -2470,7 +2471,14 @@ static Spreadsheet *spreadsheet_new (SheetCmd c)
 	sheet->varlist = gretl_list_new(1);
     } else {
 	if (sheet->cmd == SHEET_EDIT_VARLIST) {
-	    sheet->varlist = main_window_selection_as_list();
+	    if (varnum > 0) {
+		sheet->varlist = gretl_list_new(1);
+		if (sheet->varlist != NULL) {
+		    sheet->varlist[1] = varnum;
+		}
+	    } else {
+		sheet->varlist = main_window_selection_as_list();
+	    }
 	} else {
 	    sheet->varlist = full_var_list(datainfo, NULL);
 	}
@@ -2949,7 +2957,26 @@ void show_spreadsheet (SheetCmd c)
 	return;
     }
 
-    sheet = spreadsheet_new(c);
+    sheet = spreadsheet_new(c, 0);
+    if (sheet == NULL) {
+	return;
+    }
+
+    real_show_spreadsheet(&sheet, c, 0);
+}
+
+void show_spreadsheet_for_series (int varnum) 
+{
+    static Spreadsheet *sheet;
+    int c = SHEET_EDIT_VARLIST;
+
+    if (sheet != NULL) {
+	/* FIXME? */
+	gtk_window_present(GTK_WINDOW(sheet->win));
+	return;
+    }
+
+    sheet = spreadsheet_new(c, varnum);
     if (sheet == NULL) {
 	return;
     }
@@ -2966,7 +2993,7 @@ void edit_scalars (void)
 	return;
     }
 
-    sheet = spreadsheet_new(SHEET_EDIT_SCALARS);
+    sheet = spreadsheet_new(SHEET_EDIT_SCALARS, 0);
     if (sheet == NULL) {
 	return;
     }
@@ -3227,7 +3254,7 @@ static int new_matrix_dialog (struct gui_matrix_spec *spec)
     return canceled;
 }
 
-static int matrix_from_list (selector *sr)
+static int gui_matrix_from_list (selector *sr)
 {
     struct gui_matrix_spec *s = selector_get_data(sr);
     const char *buf = selector_list(sr);
@@ -3256,6 +3283,25 @@ static int matrix_from_list (selector *sr)
 
     if (err) {
 	gui_errmsg(err);
+    } else {
+	/* record in command log */
+	PRN *prn = NULL;
+
+	if (bufopen(&prn) == 0) {
+	    int i;
+
+	    pprintf(prn, "matrix %s = {", s->name);
+	    for (i=1; i<=list[0]; i++) {
+		pprintf(prn, "%s", datainfo->varname[list[i]]);
+		if (i < list[0]) {
+		    pputc(prn, ',');
+		} else {
+		    pputc(prn, '}');
+		}
+	    }
+	    add_command_to_stack(gretl_print_get_buffer(prn));
+	    gretl_print_destroy(prn);
+	}	    
     }
 
     free(list);
@@ -3277,6 +3323,8 @@ matrix_from_formula (struct gui_matrix_spec *s)
 	s->m = get_matrix_by_name(s->name);
 	if (s->m == NULL) {
 	    err = 1;
+	} else {
+	    add_command_to_stack(genline);
 	}
     }
 
@@ -3332,7 +3380,7 @@ static void edit_matrix (gretl_matrix *m, const char *name,
 	return;
     }
 
-    sheet = spreadsheet_new(SHEET_EDIT_MATRIX);
+    sheet = spreadsheet_new(SHEET_EDIT_MATRIX, 0);
     if (sheet == NULL) {
 	return;
     }
@@ -3386,7 +3434,7 @@ static void real_gui_new_matrix (gretl_matrix *m, const char *name)
 
     if (spec.uselist) {
 	/* matrix from listed vars */
-	simple_selection(_("Define matrix"), matrix_from_list, 
+	simple_selection(_("Define matrix"), gui_matrix_from_list, 
 			 DEFINE_MATRIX, &spec);
     } else if (spec.formula != NULL) {
 	/* matrix from genr-style formula */
@@ -3445,7 +3493,7 @@ void set_dataset_locked (gboolean s)
 int dataset_locked (void)
 {
     if (locked) {
-	errbox(_("You can't do this while editing the dataset"));
+	errbox(_("The dataset cannot be modified at present"));
     }
 
     return locked;
