@@ -34,6 +34,7 @@
 #define GUIDE_PAGE  999
 #define SCRIPT_PAGE 998
 #define GFR_PAGE    997
+#define BIB_PAGE    996
 
 enum {
     PLAIN_TEXT,
@@ -725,6 +726,14 @@ static GtkTextTagTable *gretl_tags_new (void)
 		 "size", 8 * PANGO_SCALE,
 		 NULL);
     gtk_text_tag_table_add(table, tag);
+
+    tag = gtk_text_tag_new("subscript-numeral");
+    g_object_set(tag, "family", "sans",
+		 "style", PANGO_STYLE_NORMAL,
+		 "rise", -3 * PANGO_SCALE,
+		 "size", 8 * PANGO_SCALE,
+		 NULL);
+    gtk_text_tag_table_add(table, tag);
 		 
     tag = gtk_text_tag_new("literal");
     g_object_set(tag, "family", "monospace", NULL);
@@ -739,9 +748,17 @@ static GtkTextTagTable *gretl_tags_new (void)
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("code");
-    g_object_set(tag, "family", "monospace", NULL);
+    g_object_set(tag, "family", "monospace", 
+#if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 8
+		 "paragraph-background", "#e6f3ff",
+#endif
+		 NULL);
     gtk_text_tag_table_add(table, tag);
 
+    tag = gtk_text_tag_new("mono");
+    g_object_set(tag, "family", "monospace", NULL);
+    gtk_text_tag_table_add(table, tag);    
+
     return table;
 }
 
@@ -941,12 +958,22 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
 {
     GtkTextTagTable *tab = gtk_text_buffer_get_tag_table(tbuf);
     GtkTextTag *tag;
+    gchar *show = NULL;
     gchar tagname[32];
 
     if (page == GUIDE_PAGE) {
 	strcpy(tagname, "tag:guide");
     } else if (page == SCRIPT_PAGE) {
 	strcpy(tagname, text);
+    } else if (page == BIB_PAGE) {
+	char *p = strrchr(text, ';');
+
+	if (p != NULL) {
+	    strcpy(tagname, p + 1);
+	    show = g_strndup(text, p - text);
+	} else {
+	    strcpy(tagname, text);
+	}
     } else {
 	sprintf(tagname, "tag:p%d", page);
     }
@@ -954,14 +981,12 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
     tag = gtk_text_tag_table_lookup(tab, tagname);
 
     if (tag == NULL) {
-	if (page == GUIDE_PAGE) {
+	if (page == GUIDE_PAGE || page == BIB_PAGE) {
 	    tag = gtk_text_buffer_create_tag(tbuf, tagname, "foreground", "blue", 
 					     "family", "sans", NULL);
 	} else if (page == SCRIPT_PAGE) {
 	    tag = gtk_text_buffer_create_tag(tbuf, tagname, "foreground", "blue", 
 					     "family", "monospace", NULL);
-	    g_object_set_data_full(G_OBJECT(tag), "fname", g_strdup(text), 
-				   g_free);
 	} else if (indent != NULL) {
 	    tag = gtk_text_buffer_create_tag(tbuf, tagname, "foreground", "blue", 
 					     "left_margin", 30, NULL);
@@ -971,7 +996,12 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
 	g_object_set_data(G_OBJECT(tag), "page", GINT_TO_POINTER(page));
     } 
 
-    gtk_text_buffer_insert_with_tags(tbuf, iter, text, -1, tag, NULL);
+    if (show != NULL) {
+	gtk_text_buffer_insert_with_tags(tbuf, iter, show, -1, tag, NULL);
+	g_free(show);
+    } else {
+	gtk_text_buffer_insert_with_tags(tbuf, iter, text, -1, tag, NULL);
+    }
 }
 
 static void insert_xlink (GtkTextBuffer *tbuf, GtkTextIter *iter, 
@@ -1011,15 +1041,17 @@ static void insert_xlink (GtkTextBuffer *tbuf, GtkTextIter *iter,
     gtk_text_buffer_insert_with_tags(tbuf, iter, text, -1, tag, NULL);
 }
 
-static void link_open_script (GtkTextTag *tag)
+static void open_script_link (GtkTextTag *tag)
 {
-    const char *fname = g_object_get_data(G_OBJECT(tag), "fname");
     const char *gretldir = gretl_home();
+    gchar *fname = NULL;
     char fullname[MAXLEN];
     FILE *fp;
 
+    g_object_get(G_OBJECT(tag), "name", &fname, NULL);
     sprintf(fullname, "%sscripts%cmisc%c%s", gretldir, 
 	    SLASH, SLASH, fname);
+    g_free(fname);
 
     fp = gretl_fopen(fullname, "r");
     if (fp != NULL) {
@@ -1032,6 +1064,56 @@ static void link_open_script (GtkTextTag *tag)
     view_file(fullname, 0, 0, 78, 370, VIEW_SCRIPT);
 }
 
+static void make_bibitem_window (const char *buf,
+				 GtkWidget *tview)
+{
+    windata_t *vwin;
+    GtkWidget *top;
+
+    vwin = view_formatted_text_buffer(NULL, buf, 64, 100);
+    top = gtk_widget_get_toplevel(tview);
+    gtk_window_set_transient_for(GTK_WINDOW(vwin->main),
+				 GTK_WINDOW(top));
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(vwin->main), 
+				       TRUE);
+    gtk_window_set_position(GTK_WINDOW(vwin->main), 
+			    GTK_WIN_POS_CENTER_ON_PARENT);
+    gtk_widget_show(vwin->main);
+}
+
+static void open_bibitem_link (GtkTextTag *tag, GtkWidget *tview)
+{
+    const char *gretldir = gretl_home();
+    gchar *key = NULL;
+    char fullname[MAXLEN];
+    FILE *fp;
+
+    g_object_get(G_OBJECT(tag), "name", &key, NULL);
+    sprintf(fullname, "%sgretlhelp.refs", gretldir);
+    fp = gretl_fopen(fullname, "r");
+
+    if (fp != NULL) {
+	char *buf, line[4096];
+	int n = strlen(key);
+
+	while (fgets(line, sizeof line, fp)) {
+	    if (!strncmp(line, "<@key=\"", 7)) {
+		if (!strncmp(line + 7, key, n)) {
+		    buf = strchr(line + 7, '>');
+		    if (buf != NULL) {
+			make_bibitem_window(buf + 1, tview);
+		    }
+		    break;
+		}
+	    }
+	}
+
+	fclose(fp);
+    }    
+
+    g_free(key);
+}
+
 static int object_get_int (gpointer p, const char *key)
 {
     return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(p), key));
@@ -1053,7 +1135,9 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 	    if (page == GUIDE_PAGE) {
 		display_pdf_help(NULL);
 	    } else if (page == SCRIPT_PAGE) {
-		link_open_script(tag);
+		open_script_link(tag);
+	    } else if (page == BIB_PAGE) {
+		open_bibitem_link(tag, tview);
 	    } else {
 		int role = object_get_int(tview, "role");
 
@@ -1435,7 +1519,7 @@ help_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 {
     GdkModifierType mods = widget_get_pointer_mask(w);
 
-    if (mods & GDK_BUTTON3_MASK) {
+    if (RIGHT_CLICK(mods)) {
 	windata_t *hwin = (windata_t *) p;
 
 	if (hwin->active_var == 0 && pop_backpage(w) == 0) {
@@ -2411,7 +2495,7 @@ script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 {
     GdkModifierType mods = widget_get_pointer_mask(w);
 
-    if (mods & GDK_BUTTON3_MASK) {
+    if (RIGHT_CLICK(mods)) {
 	windata_t *vwin = (windata_t *) p;
 	struct textbit *tc = NULL;
 
@@ -2443,13 +2527,16 @@ enum {
     INSERT_FIG,
     INSERT_REPL,
     INSERT_LIT,
+    INSERT_OPT,
     INSERT_ITAL,
     INSERT_SUP,
     INSERT_SUB,
     INSERT_TEXT,
     INSERT_PDFLINK,
     INSERT_INPLINK,
-    INSERT_GFRLINK
+    INSERT_GFRLINK,
+    INSERT_BIBLINK,
+    INSERT_BOLD
 };
 
 static void insert_help_figure (GtkTextBuffer *tbuf, GtkTextIter *iter,
@@ -2484,11 +2571,21 @@ static void insert_tagged_text (GtkTextBuffer *tbuf, GtkTextIter *iter,
     case INSERT_LIT:
 	ftag = "literal";
 	break;
+    case INSERT_OPT:
+	ftag = "literal"; /* FIXME? */
+	break;
     case INSERT_SUP:
 	ftag = "superscript";
 	break;
     case INSERT_SUB:
-	ftag = "subscript";
+	if (integer_string(s)) {
+	    ftag = "subscript-numeral";
+	} else {
+	    ftag = "subscript";
+	}
+	break;
+    case INSERT_BOLD:
+	ftag = "heading";
 	break;
     default:
 	break;
@@ -2517,6 +2614,8 @@ static int get_instruction_and_string (const char *p, char *str)
 	ins = INSERT_REPL;
     } else if (!strncmp(p, "lit", 3)) {
 	ins = INSERT_LIT;
+    } else if (!strncmp(p, "opt", 3)) {
+	ins = INSERT_OPT;
     } else if (!strncmp(p, "sup", 3)) {
 	ins = INSERT_SUP;
     } else if (!strncmp(p, "sub", 3)) {
@@ -2527,6 +2626,10 @@ static int get_instruction_and_string (const char *p, char *str)
 	ins = INSERT_INPLINK;
     } else if (!strncmp(p, "gfr", 3)) {
 	ins = INSERT_GFRLINK;
+    } else if (!strncmp(p, "bib", 3)) {
+	ins = INSERT_BIBLINK;
+    } else if (!strncmp(p, "hd1", 3)) {
+	ins = INSERT_BOLD;
     }
 
     if (ins != INSERT_NONE) {
@@ -2569,16 +2672,20 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 {
     static char targ[128];
     const char *indent = NULL;
-    const char *code = NULL;
     const char *p;
+    int code = 0;
+    int mono = 0;
     int itarg, ins;
 
     while ((p = strstr(s, "<"))) {
 	int skip = 0;
 
-	if (code != NULL) {
+	if (code) {
 	    gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, p - s,
 						     "code", indent, NULL);
+	} else if (mono) {
+	    gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, p - s,
+						     "mono", indent, NULL);
 	} else {
 	    gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, p - s,
 						     "text", indent, NULL);
@@ -2607,6 +2714,8 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 		insert_link(tbuf, iter, targ, GUIDE_PAGE, indent);
 	    } else if (ins == INSERT_INPLINK) {
 		insert_link(tbuf, iter, targ, SCRIPT_PAGE, indent);
+	    } else if (ins == INSERT_BIBLINK) {
+		insert_link(tbuf, iter, targ, BIB_PAGE, indent);
 	    } else if (ins == INSERT_GFRLINK) {
 		insert_xlink(tbuf, iter, targ, GFR_PAGE, indent);
 	    } else if (ins == INSERT_FIG) {
@@ -2622,10 +2731,16 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 	    indent = NULL;
 	    skip = 8 + (*(p+8) == '\n');
 	} else if (!strncmp(p, "code", 4)) {
-	    code = "code";
+	    code = 1;
 	    skip = get_code_skip(p + 5);
 	} else if (!strncmp(p, "/code", 5)) {
-	    code = NULL;
+	    code = 0;
+	    skip = 6 + (*(p+6) == '\n');
+	} else if (!strncmp(p, "mono", 4)) {
+	    mono = 1;
+	    skip = get_code_skip(p + 5);
+	} else if (!strncmp(p, "/mono", 5)) {
+	    mono = 0;
 	    skip = 6 + (*(p+6) == '\n');
 	} else {
 	    /* literal "<" */
@@ -2635,9 +2750,12 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 	s = p + skip;
     }
 
-    if (code != NULL) {
+    if (code) {
 	gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, -1,
 						 "code", indent, NULL);
+    } else if (mono) {
+	gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, -1,
+						 "mono", indent, NULL);
     } else {
 	gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, -1,
 						 "text", indent, NULL);
@@ -2744,6 +2862,19 @@ int set_help_topic_buffer (windata_t *hwin, int pos, int en)
     return 1;
 }
 
+void gretl_viewer_set_formatted_buffer (windata_t *vwin, const char *buf)
+{
+    GtkTextBuffer *textb;
+    GtkTextIter iter;
+
+    textb = gretl_text_buf_new();
+    gtk_text_buffer_get_iter_at_offset(textb, &iter, 0);
+    insert_text_with_markup(textb, &iter, buf, FUNCS_HELP);
+
+    gtk_text_view_set_buffer(GTK_TEXT_VIEW(vwin->text), textb);
+    cursor_to_top(vwin);
+}
+
 static int get_screen_height (void)
 {
     static int screen_height;
