@@ -52,6 +52,7 @@
 #include "../pixmaps/mini.split_h.xpm"
 #include "../pixmaps/mini.split_v.xpm"
 #include "../pixmaps/mini.compass.xpm"
+#include "../pixmaps/mini.spreadsheet.xpm"
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 8)
 # include "../pixmaps/info_24.xpm"
 #endif
@@ -67,6 +68,7 @@
 #include "../pixmaps/mini.plot.xpm"
 #include "../pixmaps/mini.model.xpm"
 #include "../pixmaps/mini.func.xpm"
+#include "../pixmaps/mini.db.xpm"
 
 enum {
     SAVE_ITEM = 1,
@@ -91,7 +93,8 @@ enum {
     ALPHA_ITEM,
     REFRESH_ITEM,
     OPEN_ITEM,
-    SPLIT_ITEM
+    SPLIT_ITEM,
+    EDITOR_ITEM
 } viewbar_flags;
 
 static GtkIconFactory *gretl_stock_ifac;
@@ -123,7 +126,9 @@ void gretl_stock_icons_init (void)
 	mini_en_xpm,
 	mini_split_h_xpm,
 	mini_split_v_xpm,
-	mini_compass_xpm
+	mini_compass_xpm,
+	mini_spreadsheet_xpm,
+	mini_db_xpm
     };
     const char *stocks[] = {
 #if NO_INFO_ICON
@@ -150,7 +155,9 @@ void gretl_stock_icons_init (void)
 	GRETL_STOCK_EN,
 	GRETL_STOCK_SPLIT_H,
 	GRETL_STOCK_SPLIT_V,
-	GRETL_STOCK_COMPASS
+	GRETL_STOCK_COMPASS,
+	GRETL_STOCK_SHEET,
+	GRETL_STOCK_DB
     };
     int n = G_N_ELEMENTS(stocks);
 
@@ -301,6 +308,15 @@ static void cmd_log_refresh (GtkWidget *w, windata_t *vwin)
     }
 }
 
+static void toolbar_refresh (GtkWidget *w, windata_t *vwin)
+{
+    if (vwin->role == VIEW_LOG) {
+	cmd_log_refresh(w, vwin);
+    } else if (vwin->role == VIEW_SERIES) {
+	series_view_refresh(w, vwin);
+    }
+}
+
 static void matrix_savename (GtkWidget *w, dialog_t *dlg)
 {
     char *newname = (char *) edit_dialog_get_data(dlg);
@@ -624,10 +640,11 @@ static GretlToolItem viewbar_items[] = {
     { N_("Send To..."), GRETL_STOCK_MAIL, G_CALLBACK(mail_script_callback), MAIL_ITEM },
     { N_("Scripts index"), GTK_STOCK_INDEX, G_CALLBACK(script_index), INDEX_ITEM },
     { N_("Confidence level..."), GRETL_STOCK_ALPHA, G_CALLBACK(coeffint_set_alpha), ALPHA_ITEM },
-    { N_("Refresh"), GTK_STOCK_REFRESH, G_CALLBACK(cmd_log_refresh), REFRESH_ITEM },
     { N_("LaTeX"), GRETL_STOCK_TEX, G_CALLBACK(window_tex_callback), TEX_ITEM },
     { N_("Graph"), GRETL_STOCK_TS, G_CALLBACK(series_view_graph), PLOT_ITEM },
     { N_("Reformat..."), GTK_STOCK_CONVERT, G_CALLBACK(reformat_callback), FORMAT_ITEM },
+    { N_("Edit values..."), GRETL_STOCK_SHEET, G_CALLBACK(series_view_edit), EDITOR_ITEM },
+    { N_("Refresh"), GTK_STOCK_REFRESH, G_CALLBACK(toolbar_refresh), REFRESH_ITEM },
     { N_("Add to dataset..."), GTK_STOCK_ADD, G_CALLBACK(add_data_callback), ADD_DATA_ITEM },
     { N_("Add as matrix..."), GTK_STOCK_ADD, G_CALLBACK(add_matrix_callback), ADD_MATRIX_ITEM },
     { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(set_output_sticky), STICKIFY_ITEM },
@@ -720,6 +737,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (!format_ok && f == FORMAT_ITEM) {
 	return NULL;
+    } else if (r != VIEW_SERIES && f == EDITOR_ITEM) {
+	return NULL;
     } else if (r != EDIT_SCRIPT && r != EDIT_PKG_CODE && 
 	       r != EDIT_PKG_SAMPLE && f == EDIT_SCRIPT_ITEM) {
 	return NULL;
@@ -729,7 +748,7 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (r != COEFFINT && f == ALPHA_ITEM) {
 	return NULL;
-    } else if (r != VIEW_LOG && f == REFRESH_ITEM) {
+    } else if (r != VIEW_LOG && r != VIEW_SERIES && f == REFRESH_ITEM) {
 	return NULL;
     } else if (r != EDIT_GP && f == GP_HELP_ITEM) {
 	return NULL;
@@ -1043,6 +1062,7 @@ static GretlToolItem mainbar_items[] = {
     { N_("command reference"),  GTK_STOCK_HELP,      G_CALLBACK(tbar_command_ref), 0 },
     { N_("X-Y graph"),          GRETL_STOCK_SCATTER, G_CALLBACK(tbar_xy_graph), 0 },
     { N_("OLS model"),          GRETL_STOCK_MODEL,   G_CALLBACK(tbar_model), 0 },
+    { N_("gretl database"),     GRETL_STOCK_DB,      G_CALLBACK(show_native_dbs), 0 },
     { N_("open dataset"),       GTK_STOCK_OPEN,      G_CALLBACK(tbar_open_data), 0 }
 };
 
