@@ -39,6 +39,24 @@
  * libgretl; see johansen_test() and johansen_test_simple().
  */
 
+typedef struct adf_info_ adf_info;
+typedef struct kpss_info_ kpss_info;
+
+struct adf_info_ {
+    int T;
+    int df;
+    int order;
+    double b;
+    double tau;
+    double pval;
+};
+
+struct kpss_info_ {
+    int T;
+    double test;
+    double pval;
+};
+
 enum {
     UR_NO_CONST = 1,
     UR_CONST,
@@ -49,7 +67,8 @@ enum {
 
 enum {
     ADF_EG_TEST   = 1 << 0,
-    ADF_EG_RESIDS = 1 << 1
+    ADF_EG_RESIDS = 1 << 1,
+    ADF_PANEL     = 1 << 2
 } adf_flags;
 
 /* replace y with demeaned or detrended y */
@@ -80,9 +99,9 @@ static int GLS_demean_detrend (double *y, int T, int test)
 
     c = (test == UR_CONST)? (1.0 - 7.0/T) : (1.0 - 13.5/T);
 
-    gretl_vector_set(ya, 0, y[0]);
+    gretl_vector_set(ya, 0, y[0] /* (1 - c) * y[0] ?? */);
     for (t=1; t<T; t++) {
-	gretl_vector_set(ya, t, y[t] - y[t-1] * c);
+	gretl_vector_set(ya, t, y[t] - c * y[t-1]);
     }
 
     gretl_matrix_set(Za, 0, 0, 1);
@@ -158,6 +177,9 @@ adf_prepare_vars (int order, int varno, int nseas, int *d0,
 		    offset = t+1;
 		}
 	    }
+	    if (offset < pdinfo->t1 - order) {
+		offset = pdinfo->t1 - order;
+	    }
 	    T = pdinfo->t2 - offset + 1;
 	    err = GLS_demean_detrend((*pZ)[v] + offset, T, test);
 	}
@@ -289,11 +311,7 @@ static void DF_header (const char *s, int p, PRN *prn)
     }
 }
 
-static void 
-print_adf_results (int order, int auto_order, double DFt, double pv, 
-		   MODEL *dfmod, int dfnum, const char *vname, 
-		   int *blurb_done, unsigned char flags, int i, 
-		   int niv, int nseas, gretlopt opt, PRN *prn)
+static const char *DF_model_string (int i)
 {
     const char *models[] = {
 	"(1-L)y = (a-1)*y(-1) + e",
@@ -301,18 +319,52 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
 	"(1-L)y = b0 + b1*t + (a-1)*y(-1) + e",
 	"(1-L)y = b0 + b1*t + b2*t^2 + (a-1)*y(-1) + e"
     };
-    const char *aug_models[] = {
+
+    if (i >= 0 && i < 4) {
+	return models[i];
+    } else {
+	return "";
+    }
+}
+
+static const char *ADF_model_string (int i)
+{
+    const char *models[] = {
 	"(1-L)y = (a-1)*y(-1) + ... + e",
 	"(1-L)y = b0 + (a-1)*y(-1) + ... + e",
 	"(1-L)y = b0 + b1*t + (a-1)*y(-1) + ... + e",
 	"(1-L)y = b0 + b1*t + b2*t^2 + (a-1)*y(-1) + ... + e"
     };
-    const char *teststrs[] = {
+
+    if (i >= 0 && i < 4) {
+	return models[i];
+    } else {
+	return "";
+    }
+}
+
+static const char *DF_test_string (int i)
+{
+    const char *tests[] = {
 	N_("test without constant"),
 	N_("test with constant"),
 	N_("with constant and trend"),
 	N_("with constant and quadratic trend")
     };
+
+    if (i >= 0 && i < 4) {
+	return tests[i];
+    } else {
+	return "";
+    }
+}
+
+static void 
+print_adf_results (int order, int auto_order, double DFt, double pv, 
+		   MODEL *dfmod, int dfnum, const char *vname, 
+		   int *blurb_done, unsigned char flags, int i, 
+		   int niv, int nseas, gretlopt opt, PRN *prn)
+{
     const char *urcstrs[] = {
 	"nc", "c", "ct", "ctt"
     };
@@ -337,13 +389,17 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
     if (*blurb_done == 0) {
 	DF_header(vname, order, prn);
 	pprintf(prn, _("sample size %d\n"), dfmod->nobs);
-	pputs(prn, _("unit-root null hypothesis: a = 1"));
-	pputs(prn, "\n\n");
+	if (flags & ADF_PANEL) {
+	    pputc(prn, '\n');
+	} else {
+	    pputs(prn, _("unit-root null hypothesis: a = 1"));
+	    pputs(prn, "\n\n");
+	}
 	*blurb_done = 1;
     }
 
     if (!(flags & ADF_EG_RESIDS)) {
-	pprintf(prn, "   %s ", _(teststrs[i]));
+	pprintf(prn, "   %s ", _(DF_test_string(i)));
 	if (opt & OPT_G) {
 	    pputs(prn, "(GLS) ");
 	}
@@ -354,7 +410,7 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
     }
 
     pprintf(prn, "   %s: %s\n", _("model"), 
-	    (order > 0)? aug_models[i] : models[i]);
+	    (order > 0)? ADF_model_string(i) : DF_model_string(i));
 
     if (!na(dfmod->rho)) {
 	pprintf(prn, "   %s: %.3f\n", _("1st-order autocorrelation coeff. for e"), 
@@ -398,7 +454,7 @@ static int auto_adjust_order (int *list, int order_max,
 
     for (k=order_max; k>0; k--) {
 
-	kmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	kmod = lsq(list, *pZ, pdinfo, OLS, OPT_A);
 
 	if (kmod.errcode) {
 	    fprintf(stderr, "auto_adjust_order: k = %d, err = %d\n", k,
@@ -584,10 +640,11 @@ static int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
 static int real_adf_test (int varno, int order, int niv,
 			  double ***pZ, DATAINFO *pdinfo, 
 			  gretlopt opt, unsigned char flags,
-			  PRN *prn)
+			  adf_info *ainfo, PRN *prn)
 {
     MODEL dfmod;
     gretlopt eg_opt = OPT_NONE;
+    gretlopt df_mod_opt = OPT_A | OPT_Z;
     int orig_nvars = pdinfo->v;
     int blurb_done = 0;
     int auto_order = 0;
@@ -604,6 +661,11 @@ static int real_adf_test (int varno, int order, int niv,
     fprintf(stderr, "real_adf_test: got order = %d\n", order);
 #endif
 
+    if (gretl_isconst(pdinfo->t1, pdinfo->t2, (*pZ)[varno])) {
+	gretl_errmsg_sprintf(_("%s is a constant"), pdinfo->varname[varno]);
+	return E_DATA;
+    }    
+
     if (opt & OPT_E) {
 	/* testing down */
 	auto_order = 1;
@@ -631,6 +693,16 @@ static int real_adf_test (int varno, int order, int niv,
 	opt = OPT_N;
     }
 
+#if 0
+    if (flags & ADF_PANEL) {
+	/* testing for unit root with panel data: compute standard
+	   errors (and so the ADF t-statistic) without a degrees-of-
+	   freedom adjustment in the ADF regression
+	*/
+	df_mod_opt |= OPT_N;
+    }
+#endif
+
     if (opt & OPT_F) {
 	/* difference the variable before testing */
 	int t1 = pdinfo->t1;
@@ -653,7 +725,7 @@ static int real_adf_test (int varno, int order, int niv,
 	if (opt & OPT_G) {
 	    opt |= OPT_C;
 	} else {
-	    opt |= (OPT_C | OPT_T | OPT_R);
+	    opt |= (OPT_C | OPT_T);
 	}
     }
 
@@ -743,7 +815,7 @@ static int real_adf_test (int varno, int order, int niv,
 	printlist(list, "final ADF regression list");
 #endif
 
-	dfmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	dfmod = lsq(list, *pZ, pdinfo, OLS, df_mod_opt);
 	if (dfmod.errcode) {
 	    fprintf(stderr, "adf_test: dfmod.errcode = %d\n", 
 		    dfmod.errcode);
@@ -775,13 +847,22 @@ static int real_adf_test (int varno, int order, int niv,
 				niv, itv, opt);
 	}
 
-	if (!(opt & OPT_Q)) {
+	if (ainfo != NULL) {
+	    ainfo->T = dfmod.nobs;
+	    ainfo->df = dfmod.dfd;
+	    ainfo->order = order;
+	    ainfo->b = dfmod.coeff[dfnum];
+	    ainfo->tau = DFt;
+	    ainfo->pval = pv;
+	}
+
+	if (!(opt & OPT_Q) && !(flags & ADF_PANEL)) {
 	    print_adf_results(order, auto_order, DFt, pv, &dfmod, dfnum, 
 			      pdinfo->varname[varno], &blurb_done, flags,
 			      itv, niv, nseas, opt, prn);
 	}
 
-	if (opt & OPT_V) {
+	if ((opt & OPT_V) && !(flags & ADF_PANEL)) {
 	    /* verbose */
 	    dfmod.aux = (order > 0)? AUX_ADF : AUX_DF;
 	    if (!na(pv)) {
@@ -789,7 +870,7 @@ static int real_adf_test (int varno, int order, int niv,
 		gretl_model_set_double(&dfmod, "dfpval", pv);
 	    }
 	    printmodel(&dfmod, pdinfo, OPT_NONE, prn);
-	} else if (!(opt & OPT_Q) && !(flags & ADF_EG_RESIDS)) {
+	} else if (!(opt & OPT_Q) && !(flags & (ADF_EG_RESIDS | ADF_PANEL))) {
 	    pputc(prn, '\n');
 	}
 
@@ -797,7 +878,7 @@ static int real_adf_test (int varno, int order, int niv,
     }
 
     if (!err) {
-	if (!(flags & ADF_EG_TEST) || (flags & ADF_EG_RESIDS)) {
+	if (!(flags & (ADF_EG_TEST | ADF_PANEL)) || (flags & ADF_EG_RESIDS)) {
 	    record_test_result(DFt, pv, "Dickey-Fuller");
 	}
     }
@@ -815,6 +896,357 @@ static int real_adf_test (int varno, int order, int niv,
     return err;
 }
 
+/* print critical value for ADF-IPS or KPSS */
+
+static void print_critical_values (double *a, double *cv, 
+				   int ci, PRN *prn)
+{
+    const char *label = N_("Critical values");
+    int figs = (ci == ADF)? 2 : 3;
+
+    pprintf(prn, "%*s    ", TRANSLATED_WIDTH(_(label)), " ");
+    pprintf(prn, "%g%%      %g%%      %g%%\n", 
+	    100*a[0], 100*a[1], 100*a[2]);
+    pprintf(prn, "%s: %.*f   %.*f   %.*f\n", 
+	    _(label), figs, cv[0], figs, cv[1], figs, cv[2]);
+}
+
+static int panel_adjust_ADF_opt (gretlopt *opt)
+{
+    int err = 0;
+
+    /* Has the user selected an option governing the 
+       deterministic terms to be included? If so, don't 
+       mess with it.
+    */
+    if (*opt & (OPT_N | OPT_C | OPT_R | OPT_T)) {
+	; /* no-op */
+    } else {
+	/* panel default: test with constant */
+	*opt |= OPT_C;
+    }
+
+    return err;
+}
+
+static int DF_index (gretlopt opt)
+{
+    if (opt & OPT_N) {
+	return 0;
+    } else if (opt & OPT_C) {
+	return 1;
+    } else if (opt & OPT_T) {
+	return 2;
+    } else {
+	return 3;
+    }
+}
+
+/* See Im, Pesaran and Shin, "Testing for unit roots in
+   heterogeneous panels", Journal of Econometrics 115 (2003),
+   53-74.
+*/
+
+static int do_IPS_test (double tbar, int n, const int *Ti, 
+			int order, const int *Oi,
+			const DATAINFO *pdinfo,
+			gretlopt opt, PRN *prn)
+{
+    int (*get_IPS_critvals) (int, int, int, double *);
+    int (*IPS_tbar_moments) (int, double *, double *);
+    int (*IPS_tbar_rho_moments) (int, int, int, double *, double *);
+    void *handle = NULL;
+    int Tmin = Ti[1], Tmax = Ti[1];
+    int i, T, err = 0;
+
+    for (i=2; i<=n; i++) {
+	if (Ti[i] > Tmax) {
+	    Tmax = Ti[i];
+	}
+	if (Ti[i] < Tmin) {
+	    Tmin = Ti[i];
+	}
+    }
+
+    if (Oi != NULL || order > 0 ) {
+	/* non-zero lag order: use IPS's W_{tbar} statistic */
+	double E, V, Etbar, Vtbar, Wtbar;
+	int order_i;
+
+	IPS_tbar_rho_moments = get_plugin_function("IPS_tbar_rho_moments", &handle);
+
+	if (IPS_tbar_rho_moments != NULL) {
+	    for (i=0; i<n && !err; i++) {
+		T = Ti[i+1];
+		order_i = (Oi != NULL)? Oi[i+1] : order;
+		err = IPS_tbar_rho_moments(order_i, T, (opt & OPT_T), &E, &V);
+		Etbar += E;
+		Vtbar += V;
+	    }
+
+	    if (!err) {
+		Etbar /= n;
+		Vtbar /= n;
+		Wtbar = sqrt(n) * (tbar - Etbar) / sqrt(Vtbar);
+		pprintf(prn, "N = %d, Tmin = %d, Tmax = %d\n", n, Tmin, Tmax);
+		pprintf(prn, "Im-Pesaran-Shin W_tbar = %g [%.4f]\n", Wtbar, 
+			normal_pvalue_1(-Wtbar));
+	    }
+	}
+    } else if (Tmax > Tmin) {
+	/* sample sizes differ: use IPS's Z_{tbar} */
+	double E, V, Etbar, Vtbar, Ztbar;
+
+	IPS_tbar_moments = get_plugin_function("IPS_tbar_moments", &handle);
+
+	if (IPS_tbar_moments != NULL) {
+	    for (i=0; i<n && !err; i++) {
+		T = Ti[i+1];
+		err = IPS_tbar_moments(T, &E, &V);
+		Etbar += E;
+		Vtbar += V;
+	    }
+
+	    if (!err) {
+		Etbar /= n;
+		Vtbar /= n;
+		Ztbar = sqrt(n) * (tbar - Etbar) / sqrt(Vtbar);
+		pprintf(prn, "N = %d, Tmin = %d, Tmax = %d\n", n, Tmin, Tmax);
+		pprintf(prn, "Im-Pesaran-Shin Z_tbar = %g [%.4f]\n", Ztbar, 
+			normal_pvalue_1(-Ztbar));
+	    }
+	}
+    } else {
+	/* simple case: use tbar with exact critical values */
+	pprintf(prn, "N,T = (%d,%d)\n", n, Tmax);
+	pprintf(prn, "Im-Pesaran-Shin t-bar = %g\n", tbar);
+
+	get_IPS_critvals = get_plugin_function("get_IPS_critvals", &handle);
+
+	if (get_IPS_critvals != NULL) {
+	    double a[] = { 0.1, 0.05, 0.01 };
+	    double cv[3];
+		
+	    err = (*get_IPS_critvals) (n, Tmax, (opt & OPT_T), cv);
+	    if (!err) {
+		print_critical_values(a, cv, ADF, prn);
+	    }
+	}
+    }
+
+    if (handle != NULL) {
+	close_plugin(handle);
+    }
+
+    return err;
+}
+
+/* See In Choi, "Unit root tests for panel data", Journal of
+   International Money and Finance 20 (2001), 249-272.
+*/
+
+static void do_choi_test (double ppv, double zpv, double lpv, 
+			  int n, PRN *prn)
+{
+    double P = -2 * ppv;
+    double Z = zpv / sqrt((double) n);
+    int tdf = 5 * n + 4;
+    double k = (3.0*tdf)/(M_PI*M_PI*n*(5*n+2));
+    double L = sqrt(k) * lpv;
+
+    pprintf(prn, "%s\n", _("Choi meta-tests:"));
+    pprintf(prn, "   %s(%d) = %g [%.4f]\n", _("Inverse chi-square"),
+	    2*n, P, chisq_cdf_comp(2*n, P));
+    pprintf(prn, "   %s = %g [%.4f]\n", _("Inverse normal test"),
+	    Z, normal_pvalue_1(-Z));
+    pprintf(prn, "   %s: t(%d) = %g [%.4f]\n", _("Logit test"),
+	    tdf, L, student_pvalue_1(tdf, -L));
+}
+
+static int panel_DF_test (int v, int order, 
+			  double ***pZ, DATAINFO *pdinfo, 
+			  gretlopt opt, PRN *prn)
+{
+    int u0 = pdinfo->t1 / pdinfo->pd;
+    int uN = pdinfo->t2 / pdinfo->pd;
+    int quiet = (opt & OPT_Q);
+    int verbose = (opt & OPT_V);
+    double ppv = 0.0, zpv = 0.0, lpv = 0.0;
+    double pval, tbar = 0.0;
+    int *Ti = NULL, *Oi = NULL;
+    int i, n, err;
+
+    err = panel_adjust_ADF_opt(&opt);
+    if (err) {
+	return err;
+    }
+
+    if (opt & OPT_G) {
+	/* GLS option: can't do IPS t-bar test */
+	tbar = NADBL;
+    } else {
+	Ti = gretl_list_new(uN - u0 + 1);
+	if (Ti == NULL) {
+	    return E_ALLOC;
+	}
+	if ((opt & OPT_E) || order < 0) {
+	    /* testing down: lag order may vary by unit */
+	    Oi = gretl_list_new(uN - u0 + 1);
+	    if (Oi == NULL) {
+		free(Ti);
+		return E_ALLOC;
+	    }
+	}	    
+    }
+
+    if (!quiet) {
+	int j = DF_index(opt);
+
+	DF_header(pdinfo->varname[v], (opt & OPT_E)? 0 : order, prn);
+	pprintf(prn, "   %s ", _(DF_test_string(j)));
+	if (opt & OPT_G) {
+	    pputs(prn, "(GLS) ");
+	}
+	pputc(prn, '\n');
+	pprintf(prn, "   %s: %s\n\n", _("model"), 
+		(order > 0)? ADF_model_string(j) : DF_model_string(j));
+    }
+
+    /* number of units in sample range */
+    n = uN - u0 + 1;
+
+    /* run a Dickey-Fuller test for each unit and record the
+       results */
+
+    for (i=u0; i<=uN && !err; i++) {
+	adf_info ainfo;
+
+	pdinfo->t1 = i * pdinfo->pd;
+	pdinfo->t2 = pdinfo->t1 + pdinfo->pd - 1;
+	err = series_adjust_sample((*pZ)[v], &pdinfo->t1, &pdinfo->t2);
+
+	if (!err) {
+	    err = real_adf_test(v, order, 1, pZ, pdinfo, opt, 
+				ADF_PANEL, &ainfo, prn);
+	    if (!err && verbose) {
+		pprintf(prn, "%s %d, T = %d, %s = %d\n", _("Unit"), i + 1, 
+			ainfo.T, _("lag order"), ainfo.order);
+		pprintf(prn, "   %s: %g\n"
+			"   %s = %g", 
+			_("estimated value of (a - 1)"), ainfo.b,
+			_("test statistic"), ainfo.tau);
+		if (na(ainfo.pval)) {
+		    pputs(prn, "\n\n");
+		} else {
+		    pprintf(prn, " [%.4f]\n\n", ainfo.pval);
+		}
+	    }	    
+	}
+
+	if (!err) {
+	    if (Ti != NULL) {
+		Ti[i-u0+1] = ainfo.T;
+	    }
+	    if (Oi != NULL) {
+		Oi[i-u0+1] = ainfo.order;
+	    }	    
+	    if (!na(tbar)) {
+		tbar += ainfo.tau;
+	    }
+	    pval = ainfo.pval;
+	    if (na(pval)) {
+		ppv = zpv = lpv = NADBL;
+	    } else if (!na(ppv)) {
+		ppv += log(pval);
+		zpv += normal_cdf_inverse(pval);
+		lpv += log(pval / (1-pval));
+	    }
+	}
+    }
+
+    /* process the results as per Im-Pesaran-Shin and/or Choi */
+
+    if (!err) {
+	pprintf(prn, "%s\n\n", _("H0: all groups have unit root"));
+	if (!na(tbar)) {
+	    tbar /= n;
+	    do_IPS_test(tbar, n, Ti, order, Oi, pdinfo, opt, prn);
+	}
+	if (!na(ppv)) {
+	    pputc(prn, '\n');
+	    do_choi_test(ppv, zpv, lpv, n, prn);
+	}
+	pputc(prn, '\n');
+    }
+
+    free(Ti);
+    free(Oi);
+
+    return err;
+}
+
+/**
+ * levin_lin_test:
+ * @vnum: ID number of variable to test.
+ * @plist: list of ADF lag orders.
+ * @Z: data array.
+ * @pdinfo: data information struct.
+ * @opt: option flags.
+ * @prn: gretl printing struct.
+ *
+ * Carries out and prints the results of the Levin-Lin-Chu test
+ * for a unit root in panel data. 
+ *
+ * The list @plist should contain either a single lag order
+ * to be applied to all units, or a set of unit-specific
+ * orders; in the latter case the length of the list must
+ * equal the number of panel units in the current sample
+ * range. (This is a gretl list: the first element holds
+ * a count of the number of elements following.)
+ *
+ * By default a test with constant is performed, but the
+ * (mutually exclusive) options OPT_N and OPT_T in @opt switch to
+ * the case of no constant or constant plus trend respectively.
+ * The OPT_Q flag may be used to suppress printed output.
+ *
+ * Returns: 0 on successful completion, non-zero on error.
+ */
+
+int levin_lin_test (int vnum, const int *plist,
+		    double **Z, DATAINFO *pdinfo, 
+		    gretlopt opt, PRN *prn)
+{
+    int (*real_levin_lin) (int, const int *, double **,
+			   DATAINFO *, gretlopt, PRN *);
+    void *handle;
+    int panelmode;
+    int err = 0;
+
+    panelmode = multi_unit_panel_sample(pdinfo);
+
+    if (!panelmode || incompatible_options(opt, OPT_N | OPT_T)) {
+	return E_BADOPT;
+    }
+
+    real_levin_lin = get_plugin_function("real_levin_lin", &handle);
+
+    if (real_levin_lin == NULL) {
+	fputs(I_("Couldn't load plugin function\n"), stderr);
+	err = E_FOPEN;
+    } else {
+	int save_t1 = pdinfo->t1;
+	int save_t2 = pdinfo->t2;
+ 
+	err = (*real_levin_lin) (vnum, plist, Z, pdinfo, opt, prn);
+	close_plugin(handle);
+
+	pdinfo->t1 = save_t1;
+	pdinfo->t2 = save_t2;
+    }
+
+    return err;
+}
+
 /**
  * adf_test:
  * @order: lag order for the (augmented) test.
@@ -848,10 +1280,10 @@ static int real_adf_test (int varno, int order, int niv,
 int adf_test (int order, const int *list, double ***pZ,
 	      DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
-    int t1 = pdinfo->t1;
-    int t2 = pdinfo->t2;
-    int v, vlist[2] = {1,0};
-    int i, err;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
+    int panelmode;
+    int err;
 
     /* GLS incompatible with no const, quadratic trend or seasonals */
     err = incompatible_options(opt, OPT_N | OPT_R | OPT_G);
@@ -864,40 +1296,119 @@ int adf_test (int order, const int *list, double ***pZ,
 	err = incompatible_options(opt, OPT_C | OPT_T);
     }
 
-    for (i=1; i<=list[0] && !err; i++) {
-	v = list[i];
-	vlist[1] = v;
-	err = list_adjust_t1t2(vlist, (const double **) *pZ, pdinfo);
-	if (!err) {
-	    err = real_adf_test(v, order, 1, pZ, pdinfo, opt, 
-				0, prn);
+    panelmode = multi_unit_panel_sample(pdinfo);
+
+    if (panelmode) {
+	err = panel_DF_test(list[1], order, pZ, pdinfo, opt, prn);
+    } else {
+	/* regular time series case */
+	int i, v, vlist[2] = {1, 0};
+
+	for (i=1; i<=list[0] && !err; i++) {
+	    vlist[1] = v = list[i];
+	    err = list_adjust_sample(vlist, &pdinfo->t1, &pdinfo->t2, 
+				     (const double **) *pZ);
+	    if (!err) {
+		err = real_adf_test(v, order, 1, pZ, pdinfo, opt, 
+				    0, NULL, prn);
+	    }
+	    pdinfo->t1 = save_t1;
+	    pdinfo->t2 = save_t2;
 	}
-	pdinfo->t1 = t1;
-	pdinfo->t2 = t2;
     }
 
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
+
     return err;
 }
 
+/* See Peter S. Sephton, "Response surface estimates of the KPSS 
+   stationarity test", Economics Letters 47 (1995) 255-261.
+
+   The estimates below of \beta_\infty and \beta_1 allow the
+   construction of better critical values for finite samples
+   than the values given in the original KPSS article.
+*/
+
+static void kpss_parms (double a, int trend, double *b)
+{
+    const double b0_level[] = { 0.74375, 0.46119, 0.34732 };
+    const double b1_level[] = { -0.99187, 0.45911, 0.20695 };
+    const double b0_trend[] = { 0.21778, 0.14795, 0.119298 };
+    const double b1_trend[] = { -0.235089, 0.035327, 0.100804 };
+    int i = (a == .01)? 0 : (a == .05)? 1 : 2;
+
+    if (trend) {
+	b[0] = b0_trend[i];
+	b[1] = b1_trend[i];
+    } else {
+	b[0] = b0_level[i];
+	b[1] = b1_level[i];
+    }	
+}
+
+static double kpss_critval (double alpha, int T, int trend)
+{
+    double b[2];
+
+    kpss_parms(alpha, trend, b);
+    return b[0] + b[1]/T;
+}
+
+#define PV_GT10 1.1
+#define PV_LT01 -1.0
+
+static double kpss_interp (double s, int T, int trend)
+{
+    double c10, c05, c01;
+    double pv;
+
+    c10 = kpss_critval(.10, T, trend);
+    if (s < c10) {
+	return PV_GT10;
+    }
+
+    c01 = kpss_critval(.01, T, trend);
+    if (s > c01) {
+	return PV_LT01;
+    }  
+
+    /* OK, p-value must lie between .01 and .10 */
+
+    c05 = kpss_critval(.05, T, trend);
+    if (s > c05) {
+	pv = .01 + .04 * (c01 - s) / (c01 - c05);
+    } else {
+	pv = .05 + .05 * (c05 - s) / (c05 - c10);
+    }
+
+    return pv;
+}
+
 static int 
 real_kpss_test (int order, int varno, double ***pZ,
-		DATAINFO *pdinfo, gretlopt opt, PRN *prn)
+		DATAINFO *pdinfo, gretlopt opt, 
+		kpss_info *kinfo, PRN *prn)
 {
     MODEL KPSSmod;
     int list[4];
     int hastrend = 0;
-    double s2 = 0.0;
+    double et, s2 = 0.0;
     double cumsum = 0.0, cumsum2 = 0.0;
-    double teststat;
+    double teststat, pval = NADBL;
     double *autocov;
-    double et;
-
-    int i, t;
     int t1, t2, T;
+    int i, t;
 
     /* sanity check */
-    if (order < 0 || varno <= 0 || varno >= pdinfo->v) {
-	return 1;
+    if (varno <= 0 || varno >= pdinfo->v) {
+	return E_DATA;
+    }
+
+    if (gretl_isconst(pdinfo->t1, pdinfo->t2, (*pZ)[varno])) {
+	gretl_errmsg_sprintf(_("%s is a constant"), pdinfo->varname[varno]);
+	return E_DATA;
     }
 
     if (opt & OPT_F) {
@@ -924,7 +1435,7 @@ real_kpss_test (int order, int varno, double ***pZ,
     }
 
     /* OPT_M: reject missing values within sample range */
-    KPSSmod = lsq(list, pZ, pdinfo, OLS, OPT_A | OPT_M);
+    KPSSmod = lsq(list, *pZ, pdinfo, OLS, OPT_A | OPT_M);
     if (KPSSmod.errcode) {
 	clear_model(&KPSSmod);
 	return KPSSmod.errcode;
@@ -934,7 +1445,11 @@ real_kpss_test (int order, int varno, double ***pZ,
     t2 = KPSSmod.t2;
     T = KPSSmod.nobs;
 
-    if (opt & OPT_V) {
+    if (order <= 0) {
+	order = 4.0 * pow(T / 100.0, 0.25);
+    }
+
+    if (kinfo == NULL && (opt & OPT_V)) {
 	KPSSmod.aux = AUX_KPSS;
 	printmodel(&KPSSmod, pdinfo, OPT_NONE, prn);
     }
@@ -943,7 +1458,7 @@ real_kpss_test (int order, int varno, double ***pZ,
     if (autocov == NULL) {
 	return E_ALLOC;
     }
-  
+
     for (i=0; i<order; i++) {
 	autocov[i] = 0.0;
     }
@@ -976,44 +1491,173 @@ real_kpss_test (int order, int varno, double ***pZ,
     }
 
     s2 /= T;
-    teststat = cumsum2 / (s2 * T * T);
 
-    if (opt & OPT_V) {
-	pprintf(prn, "  %s: %g\n", _("Robust estimate of variance"), s2);
-	pprintf(prn, "  %s: %g\n", _("Sum of squares of cumulated residuals"), 
-		cumsum2);
+    if (s2 <= 0.0) {
+	teststat = pval = NADBL;
+    } else {
+	teststat = cumsum2 / (s2 * T * T);
+	pval = kpss_interp(teststat, T, hastrend);
     }
 
-    if (!(opt & OPT_Q)) {
-	pprintf(prn, _("\nKPSS test for %s %s\n\n"), pdinfo->varname[varno],
-		(hastrend)? _("(including trend)") : _("(without trend)"));
-	pprintf(prn, _("Lag truncation parameter = %d\n"), order);
-	pprintf(prn, "%s = %g\n\n", _("Test statistic"), teststat);
-	pprintf(prn, "%*s    ", TRANSLATED_WIDTH(_("Critical values")), " ");
-	pprintf(prn, "%g%%      %g%%    %g%%      %g%%\n", 10.0, 5.0, 2.5, 1.0);
-	if (hastrend) {
-	    pprintf(prn, "%s: %.3f   %.3f   %.3f   %.3f\n\n", 
-		    _("Critical values"), 0.119, 0.146, 0.176, 0.216);
-	} else {
-	    pprintf(prn, "%s: %.3f   %.3f   %.3f   %.3f\n\n", 
-		    _("Critical values"), 0.347, 0.463, 0.574, 0.739);
+    if (kinfo != NULL) {
+	/* storing info for panel test */
+	kinfo->T = T;
+	kinfo->test = teststat;
+	kinfo->pval = pval;
+    } else {
+	/* testing individual time series */
+	if (pval == PV_GT10 || pval == PV_LT01) {
+	    pval = NADBL;
+	}
+	if (opt & OPT_V) {
+	    pprintf(prn, "  %s: %g\n", _("Robust estimate of variance"), s2);
+	    pprintf(prn, "  %s: %g\n", _("Sum of squares of cumulated residuals"), 
+		    cumsum2);
 	}
     }
 
-    record_test_result(teststat, NADBL, "KPSS");
-    clear_model(&KPSSmod);
+    if (!(opt & OPT_Q)) {
+	double a[] = { 0.1, 0.05, 0.01 };
+	double cv[3];
 
+	cv[0] = kpss_critval(a[0], T, hastrend);
+	cv[1] = kpss_critval(a[1], T, hastrend);
+	cv[2] = kpss_critval(a[2], T, hastrend);
+
+	pprintf(prn, _("\nKPSS test for %s %s\n\n"), pdinfo->varname[varno],
+		(hastrend)? _("(including trend)") : _("(without trend)"));
+	pprintf(prn, "T = %d\n", T);
+	pprintf(prn, _("Lag truncation parameter = %d\n"), order);
+	pprintf(prn, "%s = %g\n\n", _("Test statistic"), teststat);
+	print_critical_values(a, cv, KPSS, prn);
+	if (pval == PV_GT10) {
+	    pprintf(prn, "%s > .10\n", _("P-value"));
+	    pval = NADBL; /* invalidate for record_test_result */
+	} else if (pval == PV_LT01) {
+	    pprintf(prn, "%s < .01\n", _("P-value"));
+	    pval = NADBL;
+	} else if (!xna(pval)) {
+	    pprintf(prn, "%s %.3f\n", _("Interpolated p-value"), pval);
+	}
+	pputc(prn, '\n');
+    }
+
+    if (kinfo == NULL) {
+	record_test_result(teststat, pval, "KPSS");
+    }
+
+    clear_model(&KPSSmod);
     free(autocov);
 
     return 0;
 }
 
+static int panel_kpss_test (int order, int v, 
+			    double ***pZ, DATAINFO *pdinfo, 
+			    gretlopt opt, PRN *prn)
+{
+    kpss_info kinfo;
+    int u0 = pdinfo->t1 / pdinfo->pd;
+    int uN = pdinfo->t2 / pdinfo->pd;
+    int n = uN - u0 + 1;
+    int verbose = (opt & OPT_V);
+    double ppv = 0.0, zpv = 0.0, lpv = 0.0;
+    int gt_10 = 0, lt_01 = 0;
+    double pval;
+    int i, err;
+
+    /* run a KPSS test for each unit and record the
+       results */
+
+    pprintf(prn, _("\nKPSS test for %s %s\n"), pdinfo->varname[v],
+	    (opt & OPT_T)? _("(including trend)") : _("(without trend)"));
+    pprintf(prn, _("Lag truncation parameter = %d\n"), order);
+    pputc(prn, '\n');
+
+    for (i=u0; i<=uN && !err; i++) {
+	pdinfo->t1 = i * pdinfo->pd;
+	pdinfo->t2 = pdinfo->t1 + pdinfo->pd - 1;
+	err = series_adjust_sample((*pZ)[v], &pdinfo->t1, &pdinfo->t2);
+	if (!err) {
+	    err = real_kpss_test(order, v, pZ, pdinfo, opt | OPT_Q, &kinfo, prn);
+	    if (!err && verbose) {
+		pprintf(prn, "Unit %d, T = %d\n", i + 1, kinfo.T);
+		if (na(kinfo.pval)) {
+		    pputs(prn, "\n\n");
+		} else {
+		    pprintf(prn, "test = %g, ", kinfo.test);
+		    if (kinfo.pval == PV_GT10) {
+			pprintf(prn, "%s > .10\n", _("p-value"));
+		    } else if (kinfo.pval == PV_LT01) {
+			pprintf(prn, "%s < .01\n", _("p-value"));
+		    } else {
+			pprintf(prn, "%s %.3f\n", _("interpolated p-value"), kinfo.pval);
+		    }
+		    pputc(prn, '\n');
+		}
+	    }
+	}
+
+	if (!err) {
+	    pval = kinfo.pval;
+
+	    if (pval == PV_GT10) {
+		gt_10++;
+		if (lt_01 == 0) {
+		    /* record lower bound */
+		    pval = .10;
+		} else {
+		    pval = NADBL;
+		}
+	    } else if (pval == PV_LT01) {
+		lt_01++;
+		if (gt_10 == 0) {
+		    /* record upper bound */
+		    pval = .01;
+		} else {
+		    pval = NADBL;
+		}
+	    }
+
+	    if (xna(pval)) {
+		ppv = zpv = lpv = NADBL;
+	    } else if (!na(ppv)) {
+		ppv += log(pval);
+		zpv += normal_cdf_inverse(pval);
+		lpv += log(pval / (1-pval));
+	    }
+	}
+    }
+
+    if (!err && !na(ppv)) {
+	/* process the results as per Choi, as best we can */
+	pprintf(prn, "%s\n\n", _("H0: all groups are stationary"));
+	do_choi_test(ppv, zpv, lpv, n, prn);
+	if (gt_10 > 0) {
+	    pputs(prn, "   Note: these are LOWER BOUNDS "
+		  "on the true p-values\n");
+	    pprintf(prn, "   (Individual p-values > .10, and recorded as .10: %d)\n",
+		    gt_10);
+	} else if (lt_01 > 0) {
+	    pputs(prn, "   Note: these are UPPER BOUNDS "
+		  "on the true p-values\n");
+	    pprintf(prn, "   (Individual p-values < .01, and recorded as .01: %d)\n",
+		    lt_01);
+	} 
+	pputc(prn, '\n');
+    } else {
+	pprintf(prn, "Choi test: cannot be calculated\n");
+    }
+
+    return err;
+}
+
 /**
  * kpss_test:
  * @order: window size for Bartlett smoothing.
  * @list: list of variables to test.
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
  * @opt: option flags.
  * @prn: gretl printing struct.
  *
@@ -1029,13 +1673,32 @@ real_kpss_test (int order, int varno, double ***pZ,
 int kpss_test (int order, const int *list, double ***pZ,
 	       DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
-    int i, err = 0;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
+    int err = 0;
 
-    for (i=1; i<=list[0] && !err; i++) {
-	err = real_kpss_test(order, list[i], pZ, pdinfo,
-			     opt, prn);
+    if (multi_unit_panel_sample(pdinfo)) {
+	err = panel_kpss_test(order, list[1], pZ, pdinfo, opt, prn);
+    } else {
+	/* regular time series case */
+	int i, v, vlist[2] = {1, 0};
+
+	for (i=1; i<=list[0] && !err; i++) {
+	    v = list[i];
+	    vlist[1] = v;
+	    err = list_adjust_sample(vlist, &pdinfo->t1, &pdinfo->t2, 
+				     (const double **) *pZ);
+	    if (!err) {
+		err = real_kpss_test(order, v, pZ, pdinfo, opt, NULL, prn);
+	    }
+	    pdinfo->t1 = save_t1;
+	    pdinfo->t2 = save_t2;
+	}
     }
 
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
+
     return err;
 }
 
@@ -1207,6 +1870,12 @@ int engle_granger_test (int order, const int *list, double ***pZ,
     int *clist = NULL;
     int err = 0;
 
+    if (multi_unit_panel_sample(pdinfo)) {
+	gretl_errmsg_set("Sorry, this command is not yet available "
+			 "for panel data");
+	return E_DATA;
+    }
+
     err = coint_check_opts(opt, &detcode, &adf_opt);
     if (err) {
 	return err;
@@ -1236,7 +1905,7 @@ int engle_granger_test (int order, const int *list, double ***pZ,
 	    pprintf(prn, _("Step %d: testing for a unit root in %s\n"),
 		    step++, pdinfo->varname[clist[i]]);
 	    real_adf_test(clist[i], order, 1, pZ, pdinfo, adf_opt, 
-			  ADF_EG_TEST, prn);
+			  ADF_EG_TEST, NULL, prn);
 	}
     }
 
@@ -1251,7 +1920,7 @@ int engle_granger_test (int order, const int *list, double ***pZ,
     pdinfo->t1 = orig_t1;
     pdinfo->t2 = orig_t2;
 
-    cmod = lsq(clist, pZ, pdinfo, OLS, OPT_NONE);
+    cmod = lsq(clist, *pZ, pdinfo, OLS, OPT_NONE);
     err = cmod.errcode;
     if (err) {
 	goto bailout;
@@ -1278,7 +1947,7 @@ int engle_granger_test (int order, const int *list, double ***pZ,
 
     /* Run (A)DF test on the residuals */
     real_adf_test(k, order, nv, pZ, pdinfo, adf_opt, 
-		  ADF_EG_TEST | ADF_EG_RESIDS, prn); 
+		  ADF_EG_TEST | ADF_EG_RESIDS, NULL, prn); 
 
     pputs(prn, _("\nThere is evidence for a cointegrating relationship if:\n"
 		 "(a) The unit-root hypothesis is not rejected for the individual"
