@@ -123,7 +123,7 @@ mask_from_test_list (const int *list, const MODEL *pmod, int *err)
 	return NULL;
     }
 
-    if (pmod->ci == ARBOND) {
+    if (pmod->ci == ARBOND || pmod->ci == DPANEL) {
 	/* find correct offset into independent vars in list */
 	for (i=2; i<=pmod->list[0]; i++) {
 	    if (pmod->list[i] == LISTSEP) {
@@ -636,7 +636,7 @@ full_model_list (const MODEL *pmod, const int *inlist)
     return flist;
 }
 
-static gretlopt retrieve_arbond_opts (const MODEL *pmod)
+static gretlopt retrieve_dpanel_opts (const MODEL *pmod)
 {
     gretlopt opt = OPT_NONE;
 
@@ -683,8 +683,8 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 {
     MODEL rep;
     const char *param = NULL;
+    const int *auxlist = NULL;
     char altparm[32] = {0};
-    double rho = 0.0;
     int *list = *plist;
     int mc = get_model_count();
     int repci = orig->ci;
@@ -707,7 +707,6 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	} else if (orig->opt & OPT_P) {
 	    myopt |= OPT_P;
 	}
-	rho = estimate_rho(list, pZ, pdinfo, myopt, prn, &rep.errcode);
     } else if (orig->ci == WLS || orig->ci == AR || get_extra_var(orig)) {
 	int *full_list = full_model_list(orig, list);
 
@@ -719,7 +718,11 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	}
     } else if (orig->ci == ARBOND) {
 	param = gretl_model_get_data(orig, "istr");
-	myopt |= retrieve_arbond_opts(orig);
+	myopt |= retrieve_dpanel_opts(orig);
+    } else if (orig->ci == DPANEL) {
+	param = gretl_model_get_data(orig, "istr");
+	auxlist = gretl_model_get_data(orig, "ylags");
+	myopt |= retrieve_dpanel_opts(orig);
     } else if (orig->ci == ARCH) {
 	order = gretl_model_get_int(orig, "arch_order");
     } else if (orig->ci == LOGIT || orig->ci == PROBIT) {
@@ -769,10 +772,17 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
     case AR:
 	rep = ar_model(list, pZ, pdinfo, myopt, prn);
 	break;
+    case AR1:
+	rep = ar1_model(list, pZ, pdinfo, myopt, prn);
+	break;
     case ARBOND:
 	rep = arbond_model(list, param, (const double **) *pZ, 
 			   pdinfo, myopt, prn);
 	break;
+    case DPANEL: 
+	rep = dpd_model(list, auxlist, param, (const double **) *pZ, 
+			pdinfo, myopt, prn);
+	break;
     case ARCH:
 	rep = arch_model(list, order, pZ, pdinfo, myopt, prn);
 	break;
@@ -787,7 +797,7 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	if (gretl_model_get_int(orig, "rq")) {
 	    rep = quantreg_driver(altparm, list, pZ, pdinfo, myopt, NULL);
 	} else {
-	    rep = lad(list, pZ, pdinfo);
+	    rep = lad(list, *pZ, pdinfo);
 	}
 	break;
     case POISSON:
@@ -795,7 +805,7 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	rep = count_model(list, orig->ci, pZ, pdinfo, myopt, NULL);
 	break;
     case DURATION:
-	rep = duration_model(list, pZ, pdinfo, myopt, NULL);
+	rep = duration_model(list, *pZ, pdinfo, myopt, NULL);
 	break;
     case HECKIT:
 	rep = heckit_model(list, pZ, pdinfo, myopt, NULL);
@@ -813,15 +823,16 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
     case PANEL:
 	rep = panel_model(list, pZ, pdinfo, myopt, prn);
 	break;
+    case HSK:
+	rep = hsk_model(list, pZ, pdinfo);
+	break;
     default:
-	/* handles OLS, AR1, WLS, HSK, etc. */
-	if (rho != 0.0) {
-	    rep = ar1_lsq(list, pZ, pdinfo, repci, myopt, rho);
-	} else if (gretl_model_get_int(orig, "pooled")) {
+	/* handles OLS, WLS, etc. */
+	if (gretl_model_get_int(orig, "pooled")) {
 	    myopt |= OPT_P;
 	    rep = panel_model(list, pZ, pdinfo, myopt, prn);
 	} else {
-	    rep = lsq(list, pZ, pdinfo, repci, myopt);
+	    rep = lsq(list, *pZ, pdinfo, repci, myopt);
 	}
 	break;
     }
@@ -889,7 +900,7 @@ real_nonlinearity_test (MODEL *pmod, int *list,
     /* replace the dependent var */
     list[1] = pdinfo->v - 1;
 
-    aux = lsq(list, pZ, pdinfo, OLS, OPT_A);
+    aux = lsq(list, *pZ, pdinfo, OLS, OPT_A);
     if (aux.errcode) {
 	err = aux.errcode;
 	fprintf(stderr, "auxiliary regression failed\n");
@@ -1093,7 +1104,8 @@ int add_test (const int *addvars, MODEL *orig, MODEL *pmod,
     /* create augmented regression list */
     if (orig->ci == IVREG) {
 	tmplist = ivreg_list_add(orig->list, addvars, opt, &err);
-    } else if (orig->ci == PANEL || orig->ci == ARBOND) {
+    } else if (orig->ci == PANEL || orig->ci == ARBOND || 
+	       orig->ci == DPANEL) {
 	tmplist = panel_list_add(orig, addvars, &err);
     } else {
 	tmplist = gretl_list_add(orig->list, addvars, &err);
@@ -1173,7 +1185,8 @@ static int wald_omit_test (const int *list, MODEL *pmod,
 
     if (pmod->ci == IVREG) {
 	test = ivreg_list_omit(pmod->list, list, opt, &err);
-    } else if (pmod->ci == PANEL || pmod->ci == ARBOND) {
+    } else if (pmod->ci == PANEL || pmod->ci == ARBOND ||
+	       pmod->ci == DPANEL) {
 	test = panel_list_omit(pmod, list, &err);
     } else {
 	test = gretl_list_omit(pmod->list, list, 2, &err);
@@ -1366,7 +1379,8 @@ static int make_short_list (MODEL *orig, const int *omitvars,
 
     if (orig->ci == IVREG) {
 	list = ivreg_list_omit(orig->list, omitvars, opt, &err);
-    } else if (orig->ci == PANEL || orig->ci == ARBOND) {
+    } else if (orig->ci == PANEL || orig->ci == ARBOND ||
+	       orig->ci == DPANEL) {
 	list = panel_list_omit(orig, omitvars, &err);
     } else if (*omitlast) {
 	/* special: just drop the last variable */
@@ -1560,7 +1574,7 @@ double get_DW_pvalue_for_model (const MODEL *pmod,
     if (pmod == NULL || pmod->list == NULL) {
 	*err = E_DATA;
     } else if ((pmod->ci != OLS && pmod->ci != PANEL) || 
-	       pmod->missmask != NULL || na(pmod->dw)) {
+	       model_has_missing_obs(pmod) || na(pmod->dw)) {
 	*err = E_BADSTAT;
     } else {
 	/* check that relevant vars have not been redefined */
@@ -1706,7 +1720,7 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
+	aux = lsq(newlist, *pZ, pdinfo, OLS, OPT_A);
 	err = aux.errcode;
 	if (err) {
 	    errmsg(aux.errcode, prn);
@@ -1876,8 +1890,8 @@ static int ivreg_autocorr_test (MODEL *pmod, int order,
 	return E_NOTIMP;
     }
 
-    if (pmod->missmask != NULL) {
-	return E_DATA;
+    if (model_has_missing_obs(pmod)) {
+	return E_MISSDATA;
     }
 
     /* impose original sample range */
@@ -2029,8 +2043,8 @@ int autocorr_test (MODEL *pmod, int order,
 	return E_NOTIMP;
     }
 
-    if (pmod->missmask != NULL) {
-	return E_DATA;
+    if (model_has_missing_obs(pmod)) {
+	return E_MISSDATA;
     }
 
     if (dataset_is_panel(pdinfo)) {
@@ -2104,7 +2118,7 @@ int autocorr_test (MODEL *pmod, int order,
 	/* regression on [X~E], using original sample */
 	impose_model_smpl(pmod, pdinfo);
 	newlist[1] = v;
-	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
+	aux = lsq(newlist, *pZ, pdinfo, OLS, OPT_A);
 	err = aux.errcode;
 	if (err) {
 	   errmsg(err, prn);
@@ -2294,7 +2308,7 @@ static double QLR_get_critval (double Fmax, int dfn, int *a, int *approx)
 static int QLR_graph (const double *Ft, int t1, int t2, 
 		      int tmax, int dfn, const DATAINFO *pdinfo)
 {
-    const double *x = gretl_plotx(pdinfo);
+    const double *x = gretl_plotx(NULL, pdinfo);
     FILE *fp;
     int t, err = 0;
 
@@ -2505,7 +2519,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, double ***pZ,
 	}
 	
 	for (t=split; t<=smax; t++) {
-	    chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A);
+	    chow_mod = lsq(chowlist, *pZ, pdinfo, OLS, OPT_A);
 	    if (chow_mod.errcode) {
 		err = chow_mod.errcode;
 		errmsg(err, prn);
@@ -2553,7 +2567,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, double ***pZ,
 	    lsqopt |= OPT_R;
 	}
 
-	chow_mod = lsq(chowlist, pZ, pdinfo, OLS, lsqopt);
+	chow_mod = lsq(chowlist, *pZ, pdinfo, OLS, lsqopt);
 
 	if (chow_mod.errcode) {
 	    err = chow_mod.errcode;
@@ -2753,46 +2767,63 @@ static double vprime_M_v (double *v, double *M, int n)
     return val;
 }
 
+/* Compute scaled recursive residuals and cumulate \bar{w} via a 
+   sequence of OLS regressions. See William Greene, Econometric 
+   Analysis 5e, pp. 135-136.
+*/
+
 static int cusum_compute (MODEL *pmod, double *cresid, int T, int k,
-			  double *wbar, double ***pZ, DATAINFO *pdinfo) 
+			  double *wbar, double **Z, DATAINFO *pdinfo) 
 {
     MODEL cmod;
     gretlopt opt = OPT_X | OPT_A;
-    double *xvec;
-    double xx;
+    double yhat, den, *xt;
     int n = T - k;
     int i, j, t;
     int err = 0;
 
-    xvec = malloc(k * sizeof *xvec);
-    if (xvec == NULL) {
+    /* vector of regressors at t */
+    xt = malloc(k * sizeof *xt);
+    if (xt == NULL) {
 	return E_ALLOC;
     }
 
+    /* set minimal sample based on model to be tested */
+    pdinfo->t1 = pmod->t1;
+    pdinfo->t2 = pmod->t1 + k - 1;    
+
     for (j=0; j<n && !err; j++) {
-	cmod = lsq(pmod->list, pZ, pdinfo, OLS, opt);
+	cmod = lsq(pmod->list, Z, pdinfo, OLS, opt);
 	if (cmod.errcode) {
 	    err = cmod.errcode;
 	} else {
-	    /* compute ex post prediction error */
+	    /* compute one step ahead prediction error: \hat{y}_t
+	       based on estimates through t - 1
+	    */
 	    t = pdinfo->t2 + 1;
-	    xx = 0.0;
+	    yhat = 0.0;
 	    for (i=0; i<cmod.ncoeff; i++) {
-		xvec[i] = (*pZ)[cmod.list[i+2]][t];
-		xx += cmod.coeff[i] * xvec[i];
+		xt[i] = Z[cmod.list[i+2]][t];
+		yhat += cmod.coeff[i] * xt[i];
 	    }
-	    cresid[j] = (*pZ)[pmod->list[1]][t] - xx;
+	    cresid[j] = Z[pmod->list[1]][t] - yhat;
 	    cmod.ci = CUSUM;
-	    err = makevcv(&cmod, 1.0);
-	    xx = vprime_M_v(xvec, cmod.vcv, cmod.ncoeff);
-	    cresid[j] /= sqrt(1.0 + xx);
-	    *wbar += cresid[j];
-	    pdinfo->t2 += 1;
+	    err = makevcv(&cmod, 1.0); /* (X'X)^{-1} */
+	    if (!err) {
+		/* compute scaled residual: divide by
+		   \sqrt{1 + x_t'(X_{t-1}'X_{t-1})^{-1} x_t}
+		*/
+		den = vprime_M_v(xt, cmod.vcv, cmod.ncoeff);
+		cresid[j] /= sqrt(1.0 + den);
+		*wbar += cresid[j];
+		/* extend the sample by one observation */
+		pdinfo->t2 += 1;
+	    }
 	}
 	clear_model(&cmod); 
     }
 
-    free(xvec);
+    free(xt);
 
     return err;
 }
@@ -2817,7 +2848,7 @@ static int cusum_do_graph (double a, double b, const double *W,
     if (dataset_is_time_series(pdinfo) && okfreq(pdinfo->pd)) {
 	b *= pdinfo->pd;
 	frac /= pdinfo->pd;
-        obs = gretl_plotx(pdinfo);
+        obs = gretl_plotx(NULL, pdinfo);
 	if (obs != NULL) {
 	    x0 = obs[t1 + k];
 	}
@@ -2895,7 +2926,7 @@ static void cusum_harvey_collier (double wbar, double sigma, int m,
 /**
  * cusum_test:
  * @pmod: pointer to model to be tested.
- * @pZ: pointer to data matrix.
+ * @Z: data array.
  * @pdinfo: information on the data set.
  * @opt: if flags include %OPT_S, save results of test to model.
  * @prn: gretl printing struct.
@@ -2907,7 +2938,7 @@ static void cusum_harvey_collier (double wbar, double sigma, int m,
  * Returns: 0 on successful completion, error code on error.
  */
 
-int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
+int cusum_test (MODEL *pmod, double **Z, DATAINFO *pdinfo, 
 		gretlopt opt, PRN *prn) 
 {
     int save_t1 = pdinfo->t1;
@@ -2929,17 +2960,13 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return 0;
     }
 
-    if (has_missing_obs(pmod)) {
-	return E_DATA;
+    if (model_has_missing_obs(pmod)) {
+	return E_MISSDATA;
     }
 
-    /* number of forecasts */
+    /* the number of forecasts */
     m = T - k; 
 
-    /* set sample based on model to be tested */
-    pdinfo->t1 = pmod->t1;
-    pdinfo->t2 = pmod->t1 + k - 1;    
-
     cresid = malloc(m * sizeof *cresid);
     W = malloc(m * sizeof *W);
 
@@ -2948,7 +2975,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	err = cusum_compute(pmod, cresid, T, k, &wbar, pZ, pdinfo);
+	err = cusum_compute(pmod, cresid, T, k, &wbar, Z, pdinfo);
 	if (err) {
 	    errmsg(err, prn);
 	}
@@ -3143,7 +3170,7 @@ int comfac_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	/* re-impose the sample that was in force when the original model
 	   was estimated */
 	impose_model_smpl(pmod, pdinfo);
-	cmod = lsq(biglist, pZ, pdinfo, OLS, OPT_A);
+	cmod = lsq(biglist, *pZ, pdinfo, OLS, OPT_A);
 	clearit = 1;
 	err = cmod.errcode;
     }
@@ -3402,7 +3429,7 @@ int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 /**
  * vif_test:
  * @pmod: pointer to model to be tested.
- * @pZ: pointer to data matrix.
+ * @Z: pointer to data matrix.
  * @pdinfo: information on the data set.
  * @prn: gretl printing struct.
  *
@@ -3412,10 +3439,10 @@ int leverage_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
  * Returns: 0 on successful completion, error code on error.
  */
 
-int vif_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn)
+int vif_test (MODEL *pmod, double **Z, DATAINFO *pdinfo, PRN *prn)
 {
     void *handle;
-    int (*print_vifs) (MODEL *, double ***, DATAINFO *, PRN *);
+    int (*print_vifs) (MODEL *, double **, DATAINFO *, PRN *);
     int err;
 
     gretl_error_clear();
@@ -3425,7 +3452,7 @@ int vif_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	return 1;
     }
 
-    err = (*print_vifs)(pmod, pZ, pdinfo, prn);
+    err = (*print_vifs)(pmod, Z, pdinfo, prn);
 
     close_plugin(handle);
 
