@@ -797,10 +797,11 @@ real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
  * 
  * Given a "current" date string, a periodicity, and a starting
  * date string, returns the observation number corresponding to
- * the current date string, counting from zero.
+ * the current date string, counting from zero. It is an error
+ * if @date represents an observation that lies outside of the
+ * full data range specified in @pdinfo.
  * 
- * Returns: integer observation number.
- *
+ * Returns: integer observation number, or -1 on error.
  */
 
 int dateton (const char *date, const DATAINFO *pdinfo)
@@ -808,25 +809,37 @@ int dateton (const char *date, const DATAINFO *pdinfo)
     return real_dateton(date, pdinfo, 0);
 }
 
-/* special for appending data: allow the date to be outside of
-   the range of the current dataset */
+/**
+ * merge_dateton:
+ * @date: string representation of date for processing.
+ * @pdinfo: pointer to data information struct.
+ * 
+ * Works just as dateton(), except that for this function it
+ * is not an error if @date represents an observation that
+ * lies beyond the data range specified in @pdinfo. This is 
+ * inended for use when merging data, or when creating a new
+ * dataset.
+ * 
+ * Returns: integer observation number, or -1 on error.
+ */
 
-static int merge_dateton (const char *date, const DATAINFO *pdinfo)
+int merge_dateton (const char *date, const DATAINFO *pdinfo)
 {
     return real_dateton(date, pdinfo, 1);
 }
 
-static char *
-out_of_range_panel_obs (char *s, int t, const DATAINFO *pdinfo)
+static char *panel_obs (char *s, int t, const DATAINFO *pdinfo)
 {
     int i = t / pdinfo->pd + 1;
     int j = (t + 1) % pdinfo->pd;
+    int d = 1 + floor(log10(pdinfo->pd));
 
     if (j == 0) {
 	j = pdinfo->pd;
     }
 
-    sprintf(s, "%d:%0*d", i, pdinfo->paninfo->olen, j);
+    sprintf(s, "%d:%0*d", i, d, j);
+
     return s;
 }
 
@@ -869,14 +882,8 @@ char *ntodate (char *datestr, int t, const DATAINFO *pdinfo)
 	x = pdinfo->sd0 + 10 * t;
 	sprintf(datestr, "%d", (int) x);
 	return datestr;
-    } else if (pdinfo->paninfo != NULL) {
-	/* indexed panel data */
-	if (t < 0 || t >= pdinfo->n) {
-	    return out_of_range_panel_obs(datestr, t, pdinfo);
-	}
-	sprintf(datestr, "%d:%0*d", pdinfo->paninfo->unit[t] + 1,
-		pdinfo->paninfo->olen,
-		pdinfo->paninfo->period[t] + 1);
+    } else if (dataset_is_panel(pdinfo)) {
+	panel_obs(datestr, t, pdinfo);
 	return datestr;
     }
 
@@ -1252,6 +1259,7 @@ int write_data (const char *fname, int *list,
     char datfile[MAXLEN], hdrfile[MAXLEN], lblfile[MAXLEN];
     int tsamp = sample_size(pdinfo);
     int n = pdinfo->n;
+    int pop_locale = 0;
     char delim = 0;
     FILE *fp = NULL;
     int *pmax = NULL;
@@ -1352,6 +1360,7 @@ int write_data (const char *fname, int *list,
 
     if (fmt != GRETL_FMT_CSV || pdinfo->decpoint != ',') {
 	gretl_push_c_numeric_locale();
+	pop_locale = 1;
     }
 
     if (fmt == GRETL_FMT_TRAD) { 
@@ -1531,7 +1540,7 @@ int write_data (const char *fname, int *list,
 	}
     }
 
-    if (fmt != GRETL_FMT_CSV || pdinfo->decpoint != ',') {
+    if (pop_locale) {
 	gretl_pop_c_numeric_locale();
     }
 
@@ -1887,10 +1896,6 @@ int gretl_get_data (char *fname, double ***pZ, DATAINFO *pdinfo,
 	err = switch_panel_orientation(tmpZ, tmpdinfo);
     }
 
-    if (!err && tmpdinfo->structure == STACKED_TIME_SERIES) {
-	err = dataset_add_default_panel_indices(tmpdinfo);
-    }
-
     if (err) goto bailout;
 
     /* print out basic info from the files read */
@@ -2123,8 +2128,8 @@ static int compare_ranges (const DATAINFO *targ,
 static int panel_expand_ok (DATAINFO *pdinfo, DATAINFO *addinfo,
 			    gretlopt opt)
 {
-    int n = pdinfo->paninfo->nunits;
-    int T = pdinfo->paninfo->Tmax;
+    int n = pdinfo->n / pdinfo->pd;
+    int T = pdinfo->pd;
     int ok = 0;
 
     if (addinfo->n == T) {
@@ -2146,8 +2151,8 @@ static int panel_append_special (int addvars,
 				 gretlopt opt,
 				 PRN *prn)
 {
-    int n = pdinfo->paninfo->nunits;
-    int T = pdinfo->paninfo->Tmax;
+    int n = pdinfo->n / pdinfo->pd;
+    int T = pdinfo->pd;
     int k = pdinfo->v;
     int tsdata;
     int i, j, s, p, t;
@@ -2261,6 +2266,7 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 		       double **addZ, DATAINFO *addinfo,
 		       gretlopt opt, PRN *prn)
 {
+    int dayspecial = 0;
     int addsimple = 0;
     int addpanel = 0;
     int addvars = 0;
@@ -2274,6 +2280,11 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
+    if (dated_daily_data(pdinfo) && dated_daily_data(addinfo)) {
+	fprintf(stderr, "special: merging daily data\n");
+	dayspecial = 1;
+    }
+
     /* below: had additional condition: simple_structure(pdinfo)
        relaxed this on 2009-05-15 */
 
@@ -2386,11 +2397,26 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 		copy_varinfo(pdinfo->varinfo[v], addinfo->varinfo[i]);
 	    } 
 
-	    for (t=0; t<pdinfo->n; t++) {
-		if (t >= offset && t - offset < addinfo->n) {
-		    (*pZ)[v][t] = addZ[i][t - offset];
-		} else if (newvar) {
-		    (*pZ)[v][t] = NADBL;
+	    if (dayspecial) {
+		char obs[OBSLEN];
+		int s;
+
+		for (t=0; t<pdinfo->n; t++) {
+		    ntodate(obs, t, pdinfo);
+		    s = dateton(obs, addinfo);
+		    if (s >= 0 && s < addinfo->n) {
+			(*pZ)[v][t] = addZ[i][s];
+		    } else {
+			(*pZ)[v][t] = NADBL;
+		    }
+		}
+	    } else {
+		for (t=0; t<pdinfo->n; t++) {
+		    if (t >= offset && t - offset < addinfo->n) {
+			(*pZ)[v][t] = addZ[i][t - offset];
+		    } else if (newvar) {
+			(*pZ)[v][t] = NADBL;
+		    }
 		}
 	    }
 	}
