@@ -29,6 +29,15 @@
 
 static int dataset_changed;
 
+/**
+ * check_dataset_is_changed:
+ *
+ * Returns: 1 if the current dataset has been modified since
+ * the last call to this function, or since the library was
+ * initialized if this function has not yet been called;
+ * otherwise 0.
+ */
+
 int check_dataset_is_changed (void)
 {
     int ret = dataset_changed;
@@ -37,6 +46,12 @@ int check_dataset_is_changed (void)
     return ret;
 }
 
+/**
+ * set_dataset_is_changed:
+ *
+ * Sets the internal "dataset changed" flag to 1
+ */
+
 void set_dataset_is_changed (void)
 {
     dataset_changed = 1;
@@ -45,12 +60,12 @@ void set_dataset_is_changed (void)
 /**
  * free_Z:
  * @Z: data matrix.
- * @pdinfo: data information struct.
+ * @pdinfo: dataset information.
  *
  * Does a deep free on the data matrix.
  */
 
-void free_Z (double **Z, DATAINFO *pdinfo)
+void free_Z (double **Z, const DATAINFO *pdinfo)
 {
     if (Z != NULL && pdinfo != NULL) {
 	int i;
@@ -88,17 +103,6 @@ void dataset_destroy_obs_markers (DATAINFO *pdinfo)
     } 
 }
 
-void dataset_destroy_panel_info (DATAINFO *pdinfo)
-{
-    if (pdinfo->paninfo != NULL) {
-	free(pdinfo->paninfo->unit);
-	free(pdinfo->paninfo->period);
-	free(pdinfo->paninfo->padmask);
-	free(pdinfo->paninfo);
-	pdinfo->paninfo = NULL;
-    }
-}
-
 static void free_varinfo (DATAINFO *pdinfo, int v)
 {
     free(pdinfo->varinfo[v]);
@@ -109,7 +113,8 @@ static void free_varinfo (DATAINFO *pdinfo, int v)
  * @pdinfo: data information struct.
  * @code: either %CLEAR_FULL or %CLEAR_SUBSAMPLE.
  *
- * Frees the allocated content of a data information struct.
+ * Frees the allocated content of a data information struct;
+ * note that @pdinfo itself is not freed.
  */
 
 void clear_datainfo (DATAINFO *pdinfo, int code)
@@ -132,11 +137,8 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 	pdinfo->restriction = NULL;
     }	
 
-    if (pdinfo->paninfo != NULL) {
-	dataset_destroy_panel_info(pdinfo);
-    }    
-
     /* if this is not a sub-sample datainfo, free varnames, labels, etc. */
+
     if (code == CLEAR_FULL) {
 	if (pdinfo->varname != NULL) {
 	    for (i=0; i<pdinfo->v; i++) {
@@ -168,7 +170,8 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
  * @Z: data array.
  * @pdinfo: dataset information struct.
  *
- * Frees all resources associated with @Z and @pdinfo.
+ * Frees all resources associated with @Z and @pdinfo, 
+ * including the pointers @Z and @pdinfo themselves.
  */
 
 void destroy_dataset (double **Z, DATAINFO *pdinfo)
@@ -228,7 +231,7 @@ void dataset_obs_info_default (DATAINFO *pdinfo)
  * strings.  Note that These strings have a fixed maximum 
  * length of #OBSLEN - 1.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_allocate_obs_markers (DATAINFO *pdinfo)
@@ -253,232 +256,6 @@ int dataset_allocate_obs_markers (DATAINFO *pdinfo)
     return err;
 }
 
-/**
- * dataset_allocate_panel_info:
- * @pdinfo: dataset information struct
- *
- * Allocates space in @pdinfo for two indices representing
- * the unit or group and time-period, respectively, of each 
- * observation in a panel data set.
- *
- * Returns: 0 on success, %E_ALLOC on error.
- */
-
-int dataset_allocate_panel_info (DATAINFO *pdinfo)
-{
-    PANINFO *pan;
-    int i;
-
-#if DDEBUG
-    fprintf(stderr, "dataset_allocate_panel_info: pdinfo at %p\n",
-	    (void *) pdinfo);
-#endif
-
-    /* just in case, clean out any previous stuff */
-    dataset_destroy_panel_info(pdinfo);  
-
-    pan = malloc(sizeof *pan);
-    if (pan == NULL) {
-	return E_ALLOC;
-    }
-
-    pan->unit = pan->period = NULL;
-    pan->padmask = NULL;
-
-    pan->unit = malloc(pdinfo->n * sizeof *pan->unit);
-    pan->period = malloc(pdinfo->n * sizeof *pan->period);
-
-    if (pan->unit == NULL || pan->period == NULL) {
-	free(pan->unit);
-	free(pan->period);
-	free(pan);
-	return E_ALLOC;
-    }
-
-    for (i=0; i<pdinfo->n; i++) {
-	pan->unit[i] = pan->period[i] = -1; /* deliberately invalid */
-    }
-
-    pan->nunits = 0;
-    pan->Tmin = pan->Tmax = 0;
-    pan->olen = 0;
-
-    pdinfo->paninfo = pan;
-
-    return 0;
-}
-
-static int resize_panel_info (DATAINFO *pdinfo, int n)
-{
-    PANINFO *pan = pdinfo->paninfo;
-    int *unit, *period;
-    int err = 0;
-
-    unit = realloc(pan->unit, n * sizeof *unit);
-    if (unit == NULL) {
-	return E_ALLOC;
-    }
-    pan->unit = unit;
-
-    period = realloc(pan->period, n * sizeof *period);
-    if (period == NULL) {
-	return E_ALLOC;
-    }
-    pan->period = period;
-
-    if (n > pdinfo->n) {
-	int uadd = (n - pdinfo->n) / pdinfo->pd;
-	int i, t, j = pdinfo->paninfo->nunits;
-	int s = pdinfo->n;
-
-	for (i=0; i<uadd && !err; i++) {
-	    for (t=0; t<pdinfo->pd && !err; t++) {
-		if (s+i >= n) {
-		    err = E_DATA;
-		    fprintf(stderr, "panel info out of bounds\n");
-		} else {
-		    pan->unit[s+i] = j;
-		    pan->period[s+i] = t;
-		    s++;
-		}
-	    }
-	    j++;
-	}
-    }
-
-    pdinfo->paninfo->nunits = n / pdinfo->pd;
-
-    return err;
-}
-
-/**
- * dataset_add_default_panel_indices:
- * @pdinfo: dataset information struct.
- *
- * Adds a pair of indices for panel unit and panel period.
- * The default is that both are zero-based and increase
- * consecutively, per unit and per period respectively.
- * This function assumes a balanced panel.
- *
- * Returns: 0 on success, non-zero code on error.
- */
-
-int dataset_add_default_panel_indices (DATAINFO *pdinfo)
-{
-    int nunits, T;
-    int i, s, t, err;
-
-    if (pdinfo->n % pdinfo->pd != 0) {
-	fprintf(stderr, "dataset_add_default_panel_indices: "
-		"error: pdinfo->n %% pdinfo->pd = %d\n", 
-		pdinfo->n % pdinfo->pd);	
-	return E_DATA;
-    }
-
-    err = dataset_allocate_panel_info(pdinfo);
-
-    if (!err) {
-	char test[32];
-
-	T = pdinfo->pd;
-	nunits = pdinfo->n / pdinfo->pd;
-	s = 0;
-	for (i=0; i<nunits; i++) {
-	    for (t=0; t<T; t++) {
-		pdinfo->paninfo->unit[s] = i;
-		pdinfo->paninfo->period[s] = t;
-		s++;
-	    }
-	}
-	pdinfo->paninfo->nunits = nunits;
-	pdinfo->paninfo->Tmin = T;
-	pdinfo->paninfo->Tmax = T;
-
-	sprintf(test, "%d", T);
-	pdinfo->paninfo->olen = strlen(test);
-    }
-
-    return err;
-}
-
-/**
- * dataset_finalize_panel_indices:
- * @pdinfo: dataset information struct.
- *
- * Having already added a pair of indices for panel unit 
- * and panel period, check these for consistency and
- * calculate the number of panel units and the minimum
- * and maximum observations per unit.  If it turns out
- * there's only one unit, or only one period, in the
- * dataset, then it's not really a panel: we destroy
- * the panel info and return %E_PDWRONG.
- *
- * Returns: 0 on success, non-zero code on error.
- */
-
-int dataset_finalize_panel_indices (DATAINFO *pdinfo)
-{
-    PANINFO *pan = pdinfo->paninfo;
-    char test[32];
-    int u0, ubak;
-    int i, Ti, len;
-
-    if (pan == NULL) {
-	return E_DATA;
-    }
-
-    pan->nunits = 1;
-    pan->Tmax = 0;
-    pan->Tmin = 999999;
-    pan->olen = 0;
-
-    u0 = ubak = pan->unit[0];
-    Ti = 0;
-
-    /* basic validity check */
-    for (i=0; i<pdinfo->n; i++) {
-	if (pan->unit[i] < 0 || pan->period[i] < 0) {
-	    gretl_errmsg_set("Panel index information is corrupted");
-	    return E_DATA;
-	}
-	sprintf(test, "%d", pan->period[i] + 1);
-	len = strlen(test);
-	if (len > pan->olen) {
-	    pan->olen = len;
-	}
-    }
-
-    for (i=0; i<pdinfo->n; i++) {
-	if (pan->unit[i] == ubak) {
-	    Ti++;
-	} else {
-	    pan->nunits += 1;
-	    if (Ti > pan->Tmax) {
-		pan->Tmax = Ti;
-	    }
-	    if (Ti < pan->Tmin) {
-		pan->Tmin = Ti;
-	    }	    
-	    Ti = 1;
-	    ubak = pan->unit[i];
-	}
-    }
-
-    if (pan->nunits == 1 || pan->Tmax < 2) {
-	dataset_destroy_panel_info(pdinfo);
-	pdinfo->structure = CROSS_SECTION;
-	return E_PDWRONG;
-    }
-
-#if DDEBUG
-    fprintf(stderr, "dataset_finalize_panel_indices:\n "
-	    "nunits=%d, Tmin=%d, Tmax=%d, olen=%d\n", pan->nunits,
-	    pan->Tmin, pan->Tmax, pan->olen);
-#endif
-
-    return 0;
-}
-
 static void gretl_varinfo_init (VARINFO *vinfo)
 {
     vinfo->label[0] = '\0';
@@ -523,10 +300,11 @@ void copy_varinfo (VARINFO *targ, const VARINFO *src)
  *
  * Given a blank @pdinfo, which should have been obtained using
  * datainfo_new(), allocate space for the names of variables.
- * The @v member of @pdinfo (number of variables) must be
+ * The @v member of @pdinfo (representing the number of variables,
+ * including the automatically added constant at position 0) must be
  * set before calling this function.
  * 
- * Returns: 0 on sucess, %E_ALLOC on failure.
+ * Returns: 0 on sucess, E_ALLOC on failure.
  */
 
 int dataset_allocate_varnames (DATAINFO *pdinfo)
@@ -568,63 +346,75 @@ int dataset_allocate_varnames (DATAINFO *pdinfo)
     return err;
 }
 
+/**
+ * datainfo_init:
+ * @pdinfo: pointer to DATAINFO struct.
+ *
+ * Zeros all members of @pdinfo and sets it as a plain cross-section. 
+ * Designed for use with a DATAINFO structure that has not been 
+ * obtained via datainfo_new().
+ */
+
+void datainfo_init (DATAINFO *pdinfo)
+{
+    pdinfo->v = 0;
+    pdinfo->n = 0;
+    pdinfo->pd = 1;
+    pdinfo->sd0 = 1.0;
+    pdinfo->t1 = 0;
+    pdinfo->t2 = 0;
+    pdinfo->stobs[0] = '\0';
+    pdinfo->endobs[0] = '\0';
+
+    pdinfo->varname = NULL;
+    pdinfo->varinfo = NULL;    
+
+    pdinfo->markers = NO_MARKERS;  
+    pdinfo->delim = ',';
+    pdinfo->decpoint = '.';
+
+    pdinfo->S = NULL;
+    pdinfo->descrip = NULL;
+    pdinfo->submask = NULL;
+    pdinfo->restriction = NULL;
+
+    pdinfo->structure = CROSS_SECTION;
+}
+
 /**
  * datainfo_new:
  *
  * Creates a new data information struct pointer from scratch,
- * properly initialized as empty.
+ * properly initialized as empty (no variables, no observations).
  * 
  * Returns: pointer to data information struct, or NULL on error.
  */
 
 DATAINFO *datainfo_new (void)
 {
-    DATAINFO *dinfo;
+    DATAINFO *pdinfo = malloc(sizeof *pdinfo);
 
-    dinfo = malloc(sizeof *dinfo);
-    if (dinfo == NULL) {
-	return NULL;
+    if (pdinfo != NULL) {
+	datainfo_init(pdinfo);
     }
 
-    dinfo->v = 0;
-    dinfo->n = 0;
-    dinfo->pd = 1;
-    dinfo->sd0 = 1.0;
-    dinfo->t1 = 0;
-    dinfo->t2 = 0;
-    dinfo->stobs[0] = '\0';
-    dinfo->endobs[0] = '\0';
-
-    dinfo->varname = NULL;
-    dinfo->varinfo = NULL;    
-    dinfo->paninfo = NULL;
-
-    dinfo->markers = NO_MARKERS;  
-    dinfo->delim = ',';
-    dinfo->decpoint = '.';
-
-    dinfo->S = NULL;
-    dinfo->descrip = NULL;
-    dinfo->submask = NULL;
-    dinfo->restriction = NULL;
-
-    dinfo->structure = CROSS_SECTION;
-
-    return dinfo;
+    return pdinfo;
 }
 
 /**
  * create_new_dataset:
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @nvar: number of variables.
- * @nobs: number of observations per variable 
- * @markers: 1 if there are case markers for the observations, 0
- * otherwise.
+ * @nobs: number of observations per variable.
+ * @markers: 1 if space should be allocated for "case markers" for 
+ * the observations, 0 otherwise.
  *
- * Creates a new data information struct corresponding to a given
- * data matrix.
+ * Allocates space in the two-dimensional array to which @pZ points,
+ * to hold the specified number of variables and observations.
+ * Also allocates a new dataset information struct that matches the
+ * dimensions of the data array.
  * 
- * Returns: pointer to data information struct, or %NULL on error.
+ * Returns: pointer to data information struct, or NULL on error.
  */
 
 DATAINFO *
@@ -677,12 +467,12 @@ DATAINFO *create_auxiliary_dataset (double ***pZ, int nvar, int nobs)
  * @pdinfo: dataset information struct.
  *
  * Allocates the two-dimensional array to which @pZ points,
- * based on the %v (number of variables) and %n (number of
+ * based on the v (number of variables) and n (number of
  * observations) members of @pdinfo.  The variable at 
  * position 0 is initialized to all 1s; other variables
  * are initialized to #NADBL.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int allocate_Z (double ***pZ, const DATAINFO *pdinfo)
@@ -746,7 +536,6 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
     pdinfo->markers = NO_MARKERS;
     pdinfo->delim = ',';
     pdinfo->descrip = NULL;
-    pdinfo->paninfo = NULL;
     pdinfo->submask = NULL;
     pdinfo->restriction = NULL;
     
@@ -998,17 +787,17 @@ maybe_extend_dummies (double **Z, const DATAINFO *pdinfo, int oldn)
  * @newobs: number of observations to add.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
- * @opt: use %OPT_A to attempt to recognize and
+ * @opt: use OPT_A to attempt to recognize and
  * automatically extend simple deterministic variables such 
  * as a time trend and periodic dummy variables; 
- * use %OPT_D to drop any observation markers rather than
+ * use OPT_D to drop any observation markers rather than
  * expanding the set of markers and padding it out with
  * dummy values.
  *
  * Extends all series in the dataset by the specified number of
  * extra observations.  The added values are initialized to
  * the missing value code, #NADBL, with the exception of
- * simple deterministic variables when %OPT_A is given.
+ * simple deterministic variables when OPT_A is given.
  *
  * Returns: 0 on success, non-zero code on error.
  */
@@ -1055,13 +844,6 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (pdinfo->paninfo != NULL) {
-	err = resize_panel_info(pdinfo, bign);
-	if (err) {
-	    return err;
-	}
-    }
-    
     if (pdinfo->t2 == pdinfo->n - 1) {
 	pdinfo->t2 = bign - 1;
     }
@@ -1079,6 +861,46 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+static int 
+real_insert_observation (int pos, double ***pZ, DATAINFO *pdinfo)
+{
+    double *x;
+    int n = pdinfo->n + 1;
+    int i, t;
+    int err = 0;
+
+    for (i=0; i<pdinfo->v; i++) {
+	x = realloc((*pZ)[i], n * sizeof *x);
+	if (x == NULL) {
+	    return E_ALLOC;
+	}
+	(*pZ)[i] = x;
+	for (t=pdinfo->n; t>pos; t--) {
+	    (*pZ)[i][t] = (*pZ)[i][t-1];
+	}
+	(*pZ)[i][pos] = (i == 0)? 1.0 : NADBL;
+    }
+    
+    if (dataset_has_markers(pdinfo)) {
+	if (reallocate_markers(pdinfo, n)) {
+	    return E_ALLOC;
+	}
+	for (t=pdinfo->n; t>pos; t--) {
+	    strcpy(pdinfo->S[t], pdinfo->S[t-1]); 
+	}
+	sprintf(pdinfo->S[pos], "%d", pos + 1);
+    }
+
+    if (pdinfo->t2 == pdinfo->n - 1) {
+	pdinfo->t2 = n - 1;
+    }
+
+    pdinfo->n = n;
+    ntodate(pdinfo->endobs, n - 1, pdinfo);
+
+    return err;
+}
+
 /**
  * dataset_drop_observations:
  * @n: number of observations to drop.
@@ -1120,12 +942,6 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    if (pdinfo->paninfo != NULL) {
-	if (resize_panel_info(pdinfo, newn)) {
-	    return E_ALLOC;
-	}
-    }    
-
     if (pdinfo->t2 > newn - 1) {
 	pdinfo->t2 = newn - 1;
     }
@@ -1144,7 +960,7 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
  * @pdinfo: dataset information.
  *
  * Truncates the range of observations in the dataset, based on
- * the current values of the %t1 and %t2 members of @pdinfo.
+ * the current values of the t1 and t2 members of @pdinfo.
  *
  * Returns: 0 on success, non-zero code on error.
  */
@@ -1173,20 +989,14 @@ int dataset_shrink_obs_range (double ***pZ, DATAINFO *pdinfo)
 	    memmove(pdinfo->S, pdinfo->S + head, mvsize);
 	}
 
-	if (pdinfo->paninfo != NULL) {
-	    mvsize = rem * sizeof *pdinfo->paninfo->unit;
-	    memmove(pdinfo->paninfo->unit, pdinfo->paninfo->unit + head, mvsize);
-	    memmove(pdinfo->paninfo->period, pdinfo->paninfo->period + head, mvsize);
-	}
-
-	/* FIXME panel data */
-
 	if (pdinfo->structure == CROSS_SECTION) {
 	    ntodate(pdinfo->stobs, 0, pdinfo);
 	} else {
+	    /* FIXME panel? */
 	    ntodate(pdinfo->stobs, pdinfo->t1, pdinfo);
 	    pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
 	}
+
 	pdinfo->t1 = 0;
 	pdinfo->t2 -= head;
 	pdinfo->n -= head;
@@ -1308,7 +1118,7 @@ static int real_add_series (int newvars, double *x,
  * to the dataset.  It is the caller's responsibility to
  * initialize the numerical values of the new series.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int 
@@ -1327,7 +1137,7 @@ dataset_add_series (int newvars, double ***pZ, DATAINFO *pdinfo)
  * The array @x is not copied; it should be treated as
  * belonging to @pZ after this operation.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int 
@@ -1348,7 +1158,7 @@ dataset_add_allocated_series (double *x, double ***pZ, DATAINFO *pdinfo)
  * level "deeper" (in terms of function execution) than the
  * current level.  This is for use with user-defined functions.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_add_series_as (double *x, const char *newname,
@@ -1395,7 +1205,7 @@ int dataset_add_series_as (double *x, const char *newname,
  * function's namespace under the name it was given as a
  * parameter.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_copy_variable_as (int v, const char *newname,
@@ -1720,21 +1530,21 @@ static int real_drop_listed_vars (int *list, double ***pZ,
 	if (d0 == d1) {
 	    if (gretl_messages_on()) {
 		if (*vname != '\0') {
-		    pprintf(prn, "Deleted %s", vname);
+		    pprintf(prn, _("Deleted %s"), vname);
 		} else {
-		    pprintf(prn, "Deleted %d variables", d1);
+		    pprintf(prn, _("Deleted %d variables"), d1);
 		}
 		pputc(prn, '\n');
 	    }
 	} else {
 	    if (d1 == 0) {
-		pputs(prn, "No variables deleted");
+		pputs(prn, _("No variables were deleted"));
 	    } else if (*vname != '\0') {
-		pprintf(prn, "Deleted %s", vname);
+		pprintf(prn, _("Deleted %s"), vname);
 	    } else {
-		pprintf(prn, "Deleted %d variables", d1);
+		pprintf(prn, _("Deleted %d variables"), d1);
 	    }
-	    pputs(prn, " (some data were in use)\n");
+	    pprintf(prn, " (%s)\n", _("some data were in use"));
 	    
 	}
     }
@@ -1767,15 +1577,15 @@ static int *make_dollar_list (DATAINFO *pdinfo, int *err)
  * @pdinfo: dataset information.
  * @renumber: location for return of information on whether
  * remaining variables have been renumbered as a result, or
- * %NULL.
+ * NULL.
  * @prn: pointer to printing struct.
  *
  * Deletes the variables given in @list from the dataset.  Remaining
  * variables may have their ID numbers changed as a consequence. If
- * @renumber is not %NULL, this location receives 1 in case variables
+ * @renumber is not NULL, this location receives 1 in case variables
  * have been renumbered, 0 otherwise.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_drop_listed_variables (int *list, double ***pZ, 
@@ -1783,6 +1593,7 @@ int dataset_drop_listed_variables (int *list, double ***pZ,
 				   int *renumber,
 				   PRN *prn)
 {
+    int oldv = pdinfo->v;
     int *dlist = NULL;
     int free_dlist = 0;
     int lastvar[2];
@@ -1831,6 +1642,8 @@ int dataset_drop_listed_variables (int *list, double ***pZ,
 
     if (free_dlist) {
 	free(dlist);
+    } else if (pdinfo->v != oldv) {
+	set_dataset_is_changed();
     }
 
     return err;
@@ -1844,7 +1657,7 @@ int dataset_drop_listed_variables (int *list, double ***pZ,
  *
  * Deletes variable @v from the dataset.  
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo) 
@@ -1938,7 +1751,7 @@ int dataset_renumber_variable (int v_old, int v_new,
  * numbers greater than or equal to @vmin.  Never deletes the
  * automatically generated constant (ID number 0).
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
@@ -2140,7 +1953,7 @@ int dataset_sort_by (const int *list, double **Z, DATAINFO *pdinfo, gretlopt opt
     spoint_t *sv = NULL;
     double *x = NULL;
     char **S = NULL;
-    int ns = list[1];
+    int ns = list[0];
     int i, t, v;
     int err = 0;
 
@@ -2229,7 +2042,7 @@ static int dataset_sort (const int *list, double **Z, DATAINFO *pdinfo,
  * Deletes from the dataset the number @delvars of variables 
  * that were added most recently (that have the highest ID numbers).
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, E_ALLOC on error.
  */
 
 int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
@@ -2907,6 +2720,24 @@ static int add_obs (int n, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
+static int insert_obs (int n, double ***pZ, DATAINFO *pdinfo, PRN *prn)
+{
+    int err = 0;
+
+    if (complex_subsampled()) {
+	pprintf(prn, _("The data set is currently sub-sampled.\n"));
+	err = E_DATA;
+    } else if (dataset_is_panel(pdinfo)) {
+	err = E_PDWRONG;
+    } else if (n <= 0 || n > pdinfo->n) {
+	err = E_DATA;
+    } else {
+	err = real_insert_observation(n - 1, pZ, pdinfo);
+    }
+
+    return err;
+}
+
 int dataset_op_from_string (const char *s)
 {
     int op = DS_NONE;
@@ -2935,6 +2766,8 @@ int dataset_op_from_string (const char *s)
 	op = DS_CLEAR;
     } else if (!strcmp(s, "renumber")) {
 	op = DS_RENUMBER;
+    } else if (!strcmp(s, "insobs")) {
+	op = DS_INSOBS;
     }
 
     return op;
@@ -2965,6 +2798,10 @@ static int dataset_int_param (const char **ps, int op,
 
     if (k <= 0 || (op == DS_RESAMPLE && k < 1)) {
 	*err = E_DATA;
+    } else if (op == DS_INSOBS) {
+	if (k > pdinfo->n) {
+	    *err = E_DATA;
+	}
     } else if (op == DS_COMPACT) {
 	int ok = 0;
 
@@ -2978,19 +2815,6 @@ static int dataset_int_param (const char **ps, int op,
 	    ok = 1;
 	}
 
-	if (!ok) {
-	    *err = E_PDWRONG;
-	    gretl_errmsg_set("This conversion is not supported");
-	}
-    } else if (op == DS_EXPAND) {
-	int ok = 0;
-
-	if (pdinfo->pd == 1 && (k == 4 || k == 12)) {
-	    ok = 1;
-	} else if (pdinfo->pd == 4 && k == 12) {
-	    ok = 1;
-	} 
-
 	if (!ok) {
 	    *err = E_PDWRONG;
 	    gretl_errmsg_set("This conversion is not supported");
@@ -3192,12 +3016,13 @@ int renumber_series_with_checks (const char *s, int fixmax,
    dataset addobs            24
    dataset compact           1
    dataset compact           4 last
-   dataset expand            12
+   dataset expand            interpolate
    dataset transpose
    dataset sortby     x1
    dataset resample          500
    dataset clear 
    dataset renumber   orig   2  
+   dataset insobs     13
 
 */
 
@@ -3247,20 +3072,35 @@ int modify_dataset (int op, const int *list, const char *s,
 	return 1;
     }
 
-    if (op == DS_ADDOBS || op == DS_COMPACT || 
-	op == DS_EXPAND || op == DS_RESAMPLE) {
+    if (op == DS_ADDOBS || op == DS_INSOBS || 
+	op == DS_COMPACT || op == DS_RESAMPLE) {
 	k = dataset_int_param(&s, op, *pZ, pdinfo, &err);
 	if (err) {
 	    return err;
 	}
+    } else if (op == DS_EXPAND) {
+	if (pdinfo->pd == 1) {
+	    k = 4;
+	} else if (pdinfo->pd == 4) {
+	    k = 12;
+	} else {
+	    return E_PDWRONG;
+	}
     }
 
     if (op == DS_ADDOBS) {
 	err = add_obs(k, pZ, pdinfo, prn);
+    } else if (op == DS_INSOBS) {
+	err = insert_obs(k, pZ, pdinfo, prn);
     } else if (op == DS_COMPACT) {
 	err = compact_data_set_wrapper(s, pZ, pdinfo, k);
     } else if (op == DS_EXPAND) {
-	err = expand_data_set(pZ, pdinfo, k);
+	int interp = 0, n = strlen(s);
+
+	if (n > 0 && !strncmp(s, "interpolate", n)) {
+	    interp = 1;
+	}
+	err = expand_data_set(pZ, pdinfo, k, interp);
     } else if (op == DS_TRANSPOSE) {
 	err = transpose_data(pZ, pdinfo);
     } else if (op == DS_SORTBY) {
@@ -3308,6 +3148,44 @@ int dataset_get_structure (const DATAINFO *pdinfo)
     }
 }
 
+/**
+ * panel_sample_size:
+ * @pdinfo: pointer to data information struct.
+ * 
+ * Returns: the numbers of units/individuals in the current
+ * sample range, or 0 if the dataset is not a panel.
+ */
+
+int panel_sample_size (const DATAINFO *pdinfo)
+{
+    int ret = 0;
+
+    if (dataset_is_panel(pdinfo)) {
+	ret = (pdinfo->t2 - pdinfo->t1 + 1) / pdinfo->pd;
+    }
+
+    return ret;
+}
+
+/**
+ * multi_unit_panel_sample:
+ * @pdinfo: pointer to data information struct.
+ * 
+ * Returns: 1 if the dataset is a panel and the current sample
+ * range includes two or more individuals, otherwise 0.
+ */
+
+int multi_unit_panel_sample (const DATAINFO *pdinfo)
+{
+    int ret = 0;
+
+    if (dataset_is_panel(pdinfo)) {
+	ret = (pdinfo->t2 - pdinfo->t1 + 1) > pdinfo->pd;
+    }
+
+    return ret;
+}
+
 /**
  * dataset_purge_missing_rows:
  * @Z: data array.
