@@ -53,7 +53,8 @@ typedef enum {
     DS_RESAMPLE,
     DS_RESTORE,
     DS_CLEAR,
-    DS_RENUMBER
+    DS_RENUMBER,
+    DS_INSOBS
 } DatasetOp;
 
 typedef enum {
@@ -281,10 +282,12 @@ typedef enum {
 
 #define sample_size(p) ((p == NULL)? 0 : (p->t2 - p->t1 + 1))
 
-void free_Z (double **Z, DATAINFO *pdinfo);
+void free_Z (double **Z, const DATAINFO *pdinfo);
 
 DATAINFO *datainfo_new (void);
 
+void datainfo_init (DATAINFO *pdinfo);
+
 DATAINFO *create_new_dataset (double ***pZ, /* data matrix */
 			      int nvar,     /* number of variables */
 			      int nobs,     /* observations per variable */
@@ -306,14 +309,6 @@ int dataset_allocate_obs_markers (DATAINFO *pdinfo);
 
 void dataset_destroy_obs_markers (DATAINFO *pdinfo);
 
-int dataset_allocate_panel_info (DATAINFO *pdinfo);
-
-void dataset_destroy_panel_info (DATAINFO *pdinfo);
-
-int dataset_add_default_panel_indices (DATAINFO *pdinfo);
-
-int dataset_finalize_panel_indices (DATAINFO *pdinfo);
-
 void dataset_obs_info_default (DATAINFO *pdinfo);
 
 void copy_dataset_obs_info (DATAINFO *targ, const DATAINFO *src);
@@ -420,6 +415,10 @@ int modify_dataset (int op, const int *list, const char *s,
 
 int dataset_get_structure (const DATAINFO *pdinfo);
 
+int panel_sample_size (const DATAINFO *pdinfo);
+
+int multi_unit_panel_sample (const DATAINFO *pdinfo);
+
 int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo);
 
 int check_dataset_is_changed (void);
