@@ -24,6 +24,7 @@
 #include "gretl_www.h"
 #include "libset.h"
 #include "gretl_string_table.h"
+#include "matrix_extra.h"
 #include "dbread.h"
 
 #include <glib.h>
@@ -956,6 +957,7 @@ static void series_info_init (SERIESINFO *sinfo)
 void dbwrapper_destroy (dbwrapper *dw)
 {
     if (dw != NULL) {
+	free(dw->fname);
 	free(dw->sinfo);
 	free(dw);
     }
@@ -963,12 +965,14 @@ void dbwrapper_destroy (dbwrapper *dw)
 
 /**
  * dbwrapper_new:
- * @n:
+ * @n: initial number of series.
+ * @fname: database filename.
+ * @dbtype: database type code.
  *
  * Returns: an allocated database series wrapper.
  */
 
-dbwrapper *dbwrapper_new (int n)
+dbwrapper *dbwrapper_new (int n, const char *fname, int dbtype)
 {
     dbwrapper *dw;
     int i;
@@ -982,6 +986,9 @@ dbwrapper *dbwrapper_new (int n)
 	return NULL;
     }
 
+    dw->fname = gretl_strdup(fname);
+    dw->dbtype = dbtype;
+
     dw->sinfo = malloc(n * sizeof *dw->sinfo);
     if (dw->sinfo == NULL) {
 	free(dw);
@@ -1104,6 +1111,7 @@ static int count_in7_series (FILE *fp, int *err)
 
 /**
  * read_pcgive_db:
+ * @fname: name of database file.
  * @fp: pre-opened stream (caller to close it)
  *
  * Read the series info from a PcGive database, .in7 file
@@ -1112,7 +1120,7 @@ static int count_in7_series (FILE *fp, int *err)
  * or NULL in case of failure.
  */
 
-dbwrapper *read_pcgive_db (FILE *fp) 
+dbwrapper *read_pcgive_db (const char *fname, FILE *fp) 
 {
     dbwrapper *dw;
     int ns, err = 0;
@@ -1132,7 +1140,7 @@ dbwrapper *read_pcgive_db (FILE *fp)
 #endif
     
     /* allocate table for series rows */
-    dw = dbwrapper_new(ns);
+    dw = dbwrapper_new(ns, fname, GRETL_PCGIVE_DB);
     if (dw == NULL) {
 	gretl_errmsg_set(_("Out of memory!"));
 	return NULL;
@@ -1153,6 +1161,7 @@ dbwrapper *read_pcgive_db (FILE *fp)
 
 /**
  * read_rats_db:
+ * @fname: database filename.
  * @fp: pre-opened stream (caller to close it)
  *
  * Read the series info from a RATS 4.0 database: read the base 
@@ -1163,7 +1172,7 @@ dbwrapper *read_pcgive_db (FILE *fp)
  * or NULL in case of failure.
  */
 
-dbwrapper *read_rats_db (FILE *fp) 
+dbwrapper *read_rats_db (const char *fname, FILE *fp) 
 {
     dbwrapper *dw;
     long forward;
@@ -1184,7 +1193,7 @@ dbwrapper *read_rats_db (FILE *fp)
     }
 
     /* allocate table for series rows */
-    dw = dbwrapper_new(0);
+    dw = dbwrapper_new(0, fname, GRETL_RATS_DB);
     if (dw == NULL) {
 	gretl_errmsg_set(_("Out of memory!"));
 	return NULL;
@@ -1469,39 +1478,88 @@ double *compact_db_series (const double *src, SERIESINFO *sinfo,
     return x;
 }
 
+static double *interpolate_db_series (const double *src,
+				      int oldn, int mult,
+				      int *err)
+{
+    gretl_matrix *yx;
+    gretl_matrix *y;
+    double *ret = NULL;
+    int t;
+
+    y = gretl_column_vector_alloc(oldn);
+    if (y == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (t=0; t<oldn; t++) {
+	y->val[t] = src[t];
+    }
+
+    yx = matrix_chowlin(y, NULL, mult, err);
+    gretl_matrix_free(y);
+
+    if (!*err) {
+	ret = yx->val;
+	yx->val = NULL;
+    }
+
+    gretl_matrix_free(yx);
+
+    return ret;
+}
+
 /* Expand a single series from a database, for importation 
    into a working dataset of higher frequency.  At present
    this is permitted only for the cases:
 
-     annual    -> quarterly
-     annual    -> monthly
-     quarterly -> monthly
+   1) annual    -> quarterly
+   2) annual    -> monthly
+   3) quarterly -> monthly
+
+   Interpolation is supported for cases 1 and 3 only.
 */
 
 double *expand_db_series (const double *src, SERIESINFO *sinfo,
-			  int target_pd)
+			  int target_pd, int interpol)
 {
     char stobs[12] = {0};
     int oldn = sinfo->nobs;
     int mult, newn;
     double *x = NULL;
-    int j, s, t;
+    int j, t;
+    int err = 0;
 
     mult = target_pd / sinfo->pd;
     newn = mult * sinfo->nobs;
 
-    x = malloc(newn * sizeof *x);
-    if (x == NULL) {
-	return NULL;
-    }    
+    if (!((target_pd == 4 && sinfo->pd == 1) ||
+	  (target_pd == 12 && sinfo->pd == 4))) {
+	interpol = 0;
+    }
 
-    s = 0;
-    for (t=0; t<oldn; t++) {
-	for (j=0; j<mult; j++) {
-	    x[s++] = src[t];
+    if (interpol) {
+	x = interpolate_db_series(src, oldn, mult, &err);
+    } else {
+	x = malloc(newn * sizeof *x);
+	if (x == NULL) {
+	    err = E_ALLOC;
+	} else {  
+	    int s = 0;
+
+	    for (t=0; t<oldn; t++) {
+		for (j=0; j<mult; j++) {
+		    x[s++] = src[t];
+		}
+	    }
 	}
     }
 
+    if (err) {
+	return NULL;
+    }
+
     if (sinfo->pd == 1) {
 	strcpy(stobs, sinfo->stobs);
 	if (target_pd == 4) {
@@ -2768,10 +2826,14 @@ static double *compact_series (const double *src, int i, int n, int oldn,
 	    }
 	} else if (method == COMPACT_SUM || method == COMPACT_AVG) {
 	    int j, st, den = compfac;
+	    int n_ok = 0;
 
-	    if (idx + compfac - 1 > oldn - 1) break;
+	    if (idx + compfac - 1 > oldn - 1) {
+		break;
+	    }
 
 	    x[t] = 0.0;
+
 	    for (j=0; j<compfac; j++) {
 		st = idx + j;
 		if (na(src[st])) {
@@ -2782,9 +2844,16 @@ static double *compact_series (const double *src, int i, int n, int oldn,
 			break;
 		    }
 		} else {
+		    /* got a valid observation */
+		    n_ok++;
 		    x[t] += src[st];
 		}
 	    }
+
+	    if (n_ok == 0) {
+		x[t] = NADBL;
+	    }
+
 	    if (method == COMPACT_AVG && !na(x[t])) {
 		if (den > 0) {
 		    x[t] /= den;
@@ -2893,19 +2962,30 @@ daily_series_to_monthly (const double *src, DATAINFO *pdinfo, int i,
 	} else if (method == COMPACT_SUM ||
 		   method == COMPACT_AVG) {
 	    int j, dayt, den = mdays;
+	    int n_ok = 0;
 
 	    x[t] = 0.0;
+
 	    for (j=0; j<mdays; j++) {
 		dayt = sopt + j;
 		if (dayt >= pdinfo->n) {
 		    x[t] = NADBL;
 		    break;
-		} else if (na(z[dayt]) && method == COMPACT_AVG) {
-		    den--;
-		} else if (!na(z[dayt])) {
+		} else if (na(z[dayt])) {
+		    if (method == COMPACT_AVG) {
+			den--;
+		    }
+		} else {
+		    /* got a valid observation */
 		    x[t] += z[dayt];
+		    n_ok++;
 		}
 	    }
+
+	    if (n_ok == 0) {
+		x[t] = NADBL;
+	    } 
+
 	    if (method == COMPACT_AVG && !na(x[t])) {
 		if (den > 0) {
 		    x[t] /= (double) den;
@@ -3737,11 +3817,40 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
     return err;
 }
 
+static gretl_matrix *
+interpol_expand_dataset (const double **Z, const DATAINFO *pdinfo, 
+			 int newpd, int *err)
+{
+    gretl_matrix *Y0, *Y1 = NULL;
+    int *list;
+
+    list = gretl_consecutive_list_new(1, pdinfo->v - 1);
+    if (list == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    Y0 = gretl_matrix_data_subset(list, Z, pdinfo->t1, pdinfo->t2,
+				  M_MISSING_ERROR, err);
+
+    if (!*err) {
+	int f = newpd / pdinfo->pd;
+
+	Y1 = matrix_chowlin(Y0, NULL, f, err);
+	gretl_matrix_free(Y0);
+    }
+
+    free(list);
+
+    return Y1;
+}
+
 /**
  * expand_data_set:
  * @pZ: pointer to data array.
  * @pdinfo: data information struct.
- * @newpd: target data frequency
+ * @newpd: target data frequency.
+ * @interpol: use interpolation (0/1).
  * 
  * Expand the data set from lower to higher frequency: an "expert"
  * option.  This is supported at present only for expansion from
@@ -3750,7 +3859,8 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
  * Returns: 0 on success, non-zero error code on failure.
  */
 
-int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
+int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
+		     int interpol)
 {
     char stobs[12];
     int oldn = pdinfo->n;
@@ -3758,6 +3868,7 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
     int t1 = pdinfo->t1;
     int t2 = pdinfo->t2;
     int mult, newn, nadd;
+    gretl_matrix *X = NULL;
     double *x = NULL;
     int i, j, s, t;
     int err = 0;
@@ -3768,11 +3879,22 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
 	return E_DATA;
     } else if (oldpd == 4 && newpd != 12) {
 	return E_DATA;
+    } else if (oldpd == 1 && newpd == 12 && interpol) {
+	return E_DATA;
     }
 
-    x = malloc(oldn * sizeof *x);
-    if (x == NULL) {
-	return E_ALLOC;
+    if (interpol) {
+	X = interpol_expand_dataset((const double **) *pZ,
+				    pdinfo, newpd, &err);
+    } else {
+	x = malloc(oldn * sizeof *x);
+	if (x == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (err) {
+	return err;
     }
 
     mult = newpd / oldpd;
@@ -3784,14 +3906,22 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
 	goto bailout;
     }
 
-    for (i=1; i<pdinfo->v; i++) {
-	for (t=0; t<oldn; t++) {
-	    x[t] = (*pZ)[i][t];
+    if (interpol) {
+	for (i=1; i<pdinfo->v; i++) {
+	    for (t=0; t<newn; t++) {
+		(*pZ)[i][t] = gretl_matrix_get(X, t, i-1);
+	    }
 	}
-	s = 0;
-	for (t=0; t<oldn; t++) {
-	    for (j=0; j<mult; j++) {
-		(*pZ)[i][s++] = x[t];
+    } else {
+	for (i=1; i<pdinfo->v; i++) {
+	    for (t=0; t<oldn; t++) {
+		x[t] = (*pZ)[i][t];
+	    }
+	    s = 0;
+	    for (t=0; t<oldn; t++) {
+		for (j=0; j<mult; j++) {
+		    (*pZ)[i][s++] = x[t];
+		}
 	    }
 	}
     }
@@ -3831,6 +3961,7 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
  bailout:
     
     free(x);
+    gretl_matrix_free(X);
 
     return err;
 }
