@@ -1085,7 +1085,7 @@ static MODEL ordered_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* run initial OLS, with dummies added */
-    model = lsq(biglist, pZ, pdinfo, OLS, OPT_A);
+    model = lsq(biglist, *pZ, pdinfo, OLS, OPT_A);
     if (model.errcode) {
 	fprintf(stderr, "ordered_estimate: initial OLS failed\n");
 	free(dumlist);
@@ -1458,12 +1458,13 @@ static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
   no maximum (despite having a supremum at 1) and no MLE exists.
 */
 
-static int perfect_pred_check (const double *y, MODEL *dmod)
+static int perfect_pred_check (const double *y, MODEL *dmod,
+			       double *beta, PRN *prn)
 {
     double max0 = -1.0e200;
     double min1 = 1.0e200;
     double yht;
-    int t;
+    int t, ret;
 
     for (t=dmod->t1; t<=dmod->t2; t++) {
 	yht = dmod->yhat[t];
@@ -1477,7 +1478,18 @@ static int perfect_pred_check (const double *y, MODEL *dmod)
 	}
     }
 
-    return (min1 > max0);
+    ret = (min1 > max0);
+
+    if (ret) {
+	pputs(prn, "\nPerfect prediction obtained with\n\n");
+	for (t=0; t<dmod->ncoeff; t++) {
+	    pprintf(prn, " beta[%d] = % .15g\n", t, beta[t]);
+	}
+	pprintf(prn, "\ny = 1 when 1/(1+exp(-X*beta)) > %.15g\n\n", 
+		1/(1+exp(-max0)));
+    }
+
+    return ret;
 }
 
 /* BRMR, Davidson and MacKinnon, ETM, p. 461 */
@@ -1511,14 +1523,14 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
 
     for (iter=0; iter<itermax; iter++) {
 
-	if (perfect_pred_check(yvar, dmod)) {
+	if (perfect_pred_check(yvar, dmod, beta, prn)) {
 	    gretl_errmsg_sprintf("Perfect prediction detected at iteration %d;\nno MLE exists",
 				 iter);
 	    err = E_NOCONV;
 	    break;
 	}
 
-	/* construct BRMR dataset */
+	/* (re-)construct BRMR dataset */
 	s = 0;
 	for (t=dmod->t1; t<=dmod->t2; t++) {
 	    yt = dmod->yhat[t];
@@ -1824,9 +1836,11 @@ static int add_slopes_to_model (MODEL *pmod, const double **Z)
     return err;
 }
 
-static void binary_model_add_stats (MODEL *pmod, const double *y)
+static void binary_model_add_stats (MODEL *pmod, const double *y,
+				    const DATAINFO *pdinfo)
 {
     int *act_pred;
+    double *llt;
     double F, xx = 0.0;
     int i, t;
 
@@ -1838,6 +1852,13 @@ static void binary_model_add_stats (MODEL *pmod, const double *y)
 	}
     }
 
+    llt = malloc(pdinfo->n * sizeof *llt);
+    if (llt != NULL) {
+	for (i=0; i<pdinfo->n; i++) {
+	    llt[i] = NADBL;
+	}
+    }
+
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	double xb = pmod->yhat[t];
 	int yt;
@@ -1864,7 +1885,9 @@ static void binary_model_add_stats (MODEL *pmod, const double *y)
 	    pmod->uhat[t] = (yt != 0)? invmills(-xb) : -invmills(xb);
 	}
 
-	pmod->llt[t] = (yt != 0)? log(F) : log(1-F);
+	if (llt != NULL) {
+	    llt[t] = (yt != 0)? log(F) : log(1-F);
+	}
     }
 
     pmod->ybar = xx / pmod->nobs;
@@ -1875,11 +1898,17 @@ static void binary_model_add_stats (MODEL *pmod, const double *y)
 			     4 * sizeof *act_pred);
     }
 
+    if (llt != NULL) {
+	gretl_model_set_data(pmod, "llt", llt, 
+			     GRETL_TYPE_DOUBLE_ARRAY,
+			     pdinfo->n * sizeof *llt);
+    }
+
     mle_criteria(pmod, 0);
 }
 
 static MODEL 
-binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo, 
+binary_logit_probit (const int *inlist, double **Z, DATAINFO *pdinfo, 
 		     int ci, gretlopt opt, PRN *prn)
 {
     int oldt1 = pdinfo->t1;
@@ -1887,7 +1916,6 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     int *list = NULL;
     char *mask = NULL;
     double *beta = NULL;
-    const double **Z;
     MODEL dmod;
     int i, depvar, nx;
 
@@ -1897,7 +1925,6 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     gretl_model_init(&dmod);
 
     depvar = inlist[1];
-    Z = (const double **) *pZ;
 
     if (!gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[depvar])) {
 	gretl_errmsg_sprintf(_("'%s' is not a binary variable"), 
@@ -1912,9 +1939,10 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, Z);
+    /* FIXME should the return value be ignored here? */
+    list_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, (const double **) Z);
 
-    mask = classifier_check(list, Z, pdinfo, opt, prn,
+    mask = classifier_check(list, (const double **) Z, pdinfo, opt, prn,
 			    &dmod.errcode);
     if (dmod.errcode) {
 	goto bailout;
@@ -1928,15 +1956,12 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!dmod.errcode) {
-	dmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	dmod = lsq(list, Z, pdinfo, OLS, OPT_A);
 	if (dmod.errcode == 0 && dmod.ncoeff != nx) {
 	    dmod.errcode = E_DATA;
 	}
     }
 
-    /* re-attach */
-    Z = (const double **) *pZ;
-
     if (dmod.errcode) {
 	goto bailout;
     }
@@ -1950,7 +1975,8 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     if (beta == NULL) {
 	dmod.errcode = E_ALLOC;
     } else {
-	dmod.errcode = do_BRMR(dmod.list, &dmod, ci, beta, Z, pdinfo, 
+	dmod.errcode = do_BRMR(dmod.list, &dmod, ci, beta, 
+			       (const double **) Z, pdinfo, 
 			       (opt & OPT_V)? prn : NULL);
     }
 
@@ -1964,24 +1990,24 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
     }
 
     dmod.lnL = logit_probit_loglik(Z[depvar], &dmod, ci);
-    Lr_chisq(&dmod, Z);
+    Lr_chisq(&dmod, (const double **) Z);
     dmod.ci = ci;
 
     /* calculate standard errors etc */
-    dmod.errcode = logit_probit_vcv(&dmod, opt, Z);
+    dmod.errcode = logit_probit_vcv(&dmod, opt, (const double **) Z);
 
     if (!dmod.errcode) {
 	if (opt & OPT_P) {
 	    /* showing p-values, not slopes */
 	    dmod.opt |= OPT_P;
-	    dmod.sdy = binary_fXb(&dmod, Z);
+	    dmod.sdy = binary_fXb(&dmod, (const double **) Z);
 	} else {
-	    dmod.errcode = add_slopes_to_model(&dmod, Z);
+	    dmod.errcode = add_slopes_to_model(&dmod, (const double **) Z);
 	}
     }
 
     if (!dmod.errcode) {
-	binary_model_add_stats(&dmod, Z[depvar]);
+	binary_model_add_stats(&dmod, Z[depvar], pdinfo);
 	if (opt & OPT_A) {
 	    dmod.aux = AUX_AUX;
 	} else {
@@ -2320,7 +2346,7 @@ static void mn_logit_yhat (MODEL *pmod, mnl_info *mnl,
 	if (na(pmod->yhat[t])) {
 	    continue;
 	}
-	pmax = 1.0;
+	pmax = 0.0;
 	iymax = 0;
 	for (i=0; i<mnl->n; i++) {
 	    p = gretl_matrix_get(mnl->Xb, s, i);
@@ -2346,6 +2372,132 @@ static void mn_logit_yhat (MODEL *pmod, mnl_info *mnl,
     gretl_model_set_int(pmod, "correct", ncorrect);
 }
 
+/**
+ * mn_logit_probabilities:
+ * @pmod: pointer to multinomial logit model
+ * @Z: data array.
+ * @pdinfo: dataset information.
+ * @err: location to receive error code.
+ *
+ * Computes the estimated probabilities of the outcomes
+ * for a multinomial logit model. The returned matrix
+ * is n x m, where n is the number of observations in
+ * the sample range over which the model was estimated
+ * and m is the number of distinct outcomes. Each element
+ * represents the conditional probability of outcome j
+ * given the values of the regressors at observation i.
+ * 
+ * If any of the regressor values are missing at a given
+ * observation the probabiity is set to NaN; provided the
+ * regressor information is complete we compute the
+ * outcome probabilities even if the actual outcome is
+ * missing.
+ *
+ * Returns: allocated matrix or NULL on failure.
+ */
+
+gretl_matrix *mn_logit_probabilities (const MODEL *pmod,
+				      const double **Z,
+				      const DATAINFO *pdinfo,
+				      int *err)
+{
+    gretl_matrix *P = NULL;
+    const int *yvals = NULL;
+    double *eXbt = NULL;
+    int i, m = 0;
+
+    if (pmod == NULL || pmod->list == NULL || pmod->coeff == NULL) {
+	*err = E_DATA;
+    }
+
+    if (!*err) {
+	/* list of outcome values (including the base case) */
+	yvals = gretl_model_get_data(pmod, "yvals");
+	if (yvals == NULL) {
+	    *err = E_DATA;
+	} else {
+	    m = yvals[0];
+	}
+    }
+
+    if (!*err) {
+	for (i=1; i<=pmod->list[0]; i++) {
+	    if (pmod->list[i] >= pdinfo->v) {
+		/* a regressor has disappeared */
+		*err = E_DATA;
+		break;
+	    }
+	}
+    }
+
+    if (!*err) {
+	int n = pmod->t2 - pmod->t1 + 1;
+
+	P = gretl_matrix_alloc(n, m);
+	if (P == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    /* allow for casting results to series */
+	    gretl_matrix_set_t1(P, pmod->t1);
+	    gretl_matrix_set_t2(P, pmod->t2);
+	}
+    }
+
+    if (!*err) {
+	/* allocate required workspace */
+	eXbt = malloc(m * sizeof *eXbt);
+	if (eXbt == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	const double *b = pmod->coeff;
+	double St, ptj;
+	int j, k, t, vi, ok;
+
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    ok = 1;
+	    for (i=2; i<=pmod->list[0]; i++) {
+		vi = pmod->list[i];
+		if (na(Z[vi][t])) {
+		    ok = 0;
+		    break;
+		}
+	    }
+	    if (!ok) {
+		/* one or more regressors missing */
+		for (j=0; j<m; j++) {
+		    gretl_matrix_set(P, t, j, M_NA);
+		}
+	    } else {
+		/* base case */
+		eXbt[0] = St = 1.0;
+		k = 0;
+		/* loop across the other y-values */
+		for (j=1; j<m; j++) {
+		    /* accumulate exp(X*beta) */
+		    eXbt[j] = 0.0;
+		    for (i=2; i<=pmod->list[0]; i++) {
+			vi = pmod->list[i];
+			eXbt[j] += Z[vi][t] * b[k++];
+		    }
+		    eXbt[j] = exp(eXbt[j]);
+		    St += eXbt[j];
+		}
+		for (j=0; j<m; j++) {
+		    ptj = eXbt[j] / St;
+		    gretl_matrix_set(P, t, j, ptj);
+		}
+	    }
+	}
+    }
+
+    free(eXbt);
+
+    return P;
+}
+
 /* In case the dependent variable is not in canonical form for
    multinomial logit, construct a transformed version */
 
@@ -2594,7 +2746,7 @@ static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo,
     int i, vi, t, s;
 
     /* we'll start with OLS to flush out data issues */
-    mod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+    mod = lsq(list, *pZ, pdinfo, OLS, OPT_A);
     if (mod.errcode) {
 	return mod;
     }
@@ -2670,10 +2822,56 @@ static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo,
     return mod;
 }
 
+/**
+ * biprobit_model:
+ * @list: binary dependent variable 1, binary dependent variable 2,
+ * list of regressors for y1. If @list ends here, it is assumed that the
+ * explanatory variables for y2 are the same as y1. Otherwise, the list 
+ * must include a separator and the list of regressors for y2.
+ * @Z: data array.
+ * @pdinfo: information on the data set.
+ * @opt: can contain OPT_Q for quiet operation, OPT_V for verbose 
+ * operation, OPT_R for robust covariance matrix, OPT_G for covariance
+ * matrix based on Outer Product of Gradient.
+ * @prn: printing struct.
+ *
+ * Computes estimates of the bivariate probit model specified by @list,
+ * using maximum likelihood via Newton-Raphson.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL biprobit_model (int *list, double **Z, DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
+{
+    MODEL bpmod;
+    void *handle;
+    MODEL (* biprobit_estimate) (const int *, double **, DATAINFO *, 
+				 gretlopt, PRN *);
+
+    gretl_error_clear();
+
+    biprobit_estimate = get_plugin_function("biprobit_estimate", &handle);
+
+    if (biprobit_estimate == NULL) {
+	gretl_model_init(&bpmod);
+	bpmod.errcode = E_FOPEN;
+	return bpmod;
+    }
+
+    bpmod = (*biprobit_estimate) (list, Z, pdinfo, opt, prn);
+
+    close_plugin(handle);
+
+    set_model_id(&bpmod);
+
+    return bpmod;
+}
+
 /**
  * binary_logit:
  * @list: binary dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
+ * @Z: data array.
  * @pdinfo: information on the data set.
  * @opt: if includes OPT_R form robust (QML) estimates of standard
  * errors and covariance matrix; if OPT_P arrange for
@@ -2689,16 +2887,16 @@ static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo,
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL binary_logit (int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL binary_logit (int *list, double **Z, DATAINFO *pdinfo, 
 		    gretlopt opt, PRN *prn)
 {
-    return binary_logit_probit(list, pZ, pdinfo, LOGIT, opt, prn);
+    return binary_logit_probit(list, Z, pdinfo, LOGIT, opt, prn);
 }
 
 /**
  * binary_probit:
  * @list: binary dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
+ * @Z: data array.
  * @pdinfo: information on the data set.
  * @opt: if includes OPT_R form robust (QML) estimates of standard
  * errors and covariance matrix; if OPT_P arrange for
@@ -2714,10 +2912,10 @@ MODEL binary_logit (int *list, double ***pZ, DATAINFO *pdinfo,
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL binary_probit (int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL binary_probit (int *list, double **Z, DATAINFO *pdinfo, 
 		     gretlopt opt, PRN *prn)
 {
-    return binary_logit_probit(list, pZ, pdinfo, PROBIT, opt, prn);
+    return binary_logit_probit(list, Z, pdinfo, PROBIT, opt, prn);
 }
 
 /**
@@ -3078,7 +3276,7 @@ MODEL logistic_model (const int *list, double lmax,
     /* replace with transformed dependent variable */
     llist[1] = pdinfo->v - 1;
 
-    lmod = lsq(llist, pZ, pdinfo, OLS, OPT_A);
+    lmod = lsq(llist, *pZ, pdinfo, OLS, OPT_A);
     if (!lmod.errcode) {
 	rewrite_logistic_stats((const double **) *pZ, pdinfo, &lmod,
 			       dv, real_lmax);
@@ -3276,18 +3474,17 @@ MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo,
    the business
 */
 
-static int duration_precheck (const int *list, double ***pZ,
+static int duration_precheck (const int *list, double **Z,
 			      DATAINFO *pdinfo, MODEL *pmod,
 			      int *pcensvar)
 {
-    const double **Z = (const double **) *pZ;
     int *olslist = NULL;
     int seppos, censvar = 0;
     int l0 = list[0];
     int err = 0;
 
     /* such models must contain a constant */
-    if (!gretl_list_const_pos(list, 2, Z, pdinfo)) {
+    if (!gretl_list_const_pos(list, 2, (const double **) Z, pdinfo)) {
 	return E_NOCONST;
     }
 
@@ -3323,7 +3520,7 @@ static int duration_precheck (const int *list, double ***pZ,
 	   coefficients etc.
 	*/	
 	if (olslist != NULL) {
-	    *pmod = lsq(olslist, pZ, pdinfo, OLS, OPT_A);
+	    *pmod = lsq(olslist, Z, pdinfo, OLS, OPT_A);
 	    if (!pmod->errcode) {
 		/* remove reference to censoring var */
 		pmod->list[0] -= 1;
@@ -3333,7 +3530,7 @@ static int duration_precheck (const int *list, double ***pZ,
 	    }
 	    free(olslist);
 	} else {
-	    *pmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
+	    *pmod = lsq(list, Z, pdinfo, OLS, OPT_A);
 	}
 	err = pmod->errcode;
 	*pcensvar = censvar;
@@ -3343,7 +3540,7 @@ static int duration_precheck (const int *list, double ***pZ,
 	int t, yno = pmod->list[1];
 
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    if (!na(pmod->uhat[t]) && (*pZ)[yno][t] <= 0) {
+	    if (!na(pmod->uhat[t]) && Z[yno][t] <= 0) {
 		gretl_errmsg_set(_("Durations must be positive"));
 		err = E_DATA;
 	    }
@@ -3356,7 +3553,7 @@ static int duration_precheck (const int *list, double ***pZ,
 /**
  * duration_model:
  * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data array.
+ * @Z: data array.
  * @pdinfo: dataset information.
  * @opt: may include OPT_R for robust covariance matrix.
  * @prn: printing struct for iteration info (or NULL is this is not
@@ -3367,7 +3564,7 @@ static int duration_precheck (const int *list, double ***pZ,
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL duration_model (const int *list, double ***pZ, 
+MODEL duration_model (const int *list, double **Z, 
 		      DATAINFO *pdinfo, gretlopt opt, 
 		      PRN *prn)
 {
@@ -3381,7 +3578,7 @@ MODEL duration_model (const int *list, double ***pZ,
     gretl_error_clear();
     gretl_model_init(&dmod);
 
-    dmod.errcode = duration_precheck(list, pZ, pdinfo, &dmod,
+    dmod.errcode = duration_precheck(list, Z, pdinfo, &dmod,
 				     &censvar);
     if (dmod.errcode) {
         return dmod;
@@ -3395,7 +3592,7 @@ MODEL duration_model (const int *list, double ***pZ,
 	return dmod;
     }
 
-    (*duration_estimate) (&dmod, censvar, (const double **) *pZ, pdinfo, 
+    (*duration_estimate) (&dmod, censvar, (const double **) Z, pdinfo, 
 			  opt, prn);
 
     close_plugin(handle);
@@ -3469,7 +3666,7 @@ MODEL count_model (const int *list, int ci,
        coefficients etc.
     */
 
-    cmod = lsq(listcpy, pZ, pdinfo, OLS, OPT_A);
+    cmod = lsq(listcpy, *pZ, pdinfo, OLS, OPT_A);
     free(listcpy);
 
     if (cmod.errcode) {
