@@ -273,6 +273,22 @@ static void fit_resid_set_dec_places (FITRESID *fr)
     }
 }
 
+static int special_depvar (const MODEL *pmod) 
+{
+    int ret = 0;
+
+    if (pmod->ci == INTREG) {
+	ret = 1;
+    } else if (pmod->ci == PANEL) {
+	if (pmod->opt & OPT_B) {
+	    /* panel "between" model */
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
 /**
  * get_fit_resid:
  * @pmod: the model for which actual and fitted values
@@ -294,7 +310,7 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
     FITRESID *fr;
     int t, dv = -1;
 
-    if (pmod->ci != INTREG) {
+    if (!special_depvar(pmod)) {
 	dv = gretl_model_get_depvar(pmod);
 	if (dv < 0 || dv >= pdinfo->v) {
 	    *err = E_DATA;
@@ -334,7 +350,11 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
 	fr->resid[t] = pmod->uhat[t];
     }
 
-    fit_resid_set_dec_places(fr);
+    if (dv < 0) {
+	fr->pmax = PMAX_NOT_AVAILABLE;
+    } else {
+	fit_resid_set_dec_places(fr);
+    }
     
     if (dv < 0) {
 	strcpy(fr->depvar, "implicit y");
@@ -486,36 +506,35 @@ has_real_exog_regressors (MODEL *pmod, const int *dvlags,
 static double fcast_get_ldv (Forecast *fc, int i, int t, int lag,
 			     const double **Z)
 {
+    int p = t - lag;
     double ldv;
 
     /* initialize to actual lagged value, if available */
-    if (t - lag < 0) {
+    if (p < 0) {
 	ldv = NADBL;
     } else {
-	ldv = Z[i][t-lag];
+	ldv = Z[i][p];
     }
 
 #if AR_DEBUG
     fprintf(stderr, "fcast_get_ldv: i=%d, t=%d, lag=%d; "
-	    "initial ldv = Z[%d][%d] = %g\n", i, t, lag, i, t-lag, ldv);
+	    "initial ldv = Z[%d][%d] = %g\n", i, t, lag, i, p, ldv);
 #endif
 
     if (fc->method != FC_STATIC) {
-	int yht = t - lag;
-
 #if AR_DEBUG
-	fprintf(stderr, "fcast_get_ldv (non-static): yht = %d\n", yht);
+	fprintf(stderr, "fcast_get_ldv (non-static): p = %d\n", p);
 #endif
-	if (fc->method == FC_DYNAMIC && yht >= 0) {
-	    if (!na(fc->yhat[yht])) {
-		ldv = fc->yhat[yht];
+	if (fc->method == FC_DYNAMIC && p >= 0) {
+	    if (!na(fc->yhat[p])) {
+		ldv = fc->yhat[p];
 	    }
-	} else if (fc->method == FC_AUTO && yht >= 0) {
+	} else if (fc->method == FC_AUTO && p >= 0) {
 	    if (t > fc->model_t2 + lag || na(ldv)) {
-		ldv = fc->yhat[yht];
+		ldv = fc->yhat[p];
 #if AR_DEBUG
-		fprintf(stderr, "fcast_get_ldv: reset ldv = yhat[%d] = %g\n",
-			yht, ldv);
+		fprintf(stderr, "fcast_get_ldv (FC_AUTO): "
+			"reset ldv = yhat[%d] = %g\n", p, ldv);
 #endif
 	    } 
 	}
@@ -1152,64 +1171,24 @@ static double arma_variance (const double *phi, int p,
     return sspsi;
 }
 
-#define invalid_obs(x,t) ((t) < 0 || na(x[t]))
-
 static double arima_difference_obs (const double *x, int t, 
-				    int d, int D, int s)
+				    int *c, int k)
 {
     double dx = x[t];
 
-    if (na(dx)) {
-	return NADBL;
-    }
+    if (!na(dx)) {
+	int i, s;
 
-    if (d > 0) {
-	if (invalid_obs(x, t-1)) {
-	    return NADBL;
-	}
-	dx -= x[t-1];
-    } 
-    if (d == 2) {
-	if (invalid_obs(x, t-2)) {
-	    return NADBL;
-	}
-	dx -= x[t-1] - x[t-2];
-    }
-    if (D > 0) {
-	if (invalid_obs(x, t-s)) {
-	    return NADBL;
-	}
-	dx -= x[t-s];
-	if (d > 0) {
-	    if (invalid_obs(x, t-s-1)) {
-		return NADBL;
+	for (i=0; i<k; i++) {
+	    if (c[i] != 0) {
+		s = t - i - 1;
+		if (s < 0 || na(x[s])) {
+		    dx = NADBL;
+		    break;
+		} else {
+		    dx -= c[i] * x[s];
+		}
 	    }
-	    dx += x[t-s-1];
-	}
-	if (d == 2) {
-	    if (invalid_obs(x, t-s-2)) {
-		return NADBL;
-	    }
-	    dx += x[t-s-1] - x[t-s-2];
-	}	    
-    } 
-    if (D == 2) {
-	if (invalid_obs(x, t-2*s)) {
-	    return NADBL;
-	}
-	dx -= x[t-s] - x[t-2*s];
-	if (d > 0) {
-	    if (invalid_obs(x, t-s-1) || invalid_obs(x, t-2*s-1)) {
-		return NADBL;
-	    }
-	    dx += x[t-s-1] - x[t-2*s-1];
-	}
-	if (d == 2) {
-	    if (invalid_obs(x, t-s-2) || invalid_obs(x, t-2*s-2)) {
-		return NADBL;
-	    }
-	    dx += x[t-s-1] - x[t-2*s-1];
-	    dx -= x[t-s-2] - x[t-2*s-2];
 	}
     }
 
@@ -1233,9 +1212,10 @@ static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
 				 const double **Z)
 {
     double *Xb;
+    int *delta = NULL;
     double x;
     int miss, diff = 0;
-    int d, D, s = 0;
+    int d, D, s = 0, k = 0;
     int i, j, t, vi;
 
     Xb = malloc((fc->t2 + 1) * sizeof *Xb);
@@ -1251,6 +1231,15 @@ static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
 	diff = 1;
     }
 
+    if (diff) {
+	delta = arima_delta_coeffs(d, D, s);
+	if (delta == NULL) {
+	    free(Xb);
+	    return NULL;
+	}
+	k = s + s * D;
+    }
+
     for (t=0; t<=fc->t2; t++) {
 	Xb[t] = 0.0;
 	miss = 0;
@@ -1261,7 +1250,7 @@ static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
 		Xb[t] += pmod->coeff[0];
 	    } else {
 		if (diff) {
-		    x = arima_difference_obs(Z[vi], t, d, D, s);
+		    x = arima_difference_obs(Z[vi], t, delta, k);
 		} else {
 		    x = Z[vi][t];
 		}
@@ -1275,6 +1264,10 @@ static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
 	}
     }
 
+    if (delta != NULL) {
+	free(delta);
+    }
+
     return Xb;
 }
 
@@ -1935,15 +1928,18 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod, int yno,
     } else if (pmod->ci == LOGISTIC) {
 	lmax = gretl_model_get_double(pmod, "lmax");
     } else if (pmod->ci == LOGIT || pmod->ci == PROBIT) {
-	if (gretl_model_get_int(pmod, "ordered")) {
+	if (gretl_model_get_int(pmod, "multinom")) {
+	    /* FIXME */
+	    return E_NOTIMP;
+	} else if (gretl_model_get_int(pmod, "ordered")) {
 	    /* we need to know how many coeffs are not
 	       just estimated cut-points */
 	    k = gretl_model_get_int(pmod, "nx");
 	    if (k <= 0) {
 		return E_MISSDATA;
 	    }
-	}
-    }
+	} 
+    } 
 
     for (t=fc->t1; t<=fc->t2; t++) {
 	double yht = 0.0;
@@ -2857,8 +2853,8 @@ static int model_do_forecast (const char *str, MODEL *pmod,
     int t1, t2, k = -1;
     int err;
 
-    if (pmod->ci == ARBOND || pmod->ci == HECKIT ||
-	pmod->ci == DURATION) {
+    if (pmod->ci == ARBOND || pmod->ci == DPANEL ||
+	pmod->ci == HECKIT || pmod->ci == DURATION) {
 	/* FIXME */
 	return E_NOTIMP;
     }
@@ -2868,6 +2864,11 @@ static int model_do_forecast (const char *str, MODEL *pmod,
 	return E_NOTIMP;
     }
 
+    if (pmod->ci == LOGIT && (gretl_model_get_int(pmod, "multinom"))) {
+	/* FIXME */
+	return E_NOTIMP;
+    }	
+
     /* Reject in case model was estimated using repacked daily
        data: this case should be handled more elegantly */
     if (gretl_model_get_int(pmod, "daily_repack")) {
@@ -2895,7 +2896,7 @@ static int model_do_forecast (const char *str, MODEL *pmod,
     }
 
     if (opt & OPT_R) {
-	fr = rolling_OLS_k_step_fcast(pmod, pZ, pdinfo, t1, t2, 
+	fr = rolling_OLS_k_step_fcast(pmod, *pZ, pdinfo, t1, t2, 
 				      k, 0, &err);
     } else {
 	fr = get_forecast(pmod, t1, t2, 0, pZ, pdinfo, opt, &err);
@@ -3440,7 +3441,7 @@ static int rolling_fcast_adjust_obs (MODEL *pmod, int *t1, int t2, int k)
 /* recursive k-step ahead forecasts, for models estimated via OLS */
 
 FITRESID * 
-rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+rolling_OLS_k_step_fcast (MODEL *pmod, double **Z, DATAINFO *pdinfo,
 			  int t1, int t2, int k, int pre_n, int *err)
 {
     FITRESID *fr;
@@ -3476,7 +3477,7 @@ rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (k > 1) {
-	*err = k_step_init(pmod, pdinfo, *pZ, k, &y, &llist);
+	*err = k_step_init(pmod, pdinfo, Z, k, &y, &llist);
 	if (*err) {
 	    return NULL;
 	}
@@ -3503,11 +3504,11 @@ rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	    pdinfo->t1, pdinfo->t2, t1, t2, k, nf);
 
     for (t=0; t<pdinfo->n; t++) {
-	fr->actual[t] = (*pZ)[pmod->list[1]][t];
+	fr->actual[t] = Z[pmod->list[1]][t];
     }
 
     for (s=0; s<nf; s++) {
-	mod = lsq(pmod->list, pZ, pdinfo, OLS, OPT_A | OPT_Z);
+	mod = lsq(pmod->list, Z, pdinfo, OLS, OPT_A | OPT_Z);
 	if (mod.errcode) {
 	    *err = mod.errcode;
 	    clear_model(&mod);
@@ -3526,7 +3527,7 @@ rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		if (p > 0 && p <= j) {
 		    xit = y[t-p];
 		} else {
-		    xit = (*pZ)[vi][t];
+		    xit = Z[vi][t];
 		}
 		if (na(xit)) {
 		    yf = NADBL;
