@@ -23,6 +23,7 @@
 #include "libset.h"
 #include "monte_carlo.h"
 #include "gretl_string_table.h"
+#include "gretl_panel.h"
 #include "gretl_fft.h"
 #include "estim_private.h"
 #include "../../cephes/cephes.h"
@@ -330,20 +331,22 @@ int orthdev_series (const double *x, double *y, const DATAINFO *pdinfo)
 int fracdiff_series (const double *x, double *y, double d,
 		     int diff, int obs, const DATAINFO *pdinfo)
 {
-    int dd, t, tmiss, T;
+    int dd, t, T;
     const double TOL = 1.0E-12;
     int t1 = pdinfo->t1;
     int t2 = (obs >= 0)? obs : pdinfo->t2;
+    int tmiss = 0;
     double phi = (diff)? -d : d;
 
 #if 0
     fprintf(stderr, "Doing fracdiff_series, with d = %g\n", d);
 #endif
 
-    tmiss = array_adjust_t1t2(x, &t1, &t2);
-
-    if (tmiss > 0 && tmiss < t2) {
-	t2 = tmiss;
+    if (series_adjust_sample(x, &t1, &t2) != 0) {
+	tmiss = first_missing_index(x, t1, t2);
+	if (tmiss > 0 && tmiss < t2) {
+	    t2 = tmiss;
+	}
     } 
  
     if (obs >= 0) {
@@ -438,7 +441,7 @@ int resample_series (const double *x, double *y, const DATAINFO *pdinfo)
     int *z = NULL;
     int i, t, n;
 
-    array_adjust_t1t2(x, &t1, &t2);
+    series_adjust_sample(x, &t1, &t2);
 
     n = t2 - t1 + 1;
     if (n <= 1) {
@@ -480,7 +483,7 @@ int block_resample_series (const double *x, double *y, int blocklen,
 	return resample_series(x, y, pdinfo);
     }    
 
-    array_adjust_t1t2(x, &t1, &t2);
+    series_adjust_sample(x, &t1, &t2);
 
     n = t2 - t1 + 1;
 
@@ -580,9 +583,9 @@ int filter_series (const double *x, double *y, const DATAINFO *pdinfo,
 	}
     }
 
-    err = array_adjust_t1t2(x, &t1, &t2);
+    err = series_adjust_sample(x, &t1, &t2);
     if (err) {
-	return E_DATA;
+	return err;
     } 
 
     n = t2 - t1 + 1;
@@ -668,7 +671,7 @@ int exponential_movavg_series (const double *x, double *y,
 
     if (n < 0) {
 	return E_INVARG;
-    } else if (array_adjust_t1t2(x, &t1, &t2) != 0) {
+    } else if (series_adjust_sample(x, &t1, &t2)) {
 	return E_MISSDATA;
     }
 
@@ -719,8 +722,13 @@ int movavg_series (const double *x, double *y, const DATAINFO *pdinfo,
     int t2 = pdinfo->t2;
     int k1 = k-1, k2 = 0;
     int i, s, t, T;
+    int err;
+
+    err = series_adjust_sample(x, &t1, &t2);
+    if (err) {
+	return err;
+    }
 
-    array_adjust_t1t2(x, &t1, &t2);
     T = t2 - t1 + 1;
     if (T < k) {
 	return E_TOOFEW;
@@ -740,8 +748,7 @@ int movavg_series (const double *x, double *y, const DATAINFO *pdinfo,
 	for (i=-k1; i<=k2; i++) {
 	    s = t + i;
 	    if (pdinfo->structure == STACKED_TIME_SERIES) {
-		if (pdinfo->paninfo->unit[s] != 
-		    pdinfo->paninfo->unit[t]) {
+		if (s / pdinfo->pd != t / pdinfo->pd) {
 		    s = -1;
 		}
 	    }
@@ -795,7 +802,7 @@ int seasonally_adjust_series (const double *x, double *y,
 	return E_PDWRONG;
     }
 
-    array_adjust_t1t2(x, &t1, &t2);
+    series_adjust_sample(x, &t1, &t2);
     T = t2 - t1 + 1;
 
     if (T < pdinfo->pd * 3) {
@@ -836,26 +843,28 @@ int seasonally_adjust_series (const double *x, double *y,
     return err;
 }
 
+static int new_unit (const DATAINFO *pdinfo, int t)
+{
+    return t > 0 && (t / pdinfo->pd != (t-1) / pdinfo->pd);
+}
+
 int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo, 
 		     int k)
 {
-    const int *unit;
     double ret = NADBL;
     double xbar = NADBL;
     double aux = NADBL;
     int smin = 0;
     int s, t, Ti = 0;
 
-    if (pdinfo->paninfo == NULL) {
+    if (!dataset_is_panel(pdinfo)) {
 	return E_DATA;
     }
 
-    unit = pdinfo->paninfo->unit;
-
     switch (k) {
     case F_PNOBS:
 	for (t=0; t<=pdinfo->n; t++) {
-	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+	    if (t == pdinfo->n || new_unit(pdinfo, t)) {
 		for (s=smin; s<t; s++) {
 		    y[s] = Ti;
 		}
@@ -872,7 +881,7 @@ int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo,
 	break;
     case F_PMIN:
 	for (t=0; t<=pdinfo->n; t++) {
-	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+	    if (t == pdinfo->n || new_unit(pdinfo, t)) {
 		for (s=smin; s<t; s++) {
 		    y[s] = aux;
 		}
@@ -891,7 +900,7 @@ int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo,
 	break;
     case F_PMAX:
 	for (t=0; t<=pdinfo->n; t++) {
-	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+	    if (t == pdinfo->n || new_unit(pdinfo, t)) {
 		for (s=smin; s<t; s++) {
 		    y[s] = aux;
 		}
@@ -910,7 +919,7 @@ int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo,
 	break;
     case F_PMEAN:
 	for (t=0; t<=pdinfo->n; t++) {
-	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+	    if (t == pdinfo->n || new_unit(pdinfo, t)) {
 		if (!na(xbar)) {
 		    xbar /= Ti;
 		}
@@ -938,7 +947,7 @@ int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo,
 	break;
     case F_PSD:
 	for (t=0; t<=pdinfo->n; t++) {
-	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+	    if (t == pdinfo->n || new_unit(pdinfo, t)) {
 		if (na(xbar)) {
 		    ret = NADBL;
 		} else if (Ti == 1) {
@@ -1014,9 +1023,8 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 	hp[t] = NADBL;
     }
 
-    err = array_adjust_t1t2(x, &t1, &t2);
+    err = series_adjust_sample(x, &t1, &t2);
     if (err) {
-	err = E_MISSDATA;
 	goto bailout;
     }
 
@@ -1187,7 +1195,7 @@ int bkbp_filter (const double *x, double *bk, const DATAINFO *pdinfo,
 	return 1;
     }
 
-    err = array_adjust_t1t2(x, &t1, &t2);
+    err = series_adjust_sample(x, &t1, &t2);
     if (err) {
 	return err;
     } 
@@ -1242,6 +1250,847 @@ int bkbp_filter (const double *x, double *bk, const DATAINFO *pdinfo,
     return err;
 }
 
+/* following: material relating to the Butterworth filter */
+
+#define Min(x,y) (((x) > (y))? (y) : (x))
+#define NEAR_ZERO 1e-35
+
+static double safe_pow (double x, int n)
+{
+    double lp;
+
+    x = fabs(x);
+
+    if (x < NEAR_ZERO) {
+	return 0.0;
+    } else {
+	lp = n * log(x);
+	if (lp < -80) {
+	    return 0.0;
+	} else if (lp > 80) {
+	    return exp(80.0);
+	} else {
+	    return exp(lp);
+	}
+    }
+}
+
+static double cotan (double theta)
+{
+    double s = sin(theta);
+    
+    if (fabs(s) < NEAR_ZERO) {
+	s = copysign(NEAR_ZERO, s);
+    }
+
+    return cos(theta) / s;
+} 
+
+/**
+ * hp_gain:
+ * @lambda: H-P parameter.
+ * @hipass: 1 for high-pass filter, 0 for low-pass.
+ *
+ * Returns: a matrix holding omega values from 0 to \pi in
+ * column 0, and the corresponding filter gain in column 1.
+ */
+
+gretl_matrix *hp_gain (double lambda, int hipass)
+{
+    gretl_matrix *G;
+    int i, width = 180;
+    double inc = M_PI / width;
+    double x, gain, omega = 0.0;
+
+    G = gretl_matrix_alloc(width + 1, 2);
+    if (G == NULL) {
+	return NULL;
+    }
+    
+    for (i=0; i<=width; i++) {
+	x = 2 * sin(omega / 2);
+	gain = 1 / (1 + lambda * pow(x, 4));
+	if (hipass) {
+	    gain = 1.0 - gain;
+	}
+	gretl_matrix_set(G, i, 0, omega);
+	gretl_matrix_set(G, i, 1, gain);
+	omega += inc;
+    } 
+
+    return G;
+} 
+
+/**
+ * butterworth_gain:
+ * @n: order of the filter.
+ * @cutoff: angular cutoff in radians.
+ * @hipass: 1 for high-pass filter, 0 for low-pass.
+ *
+ * Returns: a matrix holding omega values from 0 to \pi in
+ * column 0, and the corresponding filter gain in column 1.
+ */
+
+gretl_matrix *butterworth_gain (int n, double cutoff, int hipass)
+{
+    gretl_matrix *G;
+    int i, width = 180;
+    double inc = M_PI / width;
+    double x, gain, omega = 0.0;
+
+    G = gretl_matrix_alloc(width + 1, 2);
+    if (G == NULL) {
+	return NULL;
+    }
+    
+    for (i=0; i<=width; i++) {
+	if (hipass) {
+	    x = cotan(omega / 2) * cotan((M_PI - cutoff) / 2);
+	} else {
+	    x = tan(omega / 2) * cotan(cutoff / 2);
+	}
+	gain = 1 / (1 + pow(x, 2 * n));
+	gretl_matrix_set(G, i, 0, omega);
+	gretl_matrix_set(G, i, 1, gain);
+	omega += inc;
+    } 
+
+    return G;
+} 
+
+/* Toeplitz methods:
+
+   1 = Stephen Pollock's symmetric Toeplitz solver
+   2 = gretl's netlib-based general Toeplitz solver
+   3 = build full Toeplitz matrix and apply SVD
+
+*/
+
+#define TOEPLITZ_METHOD 1
+
+#if (TOEPLITZ_METHOD == 3)
+
+/* form the complete Toeplitz matrix represented in compressed 
+   form by the coefficients in g
+*/
+
+static gretl_matrix *toeplize (const double *g, int T, int k)
+{
+    gretl_matrix *X;
+    double x;
+    int i, j;
+
+    X = gretl_zero_matrix_new(T, T);
+    if (X == NULL) {
+	return NULL;
+    }
+
+    for (i=0; i<T; i++) {
+        gretl_matrix_set(X, i, i, g[0]);
+    }
+
+    for (i=1; i<k; i++) {
+	x = g[i];
+	for (j=i; j<T; j++) {
+	    gretl_matrix_set(X, j, j-i, x);
+	    gretl_matrix_set(X, j-i, j, x);
+	}
+    }
+
+    return X;
+}
+
+#include "matrix_extra.h"
+
+static int toeplitz_solve (const double *g, double *dy, int T, int q)
+{
+    gretl_matrix *y, *X;
+    int err = 0;
+
+    fprintf(stderr, "svd_toeplitz_solve...\n");
+
+    X = toeplize(g, T, q + 1);
+    if (X == NULL) {
+	return E_ALLOC;
+    }
+
+    err = gretl_SVD_invert_matrix(X);
+
+    if (!err) {
+	y = gretl_vector_from_array(dy, T, GRETL_MOD_NONE);
+	if (y == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    gretl_matrix s;
+
+	    s.t1 = s.t2 = 0;
+	    s.rows = T;
+	    s.cols = 1;
+	    s.val = dy;
+
+	    err = gretl_matrix_multiply(X, y, &s);
+	    gretl_matrix_free(y);
+	}
+    }
+
+    gretl_matrix_free(X);
+
+    return err;
+}
+
+#elif (TOEPLITZ_METHOD == 2)
+
+static int toeplitz_solve (double *g, double *y, int T, int q)
+{
+    gretl_vector *mg = NULL;
+    gretl_vector *my = NULL;
+    gretl_vector *mx = NULL;
+    int i, err = 0;
+
+    fprintf(stderr, "netlib toeplitz solve...\n");
+
+    mg = gretl_vector_alloc(T); 
+    my = gretl_vector_alloc(T); 
+
+    if (mg == NULL || my == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<T; i++) {
+	mg->val[i] = (i <= q) ? g[i] : 0;
+	my->val[i] = y[i];
+    }
+
+    mx = gretl_toeplitz_solve(mg, mg, my, &err);
+
+    if (err) {
+	fprintf(stderr, "symm_toeplitz: err = %d\n", err);
+	for (i=0; i<T; i++) {
+	    y[i] = NADBL;
+	}
+    } else {
+	for (i=0; i<T; i++) {
+	    y[i] = mx->val[i];
+	}
+    }
+
+    gretl_vector_free(mg);
+    gretl_vector_free(my);
+    gretl_vector_free(mx);
+
+    return err;
+}
+
+#else /* TOEPLITZ_METHOD == 1 */
+
+/* This function uses a Cholesky decomposition to find the solution of
+   the equation Gx = y, where G is a symmetric Toeplitz matrix of order
+   T with q supra-diagonal and q sub-diagonal bands. The coefficients of
+   G are contained in g. The RHS vector y contains the elements of
+   the solution vector x on completion.
+*/
+
+static int toeplitz_solve (double *g, double *y, int T, int q)
+{
+    double **mu;
+    int t, j, k, jmax;
+
+    mu = doubles_array_new(q+1, T);
+    if (mu == NULL) {
+	return E_ALLOC;
+    }
+
+    /* initialize */
+    for (t=0; t<q; t++) {
+	for (j=t+1; j<=q; j++) {
+	    mu[j][t] = 0.0;
+	}
+    }
+
+    /* factorize */
+    for (t=0; t<T; t++) { 
+	for (k=Min(q, t); k>=0; k--) {
+	    mu[k][t] = g[k];
+	    jmax = q - k;
+	    for (j=1; j<=jmax && t-k-j >= 0; j++) {
+		mu[k][t] -= mu[j][t-k] * mu[j+k][t] * mu[0][t-k-j];
+	    }
+	    if (k > 0) {
+		mu[k][t] /= mu[0][t-k];
+	    }
+	}
+    }
+
+    /* forward solve */
+    for (t=0; t<T; t++) {
+	jmax = Min(t, q);
+	for (j=1; j<=jmax; j++) {
+	    y[t] -= mu[j][t] * y[t-j];
+	}
+    }
+
+    /* divide by the diagonal */
+    for (t=0; t<T; t++) {
+	y[t] /= mu[0][t];
+    }
+
+    /* backsolve */
+    for (t=T-1; t>=0; t--) {
+	jmax = Min(q, T - 1 - t);
+	for (j=1; j<=jmax; j++) {
+	    y[t] -= mu[j][t+j] * y[t+j];
+	}
+    }
+
+    doubles_array_free(mu, q+1);
+
+    return 0;
+}
+
+#endif /* alternate TOEPLITZ_METHODs */
+
+
+/* Premultiply vector y by a symmetric banded Toeplitz matrix 
+   Gamma with n nonzero sub-diagonal bands and n nonzero 
+   supra-diagonal bands. The elements of Gamma are contained
+   in g; tmp is used as workspace.
+*/
+
+static int GammaY (double *g, double *y, double *tmp, 
+		   int T, int n)
+{
+    double lx, rx;
+    int t, j;
+
+    for (j=0; j<=n; j++) {
+	tmp[j] = 0.0;
+    }
+
+    for (t=0; t<T; t++) {
+	for (j=n; j>0; j--) {
+	    tmp[j] = tmp[j-1];
+	}
+	tmp[0] = g[0] * y[t];
+	for (j=1; j<=n; j++) {
+	    lx = (t - j < 0) ? 0 : y[t-j];
+	    rx = (t + j >= T) ? 0 : y[t+j];
+	    tmp[0] += g[j] * (lx + rx);
+	}
+	if (t >= n) {
+	    y[t-n] = tmp[n];
+	}
+    }
+
+    for (j=0; j<n; j++) {
+	y[T-j-1] = tmp[j];
+    }
+
+    return 0;
+}
+
+#undef Min
+#undef NEAR_ZERO
+
+/* Form the autocovariances of an MA process of order q. */
+
+static void form_gamma (double *g, double *mu, int q)
+{
+    int j, k;
+
+    for (j=0; j<=q; j++) { 
+	g[j] = 0.0;
+	for (k=0; k<=q-j; k++) {
+	    g[j] += mu[k] * mu[k+j];
+	}
+    }
+}
+
+/* Find the coefficients of the nth power of the summation 
+   operator (if sign = 1) or of the difference operator (if
+   sign = -1).
+*/
+
+static void sum_or_diff (double *theta, int n, int sign)
+{
+    int j, q;
+
+    theta[0] = 1.0;
+
+    for (q=1; q<=n; q++) { 
+	theta[q] = 0.0;
+	for (j=q; j>0; j--) {
+	    theta[j] += sign * theta[j-1];
+	}
+    } 
+} 
+
+/* g is the target, mu is used as workspace for the
+   summation coefficients */
+
+static void form_mvec (double *g, double *mu, int n)
+{
+    sum_or_diff(mu, n, 1);
+    form_gamma(g, mu, n);
+}
+
+/* g is the target, mu is used as workspace for the
+   differencing coefficients 
+   svec = Q'SQ where Q is the 2nd-diff operator
+*/
+
+static void form_svec (double *g, double *mu, int n)
+{
+    sum_or_diff(mu, n, -1);
+    form_gamma(g, mu, n);
+}
+
+/* g is the target, mu and tmp are used as workspace */
+
+static void form_wvec (double *g, double *mu, double *tmp,
+		       int n, double lam1, double lam2)
+{
+    int i;
+
+    form_svec(tmp, mu, n);
+    for (i=0; i<=n; i++) {
+	g[i] = lam1 * tmp[i];
+    }
+
+    form_mvec(tmp, mu, n);
+    for (i=0; i<=n; i++) {
+	g[i] += lam2 * tmp[i];
+    }
+}
+
+/* Find the second differences of the elements of a vector.
+   Notice that the first two elements of the vector are lost 
+   in the process. However, we index the leading element of 
+   the differenced vector by t = 0.
+*/
+
+static void QprimeY (double *y, int T)
+{
+    int t;
+
+    for (t=0; t<T-2; t++) {
+	y[t] += y[t+2] - 2 * y[t+1];
+    }
+}
+
+/* Multiply the vector y by matrix Q of order T x T-2, 
+   where Q' is the matrix which finds the second 
+   differences of a vector.  
+*/
+
+static void form_Qy (double *y, int T)
+{
+    double tmp, lag1 = 0.0, lag2 = 0.0;
+    int t;
+
+    for (t=0; t<T-2; t++) { 
+	tmp = y[t];
+	y[t] += lag2 - 2 * lag1;
+	lag2 = lag1;
+	lag1 = tmp;
+    }
+
+    y[T-2] = lag2 - 2 * lag1;
+    y[T-1] = lag1;
+}
+
+static int mp_butterworth (const double *x, double *bw, int T,
+			   int order, double cutoff)
+{
+    void *handle;
+    int (*mpfun) (const double *, double *, int, int, double);
+    int err = 0;
+
+    mpfun = get_plugin_function("mp_bw_filter", &handle);
+
+    if (mpfun == NULL) {
+	fputs(I_("Couldn't load plugin function\n"), stderr);
+	return E_FOPEN;
+    }
+
+    err = (*mpfun) (x, bw, T, order, cutoff);
+    close_plugin(handle);
+
+    return err;
+}
+
+#if 0 
+
+/* note: cut should be in degrees */
+
+static int bw_numeric_check (double cut, int n)
+{
+    const double b0 = -8.56644923648263;
+    const double b1 = -0.66992007228155;
+    const double b2 =  2.82851981714539;
+    double x;
+
+    /* let's be conservative here */
+    x = b0 + b1 * (cut - 1.0) + b2 * (n + 1);
+    x = 1/(1+exp(-x));
+
+    return (x > 0.479717591568)
+}
+
+#endif
+
+/* maximum modulus among the poles of the filter:
+   too close to 1.0 is a problem
+*/
+
+static double bw_max_mod (double cut, int n)
+{
+    double tc = tan(cut / 2);
+    double theta = M_PI / (2 * n);
+    double delta = 1 + tc * (2 * sin(theta) + tc);
+    double x = (1 - tc * tc) / delta;
+    double y = 2 * tc * cos(theta) / delta;
+
+    return sqrt(x * x + y * y);
+}
+
+#define MAX_LAM 10000
+
+/* Calculate the Butterworth lambda based on cutoff and
+   order. Return non-zero if it seems that lambda is too
+   extreme. FIXME: conditioning on lambda alone is not an
+   adequate approach.
+*/
+
+static int 
+set_bw_lambda (double cutoff, int n, double *lam1, double *lam2)
+{
+    int ret = 0;
+
+    *lam1 = 1 / tan(cutoff / 2);
+    *lam1 = safe_pow(*lam1, n * 2);
+
+    fprintf(stderr, "for cutoff %g, order %d: lambda=%g, maxmod=%g\n",
+	    cutoff, n, *lam1, bw_max_mod(cutoff, n));
+
+    if (*lam1 > 1.0e18) {
+	/* can't cope, even with multiple precision? */
+	ret = 2;
+    } else if (*lam1 > 1.0e6) {
+	/* may be OK with multiple precision? */
+	ret = 1;
+    } else {
+	/* OK with regular double-precision? */
+	if (*lam1 > MAX_LAM) { 
+	    /* note: AC thinks the following doesn't help */
+	    *lam1 = sqrt(*lam1);
+	    *lam2 = 1 / *lam1;
+	}
+    }
+
+    return ret;
+}
+
+/**
+ * butterworth_filter:
+ * @x: array of original data.
+ * @bw: array into which to write the filtered series.
+ * @pdinfo: data set information.
+ * @n: desired lag order.
+ * @cutoff: desired angular cutoff in degrees (0, 180).
+ *
+ * Calculates the Butterworth filter. The code that does this
+ * is based on D.S.G. Pollock's IDEOLOG -- see 
+ * http://www.le.ac.uk/users/dsgp1/
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int butterworth_filter (const double *x, double *bw, const DATAINFO *pdinfo,
+			int n, double cutoff)
+{
+    double *g, *ds, *tmp, *y;
+    double lam1, lam2 = 1.0;
+    int t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int T, t, m;
+    int bad_lambda = 0;
+    int err = 0;
+
+    err = series_adjust_sample(x, &t1, &t2);
+    if (err) {
+	return err;
+    } 
+
+    if (2 * n >= t2 - t1) {
+	gretl_errmsg_set("Insufficient observations");
+	return E_DATA;
+    }
+
+    if (cutoff <= 0.0 || cutoff >= 180.0) {
+	return E_INVARG;
+    }
+
+    T = t2 - t1 + 1;
+    m = 3 * (n+1);
+
+    /* sample offset */
+    y = bw + t1;
+    x = x + t1;
+
+    /* the cutoff is expressed in radians internally */
+    cutoff *= M_PI / 180.0;
+
+    bad_lambda = set_bw_lambda(cutoff, n, &lam1, &lam2);
+
+    if (bad_lambda > 1) {
+	gretl_errmsg_set("Butterworth: infeasible lambda value");
+	return E_DATA;
+    } else if (bad_lambda) {
+	return mp_butterworth(x, y, T, n, cutoff);
+    }	
+
+    /* the workspace we need for everything except the
+       Toeplitz solver */
+    g = malloc(m * sizeof *g);
+    if (g == NULL) {
+	return E_ALLOC;
+    }
+
+    ds = g + n + 1;
+    tmp = ds + n + 1;
+
+    /* place a copy of the data in y */
+    memcpy(y, x, T * sizeof *y);
+
+    form_wvec(g, ds, tmp, n, lam1, lam2); /* W = M + lambda * Q'SQ */
+
+    QprimeY(y, T);
+
+    /* solve (M + lambda*Q'SQ)x = d for x */
+    err = toeplitz_solve(g, y, T-2, n);
+
+    if (!err) {
+	form_Qy(y, T);
+	form_svec(g, ds, n-2);
+	GammaY(g, y, tmp, T, n-2);   /* Form SQy in y */
+	/* write the trend into y (low-pass) */
+	for (t=0; t<T; t++) {
+	    y[t] = x[t] - lam1 * y[t];
+	}	
+    }    
+
+    free(g);
+
+    return err;
+}
+
+/* common code for both plain and weighted polynomial trend
+   estimation */
+
+static int real_poly_trend (const double *x, double *fx, double *w,
+			    int T, int order)
+{
+    double tmp, denom, lagdenom = 1;
+    double alpha, gamma, delta;
+    double *phi, *philag;
+    int i, t;
+
+    phi = malloc(2 * T * sizeof *phi);
+    if (phi == NULL) {
+	return E_ALLOC;
+    }
+
+    philag = phi + T;
+     
+    for (t=0; t<T; t++) {
+	phi[t] = 1;
+	philag[t] = 0;
+	fx[t] = 0;
+    }
+
+    for (i=0; i<=order; i++) {
+	double xadd;
+
+	alpha = gamma = denom = 0.0;
+
+	for (t=0; t<T; t++) {
+	    xadd = x[t] * phi[t];
+	    if (w != NULL) xadd *= w[t];
+	    alpha += xadd;
+	    xadd = phi[t] * phi[t] * t;
+	    if (w != NULL) xadd *= w[t];
+	    gamma += xadd;
+	    xadd = phi[t] * phi[t];
+	    if (w != NULL) xadd *= w[t];
+	    denom += xadd;
+	}	
+          
+	alpha /= denom;
+	gamma /= denom;
+	delta = denom / lagdenom;
+	lagdenom = denom;
+       
+	for (t=0; t<T; t++) {
+	    fx[t] += alpha * phi[t];
+	    tmp = phi[t];
+	    phi[t] = (t - gamma) * phi[t] - delta * philag[t];
+	    philag[t] = tmp;
+	} 
+    }
+
+    free(phi);
+
+    return 0;
+}
+
+/**
+ * poly_trend:
+ * @x: array of original data.
+ * @fx: array into which to write the fitted series.
+ * @pdinfo: data set information.
+ * @order: desired polynomial order.
+ *
+ * Calculates a trend via the method of orthogonal polynomials.
+ * Based on C code for D.S.G. Pollock's DETREND program.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int poly_trend (const double *x, double *fx, const DATAINFO *pdinfo, int order)
+{
+    int t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int T, err;
+
+    err = series_adjust_sample(x, &t1, &t2);
+    if (err) {
+	return err;
+    } 
+
+    T = t2 - t1 + 1;
+
+    if (order > T) {
+	return E_DF;
+    }
+
+    return real_poly_trend(x + t1, fx + t1, NULL, T, order);
+} 
+
+/**
+ * poly_weights:
+ * @w: array into which the weights will be written.
+ * @T: the length of @w.
+ * @wmax: the ratio of maximum to minimum weight.
+ * @midfrac: the size of the central section that should be given
+ * the minimum weight.
+ * @opt: weighting scheme option (OPT_Q = quadratic, OPT_B = cosine bell,
+ * OPT_C = crenelated).
+ *
+ * Calculates a set of weights; intended for use with polynomial
+ * trend fitting.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+void poly_weights (double *w, int T, double wmax, 
+		   double midfrac, gretlopt opt)
+{
+    double wt, a = 0, b = 0;
+    int t, cut, T2 = T / 2;
+
+    if (midfrac > 0) {
+        cut = round(T * (1.0 - midfrac) / 2.0);
+    } else {
+        cut = T2;
+    }
+
+    if (opt == OPT_Q) {
+	/* quadratic */
+	double z1, z2, det;
+
+	if (midfrac > 0) {
+	    z1 = cut;
+	    z2 = 2 * cut;
+	} else {
+	    z1 = (T-1) / 2.0;
+	    z2 = T-1;
+	}
+
+	det = 1.0 / (z1 * (z1 * z2 - z2 * z2));
+	a = z2 * (1 - wmax) * det;
+	b = -z2 * a;
+    } else if (opt == OPT_B) {
+	/* cosine-bell */
+	b = (wmax - 1) / 2.0;
+    }
+
+    for (t=0; t<=T2; t++) {
+        if (t <= cut) {
+	    if (opt == OPT_Q) {
+		wt = a * t + b;
+		wt = wt * t + wmax;
+	    } else if (opt == OPT_B) {
+		a = (t * M_PI) / cut;
+		wt = 1 + b * (1 + cos(a));
+	    } else {
+		/* "crenelated" */
+		wt = wmax;
+	    }
+        } else {
+	    wt = 1;
+        }
+        w[t] = w[T-1-t] = wt;
+    }
+}
+
+/**
+ * weighted_poly_trend:
+ * @x: array of original data.
+ * @fx: array into which to write the fitted series.
+ * @pdinfo: data set information.
+ * @order: desired polynomial order.
+ * @opt: weighting option (OPT_Q = quadratic, OPT_B = cosine bell,
+ * OPT_C = crenelated).
+ * @wratio: ratio of maximum to minimum weight.
+ * @midfrac: proportion of the data to be treated specially, in the
+ * middle.
+ *
+ * Calculates a trend via the method of orthogonal polynomials, using
+ * the specified weighting scheme.
+ * Based on C code for D.S.G. Pollock's DETREND program.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int weighted_poly_trend (const double *x, double *fx, const DATAINFO *pdinfo,
+			 int order, gretlopt opt, double wratio, 
+			 double midfrac)
+{
+    double *w = NULL;
+    int t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int T, err;
+
+    err = series_adjust_sample(x, &t1, &t2);
+    if (err) {
+	return err;
+    } 
+
+    T = t2 - t1 + 1;
+
+    if (order > T) {
+	return E_DF;
+    }
+
+    w = malloc(T * sizeof *w);
+    if (w == NULL) {
+	err = E_ALLOC;
+    } else {
+	poly_weights(w, T, wratio, midfrac, opt);
+	err = real_poly_trend(x + t1, fx + t1, w, T, order);
+	free(w);
+    }
+
+    return err;
+}
+
 static int n_new_dummies (const DATAINFO *pdinfo,
 			  int nunits, int nperiods)
 {
@@ -1672,7 +2521,7 @@ int gen_wkday (double ***pZ, DATAINFO *pdinfo)
 }
 
 typedef enum {
-    PLOTVAR_INDEX,
+    PLOTVAR_INDEX = 1,
     PLOTVAR_TIME,
     PLOTVAR_ANNUAL,
     PLOTVAR_QUARTERS,
@@ -1680,6 +2529,7 @@ typedef enum {
     PLOTVAR_CALENDAR,
     PLOTVAR_DECADES,
     PLOTVAR_HOURLY,
+    PLOTVAR_PANEL,
     PLOTVAR_MAX
 } plotvar_type; 
 
@@ -1717,15 +2567,15 @@ int plotvar_code (const DATAINFO *pdinfo)
  * Returns: pointer to plot x-variable, or NULL on failure.
  */
 
-const double *gretl_plotx (const DATAINFO *pdinfo)
+const double *gretl_plotx (const double **Z, const DATAINFO *pdinfo)
 {
     static double *x;
     static int ptype;
     static int Tbak;
     static double sd0bak;
-
     int t, y1, T;
-    int new_ptype;
+    int new_ptype = 0;
+    int panvar = 0;
     double sd0;
     float rm;
 
@@ -1739,7 +2589,19 @@ const double *gretl_plotx (const DATAINFO *pdinfo)
 	return NULL;
     }
 
-    new_ptype = plotvar_code(pdinfo);
+    if (dataset_is_panel(pdinfo) && 
+	sample_size(pdinfo) == pdinfo->pd && Z != NULL) {
+	/* handle the case of a time-series plot for a single panel unit */
+	panvar = plausible_panel_time_var(Z, pdinfo);
+	if (panvar > 0) {
+	    new_ptype = PLOTVAR_PANEL;
+	}
+    }
+
+    if (new_ptype == 0) {
+	new_ptype = plotvar_code(pdinfo);
+    }
+
     T = pdinfo->n;
     sd0 = pdinfo->sd0;
 
@@ -1752,9 +2614,14 @@ const double *gretl_plotx (const DATAINFO *pdinfo)
 	free(x);
     }
 
-    x = malloc(T * sizeof *x);
-    if (x == NULL) {
-	return NULL;
+    if (new_ptype == PLOTVAR_PANEL) {
+	x = copyvec(Z[panvar], pdinfo->n);
+    } else {
+	x = malloc(T * sizeof *x);
+    }
+
+    if (x == NULL || new_ptype == PLOTVAR_PANEL) {
+	return x;
     }
 
     Tbak = T;
@@ -2033,7 +2900,19 @@ static int plain_obs_number (const char *obs, const DATAINFO *pdinfo)
 int get_t_from_obs_string (const char *s, const double **Z, 
 			   const DATAINFO *pdinfo)
 {
-    int t = dateton(s, pdinfo);
+    int t;
+
+    if (*s == '"') {
+	char obs[16];
+	int err = 0;
+
+	*obs = '\0';
+	strncat(obs, s, 15);
+	gretl_unquote(obs, &err);
+	t = dateton(obs, pdinfo);
+    } else {
+	t = dateton(s, pdinfo);
+    }
 
 #if OBS_DEBUG
     fprintf(stderr, "\nget_t_from_obs_string: s ='%s', dateton gives t = %d\n", 
@@ -3313,3 +4192,60 @@ double logistic_cdf (double x)
 
     return ret;
 }
+
+/**
+ * matrix_chowlin:
+ * @Y: T x k: holds the original data to be expanded, series
+ * in columns.
+ * @X: (optionally) holds covariates of Y at the higher frequency:
+ * if these are supplied they supplement the default set of
+ * regressors, namely, constant plus quadratic trend.
+ * @f: the expansion factor: 3 for quarterly to monthly or
+ * 4 for annual to quarterly. Only these factors are
+ * supported.
+ * @err: location to receive error code.
+ *
+ * Interpolate, from annual to quarterly or quarterly to monthly,
+ * via the Chow-Lin method. See Gregory C. Chow and An-loh Lin,
+ * "Best Linear Unbiased Interpolation, Distribution, and 
+ * Extrapolation of Time Series by Related Series", The
+ * Review of Economics and Statistics, Vol. 53, No. 4 
+ * (November 1971) pp. 372-375.
+ *
+ * If @X is provided, it must have T * @f rows.
+ * 
+ * Returns: matrix containing the expanded series, or
+ * NULL on failure.
+ */
+
+gretl_matrix *matrix_chowlin (const gretl_matrix *Y, 
+			      const gretl_matrix *X, 
+			      int f, int *err)
+{
+    void *handle;
+    gretl_matrix *(*chowlin) (const gretl_matrix *, 
+			      const gretl_matrix *, 
+			      int, int *);
+    gretl_matrix *ret = NULL;
+
+    if (f != 3 && f != 4) {
+	*err = E_INVARG;
+	return NULL;
+    }
+
+    if (X != NULL && X->rows / Y->rows != f) {
+	*err = E_INVARG;
+	return NULL;
+    }
+
+    chowlin = get_plugin_function("chow_lin_interpolate", &handle);
+    
+    if (chowlin == NULL) {
+	*err = E_FOPEN;
+    } else {
+	ret = (*chowlin) (Y, X, f, err);
+	close_plugin(handle);
+    }
+    
+    return ret;
+}
