@@ -136,6 +136,8 @@ struct str_table mvars[] = {
     { M_COEFF_CI,"$coeff_ci" },
     { M_KLLT,    "$kalman_llt" },
     { M_KUHAT,   "$kalman_uhat" },
+    { M_EHAT,    "$ehat" },
+    { M_MNLPROBS, "$mnlprobs" },
     { M_XLIST,   "$xlist" },
     { 0,         NULL }
 };
@@ -208,6 +210,7 @@ struct str_table funcs[] = {
     { F_PSD,      "psd" },       /* panel std dev */
     { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
     { F_BKFILT,   "bkfilt" },    /* Baxter-King filter */
+    { F_BWFILT,   "bwfilt" },    /* Butterworth filter */
     { F_FRACDIFF, "fracdiff" },  /* fractional difference */
     { F_BOXCOX,   "boxcox" },    /* Box-Cox transformation */
     { F_COV,      "cov" },
@@ -343,8 +346,12 @@ struct str_table funcs[] = {
     { F_KDENSITY, "kdensity" },
     { F_MONTHLEN, "monthlen" },
     { F_EPOCHDAY, "epochday" },
-    { F_HASHGET,  "bundleget" },
-    { F_HASHSET,  "bundleset" },
+    { F_SETNOTE,  "setnote" },
+    { F_INVMILLS, "invmills" },
+    { F_POLYFIT,  "polyfit" },
+    { F_CHOWLIN,  "chowlin" },
+    { F_VARSIMUL, "varsimul" },
+    { F_STRSPLIT, "strsplit" },
     { 0,          NULL }
 };
 
@@ -720,7 +727,7 @@ static int might_be_date_string (const char *s, int n)
     strncat(test, s, n);
 
     if (strspn(s, "1234567890") == n) {
-	/* plain integer */
+	/* plain integer (FIXME?) */
 	return 1;
     } else if (sscanf(s, "%d:%d", &y, &m) == 2) {
 	/* quarterly, monthly date */
@@ -818,6 +825,11 @@ static void look_up_dollar_word (const char *s, parser *p)
 	    undefined_symbol_error(s, p);
 	}
     }
+
+#if LDEBUG
+    fprintf(stderr, "look_up_dollar_word: '%s' -> %d\n",
+	    s, p->idnum);
+#endif
 }
 
 #ifdef USE_RLIB
@@ -874,7 +886,7 @@ static void look_up_word (const char *s, parser *p)
 		} else if (get_user_function_by_name(s)) {
 		    p->sym = UFUN;
 		    p->idstr = gretl_strdup(s);
-		} else if (string_is_defined(s)) {
+		} else if (gretl_is_string(s)) {
 		    p->sym = VSTR;
 		    p->idstr = gretl_strdup(s);
 		} else if (p->targ == LIST &&
@@ -907,7 +919,9 @@ static void look_up_word (const char *s, parser *p)
     }
 }
 
-#define could_be_matrix(t) (model_data_matrix(t) || t == M_UHAT)
+#define could_be_matrix(t) (model_data_matrix(t) || \
+			    model_data_matrix_builder(t) || \
+			    t == M_UHAT || t == M_YHAT)
 
 static void word_check_next_char (const char *s, parser *p)
 {
@@ -944,6 +958,9 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == USERIES) {
 	    /* observation from series */
 	    p->sym = OBS;
+	} else if (p->sym == BUNDLE) {
+	    /* object from bundle */
+	    p->sym = BOBJ;
 	} else {
 	    p->err = E_PARSE;
 	} 
@@ -1451,7 +1468,9 @@ const char *getsymb (int t, const parser *p)
 	return "EMPTY";
     } else if (t == LISTVAR) {
 	return "LISTVAR";
-    } 
+    } else if (t == BOBJ) {
+	return "BOBJ";
+    }
 
     if (p != NULL) {
 	if (t == NUM) {
