@@ -103,13 +103,14 @@ enum {
 	      LISTVAR,    /* variable in list */
 	      STR,	  /* string */
 	      BUNDLE,     /* gretl bundle (hash table) */
+	      BOBJ,       /* object from inside a bundle */
 	      FARGS,	  /* set of n function arguments */
 	      EMPTY,      /* "null" */
 	      ABSENT,
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
 	      EROOT,	  /* dummy root for (...) expression */
-              UFUN,	  /* user-defined function */
-  /* 80 */    RFUN,       /* GNU R function */
+  /* 80 */    UFUN,	  /* user-defined function */
+	      RFUN,       /* GNU R function */
               VSTR,       /* string variable */
               INC,   
 	      DEC,
@@ -252,6 +253,7 @@ enum {
     F_SSCANF,
     F_IRR,
     F_LOGISTIC,
+    F_INVMILLS,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -301,7 +303,8 @@ enum {
     F_PERGM,
     F_NPV,
     F_DSUM,
-    F_HASHGET,
+    F_POLYFIT,
+    F_STRSPLIT,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
     F_BFGSMAX,
     F_MSHAPE,
@@ -318,7 +321,10 @@ enum {
     F_EPOCHDAY,
     F_MOVAVG,
     F_KDENSITY,
-    F_HASHSET,
+    F_SETNOTE,
+    F_BWFILT,
+    F_CHOWLIN,
+    F_VARSIMUL,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
     F_BKFILT,
     F_MOLS,
@@ -380,7 +386,7 @@ enum {
 
 #define evalb2(s) (binary_op(s) || func2_symb(s) || s == MSL || \
                    s == MSL2 || s == SUBSL || s == LAG || \
-                   s == OBS)
+                   s == OBS || s == BOBJ)
 
 #define b1sym(s) (unary_op(s) || func1_symb(s) || funcn_symb(s) || \
                   s == G_LPR || s == EROOT)
@@ -449,12 +455,14 @@ union val {
     int *ivec;
     gretl_matrix *m;
     matrix_subspec *mspec;
+    gretl_bundle *b;
 };
 
 enum {
-    AUX_NODE    = 1 << 0,
-    TMP_NODE    = 1 << 1,
-    TRANSP_NODE = 1 << 2
+    AUX_NODE    = 1 << 0, /* auxiliary: free on exit */
+    TMP_NODE    = 1 << 1, /* temporary: free content on exit */
+    TRANSP_NODE = 1 << 2, /* matrix node: should be transposed */
+    PTR_NODE    = 1 << 3  /* node is compatible with P_LHPTR */
 };
 
 struct node {
@@ -489,7 +497,8 @@ enum {
     P_QUIET   = 1 << 21, /* don't print any messages or labels */
     P_GETSTR  = 1 << 22, /* state: flag acceptance of plain strings */
     P_LOOP    = 1 << 23, /* executing pre-built genr in loop context */
-    P_SLAVE   = 1 << 24  /* running as "slave" of NLS/MLE/GMM */
+    P_SLAVE   = 1 << 24, /* running as "slave" of NLS/MLE/GMM */
+    P_LHPTR   = 1 << 25  /* left-hand side: pointer type wanted */
 };
 
 struct lhinfo {
