@@ -561,8 +561,11 @@ static void unwrap_string_arg (parser *p)
 
 static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 {
+    char p0 = (sym == BOBJ)? '[' : '(';
+    char p1 = (sym == BOBJ)? ']' : ')';
     const char *src = NULL;
     int wrapped = 0;
+    int strsym = STR;
 
     while (p->ch == ' ') {
 	parser_getc(p);
@@ -573,7 +576,7 @@ static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 	parser_getc(p);
     }
 
-    if (p->ch == ')') {
+    if (p->ch == p1) {
 	if (wrapped) {
 	    p->err = E_PARSE;
 	    return NULL;
@@ -588,14 +591,14 @@ static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 	/* find length of string to closing paren */
 	i = 0;
 	while (*s) {
-	    if (!quoted && *s == ')') paren--;
+	    if (!quoted && *s == p1) paren--;
 	    if (paren == 0) {
 		close = i;
 		break;
 	    }
 	    if (*s == '"') {
 		quoted = !quoted;
-	    } else if (!quoted && *s == '(') {
+	    } else if (!quoted && *s == p0) {
 		paren++;
 	    } 
 	    s++;
@@ -603,7 +606,7 @@ static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 	}
 
 	if (close < 0) {
-	    unmatched_symbol_error('(', p);
+	    unmatched_symbol_error(p0, p);
 	    return NULL;
 	}
 
@@ -636,11 +639,7 @@ static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 	const char *s = get_string_by_name(p->idstr);
 
 	if (s != NULL) {
-	    free(p->idstr);
-	    p->idstr = gretl_strdup(s);
-	    if (p->idstr == NULL) {
-		p->err = E_ALLOC;
-	    }
+	    strsym = VSTR;
 	}
     }
 
@@ -648,7 +647,7 @@ static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
     fprintf(stderr, "get_final_string_arg: '%s'\n", p->idstr);
 #endif
 
-    return (p->err)? NULL : newstr(p, STR);
+    return (p->err)? NULL : newstr(p, strsym);
 }
 
 static NODE *get_middle_string_arg (parser *p)
@@ -924,8 +923,8 @@ static void pad_parent (NODE *parent, int k, int i, parser *p)
                                    (i==k-1 && (o & RIGHT_STR)))
 
 /* Get up to k comma-separated arguments (possibly optional).
-   However, if k < 0 this is a signal for get as many arguments as we
-   can find (the number is unknown in advance).
+   However, if k < 0 this is a signal to get as many arguments as we
+   can find (the number unknown in advance).
 */
 
 static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
@@ -993,7 +992,7 @@ static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
 	    } 
 	    /* matrix functions: handle trailing transpose */
 	    if (p->ch == '\'') {
-		set_transpose(t);
+		*next = p->ch;
 		parser_getc(p);
 	    }
 	    lex(p);
@@ -1160,6 +1159,13 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b2.r = get_final_string_arg(p, sym, 1);
 	}
+    } else if (sym == BOBJ) {
+	t = newb2(sym, NULL, NULL);
+	if (t != NULL) {
+	    t->v.b2.l = newref(p, BUNDLE);
+	    lex(p);
+	    t->v.b2.r = get_final_string_arg(p, sym, 1);
+	}	
     } else if (sym == OVAR) {
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
@@ -1241,6 +1247,9 @@ static NODE *powterm (parser *p)
 		get_slice_parts(t->v.b2.r, p);
 	    }
 	}	
+    } else if (next == '\'') {
+	/* support func(args)' */
+	set_transpose(t);
     }
 
 #if SDEBUG
