@@ -35,7 +35,7 @@ typedef enum {
     GPT_XYZ            = 1 << 5,  /* X-Y, controlling for Z */
     GPT_BATCH          = 1 << 6,  /* working in batch mode */
     GPT_GUI            = 1 << 7,  /* called from GUI context */
-    GPT_FIT_OMIT       = 1 << 8,  /* User said don't draw fitted line on graph */
+    GPT_FIT_OMIT       = 1 << 8,  /* user said don't draw fitted line on graph */
     GPT_DATA_STYLE     = 1 << 9,  /* data style is set by user */
     GPT_IDX            = 1 << 10, /* plot against time or obs index */
     GPT_TS             = 1 << 11, /* doing time series plot */
@@ -51,7 +51,8 @@ typedef enum {
     GPT_FILL_SWITCH    = 1 << 21, /* switching from errorbars to fill */
     GPT_ERR_SWITCH     = 1 << 22, /* switching from fill to errorbars */
     GPT_MONO           = 1 << 23, /* monochrome output */
-    GPT_GRID           = 1 << 24  /* display default gnuplot grid */
+    GPT_GRID           = 1 << 24, /* display default gnuplot grid */
+    GPT_POLAR          = 1 << 25  /* plot is in polar mode */
 } GptFlags; 
 
 typedef struct gretlRGB_ gretlRGB;
@@ -62,6 +63,8 @@ struct gretlRGB_ {
     unsigned char b;
 };
 
+typedef struct GPT_SPEC_ GPT_SPEC;
+
 #define MAXTITLE 128
 #define N_GP_COLORS 8
 #define BOXCOLOR (N_GP_COLORS - 2)
@@ -86,7 +89,7 @@ typedef enum {
     PLOT_LEVERAGE,
     PLOT_IRFBOOT,
     PLOT_KERNEL,
-    PLOT_VAR_ROOTS,
+    PLOT_ROOTS,
     PLOT_ELLIPSE,
     PLOT_MULTI_IRF,
     PLOT_PANEL,
@@ -144,6 +147,8 @@ typedef enum {
     
 const char *get_gretl_png_term_line (PlotType ptype, GptFlags flags);
 
+const char *get_png_line_for_plotspec (const GPT_SPEC *spec);
+
 const char *get_gretl_emf_term_line (PlotType ptype, int color);
 
 const char *gp_justification_string (int j);
@@ -172,6 +177,8 @@ int gnuplot_make_graph (void);
 
 int graph_written_to_file (void);
 
+int get_current_gp_term (void);
+
 void reset_plot_count (void);
 
 int matrix_plot (gretl_matrix *m, const int *list, const char *literal, 
@@ -185,7 +192,7 @@ int multi_scatters (const int *list, const double **Z,
 		    const DATAINFO *pdinfo, gretlopt opt);
 
 int gnuplot_3d (int *list, const char *literal,
-		double ***pZ, DATAINFO *pdinfo, 
+		double **Z, DATAINFO *pdinfo, 
 		gretlopt opt);
 
 int plot_freq (FreqDist *freq, DistCode dist);
@@ -204,9 +211,8 @@ int qq_plot (const int *list, const double **Z,
 int theil_forecast_plot (const int *plotlist, const double **Z, 
 			 const DATAINFO *pdinfo, gretlopt opt);
 
-int 
-gretl_panel_ts_plot (const int *list, const double **Z, DATAINFO *pdinfo,
-		     gretlopt opt);
+int gretl_panel_ts_plot (int vnum, const double **Z, DATAINFO *pdinfo,
+			 gretlopt opt);
 
 int plot_fcast_errs (const FITRESID *fr, const double *maxerr,
 		     const DATAINFO *pdinfo, gretlopt opt);
@@ -257,6 +263,8 @@ int gnuplot_has_latin5 (void);
 
 int gnuplot_has_cp1254 (void);
 
+int gnuplot_has_cp950 (void);
+
 int gnuplot_has_bbox (void);
 
 int gnuplot_has_utf8 (void);
