@@ -41,6 +41,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { ARBOND,   "arbond" },
     { ARCH,     "arch" },
     { ARMA,     "arima" },
+    { BIPROBIT, "biprobit" },
     { BREAK,    "break" },
     { BXPLOT,   "boxplot" },
     { CHOW,     "chow" },     
@@ -69,6 +70,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { ESTIMATE, "estimate" },
     { FCAST,    "fcast" }, 
     { FOREIGN,  "foreign" }, 
+    { FRACTINT, "fractint" }, 
     { FREQ,     "freq" }, 
     { FUNC,     "function" },
     { FUNCERR,  "funcerr" },
@@ -93,6 +95,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { LAGS,     "lags" },    
     { LDIFF,    "ldiff" },
     { LEVERAGE, "leverage" },
+    { LEVINLIN, "levinlin" },
     { LOGISTIC, "logistic" },
     { LOGIT,    "logit" },
     { LOGS,     "logs" },
