@@ -31,6 +31,7 @@ typedef enum {
     ARBOND,
     ARCH,
     ARMA,
+    BIPROBIT,
     BREAK,
     BXPLOT,
     CHOW,
@@ -59,6 +60,7 @@ typedef enum {
     ESTIMATE,
     FCAST,  
     FOREIGN,
+    FRACTINT,
     FREQ, 
     FUNC,
     FUNCERR,
@@ -83,6 +85,7 @@ typedef enum {
     LAGS,    
     LDIFF,
     LEVERAGE,
+    LEVINLIN,
     LOGISTIC,
     LOGIT,
     LOGS,
@@ -176,6 +179,7 @@ typedef enum {
                         c == HAUSMAN || \
                         c == KPSS || \
                         c == LEVERAGE || \
+                        c == LEVINLIN || \
                         c == MODTEST || \
                         c == MEANTEST || \
                         c == MPOLS || \
