@@ -134,7 +134,7 @@ static int lib_run_R_sync (gretlopt opt, PRN *prn)
     cmd = g_strdup_printf("\"%s\" --no-save --no-init-file --no-restore-data "
 			  "--slave", gretl_rbin_path());
 
-    err = winfork(cmd, NULL, SW_SHOWMINIMIZED, CREATE_NEW_CONSOLE);
+    err = win_run_sync(cmd, NULL);
 
     if (!err && !(opt & OPT_Q)) {
 	FILE *fp = gretl_fopen(gretl_Rout, "r");
@@ -1186,6 +1186,7 @@ int gretl_R_function_add_scalar (double x)
 {
     current_arg = R_CDR(current_arg);
     R_SETCAR(current_arg, R_ScalarReal(x));
+
     return 0;
 }
 
@@ -1205,6 +1206,7 @@ int gretl_R_function_add_vector (const double *x, int t1, int t2)
     }
     
     R_SETCAR(current_arg, res);
+
     return 0;
 }
 
