@@ -46,6 +46,14 @@
  * position 1 to position foo[0].
  */
 
+/**
+ * LISTSEP:
+ *
+ * Symbolic name for the separator used in gretl lists; this
+ * corresponds to a semicolon in the string representation of
+ * a list.
+ */
+
 typedef struct saved_list_ saved_list;
 
 struct saved_list_ {
@@ -138,9 +146,11 @@ int max_varno_in_saved_lists (void)
 
     for (i=0; i<n_lists; i++) {
 	list = list_stack[i]->list;
-	for (j=1; j<=list[0]; j++) {
-	    if (list[j] > vmax) {
-		vmax = list[j];
+	if (list != NULL) {
+	    for (j=1; j<=list[0]; j++) {
+		if (list[j] > vmax) {
+		    vmax = list[j];
+		}
 	    }
 	}
     }    
@@ -152,7 +162,7 @@ int max_varno_in_saved_lists (void)
  * get_list_name_by_index:
  * @idx: 0-based index into array of saved lists.
  *
- * Returns: the name of the specified saved list, or %NULL if
+ * Returns: the name of the specified saved list, or NULL if
  * @idx is out of bounds.
  */
 
@@ -172,7 +182,7 @@ const char *get_list_name_by_index (int idx)
  * Looks up @name in the stack of saved lists, at the current level
  * of function execution, and retrieves the associated list.
  *
- * Returns: the list, or %NULL if the lookup fails. 
+ * Returns: the list, or NULL if the lookup fails. 
  */
 
 int *get_list_by_name (const char *name)
@@ -672,14 +682,14 @@ static int var_is_deleted (const int *dlist, int dmin, int i)
 
 /**
  * gretl_lists_revise:
- * @dlist: list of variables to be deleted (or %NULL).
+ * @dlist: list of variables to be deleted (or NULL).
  * @dmin: lowest ID number of deleted var (referenced only
- * if @dlist is %NULL).
+ * if @dlist is NULL).
  *
  * Goes through any saved lists, adjusting the ID numbers
  * they contain to reflect the deletion from the dataset of
  * certain variables: those referenced in @dlist, if given, 
- * or if @dlist is %NULL, those variables with IDs greater 
+ * or if @dlist is NULL, those variables with IDs greater 
  * than or equal to @dmin.
  *
  * Returns: 0 on success, non-zero code on failure.
@@ -709,9 +719,11 @@ int gretl_lists_revise (const int *dlist, int dmin)
     /* find highest ID ref'd in any saved list */
     for (j=0; j<n_lists; j++) {
 	list = list_stack[j]->list;
-	for (i=1; i<=list[0]; i++) {
-	    if (list[i] > lmax) {
-		lmax = list[i];
+	if (list != NULL) {
+	    for (i=1; i<=list[0]; i++) {
+		if (list[i] > lmax) {
+		    lmax = list[i];
+		}
 	    }
 	}
     }
@@ -741,6 +753,9 @@ int gretl_lists_revise (const int *dlist, int dmin)
     /* use mapping to revise saved lists */
     for (j=0; j<n_lists; j++) {
 	list = list_stack[j]->list;
+	if (list == NULL) {
+	    continue;
+	}
 	for (i=list[0]; i>0; i--) {
 	    k = list[i] - dmin + 1;
 	    if (k >= 1) {
@@ -790,7 +805,7 @@ void gretl_lists_cleanup (void)
  * element of the array is set to 4.  The other elements of 
  * the list are initialized to 0.
  *
- * Returns: the newly allocated list, or %NULL on failure.
+ * Returns: the newly allocated list, or NULL on failure.
  */
 
 int *gretl_list_new (int nterms)
@@ -814,6 +829,57 @@ int *gretl_list_new (int nterms)
     return list;
 }
 
+/**
+ * gretl_list_array_new:
+ * @nlists: the number of lists to create.
+ * @nterms: the maximum number of elements to be stored in each list.
+ * 
+ * Creates an array of newly allocated lists, each of which as described
+ * in connection with gretl_list_new().
+ *
+ * Returns: the newly allocated lists, or NULL on failure.
+ */
+
+int **gretl_list_array_new (int nlists, int nterms)
+{
+    int **lists = NULL;
+    int i;
+    
+    if (nlists < 0) {
+	return NULL;
+    }
+
+    lists = malloc(nlists * sizeof *lists);
+    
+    if (lists != NULL) {
+	for (i=0; i<nlists; i++) {
+	    lists[i] = gretl_list_new(nterms);
+	}
+    }
+
+    return lists;
+}
+
+/**
+ * gretl_list_array_free:
+ * @lists: array of gretl lists.
+ * @nlists: the number of lists in @lists.
+ * 
+ * Frees all the lists in @lists and also the top-level pointer.
+ */
+
+void gretl_list_array_free (int **lists, int nlists)
+{
+    if (lists != NULL) {
+	int i;
+
+	for (i=0; i<nlists; i++) {
+	    free(lists[i]);
+	}
+	free(lists);
+    }
+}
+
 /**
  * gretl_consecutive_list_new:
  * @lmin: starting value for consecutive list elements.
@@ -822,7 +888,7 @@ int *gretl_list_new (int nterms)
  * Creates a newly allocated list whose elements run from
  * @lmin to @lmax consecutively.
  *
- * Returns: the newly allocated list, or %NULL on failure.
+ * Returns: the newly allocated list, or NULL on failure.
  */
 
 int *gretl_consecutive_list_new (int lmin, int lmax)
@@ -856,7 +922,7 @@ int *gretl_consecutive_list_new (int lmin, int lmax)
  * list is longer than the old, the extra elements are initialized 
  * to zero.
  *
- * Returns: the resized list, or %NULL on failure.
+ * Returns: the resized list, or NULL on failure.
  */
 
 int *gretl_list_resize (int **oldlist, int nterms)
@@ -901,7 +967,7 @@ int *gretl_list_resize (int **oldlist, int nterms)
  * so that it can hold one extra element, and sets the last
  * element to @v.
  *
- * Returns: the augmented list, or %NULL on failure.
+ * Returns: the augmented list, or NULL on failure.
  */
 
 int *gretl_list_append_term (int **plist, int v)
@@ -994,7 +1060,7 @@ int gretl_list_cmp (const int *list1, const int *list2)
  * Creates a newly allocated "list" with only one member, 
  * which is set to zero.
  *
- * Returns: the newly allocated list, or %NULL on failure.
+ * Returns: the newly allocated list, or NULL on failure.
  */
 
 int *gretl_null_list (void)
@@ -1013,7 +1079,7 @@ int *gretl_null_list (void)
  * @src: an array of integers, the first element of which holds
  * a count of the number of elements following.
  *
- * Returns: an allocated copy @src (or %NULL if @src is %NULL).
+ * Returns: an allocated copy @src (or NULL if @src is NULL).
  */
 
 int *gretl_list_copy (const int *src)
@@ -1039,7 +1105,7 @@ int *gretl_list_copy (const int *src)
  * a count of the number of elements following.
  *
  * Returns: an allocated copy @src from position @pos onward
- * (or %NULL on failure).
+ * (or NULL on failure).
  */
 
 int *gretl_list_copy_from_pos (const int *src, int pos)
@@ -1072,7 +1138,7 @@ int *gretl_list_copy_from_pos (const int *src, int pos)
  * (increasing) integers in the list, using the notation, e.g., 
  * "1-4" as shorthand for "1 2 3 4".
  *
- * Returns: the allocated array, or %NULL on failure.
+ * Returns: the allocated array, or NULL on failure.
  */
 
 int *gretl_list_from_string (const char *str, int *err)
@@ -1100,11 +1166,13 @@ int *gretl_list_from_string (const char *str, int *err)
     while (isspace(*s)) s++;
     tailstrip(s);
 
-    /* strip parentheses, if present */
-    if (*s == '(') {
+    /* strip parentheses or braces, if present */
+    if (*s == '(' || *s == '{') {
+	char close = (*s == '(')? ')' : '}';
+
 	n = strlen(s);
-	if (s[n-1] != ')') {
-	    /* got opening paren but no close */
+	if (s[n-1] != close) {
+	    /* got opening grouping character but no close */
 	    *err = E_PARSE;
 	    return NULL;
 	}
@@ -1112,7 +1180,7 @@ int *gretl_list_from_string (const char *str, int *err)
 	s++;
 	while (isspace(*s)) s++;
 	tailstrip(s);
-    }
+    } 
 
     q = s; /* copy relevant starting point */
 
@@ -1203,6 +1271,36 @@ int *gretl_list_from_string (const char *str, int *err)
     return list;
 }
 
+/**
+ * gretl_list_from_vector:
+ * @v: source vector.
+ *
+ * Returns: a newly allocated gretl list containing the values
+ * in @v, or NULL on failure. Note that it is an error if
+ * @v is NULL, or is not a vector.
+ */
+
+int *gretl_list_from_vector (const gretl_vector *v, int *err)
+{
+    int i, n = gretl_vector_get_length(v);
+    int *list = NULL;
+
+    if (n == 0) {
+	*err = E_DATA;
+    } else {
+	list = gretl_list_new(n);
+	if (list == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    for (i=0; i<n; i++) {
+		list[i+1] = (int) v->val[i];
+	    }
+	}
+    }
+
+    return list;
+}
+
 /**
  * gretl_list_to_string:
  * @list: array of integers.
@@ -1216,7 +1314,7 @@ int *gretl_list_from_string (const char *str, int *err)
  * than 9998.
  *
  * Returns: The string representation of the list on success,
- * or %NULL on failure.
+ * or NULL on failure.
  */
 
 char *gretl_list_to_string (const int *list)
@@ -1264,7 +1362,7 @@ char *gretl_list_to_string (const int *list)
  * if the list contains any numbers greater than 998.
  *
  * Returns: The string representation of the list on success,
- * or %NULL on failure.
+ * or NULL on failure.
  */
 
 char *gretl_list_to_lags_string (const int *list, int *err)
@@ -1460,6 +1558,43 @@ int gretl_list_purge_const (int *list, const double **Z,
     return gotc;
 }
 
+/**
+ * gretl_list_min_max:
+ * @list: gretl list.
+ * @lmin: location to receive minimum value.
+ * @lmax: location to receive maximum value.
+ *
+ * Reads @list from position 1 onward and writes to @lmin
+ * and @lmax the minimum and maximum values among the elements
+ * of the list. Reading stops at the end of the list or when
+ * a list separator is encountered.
+ *
+ * Returns: 0 on successful completion, error code if the
+ * list is NULL or empty.
+ */
+
+int gretl_list_min_max (const int *list, int *lmin, int *lmax)
+{
+    if (list == NULL || list[0] == 0) {
+	return E_DATA;
+    } else {
+	int i;
+
+	*lmin = *lmax = list[1];
+
+	for (i=2; i<=list[0]; i++) {
+	    if (list[i] < *lmin) {
+		*lmin = list[i];
+	    }
+	    if (list[i] > *lmax) {
+		*lmax = list[i];
+	    }
+	}
+
+	return 0;
+    }
+}
+
 static int *real_gretl_list_union (const int *orig, const int *add, 
 				   int *err, int duperr)
 {
@@ -1524,7 +1659,7 @@ static int *real_gretl_list_union (const int *orig, const int *add,
  * @add were already present in @orig, the error code is
  * %E_ADDDUP.
  *
- * Returns: new list on success, %NULL on error.
+ * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_add (const int *orig, const int *add, int *err)
@@ -1540,7 +1675,7 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
  *
  * Creates a list holding the union of @l1 and @l2.
  *
- * Returns: new list on success, %NULL on error.
+ * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_union (const int *l1, const int *l2, int *err)
@@ -1556,7 +1691,7 @@ int *gretl_list_union (const int *l1, const int *l2, int *err)
  *
  * Creates a list holding the intersection of @l1 and @l2.
  *
- * Returns: new list on success, %NULL on error.
+ * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_intersection (const int *l1, const int *l2, int *err)
@@ -1606,7 +1741,7 @@ int *gretl_list_intersection (const int *l1, const int *l2, int *err)
  *
  * Creates a list containing all but the last element of @orig.
  *
- * Returns: new list on success, %NULL on error.
+ * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_omit_last (const int *orig, int *err)
@@ -1750,7 +1885,7 @@ static int *real_gretl_list_omit (const int *orig, const int *omit,
  * present in @omit.  It is an error if the @omit list contains
  * members that are not present in @orig.
  *
- * Returns: new list on success, %NULL on error.
+ * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
@@ -1770,7 +1905,7 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
  * starts from position 1 in @orig, and it is not an error if
  * some members of @drop are not present in @orig.  
  *
- * Returns: new list on success, %NULL on error.
+ * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_drop (const int *orig, const int *drop, int *err)
@@ -1835,7 +1970,7 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
  *
  * Returns: a newly allocated list including the elements of @biglist,
  * from position @minpos onwards, that are not present in @sublist, 
- * again from @minpos onwards, or %NULL on failure.  Note that
+ * again from @minpos onwards, or NULL on failure.  Note that
  * comparison stops whenever a list separator is found; i.e. only
  * the pre-separator portions of the lists are compared.
  */
@@ -2219,11 +2354,11 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
  *
  * Concatenates the content of @list2 onto @list1, after first
  * appending the list separator.  It is acceptable that @list1
- * be %NULL, in which case the returned list is just @list2
+ * be NULL, in which case the returned list is just @list2
  * with the separator prepended.  But it is not acceptable that
- * @list2 be null; in that this function returns %NULL.
+ * @list2 be null; in that this function returns NULL.
  *
- * Returns: alllcated list  on success or %NULL on failure.
+ * Returns: alllcated list  on success or NULL on failure.
  */
 
 int *gretl_lists_join_with_separator (const int *list1, const int *list2)
@@ -2341,7 +2476,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 	if (ret == -1) {
 	    ret = real_list_dup(list, 1, start - 2);
 	}
-    } else if (ci == ARBOND) {
+    } else if (ci == ARBOND || ci == DPANEL) {
 	int stop = 0;
 
 	multi = 1;
@@ -2367,6 +2502,23 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 	    }
 	}
 	multi = 1;
+    } else if (ci == BIPROBIT) {
+	multi = 1;
+	if (list[1] == list[2]) {
+	    ret = 1;
+	} 
+	if (ret == -1) {
+	    for (i=3; i<list[0]; i++) {
+		if (list[i] == LISTSEP) {
+		    start = i+1;
+		    break;
+		}
+	    }
+	    ret = real_list_dup(list, start, list[0]);
+	    if (ret == -1) {
+		ret = real_list_dup(list, 3, start - 2);
+	    }
+	}
     } 
 
     if (!multi) {
@@ -2402,6 +2554,34 @@ int gretl_lists_share_members (const int *list1, const int *list2)
     return 0;
 }
 
+/**
+ * gretl_list_n_distinct_members:
+ * @list: list to test.
+ *
+ * Returns: the count of distinct elements in list from position
+ * 1 onward, not counting #LISTSEP if present.
+ */
+
+int gretl_list_n_distinct_members (const int *list)
+{
+    int i, j, n = list[0];
+
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] == LISTSEP) {
+	    n--;
+	} else {
+	    for (j=2; j<i; j++) {
+		if (list[i] == list[j]) {
+		    n--;
+		    break;
+		}
+	    }
+	}
+    }
+
+    return n;
+}
+
 /**
  * full_var_list:
  * @pdinfo: dataset information.
@@ -2410,14 +2590,14 @@ int gretl_lists_share_members (const int *list1, const int *list2)
  * Creates a newly allocated list including all series in the
  * dataset that are not hidden variables, and are accessible
  * at the current level of function execution.
- * The return value is %NULL in case either (a) allocation of
+ * The return value is NULL in case either (a) allocation of
  * memory failed, or (b) the resulting list would be empty.
  * The caller can distinguish between these possibilities by
  * examining the value returned in @nvars, which will be zero if
  * and only if the resulting list would be empty.  If this is
- * not of interest to the caller, @nvars may be given as %NULL.
+ * not of interest to the caller, @nvars may be given as NULL.
  *
- * Returns: the allocated list, or %NULL.
+ * Returns: the allocated list, or NULL.
  */
 
 int *full_var_list (const DATAINFO *pdinfo, int *nvars)
@@ -2518,7 +2698,7 @@ int gretl_list_is_consecutive (const int *list)
  * the ID numbers of variables, the names of variables, and/or the
  * names of previously defined lists (all separated by spaces).
  *
- * Returns: the constructed list, or %NULL on failure.
+ * Returns: the constructed list, or NULL on failure.
  */
 
 int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err)
@@ -2745,7 +2925,7 @@ void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
  * @pattern: pattern to be matched.
  *
  * Returns: a list of ID numbers of variables whose names
- * match @pattern, or %NULL if there are no matches.
+ * match @pattern, or NULL if there are no matches.
  */
 
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
