@@ -28,6 +28,7 @@
    as non-zero for the purpose of determining the rank of X */
 
 #define R_DIAG_MIN 1.0e-8
+#define BLOCKT -666
 
 typedef enum {
     GRETL_MOD_NONE = 0,
@@ -216,6 +217,8 @@ gretl_matrix *gretl_matrix_replace (gretl_matrix **pa,
 
 void gretl_matrix_block_destroy (gretl_matrix_block *B);
 
+void gretl_matrix_block_zero (gretl_matrix_block *B);
+
 gretl_matrix_block *gretl_matrix_block_new (gretl_matrix **pm, ...);
 
 gretl_matrix *gretl_identity_matrix_new (int n);
@@ -394,10 +397,10 @@ gretl_matrix *gretl_matrix_pow (const gretl_matrix *A,
 
 double gretl_matrix_dot_product (const gretl_matrix *a, GretlMatrixMod amod,
 				 const gretl_matrix *b, GretlMatrixMod bmod,
-				 int *errp);
+				 int *err);
 
 double gretl_vector_dot_product (const gretl_vector *a, const gretl_vector *b,
-				 int *errp);
+				 int *err);
 
 gretl_matrix *gretl_matrix_row_sum (const gretl_matrix *m, int *err);
 
@@ -604,7 +607,7 @@ int gretl_matrix_qform (const gretl_matrix *A, GretlMatrixMod amod,
 
 double gretl_scalar_qform (const gretl_vector *b, 
 			   const gretl_matrix *X,
-			   int *errp);
+			   int *err);
 
 int gretl_matrix_columnwise_product (const gretl_matrix *A,
 				     const gretl_matrix *B,
@@ -634,10 +637,15 @@ int gretl_matrices_are_equal (const gretl_matrix *a, const gretl_matrix *b,
 			      int *err);
 
 gretl_matrix *gretl_covariance_matrix (const gretl_matrix *m, int corr,
-				       int *errp);
+				       int *err);
 
 gretl_matrix *gretl_matrix_GG_inverse (const gretl_matrix *G, int *err);
 
+gretl_matrix *gretl_matrix_varsimul (const gretl_matrix *A, 
+				     const gretl_matrix *U, 
+				     const gretl_matrix *x0, 
+				     int *err);
+
 gretl_matrix **gretl_matrix_array_new (int n);
 
 gretl_matrix **
