@@ -23,16 +23,10 @@
 #include "objstack.h"
 
 typedef enum {
-    ARMA_SEAS  = 1 << 0, /* includes seasonal component */
-    ARMA_DSPEC = 1 << 1, /* input list includes differences */
-    ARMA_X12A  = 1 << 2, /* using X-12-ARIMA to generate estimates */
-    ARMA_EXACT = 1 << 3, /* using exact ML */
-    ARMA_VECH  = 1 << 4, /* using vech representation when computing
-			    variance matrix of state for Kalman filter
-			 */
-    ARMA_LS    = 1 << 5, /* using conditional ML, and O/NLS == CML */
-    ARMA_XDIFF = 1 << 6, /* ARIMA: exogenous regressors are differenced */
-    ARMA_LBFGS = 1 << 7  /* using L-BFGS-B with native exact ML */
+    ARMA_X12A  = 1 << 0, /* using X-12-ARIMA to generate estimates */
+    ARMA_EXACT = 1 << 1, /* using exact ML */
+    ARMA_LS    = 1 << 2, /* using conditional ML, and O/NLS == CML */
+    ARMA_OLS   = 1 << 3  /* OLS == MLE */
 } ArmaFlags;
 
 typedef struct CoeffIntervals_ CoeffIntervals;
@@ -121,6 +115,7 @@ struct VCVInfo_ {
                           c == PANEL || \
                           c == POISSON || \
                           c == PROBIT || \
+                          c == BIPROBIT || \
                           c == QUANTREG || \
                           c == TOBIT || \
                           c == WLS)
@@ -168,7 +163,8 @@ struct VCVInfo_ {
 			     c == NEGBIN || \
                              c == POISSON || \
                              c == PROBIT || \
-                             c == TOBIT)
+                             c == TOBIT || \
+                             c == BIPROBIT)
 
 /* model where the specification is not based on a list
    of variables */
@@ -215,6 +211,7 @@ typedef enum {
     GRETL_TEST_BP,
     GRETL_TEST_CHOWDUM,
     GRETL_TEST_COMFAC,
+    GRETL_TEST_INDEP,
     GRETL_TEST_MAX
 } ModelTestType;
 
@@ -333,6 +330,8 @@ int regarima_model_get_AR_coeffs (const MODEL *pmod,
 				  double **phi0,
 				  int *pp);
 
+int *arima_delta_coeffs (int d, int D, int s);
+
 int gretl_model_set_coeff_separator (MODEL *pmod, const char *s, int pos);
 
 int gretl_model_get_coeff_separator (const MODEL *pmod, const char **ps, int *ppos);
