@@ -1212,7 +1212,7 @@ char *addpath (char *fname, int script)
 
     /* try looking in default workdir? */
     if (1) {
-	char *dwork = gretl_default_workdir();
+	const char *dwork = gretl_default_workdir();
 
 	if (dwork != NULL) {
 	    int ok = 0;
@@ -1222,7 +1222,6 @@ char *addpath (char *fname, int script)
 	    if ((fname = search_dir(fname, dwork, USER_SEARCH))) { 
 		ok = 1;
 	    }
-	    free(dwork);
 	    if (ok) {
 		return fname;
 	    }
@@ -1548,23 +1547,22 @@ const char *gretl_workdir (void)
     return paths.workdir;
 }
 
+static char default_workdir[MAXLEN];
+
 #ifdef WIN32
 
-/* If the default workdir is a valid directory, and
-   not equal to the current workdir, return the path
-   to it.
-*/
-
-char *gretl_default_workdir (void)
+static const char *win32_default_workdir (void)
 {
+    const char *ret = NULL;
     char *base = mydocs_path();
-    char *ret = NULL;
     int ok = 0;
 
     if (base != NULL) {
-	ret = g_strdup_printf("%s\\gretl\\", base);
-	if (strcmp(ret, paths.workdir)) {
-	    DIR *dir = win32_opendir(ret);
+	sprintf(default_workdir, "%s\\gretl\\", base);
+	if (!strcmp(default_workdir, paths.workdir)) {
+	    *default_workdir = '\0';
+	} else {
+	    DIR *dir = win32_opendir(default_workdir);
 
 	    if (dir != NULL) {
 		closedir(dir);
@@ -1574,9 +1572,8 @@ char *gretl_default_workdir (void)
 	free(base);
     }
 
-    if (ret && !ok) {
-	free(ret);
-	ret = NULL;
+    if (ok) {
+	ret = default_workdir;
     }
 
     return ret;
@@ -1584,16 +1581,18 @@ char *gretl_default_workdir (void)
 
 #else /* !WIN32 */
 
-char *gretl_default_workdir (void)
+static const char *regular_default_workdir (void)
 {
+    const char *ret = NULL;
     char *home = getenv("HOME");
-    char *ret = NULL;
     int ok = 0;
 
     if (home != NULL) {
-	ret = gretl_strdup_printf("%s/gretl/", home);
-	if (strcmp(ret, paths.workdir)) {
-	    DIR *dir = opendir(ret);
+	sprintf(default_workdir, "%s/gretl/", home);
+	if (!strcmp(default_workdir, paths.workdir)) {
+	    *default_workdir = '\0';
+	} else {
+	    DIR *dir = opendir(default_workdir);
 
 	    if (dir != NULL) {
 		closedir(dir);
@@ -1602,9 +1601,8 @@ char *gretl_default_workdir (void)
 	}
     }
 
-    if (ret && !ok) {
-	free(ret);
-	ret = NULL;
+    if (ok) {
+	ret = default_workdir;
     }
 
     return ret;
@@ -1612,6 +1610,24 @@ char *gretl_default_workdir (void)
 
 #endif /* WIN32 or not */
 
+/**
+ * gretl_default_workdir:
+ *
+ * Returns: the full path to the default value of the
+ * user's gretl working directory if this differs from
+ * the user's currently-defined gretl working directory,
+ * or NULL if the two are the same.
+ */
+
+const char *gretl_default_workdir (void)
+{
+#ifdef WIN32
+    return win32_default_workdir();
+#else
+    return regular_default_workdir();
+#endif
+}
+
 static int validate_writedir (const char *dirname)
 {
     int err = 0;
@@ -1738,6 +1754,24 @@ const char *gretl_x12_arima_dir (void)
 
 const char *gretl_rbin_path (void)
 {
+#ifdef WIN32
+    /* avoid using a stale value saved to .gretl2rc */
+    static int checked;
+
+    if (!checked) {
+	char tmp[MAX_PATH];
+	int err;
+
+	err = R_path_from_registry(tmp, RTERM);
+
+	if (!err) {
+	    paths.rbinpath[0] = '\0';
+	    strncat(paths.rbinpath, tmp, MAXLEN - 1);
+	}	
+	checked = 1;
+    }
+#endif
+
     return paths.rbinpath;
 }
 
@@ -1806,14 +1840,75 @@ void show_paths (void)
     printf("gnuplot: %s\n", paths.gnuplot);
 }
 
-#ifndef WIN32
+#ifdef WIN32
+
+static char *rightmost (char *s1, char *s2)
+{
+    if (s1 == NULL) {
+	return s2;
+    } else if (s2 == NULL) {
+	return s1;
+    } else {
+	return (s2 - s1 > 0)? s2 : s1;
+    }
+}
+
+/* This aims to be general enough to handle the case where there
+   are no gretl entries in the registry; @progname is argv[0] at
+   startup.
+*/
+
+void win32_set_gretldir (const char *progname)
+{
+    int err;
+
+    *paths.gretldir = '\0';
+
+    /* we'll try the registry first */
+    err = read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", paths.gretldir);
+    if (!err) {
+	slash_terminate(paths.gretldir);
+	return;
+    }
+
+    if (g_path_is_absolute(progname)) {
+	strncat(paths.gretldir, progname, MAXLEN - 1);
+    } else {
+	char *test = getcwd(paths.gretldir, MAXLEN);
+
+	if (test != NULL) {
+	    int n = strlen(paths.gretldir);
+	    int m = strlen(progname);
+
+	    if (n + m + 1 < MAXLEN) {
+		if (paths.gretldir[n-1] != '\\' &&
+		    paths.gretldir[n-1] != '/') {
+		    strncat(paths.gretldir, "\\", 1);
+		}
+		strncat(paths.gretldir, progname, m);
+	    }
+	}
+    }
+
+    if (*paths.gretldir != '\0') {
+	char *p1 = strrchr(paths.gretldir, '\\');
+	char *p2 = strrchr(paths.gretldir, '/');
+	char *s = rightmost(p1, p2);
+
+	if (s != NULL) {
+	    *(s+1) = '\0';
+	}
+    }	
+}
+
+#else /* !WIN32 */
 
 /* We have paths.gretldir in place: now test it by seeing if we can
    open the the GPL file "COPYING", which definitely should be in that
    directory.  If that doesn't work, try some remedial measures.  
    Note, @config_path is the path garnered from the config file,
    which we may or may not have used to write paths.gretldir (and
-   which may be an empty string).
+   which may indeed be an empty string).
 */
 
 static void check_gretldir (char *config_path)
@@ -1931,21 +2026,20 @@ static void set_helpfile_paths (gretlopt opt)
 static void initialize_gretldir (char *dirname, gretlopt opt)
 {
     char *ghome = getenv("GRETL_HOME");
-    int done = 0, err = 0;
+    int err = 0;
 
     if (ghome != NULL) {
 	/* environment setting, if any, takes precedence */
 	strcpy(paths.gretldir, ghome);
 	slash_terminate(paths.gretldir);
-	done = 1;
-    } else if (*dirname != '\0') {
-	/* use value from config/registry */
+    } else if (*dirname != '\0' && *paths.gretldir == '\0') {
+	/* use value from config/registry, unless we already got
+	   a value somehow */
 	strcpy(paths.gretldir, dirname);
 	slash_terminate(paths.gretldir);
-	done = 1;
     } 
 
-    if (!done) {
+    if (*paths.gretldir == '\0') {
 #ifdef WIN32
 	/* fall back on installation-time default */
 	char *progfiles = program_files_path();
@@ -2330,6 +2424,17 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
 
     if (strcmp(paths.dotdir, paths.workdir)) { 
 	err1 = validate_writedir(paths.workdir);
+	if (err1) {
+	    /* try falling back on the default working dir */
+	    const char *defpath = gretl_default_workdir();
+
+	    if (*defpath != '\0' && strcmp(defpath, paths.workdir)) {
+		err1 = validate_writedir(defpath);
+		if (err1 == 0) {
+		    strcpy(paths.workdir, defpath);
+		}
+	    }
+	}
     }
 
     set_up_sourceview_path();
