@@ -646,7 +646,7 @@ void printlist (const int *list, const char *msg)
  * @k: number of parameters estimated.
  * @ll: pointer to recieve loglikelihood.
  * @aic: pointer to recieve Akaike criterion.
- * @bic: pointer to recieve Schwartz Bayesian criterion.
+ * @bic: pointer to recieve Schwarz Bayesian criterion.
  * @hqc: pointer to recieve Hannan-Quinn criterion.
  *
  * Calculates model selection criteria based on @ess, @nobs and
@@ -757,7 +757,8 @@ char *format_obs (char *obs, int maj, int min, int pd)
     return real_format_obs(obs, maj, min, pd, ':');
 }
 
-static int get_stobs_maj_min (char *stobs, int *maj, int *min)
+static int get_stobs_maj_min (char *stobs, int structure,
+			      int *maj, int *min)
 {
     int dotc = 0;
     char *p = stobs;
@@ -791,7 +792,7 @@ static int get_stobs_maj_min (char *stobs, int *maj, int *min)
 	    }
 	} else {
 	    sscanf(stobs, "%d", maj);
-	    if (*maj <= 0) {
+	    if (*maj <= 0 && structure != SPECIAL_TIME_SERIES) {
 		err = 1;
 	    }
 	}
@@ -959,7 +960,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
     } else {
 	int maj = 0, min = 0;
 
-	if (get_stobs_maj_min(stobs, &maj, &min)) {
+	if (get_stobs_maj_min(stobs, structure, &maj, &min)) {
 	    gretl_errmsg_sprintf(_("starting obs '%s' is invalid"), stobs);
 	    return 1;
 	}
@@ -998,14 +999,6 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
     pdinfo->structure = structure;
     pdinfo->sd0 = sd0;
 
-    if (pdinfo->structure != STACKED_TIME_SERIES &&
-	pdinfo->structure != STACKED_CROSS_SECTION &&
-	pdinfo->paninfo != NULL) {
-	/* This could be a problem in some cases, like
-	   if the dataset is subsampled? */
-	dataset_destroy_panel_info(pdinfo);
-    }
-
     ntodate(pdinfo->stobs, 0, pdinfo); 
     ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
 
@@ -1020,10 +1013,6 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
     }	
 
-    if (!err && pdinfo->structure == STACKED_TIME_SERIES) {
-	err = dataset_add_default_panel_indices(pdinfo);
-    }
-
     return err;
 }
 
@@ -1544,6 +1533,41 @@ int gretl_copy_file (const char *src, const char *dest)
     return 0;
 }  
 
+/* handle the case of "delete bundle[key]" */
+
+static int maybe_delete_bundle_value (const char *s, PRN *prn)
+{
+    char bname[VNAMELEN];
+    char key[VNAMELEN];
+    int err = 0;
+
+    if (sscanf(s, "%15[^[][%15[^]]", bname, key) == 2) {
+	gretl_bundle *bundle;
+	const char *s;
+
+	bundle = get_gretl_bundle_by_name(bname);
+	if (bundle == NULL) {
+	    err = E_UNKVAR;
+	} else {
+	    if (*key == '"') {
+		s = gretl_unquote(key, &err);
+	    } else if (gretl_is_string(key)) {
+		s = get_string_by_name(key);
+	    } else {
+		err = E_UNKVAR;
+	    }
+	}
+
+	if (!err) {
+	    err = gretl_bundle_delete_data(bundle, s);
+	}
+    } else {
+	err = E_UNKVAR;
+    }
+
+    return err;
+}
+
 int gretl_delete_var_by_name (const char *s, PRN *prn)
 {
     int err = 0;
@@ -1557,10 +1581,10 @@ int gretl_delete_var_by_name (const char *s, PRN *prn)
     } else if (get_string_by_name(s)) {
 	err = delete_saved_string(s, prn);
     } else if (gretl_is_bundle(s)) {
-	err = gretl_bundle_delete(s, prn);
+	err = gretl_bundle_delete_by_name(s, prn);
     } else {
-	err = E_UNKVAR;
-    }
+	err = maybe_delete_bundle_value(s, prn);
+    } 
 
     return err;
 }
@@ -1673,7 +1697,7 @@ void libgretl_session_cleanup (int mode)
     gretl_saved_objects_cleanup();
     gretl_transforms_cleanup();
     gretl_lists_cleanup();
-    gretl_plotx(NULL);
+    gretl_plotx(NULL, NULL);
 
     if (mode != SESSION_PRESERVE_MATRICES) {
 	destroy_user_scalars();
@@ -1784,3 +1808,5 @@ double get_last_lnl (char *blurb)
 
 
 
+
+
