@@ -28,6 +28,7 @@
 
 #ifndef STANDALONE
 # include "libgretl.h"
+# include "libset.h"
 #endif
 
 #define WBUFSIZE 8192
@@ -58,11 +59,12 @@
 #define DBHLEN     64
 
 static int wproxy;
-static char dbhost[DBHLEN]    = "ricardo.ecn.wfu.edu";
-static char gretlhost[DBHLEN] = "ricardo.ecn.wfu.edu";
-static char datacgi[DBHLEN]   = "/gretl/cgi-bin/gretldata.cgi";
-static char updatecgi[DBHLEN] = "/gretl/cgi-bin/gretl_update.cgi";
-static char mpath[DBHLEN]     = "/pub/gretl/manual/PDF/";
+static char dbhost[DBHLEN]      = "ricardo.ecn.wfu.edu";
+static char gretlhost[DBHLEN]   = "ricardo.ecn.wfu.edu";
+static char datacgi[DBHLEN]     = "/gretl/cgi-bin/gretldata.cgi";
+static char updatecgi[DBHLEN]   = "/gretl/cgi-bin/gretl_update.cgi";
+static char manual_path[DBHLEN] = "/pub/gretl/manual/PDF/";
+static char gfndoc_path[DBHLEN] = "/pub/gretl/gfndoc/";
 
 typedef enum {
     SAVE_NONE,
@@ -732,7 +734,7 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
     int (*show_progress) (long, long, int) = NULL;
     int show = 0;
 
-    if (expected > 2 * WBUFSIZE) {
+    if (gretl_in_gui_mode() && expected > 2 * WBUFSIZE) {
 	show_progress = get_plugin_function("show_progress", 
 					    &handle);
 	if (show_progress != NULL) show = 1;
@@ -769,9 +771,13 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
     allocated = WBUFSIZE;
 
     do {
+#if WDEBUG
 	fprintf(stderr, "get_contents: calling iread...\n");
+#endif
 	res = iread(fd, cbuf, sizeof cbuf);
+#if WDEBUG
 	fprintf(stderr, " got %d bytes\n", (int) res);
+#endif
 	if (res > 0) {
 	    if (fp == NULL) {
 		if ((size_t) (*len + res) > allocated) {
@@ -838,7 +844,7 @@ static char *make_posthead (urlinfo *u)
 
 	sprintf(head, "Content-Type: multipart/form-data, boundary=%s\r\n"
 		"Content-Length: %d\r\n", 
-		partsep, strlen(u->params) + u->upsize + traillen);
+		partsep, (int) strlen(u->params) + u->upsize + traillen);
     }
 
     return head;
@@ -1523,40 +1529,63 @@ static int open_local_file (urlinfo *u)
     return err;
 }
 
-static int 
-urlinfo_set_path (urlinfo *u, const char *cgi, const char *mname)
+static int urlinfo_set_cgi_path (urlinfo *u, const char *cgi)
 {
-    int len = 1;
+    int len = strlen(cgi) + 1;
+    int err = 0;
 
     if (wproxy) {
 	len += strlen(u->host) + 7;
     }
 
-    if (cgi != NULL && *cgi != '\0') {
-	len += strlen(cgi);
-    } else if (mname != NULL && *mname != '\0') {
-	len += strlen(mpath) + strlen(mname);
+    u->path = malloc(len);
+
+    if (u->path == NULL) {
+	err = 1;
+    } else {
+	if (wproxy) {
+	    sprintf(u->path, "http://%s", u->host);
+	} else {
+	    *u->path = '\0';
+	}
+	strcat(u->path, cgi);
+    }
+	
+    return err;
+}
+
+static int urlinfo_set_dl_path (urlinfo *u, const char *dirname,
+				const char *fname)
+{
+    int len = strlen(fname) + 1;
+    int err = 0;
+
+    if (dirname != NULL) {
+	/* dirname might be folded into fname */
+	len += strlen(dirname);
+    }
+
+    if (wproxy) {
+	len += strlen(u->host) + 7;
     }
 
     u->path = malloc(len);
+
     if (u->path == NULL) {
-	return 1;
-    }
-
-    *u->path = '\0';
-
-    if (wproxy) {
-	sprintf(u->path, "http://%s", u->host);
-    }
-    
-    if (cgi != NULL && *cgi != '\0') {
-	strcat(u->path, cgi);
-    } else if (mname != NULL && *mname != '\0') {
-	strcat(u->path, mpath);
-	strcat(u->path, mname);
+	err = 1;
+    } else {
+	if (wproxy) {
+	    sprintf(u->path, "http://%s", u->host);
+	} else {
+	    *u->path = '\0';
+	}
+	if (dirname != NULL) {
+	    strcat(u->path, dirname);
+	}
+	strcat(u->path, fname);
     }
 	
-    return 0;
+    return err;
 }
 
 static int 
@@ -1599,6 +1628,8 @@ urlinfo_set_params (urlinfo *u, CGIOpt opt, const char *fname,
     return (u->params != NULL);
 }
 
+#define BINTYPE(t) (t == GRAB_PKG || t == GRAB_PDF || t == GRAB_GFNDOC)
+
 /* when we're expecting binary data, check that we didn't get
    text/plain instead, which presumably in fact means we got
    an error message from the server
@@ -1608,9 +1639,8 @@ static int maybe_register_type_error (urlinfo *u, CGIOpt opt)
 {
     int err = 0;
 
-    if ((opt == GRAB_PKG || opt == GRAB_PDF) && 
-	!strcmp(u->gottype, "text/plain")) {
-	/* we shouldn't have got plain text */
+    if (BINTYPE(opt) && !strcmp(u->gottype, "text/plain")) {
+	/* we should not have got plain text */
 	err = 1;
 	if (u->fp != NULL) {
 	    char buf[256];
@@ -1632,7 +1662,8 @@ static void maybe_revise_www_paths (void)
 	strcpy(gretlhost, "www.wfu.edu");
 	strcpy(datacgi, "/~cottrell/gretl/gretldata.cgi");
 	strcpy(updatecgi, "/~cottrell/gretl/gretl_update.cgi");
-	strcpy(mpath, "/~cottrell/gretl/manual/");
+	strcpy(manual_path, "/~cottrell/gretl/manual/");
+	strcpy(gfndoc_path, "/~cottrell/gretl/gfndoc/");
     }
 }
 
@@ -1661,13 +1692,18 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
     if (u == NULL) {
 	return E_ALLOC;
     }
-    
-    if (opt == GRAB_FILE) {
-	urlinfo_set_path(u, updatecgi, NULL);
+
+    if (opt == GRAB_FOREIGN) {
+	/* note: not using the gretl server */
+	urlinfo_set_dl_path(u, NULL, fname);
     } else if (opt == GRAB_PDF) {
-	urlinfo_set_path(u, NULL, fname);
+	urlinfo_set_dl_path(u, manual_path, fname);
+    } else if (opt == GRAB_GFNDOC) {
+	urlinfo_set_dl_path(u, gfndoc_path, fname);
+    } else if (opt == GRAB_FILE) {
+	urlinfo_set_cgi_path(u, updatecgi);
     } else {
-	urlinfo_set_path(u, datacgi, NULL);
+	urlinfo_set_cgi_path(u, datacgi);
     }
 
     if (u->path == NULL) {
@@ -1675,7 +1711,7 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
 	return 1;
     }
 
-    if (opt != GRAB_PDF) {
+    if (opt != GRAB_PDF && opt != GRAB_GFNDOC && opt != GRAB_FOREIGN) {
 	urlinfo_set_params(u, opt, fname, dbseries);
 	if (u->params == NULL) {
 	    urlinfo_destroy(u, 0);
@@ -1703,10 +1739,14 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
 	return err;
     }
 
+#if WDEBUG
+    fprintf(stderr, "retrieve_url: calling try_http...\n"); 
+#endif
+
     result = try_http(u);
 
 #if WDEBUG
-    fprintf(stderr, "try_http returned %d, u->errbuf='%s'\n",
+    fprintf(stderr, "retrieve_url: try_http returned %d, u->errbuf='%s'\n",
 	    (int) result, u->errbuf);
 #endif
 
@@ -1733,6 +1773,10 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
 
     urlinfo_destroy(u, err);
 
+#if WDEBUG
+    fprintf(stderr, "retrieve_url: returning %d\n", err);
+#endif
+
     return err;
 }
 
@@ -1820,7 +1864,7 @@ int get_update_info (char **saver, time_t filedate, int queryopt)
 	return E_ALLOC;
     }
 
-    urlinfo_set_path(u, updatecgi, NULL);
+    urlinfo_set_cgi_path(u, updatecgi);
     urlinfo_set_update_params(u, filedate, queryopt);
     getbuf_init(u);
 
@@ -1902,7 +1946,7 @@ int upload_function_package (const char *login, const char *pass,
 	return E_ALLOC;
     }
 
-    urlinfo_set_path(u, datacgi, NULL);
+    urlinfo_set_cgi_path(u, datacgi);
     urlinfo_set_upload_params(u, login, pass, fname);
     getbuf_init(u);
 
@@ -2068,4 +2112,82 @@ int retrieve_manfile (const char *fname, const char *localname)
 			 localname, NULL);
 }
 
+/**
+ * retrieve_gfndoc:
+ * @fname: name of PDF file to retrieve.
+ * @localname: full path to which the file should be written
+ * on the local machine.
+ *
+ * Retrieves the specified gfn documentation in PDF format from the 
+ * gretl data server.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int retrieve_gfndoc (const char *fname, const char *localname)
+{
+    return retrieve_url (gretlhost, GRAB_GFNDOC, fname, NULL, SAVE_TO_FILE,
+			 localname, NULL);
+}
+
+/**
+ * retrieve_public_file:
+ * @uri: full URI for file to grab: host and path.
+ * @localname: full path to which the file should be written
+ * on the local machine. This cannot be NULL, but it can be
+ * empty, in which case it should be of length %MAXLEN, and
+ * on successful return it will be filled with an
+ * automatically assigned local name, based on the name
+ * of the file on the server.
+ *
+ * Retrieves the specified resource and writes it to
+ * @localname, if possible. Only handles http requests.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int retrieve_public_file (const char *uri, char *localname)
+{
+    const char *s, *path, *fname = NULL;
+    int err = 0;
+
+    if (strncmp(uri, "http://", 7)) {
+	err = E_DATA;
+    }
+
+    if (!err) {
+	s = uri + 7;
+	path = strrchr(s, '/');
+	if (path == NULL) {
+	    err = E_DATA;
+	} else {
+	    fname = path + 1;
+	    if (*fname == '\0') {
+		err = E_DATA;
+	    }
+	}
+    }
+
+    if (!err) {
+	char *host = NULL;
+
+	path = strchr(s, '/');
+	host = gretl_strndup(s, path - s);
+	if (host == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    if (*localname == '\0') {
+		/* write into user's dotdir */
+		strcat(localname, gretl_dotdir());
+		strcat(localname, fname);
+	    }
+	    err = retrieve_url(host, GRAB_FOREIGN, path, NULL, SAVE_TO_FILE,
+			       localname, NULL);
+	    free(host);
+	}
+    }
+
+    return err;
+}
+
 #endif /* !STANDALONE */
