@@ -20,7 +20,7 @@
 #ifndef GRETL_XML_H
 #define GRETL_XML_H
 
-#ifndef GRETLCLI
+#ifdef FULL_XML_HEADERS
 
 #define XUC const xmlChar *
 
@@ -83,6 +83,9 @@ int gretl_xml_node_get_string (xmlNodePtr node, xmlDocPtr doc,
 int gretl_xml_node_get_trimmed_string (xmlNodePtr node, xmlDocPtr doc, 
 				       char **pstr);
 
+int gretl_xml_child_get_string (xmlNodePtr node, xmlDocPtr doc, 
+				const char *name, char **pstr);
+
 int *gretl_xml_get_int_array (xmlNodePtr node, xmlDocPtr doc,
 			      int *nelem, int *err);
 
@@ -93,14 +96,14 @@ cmplx *gretl_xml_get_cmplx_array (xmlNodePtr node, xmlDocPtr doc,
 				  int *nelem, int *err);
 
 char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
-				    int *nelem, int *err);
+				    int *nelem, int slop, int *err);
 
 gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, 
 				    int *err);
 
 int gretl_xml_get_submask (xmlNodePtr node, xmlDocPtr doc, char **pmask);
 
-#endif /* !GRETLCLI */
+#endif /* FULL_XML_HEADERS */
 
 int gretl_write_matrix_as_gdt (const char *fname, 
 			       const gretl_matrix *X,
@@ -109,15 +112,17 @@ int gretl_write_matrix_as_gdt (const char *fname,
 
 int gretl_write_gdt (const char *fname, const int *list, 
 		     const double **Z, const DATAINFO *pdinfo, 
-		     GretlDataFormat fmt, PATHS *ppaths);
+		     gretlopt opt, int progress);
 
-int gretl_read_gdt (double ***pZ, DATAINFO *pdinfo, char *fname,
-		    PATHS *ppaths, gretlopt opt, PRN *prn);
+int gretl_read_gdt (const char *fname, double ***pZ, DATAINFO *pdinfo, 
+		    gretlopt opt, PRN *prn);
 
 char *gretl_get_gdt_description (const char *fname);
 
 int load_user_matrix_file (const char *fname);
 
+int load_user_scalars_file (const char *fname);
+
 int load_user_XML_file (const char *fname);
 
 #endif /* GRETL_XML_H */
