@@ -61,7 +61,8 @@ struct kalman_ {
     int k;  /* columns of A = number of exogenous vars in obs eqn */
     int p;  /* length of combined disturbance vector */
     int T;  /* rows of y = number of observations */
-    int t;  /* current time step, when filtering */
+    int okT; /* T - number of missing observations */
+    int t;   /* current time step, when filtering */
 
     int ifc; /* boolean: obs equation includes an implicit constant? */
 
@@ -82,12 +83,16 @@ struct kalman_ {
     gretl_matrix *P1; /* r x r: MSE matrix, after updating */
     gretl_matrix *e;  /* n x 1: one-step forecast error(s), time t */
 
-    /* input data matrices: the order matters for various functions */
+    /* input data matrices: note that the order matters for various 
+       functions, including user_kalman_recheck_matrices() and the
+       matrix_is_varying() macro
+    */
     const gretl_matrix *F; /* r x r: state transition matrix */
     const gretl_matrix *A; /* k x n: coeffs on exogenous vars, obs eqn */
     const gretl_matrix *H; /* r x n: coeffs on state variables, obs eqn */
     const gretl_matrix *Q; /* r x r: contemp covariance matrix, state eqn */
     const gretl_matrix *R; /* n x n: contemp covariance matrix, obs eqn */
+    const gretl_matrix *mu; /* r x 1: constant term in state transition */
     const gretl_matrix *y; /* T x n: dependent variable vector (or matrix) */
     const gretl_matrix *x; /* T x k: independent variables matrix */
     const gretl_matrix *Sini; /* r x 1: S_{1|0} */
@@ -131,10 +136,12 @@ struct kalman_ {
     gretl_matrix *Tmprr_2b;
     gretl_matrix *Tmpr1;
 
-    PRN *prn; /* verbose printer */
+    void *data; /* handle for attching additional info */
+    PRN *prn;   /* verbose printer */
 };
 
-#define NMATCALLS 5  /* max number of time-varying matrices */
+/* max number of time-varying matrices: F, A, H, Q, R, mu */
+#define NMATCALLS 6
 
 #define arma_ll(K) (K->flags & KALMAN_ARMA_LL)
 
@@ -169,6 +176,7 @@ enum {
     K_H,
     K_Q,
     K_R,
+    K_m,
     K_y,
     K_x,
     K_S,
@@ -231,7 +239,7 @@ void kalman_free (kalman *K)
     if (K->mnames != NULL) {
 	const gretl_matrix **mptr[] = {
 	    &K->F, &K->A, &K->H, &K->Q, &K->R,
-	    &K->y, &K->x, &K->Sini, &K->Pini
+	    &K->mu, &K->y, &K->x, &K->Sini, &K->Pini
 	};
 	int i;
 
@@ -274,6 +282,7 @@ static kalman *kalman_new_empty (int flags)
 	K->Q = K->R = NULL;
 	K->E = K->V = K->S = K->P = K->K = NULL;
 	K->y = K->x = NULL;
+	K->mu = NULL;
 	K->mnames = NULL;
 	K->matcalls = NULL;
 	K->cross = NULL;
@@ -282,6 +291,7 @@ static kalman *kalman_new_empty (int flags)
 	K->fnlevel = 0;
 	K->t = 0;
 	K->prn = NULL;
+	K->data = NULL;
     }
 
     return K;
@@ -397,10 +407,10 @@ static int check_matrix_dims (kalman *K, const gretl_matrix *m, int i)
 	c = K->n;
     } else if (i == K_R)  {
 	r = c = K->n;
-    } else if (i == K_S) {
+    } else if (i == K_S || i == K_m) {
 	r = K->r;
 	c = 1;
-    }
+    } 
 
     if (m->rows != r || m->cols != c) {
 	gretl_errmsg_sprintf("kalman: %s is %d x %d, should be %d x %d\n", 
@@ -514,6 +524,11 @@ static int kalman_check_dimensions (kalman *K)
 	}
     }
 
+    /* mu should be r x 1, if present */
+    if (!err && K->mu != NULL) {
+	err = check_matrix_dims(K, K->mu, K_m);
+    }    
+
     if (err) {
 	return err;
     }
@@ -756,6 +771,8 @@ static void kalman_set_dimensions (kalman *K, gretlopt opt)
     K->T = gretl_matrix_rows(K->y); /* y->rows defines T */
     K->n = gretl_matrix_cols(K->y); /* y->cols defines n */
 
+    K->okT = K->T;
+
     /* K->p is non-zero only under cross-correlation; in that case the
        matrix given as 'Q' in Kalman set-up in fact represents B (as
        in v_t = B \varepsilon_t) and it must be r x p, where p is the
@@ -777,11 +794,12 @@ static void kalman_set_dimensions (kalman *K, gretlopt opt)
  * @Q: r x r contemporaneous covariance matrix for the errors in the
  * state equation.
  * @R: n x n contemporaneous covariance matrix for the errors in the 
- * observation equation (or %NULL if this is not applicable).
+ * observation equation (or NULL if this is not applicable).
  * @y: T x n matrix of dependent variable(s).
- * @x: T x k matrix of exogenous variable(s).  May be %NULL if there
+ * @x: T x k matrix of exogenous variable(s).  May be NULL if there
  * are no exogenous variables, or if there's only a constant.
- * @E: T x n matrix in which to record forecast errors (or %NULL if
+ * @m: r x 1 vector of constants in the state transition, or NULL.
+ * @E: T x n matrix in which to record forecast errors (or NULL if
  * this is not required).
  * @err: location to receive error code.
  *
@@ -791,7 +809,7 @@ static void kalman_set_dimensions (kalman *K, gretlopt opt)
  * Series Analysis (1994, chapter 13), except that "S" is used in
  * place of Hamilton's \xi for the state vector.
  *
- * Returns: pointer to allocated struct, or %NULL on failure, in
+ * Returns: pointer to allocated struct, or NULL on failure, in
  * which case @err will receive a non-zero code.
  */
 
@@ -799,8 +817,8 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
 		    const gretl_matrix *F, const gretl_matrix *A,
 		    const gretl_matrix *H, const gretl_matrix *Q,
 		    const gretl_matrix *R, const gretl_matrix *y,
-		    const gretl_matrix *x, gretl_matrix *E,
-		    int *err)
+		    const gretl_matrix *x, const gretl_matrix *m,
+		    gretl_matrix *E, int *err)
 {
     kalman *K;
 
@@ -827,6 +845,7 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
     K->x = x;
     K->Sini = S;
     K->Pini = P;
+    K->mu = m;
 
     /* non-const, but again use external pointer */
     K->E = E;
@@ -863,7 +882,7 @@ enum {
    and K->C and form the 'real' Q and R.  But note that in the
    time-step case it may be that only one of Q, R needs to be treated
    in this way (if only one is time-varying, only one will have 
-   been redefined via function call).
+   been redefined via a function call).
 */
 
 static int kalman_update_crossinfo (kalman *K, int mode)
@@ -1151,7 +1170,7 @@ static int multiply_by_F (kalman *K, const gretl_matrix *A,
    although F and P are (r x r).
 */
 
-static int kalman_arma_iter_1 (kalman *K)
+static int kalman_arma_iter_1 (kalman *K, int missobs)
 {
     double Ve;
     int i, err = 0;
@@ -1159,6 +1178,10 @@ static int kalman_arma_iter_1 (kalman *K)
     /* write F*S into S+ */
     err += multiply_by_F(K, K->S0, K->S1, 0);
 
+    if (missobs) {
+	return err;
+    }
+
     /* form e = y - A'x - H'S */
     K->e->val[0] -= K->Ax->val[0];
     for (i=0; i<K->r; i++) {
@@ -1182,6 +1205,10 @@ static int kalman_arma_iter_1 (kalman *K)
 	K->SSRw += Ve * K->e->val[0];
     }
 
+    if (K->flags & KALMAN_ETT) {
+	K->e->val[0] = Ve;
+    }
+
     return err;
 }
 
@@ -1199,20 +1226,25 @@ static int kalman_iter_1 (kalman *K, int missobs, double *llt)
     /* write F*S into S+ */
     err += multiply_by_F(K, K->S0, K->S1, 0);
 
+    /* add \mu if present */
+    if (K->mu != NULL) {
+	gretl_matrix_add_to(K->S1, K->mu);
+    }
+
     if (missobs) {
 	/* the observable is not in fact observed at t */
 	*llt = 0.0;
 	if (K->K != NULL) {
 	    set_row(K->K, K->t, 0.0);
 	}
-	return 0;
+	return err;
     }   
 
     /* form e = y - A'x - H'S (e is already initialized to y) */
     err += gretl_matrix_subtract_from(K->e, K->Ax);
-    gretl_matrix_multiply_mod(K->H, GRETL_MOD_TRANSPOSE,
-			      K->S0, GRETL_MOD_NONE,
-			      K->e, GRETL_MOD_DECREMENT);
+    err += gretl_matrix_multiply_mod(K->H, GRETL_MOD_TRANSPOSE,
+				     K->S0, GRETL_MOD_NONE,
+				     K->e, GRETL_MOD_DECREMENT);
 
     if (!err) {
 	/* contribution to log-likelihood -- see Hamilton
@@ -1342,7 +1374,7 @@ int kalman_get_options (kalman *K)
    the A matrix (the first) but no explicit entry in the x matrix.
 */
 
-static void kalman_set_Ax (kalman *K)
+static void kalman_set_Ax (kalman *K, int *missobs)
 {
     double xjt, axi;
     int i, j;
@@ -1355,6 +1387,12 @@ static void kalman_set_Ax (kalman *K)
 	    } else {
 		xjt = gretl_matrix_get(K->x, K->t, j);
 	    }
+	    if (isnan(xjt)) {
+#if KDEBUG
+		fprintf(stderr, "x: got nan at obs %d\n", K->t);
+#endif
+		*missobs += 1;
+	    }
 	    axi += xjt * gretl_matrix_get(K->A, j, i);
 	}
 	gretl_vector_set(K->Ax, i, axi);
@@ -1374,6 +1412,9 @@ static void kalman_initialize_error (kalman *K, int *missobs)
 	yti = gretl_matrix_get(K->y, K->t, i);
 	K->e->val[i] = yti;
 	if (isnan(yti)) {
+#if KDEBUG
+	    fprintf(stderr, "y: got nan at obs %d\n", K->t);
+#endif
 	    *missobs += 1;
 	}
     }
@@ -1407,7 +1448,7 @@ static gretl_matrix *kalman_update_matrix (kalman *K, int i,
 static int kalman_refresh_matrices (kalman *K, PRN *prn)
 {
     const gretl_matrix **cptr[] = {
-	&K->F, &K->A, &K->H, &K->Q, &K->R
+	&K->F, &K->A, &K->H, &K->Q, &K->R, &K->mu
     };  
     int cross_update = 0;
     int i, err = 0;
@@ -1450,7 +1491,7 @@ static int kalman_refresh_matrices (kalman *K, PRN *prn)
 /**
  * kalman_forecast:
  * @K: pointer to Kalman struct: see kalman_new().
- * @prn: printing apparatus (or %NULL).
+ * @prn: printing apparatus (or NULL).
  *
  * Generates a series of one-step ahead forecasts for y, based on
  * information entered initially using kalman_new(), and possibly
@@ -1465,13 +1506,15 @@ static int kalman_refresh_matrices (kalman *K, PRN *prn)
 int kalman_forecast (kalman *K, PRN *prn)
 {
     double ldet;
-    int update_P = 1;
+    int smoothing, update_P = 1;
     int Tmiss = 0;
     int i, err = 0;
 
 #if KDEBUG
     fprintf(stderr, "kalman_forecast: T = %d\n", K->T);
-#endif  
+#endif 
+
+    smoothing = (K->flags & KALMAN_SMOOTH)? 1 : 0;
 
     if (K->nonshift < 0) {
 	K->nonshift = count_nonshifts(K->F);
@@ -1479,6 +1522,7 @@ int kalman_forecast (kalman *K, PRN *prn)
 
     K->SSRw = K->sumldet = K->loglik = 0.0;
     K->s2 = NADBL;
+    K->okT = K->T;
 
     if (K->x == NULL) {
 	/* no exogenous vars */
@@ -1516,13 +1560,19 @@ int kalman_forecast (kalman *K, PRN *prn)
 	/* read slice from y */
 	kalman_initialize_error(K, &missobs);
 
-	if (missobs) {
-	    Tmiss++;
+	if (K->x != NULL) {
+	    /* read from x if applicable 
+	       note 2010-09-18: this was conditional on missobs < K->n
+	       FIXME?
+	    */
+	    kalman_set_Ax(K, &missobs);
 	}
 
-	if (K->x != NULL && missobs < K->n) {
-	    /* read from x if applicable */
-	    kalman_set_Ax(K);
+	if (missobs) {
+	    /* 2010-09-18: this was right after kalman_initialize_error 
+	       FIXME?
+	     */
+	    Tmiss++;
 	}	
 
 	/* initial matrix calculations: form PH and H'PH 
@@ -1566,12 +1616,12 @@ int kalman_forecast (kalman *K, PRN *prn)
 	if (K->V != NULL) {
 	    /* record MSE for estimate of observables */
 	    if (missobs) {
-		if (K->flags & KALMAN_SMOOTH) {
+		if (smoothing) {
 		    set_row(K->V, K->t, 0.0); 
 		} else {
 		    set_row(K->V, K->t, 1.0/0.0); 
 		}
-	    } else if (K->flags & KALMAN_SMOOTH) {
+	    } else if (smoothing) {
 		/* record inverse */
 		load_to_vech(K->V, K->Vt, K->n, K->t);
 	    } else {
@@ -1581,8 +1631,8 @@ int kalman_forecast (kalman *K, PRN *prn)
 	}
 
 	/* first stage of dual iteration */
-	if (arma_ll(K)) {
-	    err = kalman_arma_iter_1(K);
+	if (arma_ll(K) && !smoothing) {
+	    err = kalman_arma_iter_1(K, missobs);
 	} else {
 	    err = kalman_iter_1(K, missobs, &llt);
 	    if (K->LL != NULL) {
@@ -1597,7 +1647,7 @@ int kalman_forecast (kalman *K, PRN *prn)
 	
 	/* record forecast errors if wanted */
 	if (!err && K->E != NULL) {
-	    if (missobs && (K->flags & KALMAN_SMOOTH)) {
+	    if (missobs && smoothing) {
 		set_row(K->E, K->t, 0.0);
 	    } else {
 		load_to_row(K->E, K->e, K->t);
@@ -1616,7 +1666,7 @@ int kalman_forecast (kalman *K, PRN *prn)
 
 	if (!err) {
 	    /* update MSE matrix, if needed */
-	    if (arma_ll(K) && update_P && K->t > 20) {
+	    if (arma_ll(K) && !smoothing && update_P && K->t > 20) {
 		if (!matrix_diff(K->P1, K->P0)) {
 		    K->P0->val[0] += 1.0;
 		    update_P = 0;
@@ -1638,13 +1688,15 @@ int kalman_forecast (kalman *K, PRN *prn)
 	goto bailout;
     }
 
+    K->okT = K->T - Tmiss;
+
     if (arma_ll(K)) {
-	double ll1 = 1.0 + LN_2_PI + log(K->SSRw / K->T);
+	double ll1 = 1.0 + LN_2_PI + log(K->SSRw / K->okT);
 
 	if (K->flags & KALMAN_AVG_LL) {
-	    K->loglik = -0.5 * (ll1 + K->sumldet / K->T);
+	    K->loglik = -0.5 * (ll1 + K->sumldet / K->okT);
 	} else {
-	    K->loglik = -0.5 * (K->T * ll1 + K->sumldet);
+	    K->loglik = -0.5 * (K->okT * ll1 + K->sumldet);
 	}
     } else {
 	/* For K->s2 see Koopman, Shephard and Doornik, "Statistical
@@ -1653,7 +1705,7 @@ int kalman_forecast (kalman *K, PRN *prn)
 	   available at http://www.ssfpack.com/ .  For the role of
 	   'd', see in addition Koopman's 1997 JASA article.
 	*/
-	int nT = K->n * (K->T - Tmiss);
+	int nT = K->n * K->okT;
 	int d = (K->flags & KALMAN_DIFFUSE)? K->r : 0;
 
 	if (d > 0) {
@@ -1710,7 +1762,7 @@ double kalman_get_arma_variance (const kalman *K)
     if (na(K->SSRw)) {
 	return NADBL;
     } else {
-	return K->SSRw / K->T;
+	return K->SSRw / K->okT;
     }
 }
 
@@ -1776,6 +1828,7 @@ struct K_input_mat K_input_mats[] = {
     { K_R, "obsvar" },
     { K_F, "statemat" },
     { K_Q, "statevar" },
+    { K_m, "stconst" },
     { K_S, "inistate" },
     { K_P, "inivar" }
 };
@@ -2016,6 +2069,8 @@ attach_input_matrix (kalman *K, const char *s, int i,
 	    K->Q = m;
 	} else if (i == K_R) {
 	    K->R = m;
+	} else if (i == K_m) {
+	    K->mu = m;
 	} else if (i == K_y) {
 	    K->y = m;
 	} else if (i == K_x) {
@@ -2101,19 +2156,19 @@ static int user_kalman_recheck_matrices (user_kalman *u)
     kalman *K = u->K;
     const gretl_matrix **cptr[] = {
 	&K->F, &K->A, &K->H, &K->Q, &K->R,
-	&K->y, &K->x, &K->Sini, &K->Pini
+	&K->mu, &K->y, &K->x, &K->Sini, &K->Pini
     };
     int i, err = 0;
 
-    for (i=0; i<=K_P; i++) {
+    for (i=0; i<K_MMAX; i++) {
 	if (!kalman_owns_matrix(K, i)) {
 	    /* pointer may be invalid, needs refreshing */
 	    *cptr[i] = NULL;
 	}
     }
 
-    for (i=0; i<=K_P && !err; i++) {
-	if (i <= K_R && matrix_is_varying(K, i)) {
+    for (i=0; i<K_MMAX && !err; i++) {
+	if (i <= K_m && matrix_is_varying(K, i)) {
 	    *cptr[i] = kalman_update_matrix(K, i, NULL, &err);
 	} else if (kalman_owns_matrix(K, i)) {
 	    err = update_scalar_matrix(*(gretl_matrix **) cptr[i], K->mnames[i]);
@@ -2239,6 +2294,18 @@ int delete_kalman (PRN *prn)
     return real_destroy_user_kalman(prn);
 }
 
+void kalman_attach_data (kalman *K, void *data)
+{
+    if (K != NULL) {
+	K->data = data;
+    }
+}
+
+void *kalman_get_data (const kalman *K)
+{
+    return (K != NULL)? K->data : NULL;
+}
+
 void kalman_attach_printer (kalman *K, PRN *prn)
 {
     if (K != NULL) {
@@ -2348,8 +2415,8 @@ static const char *kalman_matrix_name (int sym)
  * @line: "kalman" to start, "end kalman" to end; otherwise
  * this string should contain a matrix specification on the 
  * pattern "key value".
- * @Z: data array (may be %NULL).
- * @pdinfo: dataset information (may be %NULL).
+ * @Z: data array (may be NULL).
+ * @pdinfo: dataset information (may be NULL).
  * @opt: may contain %OPT_D for diffuse initialization of the
  * Kalman filter, %OPT_C to specify that the disturbances are 
  * correlated across the two equations.
@@ -2514,7 +2581,7 @@ static void transcribe_and_free (double *x, gretl_matrix *m,
 }
 
 /* Called on behalf of the kfilter() function: run a user-defined Kalman
-   filter in forecasting mode. The doc for kfilter says that it return
+   filter in forecasting mode. The doc for kfilter says that it returns
    0 on successful completion or 1 if numerical problems are encountered.
 */
 
@@ -3017,67 +3084,124 @@ static int kalman_add_stepinfo (kalman *K)
 }
 
 /**
- * user_kalman_smooth:
- * @Pname: name of matrix in which to retrieve the MSE of the
- * smoothed state (or %NULL if this is not required).
- * @Uname: name of matrix in which to retrieve the smoothed
- * disturbances (or %NULL if this is not required).
+ * kalman_arma_smooth:
+ * @K: pointer to Kalman struct.
  * @err: location to receive error code.
  * 
- * If a user-defined Kalman filter is found, runs a filtering
- * pass followed by a backward, smoothing pass.  At present
- * the @Uname argument is experimental and a bodge: it will 
+ * Runs a filtering pass followed by a smoothing pass.
+ *
+ * Returns: matrix containing the smoothed estimate of the
+ * dependent variable, or NULL on error.
+ */
+
+gretl_matrix *kalman_arma_smooth (kalman *K, int *err)
+{
+    int nr = (K->r * K->r + K->r) / 2;
+    int nn = (K->n * K->n + K->n) / 2;
+    gretl_vector *ys = NULL;
+
+    /* Set up the matrices we need to store computed results from all
+       time steps on the forward pass: state S_{t|t-1},(inverse) error 
+       variance, gain, and MSE of state, P_{t|t-1}.
+    */
+
+    K->S = gretl_matrix_alloc(K->T, K->r);
+    K->V = gretl_matrix_alloc(K->T, nn);
+    K->K = gretl_matrix_alloc(K->T, K->r * K->n);
+    K->P = gretl_matrix_alloc(K->T, nr);
+
+    if (K->S == NULL || K->V == NULL || K->K == NULL || K->P == NULL) {
+	*err = E_ALLOC;
+    } else {
+	double yst, Sti;
+	int i, t, miss = 0;
+
+	K->flags |= KALMAN_SMOOTH;
+	*err = kalman_forecast(K, NULL);
+	K->flags &= ~KALMAN_SMOOTH;
+	K->t = 0;
+
+	if (!*err) {
+	    *err = anderson_moore_smooth(K);
+	}
+
+	if (!*err) {
+	    ys = gretl_column_vector_alloc(K->T);
+	    if (ys == NULL) {
+		*err = E_ALLOC;
+	    } else {
+		for (t=0; t<K->T; t++) {
+		    yst = 0.0;
+		    for (i=0; i<K->r; i++) {
+			Sti = gretl_matrix_get(K->S, t, i);
+			yst += K->H->val[i] * Sti;
+		    }
+		    if (K->Ax != NULL) {
+			K->t = t;
+			kalman_set_Ax(K, &miss);
+			for (i=0; i<K->n; i++) {
+			    yst += gretl_vector_get(K->Ax, i);
+			}
+		    } 
+		    gretl_vector_set(ys, t, yst);
+		}
+		K->t = 0;
+	    }
+	}
+
+	gretl_matrix_replace(&K->S, NULL);
+	gretl_matrix_replace(&K->V, NULL);
+	gretl_matrix_replace(&K->K, NULL);
+	gretl_matrix_replace(&K->P, NULL);
+    }
+
+    if (*err && ys != NULL) {
+	gretl_matrix_free(ys);
+	ys = NULL;
+    }
+
+    return ys;
+}
+
+/**
+ * kalman_smooth:
+ * @K: pointer to kalman struct.
+ * @pP: pointer to matrix in which to retrieve the MSE of the
+ * smoothed state (or NULL if this is not required).
+ * @pU: pointer to matrix in which to retrieve the smoothed
+ * disturbances (or NULL if this is not required).
+ * @err: location to receive error code.
+ * 
+ * Runs a filtering pass followed by a backward, smoothing pass. 
+ * At present the @Uname argument is experimental and a bodge: it will 
  * not actually do anything unless @Pname is left null.
  *
  * Returns: matrix containing the smoothed estimate of the
- * state, or %NULL on error.
+ * state, or NULL on error.
  */
 
-gretl_matrix *user_kalman_smooth (const char *Pname, 
-				  const char *Uname,
-				  int *err)
+gretl_matrix *kalman_smooth (kalman *K,
+			     gretl_matrix **pP,
+			     gretl_matrix **pU,
+			     int *err)
 {
-    user_kalman *u = get_user_kalman(-1);
     gretl_matrix *E, *S, *P = NULL;
     gretl_matrix *G, *V, *U = NULL;
-    kalman *K;
-    int nr, nn, user_P = 0;
+    int nr, nn;
 
-    if (u == NULL) {
-	*err = missing_kalman_error();
-	return NULL;
-    }
-
-    K = u->K;
-
-    /* optional accessor for MSE of smoothed state estimate */
-    if (Pname != NULL && strcmp(Pname, "null")) {
-	P = get_matrix_by_name(Pname);
-	if (P == NULL) {
-	    *err = E_UNKVAR;
-	} else {
-	    user_P = 1;
-	}
-    }
-
-    if (!user_P && !*err) {
+    if (pP == NULL && pU != NULL) {
 	/* optional accessor for smoothed disturbances a la Koopman:
-	   experimental, and avilable only in @Pname is not given
+	   experimental, and avilable only if @pP is not given
 	*/
-	if (Uname != NULL && strcmp(Uname, "null")) {
-	    U = get_matrix_by_name(Uname);
-	    if (U == NULL) {
-		*err = E_UNKVAR;
-	    } else {
-		int Ucols = K->r;
+	int Ucols = K->r;
 
-		if (K->R != NULL) {
-		    Ucols += K->n;
-		}
-		if (U->rows != K->T || U->cols != Ucols) {
-		    *err = gretl_matrix_realloc(U, K->T, Ucols);
-		}
-	    }
+	if (K->R != NULL) {
+	    Ucols += K->n;
+	}
+
+	U = gretl_matrix_alloc(K->T, Ucols);
+	if (U == NULL) {
+	    *err = E_ALLOC;
 	}
     }
 
@@ -3098,11 +3222,7 @@ gretl_matrix *user_kalman_smooth (const char *Pname,
     V = gretl_matrix_alloc(K->T, nn);
     G = gretl_matrix_alloc(K->T, K->r * K->n);
     S = gretl_matrix_alloc(K->T, K->r);
-
-    if (P == NULL) {
-	/* the user didn't give P on input */
-	P = gretl_matrix_alloc(K->T, nr);
-    }
+    P = gretl_matrix_alloc(K->T, nr);
 
     if (E == NULL || V == NULL || G == NULL || 
 	S == NULL || P == NULL) {
@@ -3125,10 +3245,13 @@ gretl_matrix *user_kalman_smooth (const char *Pname,
     K->S = S;
     K->P = P;
 
+#if 0
     /* and recheck dimensions */
-    *err = user_kalman_recheck_matrices(u);
+    *err = user_kalman_recheck_matrices(K, fnlevel);
+#endif
 
     if (!*err) {
+	/* forward pass */
 	K->flags |= KALMAN_SMOOTH;
 	*err = kalman_forecast(K, NULL);
 	K->flags &= ~KALMAN_SMOOTH;
@@ -3161,10 +3284,18 @@ gretl_matrix *user_kalman_smooth (const char *Pname,
     gretl_matrix_free(V);
     gretl_matrix_free(G);
 
-    if (!user_P) {
+    if (!*err && pP != NULL) {
+	*pP = P;
+    } else {
 	gretl_matrix_free(P);
     }
 
+    if (!*err && pU != NULL) {
+	*pU = U;
+    } else {
+	gretl_matrix_free(U);
+    }    
+
     if (*err) {
 	gretl_matrix_free(S);
 	S = NULL;
@@ -3173,6 +3304,73 @@ gretl_matrix *user_kalman_smooth (const char *Pname,
     return S;
 }
 
+/**
+ * user_kalman_smooth:
+ * @Pname: name of matrix in which to retrieve the MSE of the
+ * smoothed state (or NULL if this is not required).
+ * @Uname: name of matrix in which to retrieve the smoothed
+ * disturbances (or NULL if this is not required).
+ * @err: location to receive error code.
+ * 
+ * If a user-defined Kalman filter is found, runs a filtering
+ * pass followed by a backward, smoothing pass.  At present
+ * the @Uname argument is experimental and a bodge: it will 
+ * not actually do anything unless @Pname is left null.
+ *
+ * Returns: matrix containing the smoothed estimate of the
+ * state, or NULL on error.
+ */
+
+gretl_matrix *user_kalman_smooth (const char *Pname, 
+				  const char *Uname,
+				  int *err)
+{
+    user_kalman *u = get_user_kalman(-1);
+    gretl_matrix *S = NULL;
+
+    if (u == NULL) {
+	*err = missing_kalman_error();
+    } else {
+	gretl_matrix **pP = NULL, **pU = NULL;
+	gretl_matrix *UP = NULL, *UU = NULL;
+	gretl_matrix *P = NULL, *U = NULL;
+
+	if (Pname != NULL && strcmp(Pname, "null")) {
+	    UP = get_matrix_by_name(Pname);
+	    if (UP == NULL) {
+		*err = E_UNKVAR;
+	    } else {
+		pP = &P;
+	    }
+	} else if (Uname != NULL && strcmp(Uname, "null")) {
+	    UU = get_matrix_by_name(Uname);
+	    if (UU == NULL) {
+		*err = E_UNKVAR;
+	    } else {
+		pU = &U;
+	    }	
+	}
+
+	if (!*err) {
+	    *err = user_kalman_recheck_matrices(u);
+	}
+
+	if (!*err) {
+	    S = kalman_smooth(u->K, pP, pU, err);
+	}
+
+	if (!*err && P != NULL) {
+	    user_matrix_replace_matrix_by_name(Pname, P);
+	}
+
+	if (!*err && U != NULL) {
+	    user_matrix_replace_matrix_by_name(Uname, U);
+	}	
+    }
+
+    return S;
+}
+
 /* See the account in Koopman, Shephard and Doornik, Econometrics
    Journal, 1999, section 4.2, regarding the initialization of the
    state under simulation.
@@ -3253,6 +3451,8 @@ static int kalman_simulate (kalman *K,
     } 
 
     for (K->t = 0; K->t < K->T; K->t += 1) {
+	int missobs = 0;
+
 	if (filter_is_varying(K)) {
 	    err = kalman_refresh_matrices(K, prn);
 	    if (err) {
@@ -3265,7 +3465,7 @@ static int kalman_simulate (kalman *K,
 				  K->S0, GRETL_MOD_NONE,
 				  yt, GRETL_MOD_NONE);
 	if (K->x != NULL) {
-	    kalman_set_Ax(K);
+	    kalman_set_Ax(K, &missobs);
 	}
 	if (K->A != NULL) {
 	    gretl_matrix_add_to(yt, K->Ax);
@@ -3296,6 +3496,11 @@ static int kalman_simulate (kalman *K,
 	} else {	    
 	    load_from_row(K->S1, V, K->t, GRETL_MOD_CUMULATE);
 	} 
+
+	if (K->mu != NULL) {
+	    gretl_matrix_add_to(K->S1, K->mu);
+	}
+
 	gretl_matrix_copy_values(K->S0, K->S1);
     }
 
@@ -3309,7 +3514,7 @@ static int kalman_simulate (kalman *K,
  * user_kalman_simulate:
  * @V: artificial disturbance to state.
  * @W: artificial disturbance to observation.
- * @Sname: name of matrix to retrieve simulated state (or %NULL).
+ * @Sname: name of matrix to retrieve simulated state (or NULL).
  * @prn: gretl printing struct.
  * @err: location to receive error code.
  * 
@@ -3321,10 +3526,10 @@ static int kalman_simulate (kalman *K,
  * those in the observation equation (if any).  But if the 
  * disturbances are correlated, then @V should contain the
  * "combined" disturbance vector at each time step, and
- * @W should be left %NULL.
+ * @W should be left NULL.
  *
  * Returns: matrix containing the simulated values of the
- * observables, or %NULL on failure.
+ * observables, or NULL on failure.
  */
 
 gretl_matrix *user_kalman_simulate (const gretl_matrix *V, 
@@ -3448,7 +3653,7 @@ double user_kalman_get_loglik (void)
  * Retrieves a matrix, specified by @idx, from the last
  * run of a kalman forecast, if applicable.
  * 
- * Returns: allocated matrix, or %NULL on failure.
+ * Returns: allocated matrix, or NULL on failure.
  */
 
 gretl_matrix *user_kalman_get_matrix (int idx, int *err)
