@@ -20,7 +20,18 @@
 #ifndef LIBGRETL_H
 #define LIBGRETL_H
 
-#include <stdio.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_VASPRINTF
+# define _GNU_SOURCE
+# include <stdio.h>
+# undef _GNU_SOURCE
+#else
+# include <stdio.h>
+#endif
+
 #include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
@@ -33,10 +44,6 @@
 # include <libxml/parser.h>
 #endif
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 #ifdef WIN32
 # include "winconfig.h"
 #endif
@@ -81,7 +88,6 @@
 #define LN_2_PI       1.837877066409345483560659472811
 #define LN_SQRT_2_PI  0.918938533204672741780329736406
 
-#define LISTSEP            (-100)
 #define PMAX_NOT_AVAILABLE 666
 
 /* numbers smaller than the given limit will print as zero */
@@ -255,17 +261,6 @@ typedef struct VARINFO_ {
     char line_width;
 } VARINFO;
 
-/* panel indexing information */
-typedef struct PANINFO_ {
-    int nunits;    /* number of cross-sectional units */
-    int Tmin;      /* min. number of time-series observations per unit */
-    int Tmax;      /* max. number of time-series observations per unit */
-    int olen;      /* length in digits of highest time-series index value */
-    int *unit;     /* index array, cross-sectional units */
-    int *period;   /* index array, time periods */
-    char *padmask; /* mask recording padding, when subsampled */
-} PANINFO;
-
 /* information on data set */
 typedef struct DATAINFO_ { 
     int v;              /* number of variables */
@@ -278,7 +273,6 @@ typedef struct DATAINFO_ {
     char endobs[OBSLEN]; /* string representation of ending obs */
     char **varname;     /* array of names of variables */
     VARINFO **varinfo;  /* array of specific info on vars */
-    PANINFO *paninfo;   /* additional info for panel data */
     char markers;       /* NO_MARKERS (0), REGULAR MARKERS or DAILY_DATE_STRINGS */
     char delim;         /* default delimiter for "CSV" files */
     char decpoint;      /* character used to represent decimal point */
@@ -341,7 +335,6 @@ typedef struct MODEL_ {
     double *sderr;               /* array of estimated std. errors */
     double *uhat;                /* regression residuals */
     double *yhat;                /* fitted values from regression */
-    double *llt;                 /* log-likelihood by observation */
     double *xpx;                 /* X'X matrix, in packed form */
     double *vcv;                 /* VCV matrix for coefficient estimates */
     double ess, tss;             /* Error and Total Sums of Squares */
