@@ -58,8 +58,9 @@ enum {
     STATE_LOOPING         = 1 << 16, /* loop is in progress at this level */
     STATE_LOOP_QUIET      = 1 << 17, /* loop commands should be quiet */
     STATE_LOOP_PROG       = 1 << 18, /* progressive loop is in progress */
-    STATE_BFGS_RSTEP      = 1 << 19  /* use Ricardson method in BFGS numerical
+    STATE_BFGS_RSTEP      = 1 << 19, /* use Richardson method in BFGS numerical
 					gradient */
+    STATE_DPDSTYLE_ON     = 1 << 20, /* emulate dpd in dynamic panel data models */
 };    
 
 /* for values that really want a non-negative integer */
@@ -96,6 +97,7 @@ struct set_vars_ {
     int garch_robust_vcv;       /* GARCH vcv variant, robust estimation */
     int arma_vcv;               /* ARMA vcv variant */
     int rq_maxiter;             /* max iterations for quantreg, simplex */
+    int gmm_maxiter;            /* max iterations for iterated GMM */
     gretl_matrix *initvals;     /* parameter initializer */
     struct robust_opts ropts;   /* robust standard error options */
     char shelldir[MAXLEN];      /* working dir for shell commands */
@@ -126,7 +128,8 @@ struct set_vars_ {
                            !strcmp(s, SKIP_MISSING) || \
 			   !strcmp(s, R_FUNCTIONS) || \
 			   !strcmp(s, R_LIB) || \
-			   !strcmp(s, BFGS_RSTEP))
+			   !strcmp(s, BFGS_RSTEP) || \
+			   !strcmp(s, DPDSTYLE))
 
 #define libset_double(s) (!strcmp(s, BFGS_TOLER) || \
 			  !strcmp(s, BFGS_MAXGRAD) || \
@@ -137,6 +140,7 @@ struct set_vars_ {
 #define libset_int(s) (!strcmp(s, BFGS_MAXITER) || \
 		       !strcmp(s, BFGS_VERBSKIP) || \
 		       !strcmp(s, BHHH_MAXITER) || \
+		       !strcmp(s, GMM_MAXITER) || \
 		       !strcmp(s, LBFGS_MEM) || \
 		       !strcmp(s, BOOTREP) || \
 		       !strcmp(s, HAC_KERNEL) || \
@@ -347,6 +351,7 @@ static void state_vars_copy (set_vars *sv)
     sv->bootrep = state->bootrep;
     sv->loop_maxiter = state->loop_maxiter;
     sv->rq_maxiter = state->rq_maxiter;
+    sv->gmm_maxiter = state->gmm_maxiter;
     sv->nls_toler = state->nls_toler;
     sv->delim = state->delim; 
     sv->vecm_norm = state->vecm_norm;
@@ -380,6 +385,7 @@ static void state_vars_init (set_vars *sv)
     sv->nls_toler = NADBL;
     sv->loop_maxiter = 250;
     sv->rq_maxiter = 1000;
+    sv->gmm_maxiter = 250;
     sv->delim = UNSET_INT;
     sv->vecm_norm = NORM_PHILLIPS;
     sv->initvals = NULL;
@@ -1222,6 +1228,7 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_int(BHHH_MAXITER, prn, opt);
     libset_print_double(BHHH_TOLER, prn, opt);
     libset_print_int(RQ_MAXITER, prn, opt);
+    libset_print_int(GMM_MAXITER, prn, opt);
     print_initvals(state->initvals, prn, opt);
     libset_print_bool(BFGS_RSTEP, prn, opt);
     libset_print_bool(USE_LBFGS, prn, opt);
@@ -1229,6 +1236,7 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_double(NLS_TOLER, prn, opt);
     libset_print_bool(USE_SVD, prn, opt);
     libset_print_bool(USE_FCP, prn, opt);
+    libset_print_bool(DPDSTYLE, prn, opt);
 
     libset_header(N_("Random number generation"), prn, opt);
 
@@ -1587,6 +1595,8 @@ int libset_get_int (const char *key)
 	return state->bhhh_maxiter;
     } else if (!strcmp(key, RQ_MAXITER)) {
 	return state->rq_maxiter;
+    } else if (!strcmp(key, GMM_MAXITER)) {
+	return state->gmm_maxiter;
     } else if (!strcmp(key, LBFGS_MEM)) {
 	return state->lbfgs_mem;
     } else if (!strcmp(key, BOOTREP)) {
@@ -1637,6 +1647,9 @@ static int intvar_min_max (const char *s, int *min, int *max,
     } else if (!strcmp(s, RQ_MAXITER)) {
 	*min = 1;
 	*var = &state->rq_maxiter;
+    } else if (!strcmp(s, GMM_MAXITER)) {
+	*min = 1;
+	*var = &state->gmm_maxiter;
     } else if (!strcmp(s, LBFGS_MEM)) {
 	*min = 3;
 	*max = 20;
@@ -1738,6 +1751,8 @@ static int boolvar_get_flag (const char *s)
 	return STATE_SKIP_MISSING;
     } else if (!strcmp(s, BFGS_RSTEP)) {
 	return STATE_BFGS_RSTEP;
+    } else if (!strcmp(s, DPDSTYLE)) {
+	return STATE_DPDSTYLE_ON;
     } else {
 	fprintf(stderr, "libset_get_bool: unrecognized "
 		"variable '%s'\n", s);	
@@ -2118,6 +2133,8 @@ int gretl_in_gui_mode (void)
     return gui_mode;
 }
 
+/* mechanism to support callback for printing iteration info */
+
 static ITER_PRINT_FUNC ifunc;
 
 void set_iter_print_func (ITER_PRINT_FUNC func)
@@ -2141,6 +2158,28 @@ int iter_print_callback (int i, PRN *prn)
     return ret;
 }
 
+/* mechanism to support callback for representing ongoing 
+   activity in the GUI */
+
+static SHOW_ACTIVITY_FUNC sfunc;
+
+void set_show_activity_func (SHOW_ACTIVITY_FUNC func)
+{
+    sfunc = func;
+}
+
+int show_activity_func_installed (void)
+{
+    return sfunc != NULL;
+}
+
+void show_activity_callback (void)
+{
+    if (sfunc != NULL) {
+	(*sfunc)();
+    }
+}
+
 /* mechanism for interactive debugging */
 
 static DEBUG_READLINE dbg_readline;
