@@ -67,8 +67,11 @@ typedef enum {
 #define R_FUNCTIONS      "R_functions"
 #define R_LIB            "R_lib"
 #define NORMAL_RAND      "normal_rand"
+#define DPDSTYLE         "dpdstyle"
+#define GMM_MAXITER      "gmm_maxiter"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
+typedef void (*SHOW_ACTIVITY_FUNC) (void);
 typedef int (*DEBUG_READLINE) (void *);
 typedef int (*DEBUG_OUTPUT) (void *);
 
@@ -153,6 +156,10 @@ void set_iter_print_func (ITER_PRINT_FUNC func);
 int iter_print_callback (int i, PRN *prn);
 int iter_print_func_installed (void);
 
+void set_show_activity_func (SHOW_ACTIVITY_FUNC func);
+void show_activity_callback (void);
+int show_activity_func_installed (void);
+
 void set_debug_read_func (DEBUG_READLINE dfunc);
 DEBUG_READLINE get_debug_read_func (void);
 
