@@ -42,7 +42,7 @@
  *
  * Returns: pointer to a newly allocated gretl_vector containing
  * the elements of x (or their squares), or NULL on failure.  
- * Missing valies in @x are skipped.
+ * Missing values in @x are skipped.
  */
 
 gretl_vector *
@@ -85,18 +85,13 @@ gretl_vector_from_array (const double *x, int n, GretlMatrixMod mod)
 gretl_vector *gretl_vector_from_series (const double *x, 
 					int t1, int t2)
 {
-    gretl_matrix *v;
-    int t, T = t2 - t1 + 1;
+    gretl_matrix *v = NULL;
+    int T = t2 - t1 + 1;
 
-    if (T <= 0) {
-	return NULL;
-    }
-
-    v = gretl_column_vector_alloc(T);
-
-    if (v != NULL) {
-	for (t=0; t<T; t++) {
-	    v->val[t] = x[t + t1];
+    if (T > 0) {
+	v = gretl_column_vector_alloc(T);
+	if (v != NULL) {
+	    memcpy(v->val, x + t1, T * sizeof *x);
 	}
     }
 
@@ -138,7 +133,7 @@ gretl_matrix *gretl_matrix_from_2d_array (const double **X,
  *
  * Returns: allocated 1 x 1 gretl_matrix, the single element
  * of which is set to @x, or NULL on allocation failure.
- * If @x = %NADBL the matrix value is %M_NA.
+ * If @x = #NADBL the matrix value is #M_NA.
  */
 
 gretl_matrix *gretl_matrix_from_scalar (double x) 
@@ -473,7 +468,7 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
     if (mask != NULL) {
 	T -= get_mask_count(mask, Tmax);
     } else if (op == M_MISSING_TRIM) {
-	*err = check_for_missing_obs(list, &t1, &t2, Z);
+	*err = list_adjust_sample(list, &t1, &t2, Z);
 	if (*err) {
 	    return NULL;
 	} else {
@@ -846,8 +841,6 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
 	    if (fscanf(fp, "%lf", &x) != 1) {
 		*err = E_DATA;
 		fprintf(stderr, "error reading row %d, column %d\n", i+1, j+1);
-		gretl_matrix_free(A);
-		A = NULL;
 	    } else {
 		gretl_matrix_set(A, i, j, x);
 	    }
@@ -858,6 +851,11 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
     
     fclose(fp);
 
+    if (*err) {
+	gretl_matrix_free(A);
+	A = NULL;
+    }
+
     return A;
 }
 
@@ -979,7 +977,9 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 
     if (msg != NULL && *msg != '\0' && !plain) {
 	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
-	if (!(m->t1 == 0 && m->t2 == 0)) {
+	if (m->t1 == BLOCKT && m->t2 == BLOCKT) {
+	    pprintf(prn, " (part of matrix block)\n\n");
+	} else if (m->t1 != 0 || m->t2 != 0) {
 	    pprintf(prn, " [t1 = %d, t2 = %d]\n\n", m->t1 + 1, m->t2 + 1);
 	} else {
 	    pputs(prn, "\n\n");
@@ -1329,7 +1329,8 @@ int gretl_matrix_cut_cols (gretl_matrix *m, const char *mask)
  * for rows/columns to be retained.
  *
  * In-place reduction of @m based on @mask: the masked rows
- * and columns are cut out of @m.
+ * and columns are cut out of @m. (The data array within @m
+ * is not "physically" resized.)
  *
  * Returns: 0 on success, non-zero on error.
  */
@@ -1340,13 +1341,19 @@ int gretl_matrix_cut_rows_cols (gretl_matrix *m, const char *mask)
     double x;
     int i, j, k, l, n;
 
-    if (m == NULL || mask == NULL || m->rows != m->cols) {
+    if (m == NULL || mask == NULL) {
 	return E_DATA;
-    }
+    } else if (m->rows != m->cols) {
+	return E_NONCONF;
+    } 
 
     n = count_unmasked_elements(mask, m->rows);
+    if (n == 0) {
+	gretl_matrix_reuse(m, 0, 0);
+	return 0;
+    }
 
-    /* allocate smaller temp matrix */
+    /* create smaller temporary matrix */
     tmp = gretl_matrix_alloc(n, n);
     if (tmp == NULL) {
 	return E_ALLOC;
@@ -1367,7 +1374,7 @@ int gretl_matrix_cut_rows_cols (gretl_matrix *m, const char *mask)
 	}
     }
 
-    /* resize the original matrix, copy the values back,
+    /* redimension the original matrix, copy the values back,
        and free the temp matrix */
     gretl_matrix_reuse(m, n, n);
     gretl_matrix_copy_values(m, tmp);
@@ -1426,6 +1433,110 @@ char *gretl_matrix_zero_row_mask (const gretl_matrix *m, int *err)
     return mask;
 }
 
+/**
+ * gretl_matrix_zero_col_mask:
+ * @m: matrix to process.
+ * @err: location to receive error code.
+ *
+ * Checks matrix @m for columns that are all zero.  If there are
+ * any such columns, constructs a mask of length equal to the
+ * number of columns in @m, with 1s indicating zero columns, 0s
+ * elsewhere.  If there are no such columns, returns NULL.
+ * 
+ * E_ALLOC is written to @err in case a mask should have
+ * been constructed but allocation failed.
+ *
+ * Returns: allocated mask or NULL.
+ */
+
+char *gretl_matrix_zero_col_mask (const gretl_matrix *m, int *err)
+{
+    char *mask = NULL;
+    int col0, any0 = 0;
+    int i, j;
+
+    mask = calloc(m->cols, 1);
+    if (mask == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+    
+    for (j=0; j<m->cols; j++) {
+	col0 = 1;
+	for (i=0; i<m->rows; i++) {
+	    if (gretl_matrix_get(m, i, j) != 0.0) {
+		col0 = 0;
+		break;
+	    }
+	}
+	if (col0) {
+	    mask[j] = 1;
+	    any0 = 1;
+	}
+    }
+
+    if (!any0) {
+	free(mask);
+	mask = NULL;
+    }
+
+    return mask;
+}
+
+/**
+ * gretl_matrix_zero_diag_mask:
+ * @m: matrix to process.
+ * @err: location to receive error code.
+ *
+ * Checks square matrix @m for diagonal elements that are zero.  
+ * If there are any such, constructs a mask of length equal to
+ * the number of rows (and columns) of @m, with 1s indicating
+ * the zero diagonal entries. If there are no zero diagonal
+ * elements, returns NULL.
+ * 
+ * E_ALLOC is written to @err in case a mask should have
+ * been constructed but allocation failed.
+ *
+ * Returns: allocated mask or NULL.
+ */
+
+char *gretl_matrix_zero_diag_mask (const gretl_matrix *m, int *err)
+{
+    char *mask = NULL;
+    int i, trim = 0;
+
+    if (gretl_is_null_matrix(m)) {
+	return NULL;
+    }
+
+    if (m->rows != m->cols) {
+	*err = E_NONCONF;
+	return NULL;
+    }
+
+    for (i=0; i<m->rows; i++) {
+	if (gretl_matrix_get(m, i, i) == 0.0) {
+	    trim = 1;
+	    break;
+	}
+    }
+
+    if (trim) {
+	mask = calloc(m->rows, 1);
+	if (mask == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    for (i=0; i<m->rows; i++) {
+		if (gretl_matrix_get(m, i, i) == 0.0) {
+		    mask[i] = 1;
+		}
+	    }
+	}
+    }
+
+    return mask;
+}
+
 /**
  * gretl_matrix_rank_mask:
  * @m: matrix to process.
