@@ -37,18 +37,8 @@
 #define LOOP_DEBUG 0
 #define SUBST_DEBUG 0
 
-#if LOOP_DEBUG
-# undef ENABLE_GMP
-#endif
-
-#if defined(ENABLE_GMP)
-# include <gmp.h>
-  typedef mpf_t bigval;
-#elif defined(HAVE_LONG_DOUBLE)
-  typedef long double bigval;
-#else
-  typedef double bigval;
-#endif
+#include <gmp.h>
+typedef mpf_t bigval;
 
 enum loop_types {
     COUNT_LOOP,
@@ -73,6 +63,7 @@ typedef struct {
     bigval *ssq;   /* running sum of squares */
     double *xbak;  /* previous values */
     int *diff;     /* indicator for difference */
+    char *na;      /* indicator for NAs in calculation */
 } LOOP_PRINT;  
 
 /* below: used only in "progressive" loops */ 
@@ -209,12 +200,14 @@ int gretl_execute_loop (void)
 
 /* For indexed loops: get a value from a loop "limit" element (lower
    or upper).  If we got the name of a scalar variable at setup time,
-   look up its current value (and modify the sign if wanted).
-   Otherwise we should have got a numerical constant at setup, in
-   which case we just return that value.
+   look up its current value (and modify the sign if wanted).  Or if
+   we got a "genr" expression, evaluate it.  Otherwise we should have
+   got a numerical constant at setup, in which case we just return
+   that value.  
 */
 
-static double controller_get_val (controller *clr)
+static double controller_get_val (controller *clr, double ***pZ, 
+				  DATAINFO *pdinfo, int *err)
 {
     if (clr->vname[0] != '\0') {
 	if (gretl_is_scalar(clr->vname)) {
@@ -222,7 +215,9 @@ static double controller_get_val (controller *clr)
 	} else {
 	    gretl_errmsg_sprintf(_("'%s': not a scalar"), clr->vname);
 	} 
-    } 
+    } else if (clr->expr != NULL) {
+	clr->val = generate_scalar(clr->expr, pZ, pdinfo, err);
+    }
 
 #if LOOP_DEBUG
     fprintf(stderr, "controller_get_val: vname='%s', returning %g\n", 
@@ -589,7 +584,7 @@ static int loop_attach_index_var (LOOPSET *loop, const char *vname,
 */
 
 static int index_get_limit (LOOPSET *loop, controller *clr, const char *s, 
-			    const double **Z, const DATAINFO *pdinfo)
+			    double ***pZ, DATAINFO *pdinfo)
 {
     int v, err = 0;
 
@@ -614,9 +609,12 @@ static int index_get_limit (LOOPSET *loop, controller *clr, const char *s,
 	    *clr->vname = '\0';
 	    strncat(clr->vname, s, VNAMELEN - 1);
 	} else {
-	    gretl_errmsg_sprintf(_("Undefined variable '%s' in loop condition."), s);
-	    err = E_UNKVAR;
-	}
+	    /* expression to be evaluated to scalar? */
+	    clr->expr = gretl_strdup(s);
+	    if (clr->expr == NULL) {
+		err = E_ALLOC;
+	    }
+	} 
     }
 
     return err;
@@ -625,8 +623,8 @@ static int index_get_limit (LOOPSET *loop, controller *clr, const char *s,
 #define maybe_date(s) (strchr(s, ':') || strchr(s, '/'))
 
 static int parse_as_indexed_loop (LOOPSET *loop,
-				  const double **Z,
-				  const DATAINFO *pdinfo,
+				  double ***pZ,
+				  DATAINFO *pdinfo,
 				  const char *lvar, 
 				  const char *start,
 				  const char *end)
@@ -655,9 +653,9 @@ static int parse_as_indexed_loop (LOOPSET *loop,
 	    }
 	}
     } else {
-	err = index_get_limit(loop, &loop->init, start, Z, pdinfo);
+	err = index_get_limit(loop, &loop->init, start, pZ, pdinfo);
 	if (!err) {
-	    err = index_get_limit(loop, &loop->final, end, Z, pdinfo);
+	    err = index_get_limit(loop, &loop->final, end, pZ, pdinfo);
 	}
 	if (!err) {
 	    loop->type = INDEX_LOOP;
@@ -677,13 +675,13 @@ static int parse_as_indexed_loop (LOOPSET *loop,
 }
 
 static int parse_as_count_loop (LOOPSET *loop, 
-				const double **Z,
-				const DATAINFO *pdinfo,
+				double ***pZ,
+				DATAINFO *pdinfo,
 				const char *s)
 {
     int err;
 
-    err = index_get_limit(loop, &loop->final, s, Z, pdinfo);
+    err = index_get_limit(loop, &loop->final, s, pZ, pdinfo);
 
     if (!err) {
 	loop->init.val = 1;
@@ -1101,14 +1099,13 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
     if (!strncmp(s, "foreach ", 8)) {
 	err = parse_as_each_loop(loop, pdinfo, s + 8);
     } else if (sscanf(s, "%15[^= ] = %15[^.]..%15s", lvar, op, rvar) == 3) {
-	err = parse_as_indexed_loop(loop, (const double **) *pZ, pdinfo, 
-				    lvar, op, rvar);
+	err = parse_as_indexed_loop(loop, pZ, pdinfo, lvar, op, rvar);
     } else if (!strncmp(s, "for", 3)) {
 	err = parse_as_for_loop(loop, pZ, pdinfo, s + 4);
     } else if (!strncmp(s, "while", 5)) {
 	err = parse_as_while_loop(loop, pZ, pdinfo, s + 6);
     } else if (sscanf(s, "%15s", lvar) == 1) {
-	err = parse_as_count_loop(loop, (const double **) *pZ, pdinfo, lvar);
+	err = parse_as_count_loop(loop, pZ, pdinfo, lvar);
     } else {
 	printf("parse_first_loopline: failed on '%s'\n", s);
 	gretl_errmsg_set(_("No valid loop condition was given."));
@@ -1273,9 +1270,7 @@ static void controller_free (controller *clr)
 
 static int gretl_loop_prepare (LOOPSET *loop)
 {
-#ifdef ENABLE_GMP
     mpf_set_default_prec(256);
-#endif
 
     /* allocate some initial lines/commands for loop */
     loop->lines = malloc(LOOP_BLOCK * sizeof *loop->lines); 
@@ -1290,24 +1285,22 @@ static int gretl_loop_prepare (LOOPSET *loop)
 
 static void loop_model_free (LOOP_MODEL *lmod)
 {
+    int i, n;
+
 #if LOOP_DEBUG
     fprintf(stderr, "loop_model_free: lmod at %p, model0 at %p\n",
 	    (void *) lmod, (void *) lmod->model0);
 #endif
 
-#ifdef ENABLE_GMP
-    int i, n = 4 * lmod->model0->ncoeff;
+    n = 4 * lmod->model0->ncoeff;
 
     for (i=0; i<n; i++) {
 	mpf_clear(lmod->bigarray[i]);
     }
-#endif
 
     free(lmod->bigarray);
     free(lmod->cbak);
-    free(lmod->sbak);
     free(lmod->cdiff);
-    free(lmod->sdiff);
 
     gretl_model_free(lmod->model0);
 }
@@ -1321,15 +1314,10 @@ static void loop_model_zero (LOOP_MODEL *lmod)
     lmod->n = 0;
 
     for (i=0; i<lmod->nc; i++) {
-#ifdef ENABLE_GMP
 	mpf_init(lmod->sum_coeff[i]);
 	mpf_init(lmod->ssq_coeff[i]);
 	mpf_init(lmod->sum_sderr[i]);
 	mpf_init(lmod->ssq_sderr[i]);
-#else
-	lmod->sum_coeff[i] = lmod->ssq_coeff[i] = 0.0;
-	lmod->sum_sderr[i] = lmod->ssq_sderr[i] = 0.0;
-#endif
 	lmod->cbak[i] = lmod->sbak[i] = NADBL;
 	lmod->cdiff[i] = lmod->sdiff[i] = 0;
     }
@@ -1343,8 +1331,8 @@ static void loop_model_init (LOOP_MODEL *lmod, int lno)
     lmod->nc = 0;
     lmod->model0 = NULL;
     lmod->bigarray = NULL;
-    lmod->cbak = lmod->sbak = NULL;
-    lmod->cdiff = lmod->sdiff = NULL;
+    lmod->cbak = NULL;
+    lmod->cdiff = NULL;
 }
 
 /* Start up a LOOP_MODEL struct: copy pmod into place and
@@ -1352,74 +1340,78 @@ static void loop_model_init (LOOP_MODEL *lmod, int lno)
 
 static int loop_model_start (LOOP_MODEL *lmod, const MODEL *pmod)
 {
-    int nc;
+    int nc = pmod->ncoeff;
+    int err = 0;
 
 #if LOOP_DEBUG
     fprintf(stderr, "init: copying model at %p\n", (void *) pmod);
 #endif
 
-    lmod->nc = nc = pmod->ncoeff;
-
     lmod->model0 = gretl_model_copy(pmod);
     if (lmod->model0 == NULL) {
 	return E_ALLOC;
     }
 
+    lmod->nc = nc;
+
     lmod->bigarray = malloc(nc * 4 * sizeof *lmod->bigarray);
-    if (lmod->bigarray == NULL) return E_ALLOC;
+    if (lmod->bigarray == NULL) {
+	return E_ALLOC;
+    }
 
     lmod->sum_coeff = lmod->bigarray;
     lmod->ssq_coeff = lmod->sum_coeff + nc;
     lmod->sum_sderr = lmod->ssq_coeff + nc;
     lmod->ssq_sderr = lmod->sum_sderr + nc;
 
-    lmod->cbak = malloc(nc * sizeof *lmod->cbak);
-    if (lmod->cbak == NULL) goto cleanup;
+    lmod->cbak = malloc(nc * 2 * sizeof *lmod->cbak);
+    if (lmod->cbak == NULL) {
+	err = E_ALLOC;
+    } else {
+	lmod->sbak = lmod->cbak + nc;
+    }
 
-    lmod->sbak = malloc(nc * sizeof *lmod->sbak);
-    if (lmod->sbak == NULL) goto cleanup;
-
-    lmod->cdiff = malloc(nc * sizeof *lmod->cdiff);
-    if (lmod->cdiff == NULL) goto cleanup;
-
-    lmod->sdiff = malloc(nc * sizeof *lmod->sdiff);
-    if (lmod->sdiff == NULL) goto cleanup;
-
-    loop_model_zero(lmod);
+    if (!err) {
+	lmod->cdiff = malloc(nc * 2 * sizeof *lmod->cdiff);
+	if (lmod->cdiff == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    lmod->sdiff = lmod->cdiff + nc;
+	}
+    }
 
+    if (!err) {
+	loop_model_zero(lmod);
 #if LOOP_DEBUG
-    fprintf(stderr, " model copied to %p, returning 0\n", 
-	    (void *) lmod->model0);
+	fprintf(stderr, " model copied to %p, returning 0\n", 
+		(void *) lmod->model0);
 #endif
+    }
 
-    return 0;
+    if (err) {
+	free(lmod->bigarray);
+	free(lmod->cbak);
+	free(lmod->cdiff);
+    }
 
- cleanup:
-    free(lmod->bigarray);
-    free(lmod->cbak);
-    free(lmod->sbak);
-    free(lmod->cdiff);
-    free(lmod->sdiff);
-
-    return E_ALLOC;
+    return err;
 }
 
 static void loop_print_free (LOOP_PRINT *lprn)
 {
-#ifdef ENABLE_GMP
     int i;
 
     for (i=0; i<lprn->list[0]; i++) {
 	mpf_clear(lprn->sum[i]);
 	mpf_clear(lprn->ssq[i]);
     }
-#endif
 
     free(lprn->sum);
     free(lprn->ssq);
     free(lprn->list);
     free(lprn->xbak);
     free(lprn->diff);
+    free(lprn->na);
 }
 
 static void loop_print_zero (LOOP_PRINT *lprn)
@@ -1429,14 +1421,11 @@ static void loop_print_zero (LOOP_PRINT *lprn)
     lprn->n = 0;
 
     for (i=0; i<lprn->list[0]; i++) { 
-#ifdef ENABLE_GMP
 	mpf_init(lprn->sum[i]);
 	mpf_init(lprn->ssq[i]);
-#else
-	lprn->sum[i] = lprn->ssq[i] = 0.0;
-#endif
 	lprn->xbak[i] = NADBL;
 	lprn->diff[i] = 0;
+	lprn->na[i] = 0;
     }
 }
 
@@ -1448,6 +1437,7 @@ static void loop_print_init (LOOP_PRINT *lprn, int lno)
     lprn->ssq = NULL;
     lprn->xbak = NULL;
     lprn->diff = NULL;
+    lprn->na = NULL;
 }
 
 /* allocate and initialize @lprn, based on the number of
@@ -1472,6 +1462,9 @@ static int loop_print_start (LOOP_PRINT *lprn, const int *list)
     lprn->diff = malloc(k * sizeof *lprn->diff);
     if (lprn->diff == NULL) goto cleanup;
 
+    lprn->na = malloc(k);
+    if (lprn->na == NULL) goto cleanup;
+
     loop_print_zero(lprn);
 
     return 0;
@@ -1483,6 +1476,7 @@ static int loop_print_start (LOOP_PRINT *lprn, const int *list)
     free(lprn->ssq);
     free(lprn->xbak);
     free(lprn->diff);
+    free(lprn->na);
 
     return 1;
 }
@@ -1737,9 +1731,7 @@ get_loop_model_by_line (LOOPSET *loop, int lno, int *err)
 
 static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 {
-#ifdef ENABLE_GMP
     mpf_t m;
-#endif
     int j, err = 0;
 
 #if LOOP_DEBUG
@@ -1760,12 +1752,9 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 	}
     }
 
-#ifdef ENABLE_GMP
     mpf_init(m);
-#endif
 
     for (j=0; j<pmod->ncoeff; j++) {
-#ifdef ENABLE_GMP
 	mpf_set_d(m, pmod->coeff[j]);
 	mpf_add(lmod->sum_coeff[j], lmod->sum_coeff[j], m); 
 	mpf_mul(m, m, m);
@@ -1775,12 +1764,6 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 	mpf_add(lmod->sum_sderr[j], lmod->sum_sderr[j], m);
 	mpf_mul(m, m, m);
 	mpf_add(lmod->ssq_sderr[j], lmod->ssq_sderr[j], m);
-#else
-	lmod->sum_coeff[j] += pmod->coeff[j];
-	lmod->ssq_coeff[j] += pmod->coeff[j] * pmod->coeff[j];
-	lmod->sum_sderr[j] += pmod->sderr[j];
-	lmod->ssq_sderr[j] += pmod->sderr[j] * pmod->sderr[j];
-#endif
 	if (!na(lmod->cbak[j]) && realdiff(pmod->coeff[j], lmod->cbak[j])) {
 	    lmod->cdiff[j] = 1;
 	}
@@ -1791,9 +1774,7 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 	lmod->sbak[j] = pmod->sderr[j];
     }
 
-#ifdef ENABLE_GMP
     mpf_clear(m);
-#endif
 
     lmod->n += 1;
 
@@ -1805,7 +1786,7 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 }
 
 /* Update the LOOP_PRINT struct lprn using the current values from the
-   variables in list.  If this is the fist use we need to do some
+   variables in list.  If this is the first use we need to do some
    allocation first.
 */
 
@@ -1813,9 +1794,7 @@ static int loop_print_update (LOOP_PRINT *lprn,
 			      const int *list, const double **Z, 
 			      const DATAINFO *pdinfo)
 {
-#ifdef ENABLE_GMP
     mpf_t m;
-#endif
     int j, vj;
     double x;
     int err = 0;
@@ -1828,31 +1807,29 @@ static int loop_print_update (LOOP_PRINT *lprn,
 	}
     }
 
-#ifdef ENABLE_GMP
     mpf_init(m);
-#endif
     
     for (j=0; j<list[0]; j++) {
+	if (lprn->na[j]) {
+	    continue;
+	}
 	vj = list[j+1];
 	x = gretl_scalar_get_value_by_index(vj);
-#ifdef ENABLE_GMP
+	if (na(x)) {
+	    lprn->na[j] = 1;
+	    continue;
+	}
 	mpf_set_d(m, x); 
 	mpf_add(lprn->sum[j], lprn->sum[j], m);
 	mpf_mul(m, m, m);
 	mpf_add(lprn->ssq[j], lprn->ssq[j], m);
-#else
-	lprn->sum[j] += x;
-	lprn->ssq[j] += x * x;
-#endif
 	if (!na(lprn->xbak[j]) && realdiff(x, lprn->xbak[j])) {
 	    lprn->diff[j] = 1;
 	}
 	lprn->xbak[j] = x;
     }
 
-#ifdef ENABLE_GMP
     mpf_clear(m);
-#endif
 
     lprn->n += 1;
 
@@ -2048,7 +2025,6 @@ static void print_loop_coeff (const DATAINFO *pdinfo,
 			      int i, PRN *prn)
 {
     char pname[VNAMELEN];
-#ifdef ENABLE_GMP
     mpf_t c1, c2, m, sd1, sd2;
     unsigned long ln = lmod->n;
 
@@ -2098,34 +2074,9 @@ static void print_loop_coeff (const DATAINFO *pdinfo,
     mpf_clear(m);
     mpf_clear(sd1);
     mpf_clear(sd2);
-#else /* non-GMP */
-    bigval m1, m2, sd1, sd2;
-    int n = lmod->n;
-
-    m1 = lmod->sum_coeff[i] / n;
-    if (lmod->cdiff[i] == 0) {
-	sd1 = 0.0;
-    } else {
-	sd1 = (lmod->ssq_coeff[i] - n * m1 * m1) / n;
-	sd1 = (sd1 <= 0.0)? 0.0 : sqrt((double) sd1);
-    }
-
-    m2 = lmod->sum_sderr[i] / n;
-    if (lmod->sdiff[i] == 0) {
-	sd2 = 0.0;
-    } else {
-	sd2 = (lmod->ssq_sderr[i] - n * m2 * m2) / n;
-	sd2 = (sd2 <= 0.0)? 0 : sqrt((double) sd2);
-    }
-
-    gretl_model_get_param_name(lmod->model0, pdinfo, i, pname);
-    pprintf(prn, "%*s", VNAMELEN - 1, pname);
-    pprintf(prn, "%#14g %#14g %#14g %#14g\n", (double) m1, (double) sd1, 
-	    (double) m2, (double) sd2);
-#endif
 }
 
-static void print_loop_model (LOOP_MODEL *lmod, const DATAINFO *pdinfo, 
+static void loop_model_print (LOOP_MODEL *lmod, const DATAINFO *pdinfo, 
 			      PRN *prn)
 {
     char startdate[OBSLEN], enddate[OBSLEN];
@@ -2157,8 +2108,8 @@ static void print_loop_model (LOOP_MODEL *lmod, const DATAINFO *pdinfo,
     pputc(prn, '\n');
 }
 
-static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo, 
-			    PRN *prn)
+static void loop_print_print (LOOP_PRINT *lprn, const DATAINFO *pdinfo, 
+			      PRN *prn)
 {
     bigval mean, m, sd;
     int i, vi, n;
@@ -2174,7 +2125,6 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
     pputs(prn, "    ");
     pputs(prn, _("   Variable     mean         std. dev.\n"));
 
-#ifdef ENABLE_GMP
     mpf_init(mean);
     mpf_init(m);
     mpf_init(sd);
@@ -2182,6 +2132,11 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
     for (i=0; i<lprn->list[0]; i++) {
 	vi = lprn->list[i+1];
 	s = gretl_scalar_get_name(vi);
+	if (lprn->na[i]) {
+	    pprintf(prn, "%*s", VNAMELEN - 1, s);
+	    pprintf(prn, "%14s %14s\n", "NA   ", "NA   ");
+	    continue;
+	}
 	mpf_div_ui(mean, lprn->sum[i], (unsigned long) n);
 	if (lprn->diff[i] == 0) {
 	    mpf_set_d(sd, 0.0);
@@ -2203,21 +2158,7 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
     mpf_clear(mean);
     mpf_clear(m);
     mpf_clear(sd);
-#else
-    for (i=0; i<lprn->list[0]; i++) {
-	vi = lprn->list[i+1];
-	s = gretl_scalar_get_name(vi);
-	mean = lprn->sum[i] / n;
-	if (lprn->diff[i] == 0) {
-	    sd = 0.0;
-	} else {
-	    m = (lprn->ssq[i] - n * mean * mean) / n;
-	    sd = (m < 0)? 0 : sqrt((double) m);
-	}
-	pprintf(prn, "%*s", VNAMELEN - 1, s);
-	pprintf(prn, "%#14g %#14g\n", (double) mean, (double) sd);
-    }
-#endif
+ 
     pputc(prn, '\n');
 }
 
@@ -2296,11 +2237,11 @@ static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo,
 
 	if (loop_is_progressive(loop)) {
 	    if (plain_model_ci(loop->ci[i]) && !(opt & OPT_Q)) {
-		print_loop_model(&loop->lmodels[j], pdinfo, prn);
+		loop_model_print(&loop->lmodels[j], pdinfo, prn);
 		loop_model_zero(&loop->lmodels[j]);
 		j++;
 	    } else if (loop->ci[i] == PRINT) {
-		print_loop_prn(&loop->prns[k], pdinfo, prn);
+		loop_print_print(&loop->prns[k], pdinfo, prn);
 		loop_print_zero(&loop->prns[k]);
 		k++;
 	    } else if (loop->ci[i] == STORE) {
@@ -2315,18 +2256,17 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 			const double **Z, const DATAINFO *pdinfo,
 			int *subst)
 {
-    char targ[VNAMELEN + 3] = {0};
-    char ins[32];
+    char ins[32], targ[VNAMELEN + 3] = {0};
     char *p, *pins;
-    int targlen;
-    double forval = 0;
-    int idx = 0;
+    int targlen, idx = 0;
     int err = 0;
 
 #if SUBST_DEBUG
     fprintf(stderr, "subst_dollar_targ:\n original: '%s'\n", str);
 #endif
 
+    /* construct the substitution target */
+
     if (loop->type == FOR_LOOP) {
 	if (!gretl_is_scalar(loop->init.vname)) {
 	    /* nothing to substitute */
@@ -2353,10 +2293,16 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 
     pins = ins;
 
-    /* prepare substitute */
+    /* prepare substitute string */
+
     if (loop->type == FOR_LOOP) {
-	forval = gretl_scalar_get_value(loop->init.vname);
-	/* the rest is handled below */
+	double x = gretl_scalar_get_value(loop->init.vname);
+
+	if (na(x)) {
+	    strcpy(ins, "NA");
+	} else {
+	    sprintf(ins, "%g", x);
+	}
     } else if (loop->type == INDEX_LOOP) {
 	sprintf(ins, "%d", idx);
     } else if (loop->type == DATED_LOOP) {
@@ -2365,34 +2311,18 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 	pins = loop->eachstrs[idx - 1];
     }  
 
-    while ((p = strstr(str, targ)) != NULL) {
+    while ((p = strstr(str, targ)) != NULL && !err) {
 	char *q = malloc(strlen(p));
 
 	if (q == NULL) {
-	    err = 1;
-	    break;
+	    err = E_ALLOC;
+	} else {
+	    strcpy(q, p + targlen);
+	    strcpy(p, pins);
+	    strcpy(p + strlen(pins), q);
+	    free(q);
+	    *subst = 1;
 	}
-
-	strcpy(q, p + targlen);
-
-	if (loop->type == FOR_LOOP) {
-	    sprintf(ins, "%g", forval);
-	    if (p - str > 0 && *(p - 1) == '[' && *(p + targlen) == ']') {
-		/* got an obs-type string, on the pattern [$lvar] */
-		int t = dateton(ins, pdinfo);
-
-		if (t < 0) {
-		    t = atoi(ins) - 1;
-		}
-		sprintf(ins, "%d", t);
-	    } 
-	} 
-
-	strcpy(p, pins);
-	strcpy(p + strlen(pins), q);
-	free(q);
-
-	*subst = 1;
     }
 
 #if SUBST_DEBUG
@@ -2458,13 +2388,13 @@ static int top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
     if (is_list_loop(loop)) {
 	err = loop_list_refresh(loop, pdinfo);
     } else if (loop->type == INDEX_LOOP) {
-	loop->init.val = controller_get_val(&loop->init);
+	loop->init.val = controller_get_val(&loop->init, pZ, pdinfo, &err);
     } else if (loop->type == FOR_LOOP) {
 	forloop_init(loop, pZ, pdinfo, &err);
     }
 
     if (!err && (loop->type == COUNT_LOOP || indexed_loop(loop))) {
-	loop->final.val = controller_get_val(&loop->final);
+	loop->final.val = controller_get_val(&loop->final, pZ, pdinfo, &err);
 	if (na(loop->init.val) || na(loop->final.val)) {
 	    gretl_errmsg_set(_("error evaluating loop condition"));
 	    fprintf(stderr, "loop: got NA for init and/or final value\n");
@@ -2703,6 +2633,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     LOOP_PRINT *lprn;
     char errline[MAXLINE];
     int indent0, subst, lrefresh;
+    int show_activity = 0;
     int j, err = 0;
 
     /* for the benefit of the caller: register the fact that execution
@@ -2727,6 +2658,8 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     if (err) {
 	*errline = '\0';
     }
+
+    show_activity = show_activity_func_installed();
     
     while (!err && loop_condition(loop, pZ, pdinfo, &err)) {
 #if LOOP_DEBUG
@@ -2921,6 +2854,9 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		/* added 2008-01-11, AC */
 		loop_list_refresh(loop, pdinfo);
 	    }
+	    if (show_activity && (loop->iter % 10 == 0)) {
+		show_activity_callback();
+	    }
 	}
 
     } /* end iterations of loop */
