@@ -1720,7 +1720,7 @@ static MODEL GNR (nlspec *spec, const double **Z, DATAINFO *pdinfo,
 	lsqopt |= OPT_R;
     }
 
-    gnr = lsq(glist, &gZ, gdinfo, OLS, lsqopt);
+    gnr = lsq(glist, gZ, gdinfo, OLS, lsqopt);
 
 #if NLS_DEBUG
     gnr.name = gretl_strdup("GNR for NLS");
@@ -2782,14 +2782,18 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 			    gretlopt opt, PRN *prn)
 {
     MODEL nlmod;
-    int origv = pdinfo->v;
-    int i, t, err = 0;
-
-#if NLS_DEBUG
-    fprintf(stderr, "real_nl_model: starting, pdinfo->v = %d\n", origv);
-#endif
+    int i, t, origv;
+    int err = 0;
 
     gretl_model_init(&nlmod);
+
+    if (pdinfo == NULL || pdinfo->v == 0) {
+	gretl_errmsg_set(_("No dataset is in place"));
+	nlmod.errcode = E_DATA;
+	return nlmod;
+    }
+
+    origv = pdinfo->v;
     gretl_model_smpl_init(&nlmod, pdinfo);
 
     if (spec == NULL) {
@@ -2886,6 +2890,9 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	err = mle_calculate(spec, prn);
     } else if (spec->ci == GMM) {
 	err = gmm_calculate(spec, prn);
+	if (err) {
+	    fprintf(stderr, "gmm_calculate returned %d\n", err);
+	}	
     } else {
 	/* NLS: invoke the appropriate minpack driver function */
 	if (numeric_mode(spec)) {
@@ -2918,12 +2925,8 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	    make_nl_model(&nlmod, spec, pdinfo);
 	}
     } else if (nlmod.errcode == 0) { 
-	/* error code missing: supply one */
-	if (spec->generr != 0) {
-	    nlmod.errcode = spec->generr;
-	} else {
-	    nlmod.errcode = E_NOCONV;
-	}
+	/* model error code missing */
+	nlmod.errcode = err;
     }
 
     /* ensure that the canonical parameter values get back 
@@ -3294,7 +3297,7 @@ MODEL ivreg_via_gmm (const int *list, double ***pZ,
 
     if (!err) {
 	/* OLS baseline and check */
-	olsmod = lsq(mlist, pZ, pdinfo, OLS, OPT_A | OPT_M | OPT_Z);
+	olsmod = lsq(mlist, *pZ, pdinfo, OLS, OPT_A | OPT_M | OPT_Z);
 	err = olsmod.errcode;
     }
 
