@@ -43,6 +43,7 @@
 			 c == FREQ ||		\
 			 c == KPSS ||		\
 			 c == MODTEST ||	\
+                         c == LEVINLIN ||       \
 			 c == LOOP ||		\
 			 c == NORMTEST ||	\
 			 c == OLS ||		\
@@ -99,14 +100,19 @@ struct gretl_option gretl_opts[] = {
     { ARBOND,   OPT_H, "orthdev", 0 },
     { ARBOND,   OPT_T, "two-step", 0 },
     { ARMA,     OPT_C, "conditional", 0 },
+    { ARMA,     OPT_E, "save-ehat", 0 },    
     { ARMA,     OPT_G, "opg", 0 },
     { ARMA,     OPT_H, "hessian", 0 },
     { ARMA,     OPT_L, "lbfgs", 0 },
     { ARMA,     OPT_N, "nc", 0 },    
-    { ARMA,     OPT_U, "continue", 0 },
     { ARMA,     OPT_V, "verbose", 0 },
     { ARMA,     OPT_X, "x-12-arima", 0 },
     { ARMA,     OPT_Y, "y-diff-only", 0 },
+    { BIPROBIT, OPT_G, "opg", 0 },
+    { BIPROBIT, OPT_Q, "quiet", 0 },
+    { BIPROBIT, OPT_R, "robust", 0 },
+    { BIPROBIT, OPT_V, "verbose", 0 },
+    { BIPROBIT, OPT_X, "save-xbeta", 0 },
     { BXPLOT,   OPT_O, "notches", 0 },
     { BXPLOT,   OPT_U, "output", 2 },
     { CHOW,     OPT_D, "dummy", 0 },
@@ -135,7 +141,12 @@ struct gretl_option gretl_opts[] = {
     { DIFFTEST, OPT_I, "signed-rank", 0 },
     { DIFFTEST, OPT_V, "verbose", 0 },
     { DISCRETE, OPT_R, "reverse", 0 },
+    { DPANEL,   OPT_A, "asymptotic", 0 },
+    { DPANEL,   OPT_D, "time-dummies", 0 },    
     { DPANEL,   OPT_L, "system", 0 },
+    { DPANEL,   OPT_T, "two-step", 0 },
+    { DPANEL,   OPT_V, "verbose", 0 },
+    { DPANEL,   OPT_X, "dpdstyle", 0 },
     { DUMMIFY,  OPT_F, "drop-first", 0 },
     { DUMMIFY,  OPT_L, "drop-last", 0 },
     { DURATION, OPT_W, "weibull", 0 },
@@ -169,6 +180,9 @@ struct gretl_option gretl_opts[] = {
     { FCAST,    OPT_U, "plot", 1 },
     { FOREIGN,  OPT_D, "send-data", 0 },
     { FOREIGN,  OPT_V, "verbose", 0 },
+    { FRACTINT, OPT_G, "gph", 0 },
+    { FRACTINT, OPT_A, "all", 0 },
+    { FRACTINT, OPT_Q, "quiet", 0 },
     { FREQ,     OPT_O, "gamma", 0 },
     { FREQ,     OPT_S, "silent", 0 },
     { FREQ,     OPT_Z, "normal", 0 },
@@ -179,13 +193,11 @@ struct gretl_option gretl_opts[] = {
     { GARCH,    OPT_F, "fcp", 0 }, 
     { GARCH,    OPT_N, "nc", 0 }, 
     { GARCH,    OPT_R, "robust", 0 },
-    { GARCH,    OPT_U, "continue", 0 },
     { GARCH,    OPT_V, "verbose", 0 },
     { GARCH,    OPT_Z, "stdresid", 0 },
     { GMM,      OPT_I, "iterate", 0 },
     { GMM,      OPT_L, "lbfgs", 0 },
     { GMM,      OPT_T, "two-step", 0 },
-    { GMM,      OPT_U, "continue", 0 },
     { GMM,      OPT_V, "verbose", 0 },
     { GNUPLOT,  OPT_D, "input", 2 },
     { GNUPLOT,  OPT_O, "with-lines", 0 },
@@ -202,6 +214,7 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_Y, "single-yaxis", 0 },
     { GNUPLOT,  OPT_X, "matrix", 2 },
     { HECKIT,   OPT_M, "ml", 0 },
+    { HECKIT,   OPT_R, "robust", 0 },
     { HECKIT,   OPT_T, "two-step", 0 },
     { HECKIT,   OPT_V, "verbose", 0 },
     { HELP,     OPT_F, "func", 0 },
@@ -220,6 +233,8 @@ struct gretl_option gretl_opts[] = {
     { KPSS,     OPT_V, "verbose", 0 },
     { KPSS,     OPT_F, "difference", 0 },
     { LEVERAGE, OPT_S, "save", 0 },
+    { LEVINLIN, OPT_N, "nc", 0 },
+    { LEVINLIN, OPT_T, "ct", 0 },
     { MODTEST,  OPT_A, "autocorr", 0 },
     { MODTEST,  OPT_B, "breusch-pagan", 0 },
     { MODTEST,  OPT_C, "comfac", 0 },
@@ -239,6 +254,7 @@ struct gretl_option gretl_opts[] = {
     { LOGIT,    OPT_V, "verbose", 0 },
     { LOOP,     OPT_P, "progressive", 0 },
     { LOOP,     OPT_V, "verbose", 0 },
+    { MAHAL,    OPT_Q, "quiet", 0 },
     { MAHAL,    OPT_S, "save", 0 },
     { MAHAL,    OPT_V, "vcv", 0 },
     { MEANTEST, OPT_O, "unequal-vars", 0 },
@@ -248,7 +264,6 @@ struct gretl_option gretl_opts[] = {
     { MLE,      OPT_L, "lbfgs", 0 },
     { MLE,      OPT_N, "numerical", 0 },
     { MLE,      OPT_R, "robust", 0 },
-    { MLE,      OPT_U, "continue", 0 },
     { MLE,      OPT_V, "verbose", 0 },
     { MODPRINT, OPT_A, "addstats", 2 },
     { MODPRINT, OPT_C, "csv", 0 },
@@ -264,7 +279,6 @@ struct gretl_option gretl_opts[] = {
     { NLS,      OPT_N, "numerical", 0 },
     { NLS,      OPT_O, "vcv", 0 },
     { NLS,      OPT_R, "robust", 0 },
-    { NLS,      OPT_U, "continue", 0 },
     { NLS,      OPT_V, "verbose", 0 },
     { NORMTEST, OPT_A, "all", 0 },
     { NORMTEST, OPT_D, "dhansen", 0 },
@@ -315,6 +329,8 @@ struct gretl_option gretl_opts[] = {
     { PCA,      OPT_O, "save", 0 },
     { PERGM,    OPT_O, "bartlett", 0 },
     { PERGM,    OPT_L, "log", 0 },
+    { PERGM,    OPT_R, "radians", 0 },
+    { PERGM,    OPT_D, "degrees", 0 },
     { PLOT,     OPT_O, "one-scale", 0 },
     { PLOT,     OPT_S, "time-series", 0 },
     { PLOT,     OPT_T, "tall", 0 },
@@ -390,6 +406,7 @@ struct gretl_option gretl_opts[] = {
     { VAR,      OPT_R, "robust", 0 }, 
     { VAR,      OPT_T, "trend", 0 }, 
     { VAR,      OPT_S, "lags", 2 },
+    { VAR,      OPT_X, "silent", 0 },
     { VARLIST,  OPT_S, "scalars", 0 },
     { VECM,     OPT_A, "crt", 0 },
     { VECM,     OPT_D, "seasonals", 0 },
@@ -399,6 +416,7 @@ struct gretl_option gretl_opts[] = {
     { VECM,     OPT_R, "rc", 0 },
     { VECM,     OPT_T, "ct", 0 },
     { VECM,     OPT_V, "verbose", 0 },
+    { VECM,     OPT_X, "silent", 0 },
     { WLS,      OPT_R, "robust", 0 },    
     { XTAB,     OPT_C, "column", 0 },
     { XTAB,     OPT_M, "matrix", 2 },
