@@ -53,7 +53,6 @@ enum {
     P_GNUMERIC_IMPORT,
     P_ODS_IMPORT,
     P_JOHANSEN,
-    P_LAD,
     P_VIF,
     P_LEVERAGE,
     P_MP_OLS,
@@ -86,7 +85,10 @@ enum {
     P_QUANTREG,
     P_INTREG,
     P_ANOVA,
-    P_DURATION
+    P_DURATION,
+    P_INTERPOLATE,
+    P_BIPROBIT,
+    P_PANURC
 } plugin_codes;
 
 struct plugin_info {
@@ -105,7 +107,6 @@ struct plugin_info plugins[] = {
     { P_GNUMERIC_IMPORT, "gnumeric_import" },
     { P_ODS_IMPORT,      "ods_import" },
     { P_JOHANSEN,        "johansen" },
-    { P_LAD,             "lad" },
     { P_VIF,             "vif" },
     { P_LEVERAGE,        "leverage" },
     { P_MP_OLS,          "mp_ols" },
@@ -138,7 +139,10 @@ struct plugin_info plugins[] = {
     { P_QUANTREG,        "quantreg" },
     { P_INTREG,          "interval" },
     { P_ANOVA,           "anova" },
-    { P_DURATION,        "duration" }
+    { P_DURATION,        "duration" },
+    { P_INTERPOLATE,     "interpolate" },
+    { P_BIPROBIT,        "biprobit" },
+    { P_PANURC,          "panurc" }
 };  
 
 struct plugin_function plugin_functions[] = { 
@@ -158,9 +162,6 @@ struct plugin_function plugin_functions[] = {
     { "johansen_boot_round",   P_JOHANSEN },
     { "vecm_test_restriction", P_JOHANSEN },
 
-    /* least absolute deviations */
-    { "lad_driver", P_LAD },
-
     /* influential observations */
     { "model_leverage",       P_LEVERAGE },
     { "leverage_data_dialog", P_LEVERAGE },
@@ -175,6 +176,7 @@ struct plugin_function plugin_functions[] = {
 #ifdef HAVE_MPFR
     { "mp_vector_ln",             P_MP_OLS },
 #endif
+    { "mp_bw_filter",             P_MP_OLS },
 
     /* principal components analysis */
     { "pca_from_cmatrix", P_PCA },
@@ -186,9 +188,12 @@ struct plugin_function plugin_functions[] = {
     { "range_mean_graph", P_RANGE_MEAN },
 
     /* statistical tables */
-    { "dw_lookup",         P_STATS_TABLES },
-    { "rank_sum_lookup",   P_STATS_TABLES },
-    { "stock_yogo_lookup", P_STATS_TABLES },
+    { "dw_lookup",            P_STATS_TABLES },
+    { "rank_sum_lookup",      P_STATS_TABLES },
+    { "stock_yogo_lookup",    P_STATS_TABLES },
+    { "get_IPS_critvals",     P_STATS_TABLES },
+    { "IPS_tbar_moments",     P_STATS_TABLES },
+    { "IPS_tbar_rho_moments", P_STATS_TABLES },
 
     /* SUR, 3SLS, FIML */
     { "system_estimate", P_SYSEST },
@@ -208,6 +213,7 @@ struct plugin_function plugin_functions[] = {
     { "count_data_estimate", P_POISSON },
     { "heckit_estimate",   P_HECKIT },
     { "interval_estimate", P_INTREG },
+    { "biprobit_estimate", P_BIPROBIT },
 
     /* audio graphs etc */
     { "midi_play_graph",   P_AUDIO },
@@ -241,7 +247,8 @@ struct plugin_function plugin_functions[] = {
     { "gretl_odbc_get_data",  P_ODBC},
 
     /* quantreg */
-    { "rq_driver", P_QUANTREG},
+    { "rq_driver",  P_QUANTREG},
+    { "lad_driver", P_QUANTREG},
 
     /* analysis of variance */
     { "gretl_anova", P_ANOVA},
@@ -249,6 +256,12 @@ struct plugin_function plugin_functions[] = {
     /* duration models */
     { "duration_estimate", P_DURATION},
 
+    /* data interpolation */
+    { "chow_lin_interpolate", P_INTERPOLATE},
+
+    /* panel unit roots/cointegration */
+    { "real_levin_lin", P_PANURC},
+
     /* sentinel */
     { NULL, 0 }
 };
@@ -400,6 +413,41 @@ void *get_plugin_function (const char *funcname, void **handle)
     return funp;
 }
 
+void *get_packaged_C_function (const char *pkgname,
+			       const char *funcname, 
+			       void **handle)
+{
+    void *funp;
+
+    *handle = get_plugin_handle(pkgname);
+    if (*handle == NULL) {
+	return NULL;
+    }
+
+#ifdef WIN32
+    funp = GetProcAddress(*handle, funcname);
+#else
+    funp = dlsym(*handle, funcname);
+    if (funp == NULL) {
+	char munged[64];
+
+	sprintf(munged, "_%s", funcname);
+	funp = dlsym(*handle, munged);
+	if (funp == NULL) {
+	    fprintf(stderr, "%s\n", dlerror());
+	}
+    }
+#endif   
+
+    if (funp == NULL) {
+	gretl_errmsg_set(_("Couldn't load plugin function"));
+	close_plugin(*handle);
+	*handle = NULL;
+    }
+
+    return funp;
+}
+
 /**
  * close_plugin:
  * @handle: pointer obtained via the handle argument to
