@@ -26,6 +26,8 @@
 #include "gretl_func.h"
 #include "matrix_extra.h"
 #include "gretl_scalar.h"
+#include "gretl_bundle.h"
+#include "gretl_string_table.h"
 
 #include <time.h>
 #include <glib.h>
@@ -1709,6 +1711,8 @@ static void print_listed_objects (const char *s,
 {
     const gretl_matrix *m;
     const int *list;
+    const char *p;
+    gretl_bundle *b;
     char *name;
 
     while ((name = gretl_word_strdup(s, &s)) != NULL) {
@@ -1718,6 +1722,10 @@ static void print_listed_objects (const char *s,
 	    gretl_matrix_print_to_prn(m, name, prn);
 	} else if ((list = get_list_by_name(name)) != NULL) {
 	    print_varlist(name, list, pdinfo, prn);
+	} else if ((b = get_gretl_bundle_by_name(name)) != NULL) {
+	    gretl_bundle_print(b, prn);
+	} else if ((p = get_string_by_name(name)) != NULL) {
+	    pprintf(prn, "%s\n", p);	
 	} 
 	    
 	free(name);
@@ -1921,11 +1929,11 @@ static int print_by_var (const int *list, const double **Z,
 /**
  * printdata:
  * @list: list of variables to print.
- * @mstr: optional string holding names of matrices to print.
+ * @mstr: optional string holding names of non-series objects to print.
  * @Z: data matrix.
  * @pdinfo: data information struct.
- * @opt: if %OPT_O, print the data by observation (series in columns);
- * if %OPT_N, use simple obs numbers, not dates.
+ * @opt: if OPT_O, print the data by observation (series in columns);
+ * if OPT_N, use simple obs numbers, not dates.
  * @prn: gretl printing struct.
  *
  * Print the data for the variables in @list over the currently
@@ -2681,7 +2689,10 @@ print_iter_info (int iter, double crit, int type, int k,
 	x += fabs(b[i] * g[i]);
 	print_iter_val(g[i], i, k, prn);
     }
-    pprintf(prn, " (norm %.2e)\n\n", sqrt(x/k));
+    pprintf(prn, " (norm %.2e)\n", sqrt(x/k));
+    if (iter >= 0) {
+	pputc(prn, '\n');
+    }
 
     if (iter < 0 || (iter % 20 == 0)) {
 	/* experimental */
