@@ -206,6 +206,23 @@ double binomial_pmf (double p, int n, int k)
     return pm;
 }
 
+static int binomial_pmf_array (double p, int n, double *x, int T)
+{
+    double pm;
+    int i, k;
+
+    for (i=0; i<T; i++) {
+	k = x[i];
+	pm = binomial_cdf(p, n, k);
+	if (k > 0 && !na(pm)) {
+	    pm -= binomial_cdf(p, n, k - 1);
+	}
+	x[i] = pm;
+    }
+
+    return 0;
+}
+
 /**
  * x_factorial:
  * @x: input value.
@@ -456,6 +473,31 @@ double normal_cdf_comp (double x)
     return p;
 }
 
+/**
+ * student_pvalue_1:
+ * @df: degrees of freedom.
+ * @x: the cutoff point in the distribution.
+ * 
+ * Returns: the probability that t(@df) is greater than @x,
+ * or #NADBL on failure.
+ */
+
+double student_pvalue_1 (int df, double x)
+{
+    double p = NADBL;
+
+    if (df > 0) {
+	p = stdtr(df, x);
+	if (get_cephes_errno()) {
+	    p = NADBL;
+	} else {
+	    p = 1 - p;
+	}
+    }
+
+    return p;
+}
+
 /**
  * student_pvalue_2:
  * @df: degrees of freedom.
@@ -1424,6 +1466,40 @@ static double poisson_cdf_comp (double lambda, int k)
     return x;
 }
 
+static int poisson_pmf_array (double lambda, double *x, int n)
+{
+    double den, l0, p;
+    int i, j, k;
+
+    if (lambda <= 0) {
+	for (i=0; i<n; i++) {
+	    x[i] = NADBL;
+	}
+	return 0;
+    }
+
+    l0 = exp(-lambda);
+
+    for (i=0; i<n; i++) {
+	k = x[i];
+	den = x_factorial((double) k);
+	if (xna(den)) {
+	    p = NADBL;
+	} else {
+	    p = l0 * pow(lambda, (double) k) / den;
+	}
+	if (xna(p)) {
+	    p = l0;
+	    for (j=1; j<=k; j++) {
+		p *= lambda / j;
+	    }
+	}
+	x[i] = p;
+    }	    
+
+    return 0;
+}
+
 /**
  * poisson_pmf:
  * @lambda: mean (also variance).
@@ -1444,13 +1520,13 @@ double poisson_pmf (double lambda, int k)
     den = x_factorial((double) k);
     l0 = exp(-lambda);
 
-    if (na(den) || isinf(den) || isnan(den)) {
+    if (xna(den)) {
 	p = NADBL;
     } else {
 	p = l0 * pow(lambda, (double) k) / den;
     }
 
-    if (na(p) || isinf(p) || isnan(p)) {
+    if (xna(p)) {
 	int i;
 
 	p = l0;
@@ -1928,6 +2004,10 @@ double gretl_get_pdf (char st, const double *parm, double x)
 	y = snedecor_pdf((int) parm[0], (int) parm[1], x);
     } else if (st == 'G') {
 	y = gamma_pdf(parm[0], parm[1], x);
+    } else if (st == 'B') {
+	y = binomial_pmf(parm[0], parm[1], x);
+    } else if (st == 'P') {
+	y = poisson_pmf(parm[0], x);
     } else if (st == 'W') {
 	y = weibull_pdf(parm[0], parm[1], x);
     } else if (st == 'E') {
@@ -1971,6 +2051,10 @@ int gretl_fill_pdf_array (char st, const double *parm,
 	err = snedecor_pdf_array((int) parm[0], (int) parm[1], x, n);
     } else if (st == 'G') {
 	err = gamma_pdf_array(parm[0], parm[1], x, n); 
+    } else if (st == 'B') {
+	err = binomial_pmf_array(parm[0], parm[1], x, n);
+    } else if (st == 'P') {
+	err = poisson_pmf_array(parm[0], x, n);
     } else if (st == 'W') {
 	err = weibull_pdf_array(parm[0], parm[1], x, n);
     } else if (st == 'E') {
@@ -2182,6 +2266,18 @@ double *gretl_get_random_series (char st, const double *parm,
 	} else {
 	    *err = gretl_rand_weibull(x, t1, t2, shape, scale);
 	}
+    } else if (st == 'E') {
+	/* GED */ 
+	double nu = parm[0];
+
+	if (serp1 != NULL) {
+	    for (t=t1; t<=t2 && !*err; t++) {
+		if (serp1 != NULL) nu = serp1[t];
+		*err = gretl_rand_GED(x, t, t, nu);
+	    }
+	} else {
+	    *err = gretl_rand_GED(x, t1, t2, nu);
+	}
     }	
 
     return x;
