@@ -45,6 +45,8 @@
 static GRand *gretl_rand;
 static unsigned int useed;
 
+static guint32 gretl_rand_octet (guint32 *sign);
+
 /**
  * gretl_rand_init:
  *
@@ -97,6 +99,7 @@ void gretl_rand_set_seed (unsigned int seed)
 {
     useed = (seed == 0)? time(NULL) : seed;
     g_rand_set_seed(gretl_rand, useed);
+    gretl_rand_octet(NULL);
 }
 
 static double gretl_rand_01 (void)
@@ -249,6 +252,11 @@ union wraprand {
 /* Split a 32-bit random value from GLib into 4 octets: each octet
    provides a 7-bit value for indexing into the Ziggurat plus a
    sign bit.  Load a new guint32 when the material is exhausted.
+
+   Passing NULL in place of the @sign pointer is a signal to
+   re-initialize; we do this when the user sets the seed, to
+   ensure that after re-setting the same seed one gets the
+   same sequence of pseudo-random values.
 */
 
 static guint32 gretl_rand_octet (guint32 *sign)
@@ -256,6 +264,11 @@ static guint32 gretl_rand_octet (guint32 *sign)
     static union wraprand wr;
     static int i;
 
+    if (sign == NULL) {
+	i = 0;
+	return 0;
+    }
+
     if (i == 0) {
 	wr.u = g_rand_int(gretl_rand);
 	i = 4;
@@ -441,8 +454,9 @@ double gretl_one_snormal (void)
  *
  * Fill the selected range of array @a with pseudo-random drawings
  * from the normal distribution with the given mean and standard
- * deviation, using the Mersenne Twister for uniform input and the 
- * Box-Muller method for converting to the normal distribution.
+ * deviation, using the Mersenne Twister for uniform input and  
+ * either the Ziggurat or the Box-Muller method for converting to 
+ * the normal distribution.
  *
  * Returns: 0 on success, 1 on invalid input.
  */
@@ -911,12 +925,51 @@ int gretl_rand_weibull (double *a, int t1, int t2, double shape,
     return err;
 }
 
+/**
+ * gretl_rand_GED:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @nu: shape parameter > 0.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the GED distribution with shape @nu. We exploit the fact that
+ * if x ~ GED(n), then |x/k|^n is a Gamma rv.
+ *
+ * Returns: 0 on success, non-zero if @nu is out of bounds.
+ */
+
+int gretl_rand_GED (double *a, int t1, int t2, double nu)
+{
+    int err, t;
+    double p, scale;
+
+    if (nu < 0) {
+	return E_INVARG;
+    }
+
+    p = 1.0/nu;
+    scale = pow(0.5, p) * sqrt(gamma_function(p) / gamma_function(3.0*p));
+    err = gretl_rand_gamma(a, t1, t2, p, 2);
+
+    if (!err) {
+	for (t=t1; t<=t2; t++) {
+	    a[t] = scale * pow(a[t], p);
+	    if (gretl_rand_01() < 0.5) {
+		a[t] = -a[t];
+	    }
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_rand_int_max:
  * @max: the maximum value (open)
  *
- * Returns: a pseudo-random unsigned int in the interval [0, @max)
- * using the Mersenne Twister.
+ * Returns: a pseudo-random unsigned int in the interval 
+ * [0, max-1] using the Mersenne Twister.
  */
 
 unsigned int gretl_rand_int_max (unsigned int max)
@@ -937,5 +990,3 @@ unsigned int gretl_rand_int (void)
 }
 
 
-
-
