@@ -49,7 +49,8 @@ int string_is_blank (const char *s)
 
     if (s != NULL) {
 	while (*s) {
-	    if (!isspace((unsigned char) *s) && *s != CTRLZ) {
+	    if (!isspace((unsigned char) *s) && 
+		*s != '\r' && *s != CTRLZ) {
 		ret = 0;
 		break;
 	    }
@@ -200,6 +201,46 @@ char *gretl_delete (char *str, int idx, int count)
     return str;
 }
 
+/**
+ * gretl_unquote:
+ * @str: the string to process.
+ * @err: location to receive error code.
+ *
+ * If @str begins with the ASCII double-quote character, checks
+ * that the last character is also a double-quote, and in that
+ * case trims the quotes from both ends. If the first character
+ * is a double quote but the last is not, flags an error. If
+ * the string is not quoted at all, returns the original
+ * string.
+ *
+ * Returns: the input string, possibly modified in place.
+ */
+
+char *gretl_unquote (char *str, int *err)
+{
+    *err = 0;
+
+    if (*str == '"') {
+	int n = strlen(str);
+
+	if (n > 1) {
+	    if (str[n-1] == '"') {
+		str[n-1] = '\0';
+	    } else {
+		*err = E_PARSE;
+	    }
+	} else {
+	    *err = E_PARSE;
+	}
+
+	if (!*err) {
+	    shift_string_left(str, 1);
+	}
+    }
+
+    return str;
+}
+
 /**
  * charpos:
  * @c: the character to look for.
@@ -318,7 +359,7 @@ int has_suffix (const char *str, const char *sfx)
     int ret = 0;
 
     if (str != NULL && sfx != NULL) {
-	p = strrchr(str, '.');
+	p = strrchr(str, *sfx);
 	if (p != NULL && strlen(p) == strlen(sfx)) {
 	    ret = 1;
 	    while (*p) {
@@ -508,24 +549,35 @@ char *gretl_strndup (const char *src, size_t n)
 char *gretl_strdup_printf (const char *format, ...)
 {
     va_list args;
-    int plen, bsize = 2048;
-    char *buf;
+    char *buf = NULL;
+    int len;
+
+#ifdef HAVE_VASPRINTF
+    va_start(args, format);
+    len = vasprintf(&buf, format, args);
+    va_end(args);
+    if (len < 0) {
+	buf = NULL;
+    }
+#else 
+    int bsize = 2048;
 
     buf = malloc(bsize);
     if (buf == NULL) {
 	return NULL;
     }
 
-    memset(buf, 0, 1);
+    memset(buf, 0, 1); 
 
     va_start(args, format);
-    plen = vsnprintf(buf, bsize, format, args);
+    len = vsnprintf(buf, bsize, format, args);
     va_end(args);
 
-    if (plen >= bsize) {
+    if (len >= bsize) {
 	fputs("gretl_strdup_printf warning: string was truncated\n",
 	      stderr);
     }
+#endif
 
     return buf;
 }
@@ -1902,7 +1954,7 @@ char *build_path (char *targ, const char *dirname, const char *fname,
     strcat(targ, dirname);
     len = strlen(targ);
     if (len == 0) {
-	return NULL;
+	return targ;
     }
 
     /* strip a trailing single dot */
