@@ -48,8 +48,23 @@ const char *tex_column_format (int i)
 
 void set_tex_use_pdf (const char *prog)
 {
-    use_pdf = strstr(prog, "pdf") != NULL ||
-	strstr(prog, "PDF") != NULL;
+    const char *p = strrchr(prog, SLASH);
+    char test[4];
+
+    /* looking for "pdflatex", possibly preceded by
+       an absolute path */
+
+    p = (p == NULL)? prog : p + 1;
+
+    *test = '\0';
+    strncat(test, p, 3);
+    lower(test);
+    use_pdf = !strcmp(test, "pdf");
+}
+
+int get_tex_use_pdf (void)
+{
+    return use_pdf;
 }
 
 static const char *tex_greek_var (const char *s)
@@ -1116,37 +1131,44 @@ static PRN *make_rtf_prn (int ID, char *fname, int *err)
 
 static char tex_preamble_file[MAXLEN];
 
-static const char *get_gretltex_local (void)
+static const char *get_gretltex_local (char *local)
 {
-    static char localtex[32] = {0};
     char *lang = getenv("LANG");
 
+    *local = '\0';
+
     if (lang != NULL) {
 	char lstr[3] = {0};
 
 	strncat(lstr, lang, 2);
-	sprintf(localtex, "gretlpre_%s.tex", lstr);
+	sprintf(local, "gretlpre_%s.tex", lstr);
     }
 
-    return localtex;
+    return local;
 }
 
 void set_gretl_tex_preamble (void)
 {
     const char *wdir = gretl_workdir();
-    char test[MAXLEN];
+    char local[16], test[MAXLEN];
     FILE *fp;
     int gotit = 0;
 
-    /* localized preamble file in working dir */
-    sprintf(test, "%s%s", wdir, get_gretltex_local());
-    fp = gretl_fopen(test, "r");
-    if (fp != NULL) {
-	strcpy(tex_preamble_file, test);
-	fclose(fp);
-	gotit = 1;
-    } else {
-	/* regular preamble file in working dir */
+    get_gretltex_local(local);
+
+    /* localized preamble file in working dir? */
+    if (*local != '\0') {
+	sprintf(test, "%s%s", wdir, local);
+	fp = gretl_fopen(test, "r");
+	if (fp != NULL) {
+	    strcpy(tex_preamble_file, test);
+	    fclose(fp);
+	    gotit = 1;
+	}
+    }
+
+    if (!gotit) {
+	/* regular preamble file in working dir? */
 	sprintf(test, "%sgretlpre.tex", wdir);
 	fp = gretl_fopen(test, "r");
 	if (fp != NULL) {
@@ -1157,17 +1179,21 @@ void set_gretl_tex_preamble (void)
     }
 
     if (!gotit) {
-	gchar *ddir = gretl_default_workdir();
+	const char *ddir = gretl_default_workdir();
 
 	if (ddir != NULL) {
-	    /* localized preamble file in standard working dir */
-	    sprintf(test, "%s%s", ddir, get_gretltex_local());
-	    fp = gretl_fopen(test, "r");
-	    if (fp != NULL) {
-		strcpy(tex_preamble_file, test);
-		fclose(fp);
-		gotit = 1;
-	    } else {
+	    /* localized preamble file in standard working dir? */
+	    if (*local != '\0') {
+		sprintf(test, "%s%s", ddir, local);
+		fp = gretl_fopen(test, "r");
+		if (fp != NULL) {
+		    strcpy(tex_preamble_file, test);
+		    fclose(fp);
+		    gotit = 1;
+		}
+	    }
+
+	    if (!gotit) {
 		/* regular preamble file in standard working dir */
 		sprintf(test, "%sgretlpre.tex", ddir);
 		fp = gretl_fopen(test, "r");
@@ -1177,9 +1203,8 @@ void set_gretl_tex_preamble (void)
 		    gotit = 1;
 		}
 	    }
-	    g_free(ddir);
 	}
-    }    
+    } 
 
     gretl_error_clear();
 }
@@ -1228,7 +1253,7 @@ void gretl_tex_preamble (PRN *prn, int fmt)
     FILE *fp = NULL;
     int userfile = 0;
 
-    if (tex_preamble_file[0] != '\0') {
+    if (*tex_preamble_file != '\0') {
 	fp = gretl_fopen(tex_preamble_file, "r");
 	if (fp != NULL) {
 	    char line[256];
@@ -1244,6 +1269,8 @@ void gretl_tex_preamble (PRN *prn, int fmt)
 	    }
 	    userfile = 1;
 	    fclose(fp);
+	    fprintf(stderr, "gretltex: using preamble file\n %s\n",
+		    tex_preamble_file);
 	}
     }
 
