@@ -431,12 +431,12 @@ static int fill_E_matrix (gretl_matrix *E, MODEL *pmod,
 
 	elist[1] = vi;
 
-	if (pmod->missmask != NULL) {
+	if (model_has_missing_obs(pmod)) {
 	    set_reference_missmask_from_model(pmod);
 	}
 
 	/* regress the given endogenous var on all the instruments */
-	emod = lsq(elist, pZ, pdinfo, OLS, OPT_A);
+	emod = lsq(elist, *pZ, pdinfo, OLS, OPT_A);
 	if ((err = emod.errcode)) {
 	    clear_model(&emod);
 	    break;
@@ -567,7 +567,7 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
 	OT_list[i] = instlist[i-1];
     }
 
-    smod = lsq(OT_list, pZ, pdinfo, OLS, OPT_A);
+    smod = lsq(OT_list, *pZ, pdinfo, OLS, OPT_A);
     if (smod.errcode) {
 	fprintf(stderr, "ivreg_sargan_test: smod.errcode = %d\n", smod.errcode);
 	err = smod.errcode;
@@ -624,7 +624,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     } 
 
     /* estimate the unrestricted model */
-    hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A);
+    hmod = lsq(HT_list, *pZ, pdinfo, OLS, OPT_A);
     if (hmod.errcode) {
 	fprintf(stderr, "tsls_hausman_test: hmod.errcode (U) = %d\n", hmod.errcode);
 	err = hmod.errcode;
@@ -632,10 +632,15 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     } 
 
 #if HTDBG
-    pprintf(dbgprn, "Hausman: unrestricted model\n");
+    pprintf(dbgprn, "Hausman: unrestricted model (df = %d)\n", hmod.dfd);
     printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
 #endif
 
+    if (hmod.dfd == 0) {
+	/* perfect fit, can't do test */
+	goto bailout;
+    }
+
     URSS = hmod.ess;
     nobs1 = hmod.nobs;
     ku = hmod.ncoeff;
@@ -659,7 +664,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     /* regress the U fitted values on the regressors of the restricted 
        model (so the sum of squares equals RRSS-URSS)
     */
-    hmod = lsq(HT_list, pZ, pdinfo, OLS, OPT_A);
+    hmod = lsq(HT_list, *pZ, pdinfo, OLS, OPT_A);
 
 #if HTDBG
     printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
@@ -727,6 +732,15 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 
     k = gretl_matrix_cols(Q);
 
+    if (k > Q->rows) {
+	/* can't do QR decomp! */
+	gretl_errmsg_sprintf(_("Number of instruments (%d) exceeds the "
+			       "number of observations (%d)"),
+			     k, Q->rows);
+	*err = E_DF;
+	goto bailout;
+    }
+
     R = gretl_matrix_alloc(k, k);
     if (R == NULL) {
 	*err = E_ALLOC;
@@ -922,7 +936,7 @@ static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
     pmod->criterion[C_BIC] = NADBL;
     pmod->criterion[C_HQC] = NADBL;
     
-    if (dataset_is_time_series(pdinfo) && pmod->missmask == NULL) {
+    if (dataset_is_time_series(pdinfo) && !model_has_missing_obs(pmod)) {
 	/* time series, no missing obs within sample range */
 	pmod->rho = rhohat(1, pmod->t1, pmod->t2, pmod->uhat);
 	pmod->dw = dwstat(1, pmod, Z);
@@ -1142,6 +1156,7 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
 	double rc = gretl_symmetric_matrix_rcond(S, &err);
 
 	if (!err && (na(rc) || rc < 1.0e-6)) {
+	    fprintf(stderr, "Stock-Yogo: rcond(S) = %g\n", rc);
 	    err = E_SINGULAR;
 	}
     }
@@ -1242,7 +1257,7 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
     if (!err) {
 	gretlopt myopt = (opt & OPT_R)? (OPT_A | OPT_R) : OPT_A;
 
-	mod1 = lsq(list1, pZ, pdinfo, OLS, myopt);
+	mod1 = lsq(list1, *pZ, pdinfo, OLS, myopt);
 	err = mod1.errcode;
     }
 
@@ -1402,7 +1417,7 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
     } else {
 	for (i=1; i<=zlist[0]; i++) {
 	    if (zlist[i] == list[1]) {
-		gretl_errmsg_set("You can't use the dependent variable as an instrument");
+		gretl_errmsg_set(_("You can't use the dependent variable as an instrument"));
 		err = E_DATA;
 		break;
 	    }
@@ -1597,7 +1612,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* second-stage regression */
-    tsls = lsq(s2list, pZ, pdinfo, OLS, (sysest)? OPT_Z : OPT_NONE);
+    tsls = lsq(s2list, *pZ, pdinfo, OLS, (sysest)? OPT_Z : OPT_NONE);
     if (tsls.errcode) {
 	fprintf(stderr, "tsls, stage 2: tsls.errcode = %d\n", tsls.errcode);
 	goto bailout;
