@@ -17,6 +17,8 @@
  * 
  */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_matrix.h"
 #include "matrix_extra.h"
@@ -25,13 +27,20 @@
 #include "usermat.h"
 #include "gretl_xml.h"
 #include "genparse.h"
+#include "gretl_bundle.h"
 
 #define MDEBUG 0
 #define LEVEL_AUTO -1
 
+typedef enum {
+    UM_PRIVATE = 1 << 0,
+    UM_SHELL   = 1 << 1
+} UMFlags;
+
 struct user_matrix_ {
     gretl_matrix *M;
     int level;
+    UMFlags flags;
     char name[VNAMELEN];
     char **colnames;
 };
@@ -39,6 +48,9 @@ struct user_matrix_ {
 static user_matrix **matrices;
 static int n_matrices;
 
+#define matrix_is_private(u) ((u->flags & UM_PRIVATE) || *u->name == '$')
+#define matrix_is_shell(u) (u->flags & UM_SHELL)
+
 #if MDEBUG > 1
 static void print_matrix_stack (const char *msg)
 {
@@ -74,15 +86,15 @@ const char *get_matrix_name_by_index (int idx)
 
 static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
 {
-    user_matrix *u;
+    user_matrix *u = malloc(sizeof *u);
 
-    u = malloc(sizeof *u);
     if (u == NULL) {
 	return NULL;
     }
 
     u->M = M;
     u->level = gretl_function_depth();
+    u->flags = 0;
     *u->name = '\0';
     strncat(u->name, name, VNAMELEN - 1);
     u->colnames = NULL;
@@ -108,29 +120,35 @@ static int matrix_is_user_matrix (const gretl_matrix *m)
 
 static void (*matrix_add_callback)(void);
 
+/**
+ * set_matrix_add_callback:
+ * @callback: function function to out in place.
+ *
+ * Sets the callback function to be invoked when a user-defined
+ * matrix is added to the stack of saved objects.  Intended
+ * for synchronizing the GUI program with the saved object
+ * state.
+ */
+
 void set_matrix_add_callback (void (*callback))
 {
     matrix_add_callback = callback; 
 }
 
-static int real_user_matrix_add (gretl_matrix *M, const char *name,
-				 int callback_ok)
+static user_matrix *real_user_matrix_add (gretl_matrix *M, const char *name,
+					  int callback_ok)
 {
     user_matrix **tmp;
+    user_matrix *u;
+    int n = n_matrices;
 
     if (M == NULL) {
 	return 0;
     }
 
-#if 0
-    if (check_varname(name)) {
-	return E_DATA;
-    }
-#endif
-
-    tmp = realloc(matrices, (n_matrices + 1) * sizeof *tmp);
+    tmp = realloc(matrices, (n + 1) * sizeof *tmp);
     if (tmp == NULL) {
-	return E_ALLOC;
+	return NULL;
     } else {
 	matrices = tmp;
     }
@@ -140,42 +158,50 @@ static int real_user_matrix_add (gretl_matrix *M, const char *name,
 	gretl_matrix *Mcpy = gretl_matrix_copy(M);
 
 	if (Mcpy == NULL) {
-	    return E_ALLOC;
+	    return NULL;
 	}
 	M = Mcpy;
     }
 
-    matrices[n_matrices] = user_matrix_new(M, name);
+    matrices[n] = u = user_matrix_new(M, name);
 
-    if (matrices[n_matrices] == NULL) {
-	return E_ALLOC;
+    if (matrices[n] == NULL) {
+	return NULL;
     }
 
     n_matrices++;
 
 #if MDEBUG
     fprintf(stderr, "add_user_matrix: allocated '%s' at %p (M at %p, %dx%d)\n",
-	    name, (void *) matrices[n_matrices-1], (void *) M,
+	    name, (void *) matrices[n], (void *) M,
 	    gretl_matrix_rows(M), gretl_matrix_cols(M));
 #endif
 
-    if (callback_ok && 
-	matrix_add_callback != NULL && 
+    if (callback_ok && matrix_add_callback != NULL && 
 	gretl_function_depth() == 0) {
 	(*matrix_add_callback)();
     }
 
-    return 0;
+    return u;
 }
 
 int user_matrix_add (gretl_matrix *M, const char *name)
 {
-    return real_user_matrix_add(M, name, 1);
+    user_matrix *u = real_user_matrix_add(M, name, 1);
+
+    return (u == NULL)? E_ALLOC : 0;
 }
 
-int matrix_copy_add (gretl_matrix *M, const char *name)
+int private_matrix_add (gretl_matrix *M, const char *name)
 {
-    return real_user_matrix_add(M, name, 0);
+    user_matrix *u = real_user_matrix_add(M, name, 0);
+
+    if (u == NULL) {
+	return E_ALLOC;
+    } else {
+	u->flags |= UM_PRIVATE;
+	return 0;
+    }
 }
 
 static int 
@@ -196,25 +222,23 @@ matrix_insert_diagonal (gretl_matrix *M, const gretl_matrix *S,
     return 0;
 }
 
-/* If slevel is LEVEL_AUTO, search at the current function execution
+/* If level is LEVEL_AUTO, search at the current function execution
    depth, otherwise search at the function execution depth given by
    slevel.
 */
 
-static gretl_matrix *
-real_get_matrix_by_name (const char *name, int slevel)
+static gretl_matrix *real_get_matrix_by_name (const char *name, 
+					      int level)
 {
-    int level, i;
+    int i;
 
-    if (slevel == LEVEL_AUTO) {
+    if (level == LEVEL_AUTO) {
 	level = gretl_function_depth();
-    } else {
-	level = slevel;
-    }
+    } 
 
     for (i=0; i<n_matrices; i++) {
-	if (!strcmp(name, matrices[i]->name) &&
-	    matrices[i]->level == level) {
+	if (matrices[i]->level == level && 
+	    !strcmp(name, matrices[i]->name)) {
 	    return matrices[i]->M;
 	}
     }
@@ -224,12 +248,11 @@ real_get_matrix_by_name (const char *name, int slevel)
 
 user_matrix *get_user_matrix_by_name (const char *name)
 {
-    int level = gretl_function_depth();
-    int i;
+    int i, level = gretl_function_depth();
 
     for (i=0; i<n_matrices; i++) {
-	if (!strcmp(name, matrices[i]->name) &&
-	    matrices[i]->level == level) {
+	if (matrices[i]->level == level && 
+	    !strcmp(name, matrices[i]->name)) {
 	    return matrices[i];
 	}
     }
@@ -246,6 +269,21 @@ user_matrix *get_user_matrix_by_index (int idx)
     }
 }
 
+static void usermat_destroy_colnames (user_matrix *u)
+{
+    if (u->colnames != NULL && u->M != NULL) {
+	free_strings_array(u->colnames, u->M->cols);
+	u->colnames = NULL;
+    }
+}
+
+static void usermat_destroy_matrix (user_matrix *u)
+{
+    if (!data_is_bundled(u->M)) {
+	gretl_matrix_free(u->M);
+    }
+}
+
 int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
 {
     if (u == NULL) {
@@ -258,10 +296,9 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
 		"matrix at %p\n", u->M, M);
 #endif
 	if (u->colnames != NULL && M->cols != u->M->cols) {
-	    free_strings_array(u->colnames, u->M->cols);
-	    u->colnames = NULL;
+	    usermat_destroy_colnames(u);
 	}
-	gretl_matrix_free(u->M);
+	usermat_destroy_matrix(u);
 	u->M = M;
     }
 
@@ -314,7 +351,7 @@ int user_matrix_replace_matrix_by_name (const char *name,
     return user_matrix_replace_matrix(u, M);
 }
 
-static user_matrix *get_user_matrix_by_data (const gretl_matrix *M)
+user_matrix *get_user_matrix_by_data (const gretl_matrix *M)
 {
     int level = gretl_function_depth();
     int i;
@@ -339,7 +376,39 @@ static user_matrix *get_user_matrix_by_data (const gretl_matrix *M)
 
 gretl_matrix *get_matrix_by_name (const char *name)
 {
-    return real_get_matrix_by_name(name, LEVEL_AUTO);
+    if (name == NULL || *name == '\0') {
+	return NULL;
+    } else {
+	return real_get_matrix_by_name(name, LEVEL_AUTO);
+    }
+}
+
+/**
+ * steal_matrix_by_name:
+ * @name: name of the matrix.
+ *
+ * Looks up a user-defined matrix by name and if found,
+ * grabs the matrix, leaving the matrix pointer on the
+ * named matrix as %NULL.
+ *
+ * Returns: pointer to matrix, or %NULL if not found.
+ */
+
+gretl_matrix *steal_matrix_by_name (const char *name)
+{
+    gretl_matrix *ret = NULL;
+
+    if (name != NULL && *name != '\0') {
+	user_matrix *u = get_user_matrix_by_name(name);
+
+	if (u != NULL) {
+	    usermat_destroy_colnames(u);
+	    ret = u->M;
+	    u->M = NULL;
+	}
+    }
+
+    return ret;
 }
 
 /**
@@ -409,18 +478,18 @@ gretl_matrix *user_matrix_get_matrix (user_matrix *u)
 /**
  * copy_named_matrix_as:
  * @orig: the name of the original matrix.
- * @new: the name to be given to the copy.
+ * @newname: the name to be given to the copy.
  *
  * If a saved matrix is found by the name @orig, a copy of
  * this matrix is added to the stack of saved matrices under the
- * name @new.  This is intended for use when a matrix is given
+ * name @newname.  This is intended for use when a matrix is given
  * as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int copy_named_matrix_as (const char *orig, const char *new)
+int copy_named_matrix_as (const char *orig, const char *newname)
 {
     user_matrix *u;
     int err = 0;
@@ -434,7 +503,7 @@ int copy_named_matrix_as (const char *orig, const char *new)
 	if (M == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    err = user_matrix_add(M, new);
+	    err = user_matrix_add(M, newname);
 	}
 	if (!err) {
 	    /* for use in functions: increment level of last-added matrix */
@@ -449,66 +518,101 @@ int copy_named_matrix_as (const char *orig, const char *new)
 /**
  * copy_matrix_as:
  * @m: the original matrix.
- * @new: the name to be given to the copy.
+ * @newname: the name to be given to the copy.
  *
  * A copy of matrix @m is added to the stack of saved matrices
- * under the name @new.  This is intended for use when a matrix is given
+ * under the name @newname.  This is intended for use when a matrix
  * is given as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int copy_matrix_as (const gretl_matrix *m, const char *new)
+int copy_matrix_as (const gretl_matrix *m, const char *newname)
 {
     gretl_matrix *m2 = gretl_matrix_copy(m);
-    int err;
+    user_matrix *u;
+    int err = 0;
 
     if (m2 == NULL) {
 	err = E_ALLOC;
     } else {
-	err = matrix_copy_add(m2, new);
-    }
-
-    if (!err) {
-	/* increment level of last-added matrix */
-	user_matrix *u = matrices[n_matrices - 1];
-
-	u->level += 1;
+	u = real_user_matrix_add(m2, newname, 0);
+	if (u == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    u->level += 1;
+	}
     }
 
     return err;
 }
 
 /**
- * user_matrix_set_name_and_level:
- * @m: matrix to be reconfigured.
- * @name: new name to be given to matrix.
- * @level: function execution level to be assigned to matrix.
+ * matrix_add_as_shell:
+ * @M: the matrix to add.
+ * @name: the name to be given to the "shell".
  *
- * If matrix @m is found on the stack of saved matrices, change
- * its name to @newname and change its level to @level.
+ * Matrix @M is added to the stack of saved matrices under
+ * the name @name with the shell flag set.  This is used
+ * when an anonymous matrix is given as a %const argument to a 
+ * user-defined function: it is temporarily given user_matrix, 
+ * status, so that it is accessible by name within the function,
+ * but the content @M is protected from destruction on exit 
+ * from the function.
  *
- * Returns: 0 on success, 1 if the matrix is not found.
+ * Returns: 0 on success, non-zero on error.
  */
 
-int user_matrix_set_name_and_level (const gretl_matrix *M, char *name, 
-				    int level)
+int matrix_add_as_shell (gretl_matrix *M, const char *name)
 {
-    user_matrix *u = get_user_matrix_by_data(M);
+    user_matrix *u = real_user_matrix_add(M, name, 0);
     int err = 0;
 
     if (u == NULL) {
-	err = 1;
+	err = E_ALLOC;
     } else {
-	*u->name = '\0';
-	strncat(u->name, name, VNAMELEN - 1);
-	u->level = level;
+	u->flags |= UM_SHELL;
+	u->level += 1;
     }
 
     return err;
 }
 
+static int msel_out_of_bounds (int *range, int n)
+{
+    int *bad = NULL;
+
+    if (range[0] < 1 || range[0] > n) {
+	bad = &range[0];
+    } else if (range[1] < 1 || range[1] > n) {
+	bad = &range[1];
+    }
+
+    if (bad != NULL) {
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), 
+			     *bad);
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
+static int vec_is_const (const gretl_matrix *m, int n)
+{
+    int i;
+
+    for (i=1; i<n; i++) {
+	if (m->val[i] != m->val[i-1]) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+/* convert matrix subspec into list of rows or columns */
+
 static int *mspec_to_list (int type, union msel *sel, int n,
 			   int *err)
 {
@@ -520,13 +624,37 @@ static int *mspec_to_list (int type, union msel *sel, int n,
     }
 
     if (type == SEL_RANGE) {
-	ns = sel->range[1] - sel->range[0] + 1;
+	if (sel->range[1] == MSEL_MAX) {
+	    sel->range[1] = n;
+	}
+	if (msel_out_of_bounds(sel->range, n)) {
+	    *err = E_DATA;
+	} else {
+	    ns = sel->range[1] - sel->range[0] + 1;
+	    if (ns <= 0) {
+		gretl_errmsg_sprintf(_("Range %d to %d is non-positive!"),
+				     sel->range[0], sel->range[1]); 
+		*err = E_DATA;
+	    }
+	}
     } else {
-	ns = gretl_vector_get_length(sel->m);
+	/* SEL_MATRIX */
+	if (sel->m == NULL) {
+	    gretl_errmsg_set(_("Range is non-positive!"));
+	    *err = E_DATA;
+	} else {
+	    ns = gretl_vector_get_length(sel->m);
+	    if (ns == 0) {
+		fprintf(stderr, "selection matrix is %d x %d\n", 
+			sel->m->rows, sel->m->cols);
+		*err = E_NONCONF;
+	    } else if (vec_is_const(sel->m, ns)) {
+		ns = 1;
+	    }
+	}
     }
 
-    if (ns <= 0) {
-	*err = E_DATA;
+    if (*err) {
 	return NULL;
     }
 
@@ -546,8 +674,8 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 
     for (i=1; i<=slice[0] && !*err; i++) {
 	if (slice[i] < 1 || slice[i] > n) {
-	    sprintf(gretl_errmsg, _("Index value %d is out of bounds"), 
-		    slice[i]);
+	    gretl_errmsg_sprintf(_("Index value %d is out of bounds"), 
+				 slice[i]);
 	    *err = 1;
 	}
     }
@@ -593,6 +721,11 @@ static int get_slices (matrix_subspec *spec,
     return err;
 }
 
+/* @M is the target for partial replacement, @S is the source to
+   substitute, and @spec tells how/where to make the
+   substitution.
+*/
+
 static int matrix_replace_submatrix (gretl_matrix *M,
 				     const gretl_matrix *S,
 				     matrix_subspec *spec)
@@ -606,7 +739,15 @@ static int matrix_replace_submatrix (gretl_matrix *M,
     int sscalar = 0;
     int err = 0;
 
+    if (spec == NULL) {
+	fprintf(stderr, "matrix_replace_submatrix: spec is NULL!\n");
+	return E_DATA;
+    }
+
     if (sr > mr || sc > mc) {
+	/* the replacement matrix won't fit into M */
+	fprintf(stderr, "matrix_replace_submatrix: target is %d x %d but "
+		"replacement part is %d x %d\n", mr, mc, sr, sc);
 	return E_NONCONF;
     }
 
@@ -614,24 +755,30 @@ static int matrix_replace_submatrix (gretl_matrix *M,
 	return matrix_insert_diagonal(M, S, mr, mc);
     }
 
+    /* parse mspec into lists of affected rows and columns */
     err = get_slices(spec, &rslice, &cslice, M);
     if (err) {
 	return err;
     }
 
 #if MDEBUG
-    printlist(rslice, "rslice");
-    printlist(cslice, "cslice");
-    fprintf(stderr, "M = %d x %d, S = %d x %d\n", mr, mc, sr, sc);
+    printlist(rslice, "rslice (rows list)");
+    printlist(cslice, "cslice (cols list)");
+    fprintf(stderr, "orig M = %d x %d, S = %d x %d\n", mr, mc, sr, sc);
 #endif
 
     if (sr == 1 && sc == 1) {
+	/* selection matrix is a scalar */
 	sscalar = 1;
 	sr = (rslice == NULL)? mr : rslice[0];
 	sc = (cslice == NULL)? mc : cslice[0];
     } else if (rslice != NULL && rslice[0] != sr) {
+	fprintf(stderr, "mspec has %d rows but substitute matrix has %d\n", 
+		rslice[0], sr);
 	err = E_NONCONF;
     } else if (cslice != NULL && cslice[0] != sc) {
+	fprintf(stderr, "mspec has %d cols but substitute matrix has %d\n", 
+		cslice[0], sc);
 	err = E_NONCONF;
     }
 
@@ -742,12 +889,18 @@ gretl_matrix *user_matrix_get_submatrix (const char *name,
     return S;
 }
 
-int user_matrix_replace_submatrix (const char *name, gretl_matrix *S,
+/* Look up the existing matrix called @name, and substitute
+   the matrix @S for part of the original, as specified by
+   @spec.
+*/
+
+int user_matrix_replace_submatrix (const char *mname, 
+				   const gretl_matrix *S,
 				   matrix_subspec *spec)
 {
     gretl_matrix *M;
 
-    M = real_get_matrix_by_name(name, LEVEL_AUTO); 
+    M = real_get_matrix_by_name(mname, LEVEL_AUTO); 
     if (M == NULL) {
 	return E_UNKVAR;
     }
@@ -787,15 +940,19 @@ static void destroy_user_matrix (user_matrix *u)
 	return;
     }
 
+    if (matrix_is_shell(u)) {
+	/* don't free content */
+	free(u);
+	return;
+    }
+
 #if MDEBUG
     fprintf(stderr, "destroy_user_matrix: freeing matrix at %p...", 
 	    (void *) u->M);
 #endif
 
-    if (u->colnames != NULL) {
-	free_strings_array(u->colnames, u->M->cols);
-    }
-    gretl_matrix_free(u->M);
+    usermat_destroy_colnames(u);
+    usermat_destroy_matrix(u);
     free(u);
 
 #if MDEBUG
@@ -805,15 +962,17 @@ static void destroy_user_matrix (user_matrix *u)
 
 #define LEV_PRIVATE -1
 
-static int levels_match (user_matrix *u, int lev)
+static int matrix_levels_match (user_matrix *u, int lev)
 {
+    int ret = 0;
+
     if (u->level == lev) {
-	return 1;
-    } else if (lev == LEV_PRIVATE && *u->name == '$') {
-	return 1;
+	ret = 1;
+    } else if (lev == LEV_PRIVATE && matrix_is_private(u)) {
+	ret = 1;
     }
 
-    return 0;
+    return ret;
 }
 
 /**
@@ -846,7 +1005,7 @@ int destroy_user_matrices_at_level (int level)
 	if (matrices[i] == NULL) {
 	    break;
 	}
-	if (levels_match(matrices[i], level)) {
+	if (matrix_levels_match(matrices[i], level)) {
 #if MDEBUG
 	    fprintf(stderr, "destroying matrix[%d] ('%s' at %p)\n",
 		    i, matrices[i]->name, (void *) matrices[i]->M);
@@ -980,7 +1139,7 @@ int umatrix_set_colnames_from_string (const gretl_matrix *M,
 				      const char *s)
 {
     user_matrix *u = get_user_matrix_by_data(M);
-    int i, n, err = 0;
+    int n, err = 0;
 
     if (u == NULL) {
 	return E_UNKVAR;
@@ -989,37 +1148,19 @@ int umatrix_set_colnames_from_string (const gretl_matrix *M,
     n = M->cols;
 
     if (s == NULL || *s == '\0') {
-	if (u->colnames != NULL) {
-	    free_strings_array(u->colnames, n);
-	    u->colnames = NULL;
-	}
-    } else if (count_fields(s) != n) {
-	err = E_NONCONF;
+	usermat_destroy_colnames(u);
     } else {
-	char **S = strings_array_new(n);
-	char *tmp;
+	char **S;
+	int ns;
 
+	S = gretl_string_split(s, &ns);
 	if (S == NULL) {
 	    err = E_ALLOC;
-	}
-
-	for (i=0; i<n && !err; i++) {
-	    tmp = gretl_word_strdup(s, &s);
-	    if (tmp != NULL) {
-		S[i] = gretl_strndup(tmp, 12);
-	    }
-	    if (S[i] == NULL) {
-		err = E_ALLOC;
-	    }
-	    free(tmp);
-	}
-
-	if (err) {
-	    free_strings_array(S, n);
+	} else if (ns != n) {
+	    err = E_NONCONF;
+	    free_strings_array(S, ns);
 	} else {
-	    if (u->colnames != NULL) {
-		free_strings_array(u->colnames, n);
-	    }
+	    usermat_destroy_colnames(u);
 	    u->colnames = S;
 	}
     }
@@ -1041,10 +1182,7 @@ int umatrix_set_colnames_from_list (const gretl_matrix *M,
     n = M->cols;
 
     if (list == NULL || list[0] == 0) {
-	if (u->colnames != NULL) {
-	    free_strings_array(u->colnames, n);
-	    u->colnames = NULL;
-	}
+	usermat_destroy_colnames(u);
     } else if (list[0] != n) {
 	err = E_NONCONF;
     } else {
@@ -1064,9 +1202,7 @@ int umatrix_set_colnames_from_list (const gretl_matrix *M,
 	if (err) {
 	    free_strings_array(S, n);
 	} else {
-	    if (u->colnames != NULL) {
-		free_strings_array(u->colnames, n);
-	    }
+	    usermat_destroy_colnames(u);
 	    u->colnames = S;
 	}
     }
@@ -1130,6 +1266,8 @@ gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f,
 	    gretl_matrix_demean_by_column(R);
 	} else if (f == F_CHOL) {
 	    *err = gretl_matrix_cholesky_decomp(R);
+	} else if (f == F_PSDROOT) {
+	    *err = gretl_matrix_psd_root(R);
 	} else if (f == F_INVPD) {
 	    *err = gretl_invpd(R);
 	} else if (f == F_GINV) {
@@ -1137,9 +1275,9 @@ gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f,
 	} else if (f == F_INV) {
 	    *err = gretl_invert_matrix(R);
 	} else if (f == F_UPPER) {
-	    *err = gretl_matrix_zero_upper(R);
-	} else if (f == F_LOWER) {
 	    *err = gretl_matrix_zero_lower(R);
+	} else if (f == F_LOWER) {
+	    *err = gretl_matrix_zero_upper(R);
 	} else {
 	    *err = E_DATA;
 	}
@@ -1160,12 +1298,6 @@ static void matrix_cannibalize (gretl_matrix *targ, gretl_matrix *src)
     free(targ->val);
     targ->val = src->val;
     src->val = NULL;
-
-#if USE_COLS
-    free(targ->col);
-    targ->col = src->col;
-    src->col = NULL;
-#endif
 }
 
 int matrix_invert_in_place (gretl_matrix *m)
@@ -1333,7 +1465,7 @@ real_user_matrix_QR_decomp (const gretl_matrix *m, gretl_matrix **Q,
     }
 
     if (err) {
-	strcpy(gretl_errmsg, _("Matrix decomposition failed"));
+	gretl_errmsg_set(_("Matrix decomposition failed"));
 	gretl_matrix_free(*Q);
 	*Q = NULL;
 	if (R != NULL) {
@@ -1345,39 +1477,40 @@ real_user_matrix_QR_decomp (const gretl_matrix *m, gretl_matrix **Q,
     return err;
 }
 
+#define nullarg(s) (s == NULL || !strcmp(s, "null"))
+
 gretl_matrix *
 user_matrix_QR_decomp (const gretl_matrix *m, const char *rname, int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
-    int wantR = 0;
+    gretl_matrix **pR = NULL;
 
     if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
 	return NULL;
     }
 
-    if (rname != NULL && strcmp(rname, "null")) {
-	wantR = 1;
+    if (!nullarg(rname)) {
 	if (get_matrix_by_name(rname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), rname);
 	    *err = E_UNKVAR;
+	} else {
+	    pR = &R;
 	}
     }
 
     if (!*err) {
-	*err = real_user_matrix_QR_decomp(m, &Q, (wantR)? &R : NULL);
+	*err = real_user_matrix_QR_decomp(m, &Q, pR);
     }
 
-    if (!*err && wantR) {
+    if (!*err && R != NULL) {
 	user_matrix_replace_matrix_by_name(rname, R);
     }
 
     return Q;
 }
 
-#define SVD_RESIZE 1
-
-#if SVD_RESIZE
 static int revise_SVD_V (gretl_matrix **pV, int r, int c)
 {
     gretl_matrix *V;
@@ -1401,7 +1534,6 @@ static int revise_SVD_V (gretl_matrix **pV, int r, int c)
 
     return 0;
 }
-#endif
 
 gretl_matrix *user_matrix_SVD (const gretl_matrix *m, 
 			       const char *uname, 
@@ -1419,16 +1551,18 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	return NULL;
     }
 
-    if (uname != NULL && strcmp(uname, "null")) {
+    if (!nullarg(uname)) {
 	if (get_matrix_by_name(uname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), uname);
 	    *err = E_UNKVAR;
 	} else {
 	    pU = &U;
 	}
     }
 
-    if (vname != NULL && strcmp(vname, "null")) {
+    if (!*err && !nullarg(vname)) {
 	if (get_matrix_by_name(vname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), vname);
 	    *err = E_UNKVAR;
 	} else {
 	    pV = &V;
@@ -1439,7 +1573,6 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	*err = gretl_matrix_SVD(m, pU, &S, pV);
     }
 
-#if SVD_RESIZE
     if (!*err && (U != NULL || V != NULL)) {
 	int tall = m->rows - m->cols;
 	int minrc = (m->rows > m->cols)? m->cols : m->rows;
@@ -1461,79 +1594,153 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
 	    }
 	}
     }
-#else
-    if (!*err) {
-	if (U != NULL) {
-	    user_matrix_replace_matrix_by_name(uname, U);
-	}
-	if (V != NULL) {
-	    user_matrix_replace_matrix_by_name(vname, V);
-	}
-    }
-#endif
 
     return S;
 }
 
+static gretl_matrix *get_ols_matrix (const char *mname, 
+				     int r, int c,
+				     int *newmat, int *err)
+{
+    gretl_matrix *m = get_matrix_by_name(mname);
+
+    if (m == NULL) {
+	gretl_errmsg_sprintf(_("'%s': no such matrix"), mname);
+	*err = E_UNKVAR;
+	return NULL;
+    } 
+
+    if (newmat != NULL && (m->rows != r || m->cols != c)) {
+	m = gretl_matrix_alloc(r, c);
+	if (m == NULL) {
+	    *err = E_ALLOC;
+	    return NULL;
+	}
+	*newmat = 1;
+    }
+
+    return m;
+}
+
 gretl_matrix *user_matrix_ols (const gretl_matrix *Y, 
 			       const gretl_matrix *X, 
 			       const char *Uname, 
+			       const char *Vname, 
+			       gretlopt opt,
 			       int *err)
 {
     gretl_matrix *B = NULL;
     gretl_matrix *U = NULL;
-    int newU = 0;
+    gretl_matrix *V = NULL;
+    int newU = 0, newV = 0;
+    double s2, *ps2 = NULL;
+    int g, k, T;
 
-    if (gretl_is_null_matrix(X) || gretl_is_null_matrix(X)) {
+    if (gretl_is_null_matrix(Y) || gretl_is_null_matrix(X)) {
 	*err = E_DATA;
 	return NULL;
     }
 
-    if (X->rows != Y->rows) {
+    T = Y->rows;
+    k = X->cols;
+    g = Y->cols;
+
+    if (X->rows != T) {
 	*err = E_NONCONF;
 	return NULL;
     }
 
-    if (Uname != NULL && strcmp(Uname, "null")) {
-	U = get_matrix_by_name(Uname);
-	if (U == NULL) {
-	    *err = E_UNKVAR;
-	    return NULL;
-	} 
-	if (U->rows != Y->rows || U->cols != Y->cols) {
-	    newU = 1;
-	}
+    if (g > 1 && (opt & OPT_M)) {
+	/* multiple precision: only one y var wanted */
+	*err = E_DATA;
+	return NULL;
     }
 
-    if (newU) {
-	U = gretl_matrix_alloc(Y->rows, Y->cols);
-	if (U == NULL) {
-	    *err = E_ALLOC;
+    if (!nullarg(Uname)) {
+	U = get_ols_matrix(Uname, T, g, &newU, err);
+	if (*err) {
 	    return NULL;
 	}
-    }
+    } 
 
-    B = gretl_matrix_alloc(X->cols, Y->cols);
-    if (B == NULL) {
-	*err = E_ALLOC;
+    if (!nullarg(Vname)) {
+	if (g > 1) {
+	    /* multiple dependent variables */
+	    get_ols_matrix(Vname, 0, 0, NULL, err);
+	    if (!*err) {
+		newV = 1;
+	    }
+	} else {
+	    /* single dependent variable */
+	    int nv = g * k;
+
+	    V = get_ols_matrix(Vname, nv, nv, &newV, err);
+	    if (!*err) {
+		ps2 = &s2;
+	    }
+	}
     }
 
     if (!*err) {
-	*err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+	B = gretl_matrix_alloc(k, g);
+	if (B == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	if (g == 1) {
+	    if (opt & OPT_M) {
+		/* use multiple precision */
+		*err = gretl_matrix_mp_ols(Y, X, B, V, U, ps2);
+	    } else {
+		*err = gretl_matrix_ols(Y, X, B, V, U, ps2);
+	    }
+	} else {
+	    if (newV) {
+		/* note: "V" will actually be (X'X)^{-1} */
+		*err = gretl_matrix_multi_ols(Y, X, B, U, &V);
+	    } else {
+		*err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+	    }
+	}
     }
 
     if (*err) {
 	gretl_matrix_free(B);
+	B = NULL;
+	if (newU) gretl_matrix_free(U);
+	if (newV) gretl_matrix_free(V);
+    } else {
 	if (newU) {
-	    gretl_matrix_free(U);
+	    user_matrix_replace_matrix_by_name(Uname, U);
 	}
-    } else if (newU) {
-	user_matrix_replace_matrix_by_name(Uname, U);
+	if (newV) {
+	    user_matrix_replace_matrix_by_name(Vname, V);
+	}	
     }
 
     return B;
 }
 
+static void maybe_eigen_trim (gretl_matrix *E)
+{
+    double x;
+    int i, allreal = 1;
+
+    for (i=0; i<E->rows; i++) {
+	x = gretl_matrix_get(E, i, 1);
+	if (x != 0.0) {
+	    allreal = 0;
+	    break;
+	}
+    }
+
+    if (allreal) {
+	gretl_matrix_reuse(E, -1, 1);
+    }
+}
+
 gretl_matrix *
 user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 			    int *err)
@@ -1547,9 +1754,15 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 	return NULL;
     }
 
-    if (rname != NULL && strcmp(rname, "null")) {
+    if (gretl_matrix_xna_check(m)) {
+	*err = E_NAN;
+	return NULL;
+    }
+
+    if (!nullarg(rname)) {
 	vecs = 1;
 	if (get_matrix_by_name(rname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), rname);
 	    *err = E_UNKVAR;
 	    return NULL;
 	}
@@ -1565,6 +1778,9 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
 	    E = gretl_symmetric_matrix_eigenvals(C, vecs, err);
 	} else {
 	    E = gretl_general_matrix_eigenvals(C, vecs, err);
+	    if (E != NULL && E->cols == 2) {
+		maybe_eigen_trim(E);
+	    }
 	}
     }
 
@@ -1605,7 +1821,7 @@ static void xml_put_user_matrix (user_matrix *u, FILE *fp)
 
     for (i=0; i<M->rows; i++) {
 	for (j=0; j<M->cols; j++) {
-	    fprintf(fp, "%.15g ", gretl_matrix_get(M, i, j));
+	    fprintf(fp, "%.16g ", gretl_matrix_get(M, i, j));
 	}
 	fputc('\n', fp);
     }
