@@ -14,7 +14,6 @@
  * 
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
  */
 
 #include "../cephes/mconf.h"
@@ -110,7 +109,9 @@ static int tramo_x12a_spawn (const char *workdir, const char *fmt, ...)
 
 #endif /* !WIN32 */
 
+#define X12A_CODE
 #include "arma_common.c"
+#undef X12A_CODE
 
 static int add_unique_output_file (MODEL *pmod, const char *path)
 {
@@ -355,7 +356,7 @@ static int get_x12a_vcv (const char *fname, MODEL *pmod, int nc)
 #endif
 
 /* Below: parse the coefficient estimates and standard errors from
-   the X12ARIMA output file foo.est
+   the X-12-ARIMA output file foo.est
 */
 
 static int 
@@ -364,16 +365,13 @@ get_estimates (const char *fname, MODEL *pmod, arma_info *ainfo)
     double *ar_coeff = pmod->coeff + ainfo->ifc;
     double *ma_coeff = ar_coeff + ainfo->np + ainfo->P;
     double *x_coeff = ma_coeff + ainfo->nq + ainfo->Q;
-
     double *ar_sderr = pmod->sderr + ainfo->ifc;
     double *ma_sderr = ar_sderr + ainfo->np + ainfo->P;
     double *x_sderr = ma_sderr + ainfo->nq + ainfo->Q;
-
     FILE *fp;
     char line[128], word[16];
     double b, se;
     int i, j, k;
-
     int err = 0;
 
     fp = gretl_fopen(fname, "r");
@@ -434,7 +432,7 @@ get_estimates (const char *fname, MODEL *pmod, arma_info *ainfo)
     return err;
 }
 
-/* Parse the residuals from the X12ARIMA output file foo.rsd */
+/* Parse the residuals from the x12a output file foo.rsd */
 
 static int 
 get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
@@ -480,28 +478,22 @@ get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
 }
 
 static void 
-populate_arma_model (MODEL *pmod, const char *path, 
-		     const double **Z, const DATAINFO *pdinfo, 
-		     arma_info *ainfo)
+populate_x12a_arma_model (MODEL *pmod, const char *path, 
+			  const double **Z, const DATAINFO *pdinfo, 
+			  arma_info *ainfo)
 {
     char fname[MAXLEN];
-    int t, err = 0;
-
-    pmod->uhat = malloc(pdinfo->n * sizeof *pmod->uhat);
-    pmod->yhat = malloc(pdinfo->n * sizeof *pmod->yhat);
-    pmod->coeff = malloc(ainfo->nc * sizeof *pmod->coeff);
-    pmod->sderr = malloc(ainfo->nc * sizeof *pmod->sderr);
-
-    if (pmod->uhat == NULL || pmod->yhat == NULL || 
-	pmod->coeff == NULL || pmod->sderr == NULL) {
-	pmod->errcode = E_ALLOC;
-	return;
-    }
+    int err;
 
+    pmod->t1 = ainfo->t1;
+    pmod->t2 = ainfo->t2;
+    pmod->ncoeff = ainfo->nc;
     pmod->full_n = pdinfo->n;
 
-    for (t=0; t<pdinfo->n; t++) {
-	pmod->uhat[t] = pmod->yhat[t] = NADBL;
+    err = gretl_model_allocate_storage(pmod);
+    if (err) {
+	pmod->errcode = err;
+	return;
     }
 
     sprintf(fname, "%s.est", path);
@@ -542,14 +534,23 @@ static void
 output_series_to_spc (const int *list, const double **Z, 
 		      int t1, int t2, FILE *fp)
 {
-    int i, t;
+    int i, t, done = 0;
 
-    fputs(" data = (\n", fp);
+    for (t=t1; t<=t2 && !done; t++) {
+	for (i=1; i<=list[0] && !done; i++) {
+	    if (na(Z[list[i]][t])) {
+		fputs(" missingcode=-99999\n", fp);
+		done = 1;
+	    }
+	}
+    }
+
+    fputs(" data=(\n", fp);
 
     for (t=t1; t<=t2; t++) {
 	for (i=1; i<=list[0]; i++) {
 	    if (na(Z[list[i]][t])) {
-		fputs("-9999.0 ", fp);
+		fputs("-99999 ", fp);
 	    } else {
 		fprintf(fp, "%.15g ", Z[list[i]][t]);
 	    }
@@ -654,10 +655,10 @@ static void x12_pdq_string (arma_info *ainfo, FILE *fp)
     fputc(')', fp);
 }
 
-static int write_spc_file (const char *fname, 
-			   const double **Z, const DATAINFO *pdinfo,
-			   arma_info *ainfo, int pdmax, 
-			   gretlopt opt)
+static int write_arma_spc_file (const char *fname, 
+				const double **Z, const DATAINFO *pdinfo,
+				arma_info *ainfo, int pdmax, 
+				gretlopt opt)
 {
     int maxobs = pdmax * 50;
     int maxfcast = pdmax * 5;
@@ -685,13 +686,13 @@ static int write_spc_file (const char *fname,
 
     gretl_push_c_numeric_locale();
 
-    fprintf(fp, "series {\n title = \"%s\"\n period = %d\n", 
+    fprintf(fp, "series{\n title=\"%s\"\n period=%d\n", 
 	    pdinfo->varname[ainfo->yno], pdinfo->pd);
 
     t1 -= ainfo->maxlag;
     
     make_x12a_date_string(t1, pdinfo, datestr);
-    fprintf(fp, " start = %s\n", datestr);
+    fprintf(fp, " start=%s\n", datestr);
 
     ylist[0] = 1;
     ylist[1] = ainfo->yno;
@@ -701,8 +702,6 @@ static int write_spc_file (const char *fname,
     if ((opt & OPT_F) && ainfo->t2 < pdinfo->n - 1) {
 	int nobs;
 
-	/* FIXME ensure we don't print any NAs */
-
 	tmax = pdinfo->n - 1;
 	nobs = tmax - ainfo->t1 + 1; /* t1? */
 	if (nobs > maxobs) {
@@ -763,7 +762,7 @@ static int write_spc_file (const char *fname,
     } else {
 	fputs(" print = (acm lkf lks mdl est rts rcm)\n", fp);
     }
-    
+
     fputs(" save = (rsd est lks acm rts rcm)\n", fp);
 
     if (opt & OPT_C) {
@@ -815,7 +814,14 @@ static void delete_old_files (const char *path)
     }
 }
 
-MODEL arma_x12_model (const int *list, const char *pqspec,
+static void x12a_maybe_allow_missvals (arma_info *ainfo)
+{
+    if (arma_exact_ml(ainfo)) {
+	ainfo->pflags |= ARMA_NAOK;
+    }
+}
+
+MODEL arma_x12_model (const int *list, const int *pqspec,
 		      const double **Z, const DATAINFO *pdinfo,
 		      int pdmax, gretlopt opt, PRN *prn)
 {
@@ -824,7 +830,8 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
     const char *workdir = gretl_x12_arima_dir();
     char yname[VNAMELEN], path[MAXLEN];
     MODEL armod;
-    arma_info ainfo;
+    arma_info ainfo_s, *ainfo;
+    int missv = 0, misst = 0;
 #ifdef WIN32
     char *cmd;
 #endif
@@ -835,18 +842,19 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 	opt |= OPT_F;
     }
 
-    arma_info_init(&ainfo, opt | OPT_X, pqspec, pdinfo);
-    ainfo.prn = set_up_verbose_printer(opt, prn);
+    ainfo = &ainfo_s;
+    arma_info_init(ainfo, opt | OPT_X, pqspec, pdinfo);
+    ainfo->prn = set_up_verbose_printer(opt, prn);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
-    ainfo.alist = gretl_list_copy(list);
-    if (ainfo.alist == NULL) {
+    ainfo->alist = gretl_list_copy(list);
+    if (ainfo->alist == NULL) {
 	err = E_ALLOC;
     }
 
     if (!err) {
-	err = arma_check_list(&ainfo, Z, pdinfo, opt);
+	err = arma_check_list(ainfo, Z, pdinfo, opt);
     }
 
     if (err) {
@@ -855,33 +863,32 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
     }     
 
     /* calculate maximum lag */
-    calc_max_lag(&ainfo);
+    calc_max_lag(ainfo);
+
+    x12a_maybe_allow_missvals(ainfo);
 
     /* adjust sample range if need be */
-    if (arma_adjust_sample(&ainfo, Z, pdinfo)) {
-        armod.errcode = E_DATA;
+    armod.errcode = arma_adjust_sample(ainfo, Z, pdinfo, &missv, &misst);
+    if (armod.errcode) {
 	goto bailout;
+    } else if (missv > 0) {
+	set_arma_missvals(ainfo);
     }
 
-    /* create differenced series if needed */
-    if (ainfo.d > 0 || ainfo.D > 0) {
-	err = arima_difference(&ainfo, Z, pdinfo);
-    }  
-
-    strcpy(yname, pdinfo->varname[ainfo.yno]);
+    ainfo->y = (double *) Z[ainfo->yno]; /* it's really const */
+    strcpy(yname, pdinfo->varname[ainfo->yno]);
 
     /* write out an .spc file */
     sprintf(path, "%s%c%s.spc", workdir, SLASH, yname);
-    write_spc_file(path, Z, pdinfo, &ainfo, pdmax, opt);
+    write_arma_spc_file(path, Z, pdinfo, ainfo, pdmax, opt);
 
-    /* remove any files from on old run, in case of error */
+    /* remove any files from an old run, in case of error */
     delete_old_files(path);
 
     /* run the program */
 #ifdef WIN32
     cmd = g_strdup_printf("\"%s\" %s -r -p -q", prog, yname);
-    err = winfork(cmd, workdir, SW_SHOWMINIMIZED, 
-		  CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS);
+    err = win_run_sync(cmd, workdir);
     g_free(cmd);
 #else
     err = tramo_x12a_spawn(workdir, prog, yname, "-r", "-p", "-q", "-n", NULL);
@@ -889,42 +896,34 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 
     if (!err) {
 	sprintf(path, "%s%c%s", workdir, SLASH, yname); 
-	armod.t1 = ainfo.t1;
-	armod.t2 = ainfo.t2;
-	populate_arma_model(&armod, path, Z, pdinfo, &ainfo);
+	populate_x12a_arma_model(&armod, path, Z, pdinfo, ainfo);
 	if (verbose && !armod.errcode) {
-	    print_iterations(path, ainfo.prn);
+	    print_iterations(path, ainfo->prn);
 	}
 	if (!armod.errcode) {
 	    if (gretl_in_gui_mode()) {
 		add_unique_output_file(&armod, path);
 	    }
-	    gretl_model_set_int(&armod, "arma_flags", (int) ainfo.flags);
+	    gretl_model_set_int(&armod, "arma_flags", (int) ainfo->flags);
 	}	
     } else {
 	armod.errcode = E_UNSPEC;
 	gretl_errmsg_set(_("Failed to execute x12arima"));
     }
 
-    if (armod.errcode && ainfo.prn != NULL) {
+    if (armod.errcode && ainfo->prn != NULL) {
 	sprintf(path, "%s%c%s.err", workdir, SLASH, yname);
-	print_x12a_error_file(path, ainfo.prn);
+	print_x12a_error_file(path, ainfo->prn);
     }
 
-    if (ainfo.prn != NULL && ainfo.prn != prn) {
-	iter_print_callback(0, ainfo.prn);
-	close_down_verbose_printer(ainfo.prn);
+    if (ainfo->prn != NULL && ainfo->prn != prn) {
+	iter_print_callback(0, ainfo->prn);
+	close_down_verbose_printer(ainfo->prn);
     }
 
  bailout:
 
-    arma_info_cleanup(&ainfo);
-
-    if (armod.errcode && (opt & OPT_U)) {
-	/* continue on error */
-	armod.opt |= OPT_U;
-    }
+    arma_info_cleanup(ainfo);
 
     return armod;
 }
-
