@@ -17,9 +17,7 @@
  * 
  */
 
-/*
-  Based on the Gnumeric excel plugin by Michael Meeks.
-*/
+/* Originally based on the Gnumeric excel plugin by Michael Meeks */
 
 #include <gtk/gtk.h>
 
@@ -27,12 +25,15 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <errno.h>
 
 #include "libgretl.h"
 #include "version.h"
+#include "csvdata.h"
 
 #include "importer.h"
 #include "biff.h"
+#include "build.h"
 
 static void free_sheet (void);
 static int allocate_row_col (int row, int col, wbook *book);
@@ -40,7 +41,6 @@ static int allocate_row_col (int row, int col, wbook *book);
 int debug_print;
 
 #ifdef WIN32
-# include "build.h"
 char debug_fname[FILENAME_MAX];
 FILE *fdb;
 static void make_debug_fname (void);
@@ -93,7 +93,7 @@ static void open_debug_stream (void)
 }
 #endif
 
-static int dprintf (const char *format, ...)
+static int dbprintf (const char *format, ...)
 {
     va_list args;
     int len = 0;
@@ -124,11 +124,7 @@ static int dprintf (const char *format, ...)
 
 static void print_version (void)
 {
-#ifdef WIN32
-    dprintf("gretl, version %s, %s\n", GRETL_VERSION, BUILD_DATE);
-#else
-    dprintf("gretl, version %s\n", GRETL_VERSION); 
-#endif
+    dbprintf("gretl, version %s, %s\n", GRETL_VERSION, BUILD_DATE);
 }
 
 static double get_le_double (const unsigned char *rec) 
@@ -201,18 +197,25 @@ static double biff_get_rk (const unsigned char *ptr)
 static char *convert8to7 (const char *s, int count) 
 {
     char *dest;
+    int n = strspn(s, " \t");
 
-    if (count > VNAMELEN - 1) {
-	count = VNAMELEN - 1;
+    count -= n;
+
+    if (count <= 0) {
+	dest = g_strdup("");
+    } else {
+	if (count > VNAMELEN - 1) {
+	    count = VNAMELEN - 1;
+	}
+	dest = malloc(VNAMELEN);
+	*dest = '\0';
+	s += n;
+	strncat(dest, s, count);
+	iso_to_ascii(dest);
+	tailstrip(dest);
     }
 
-    dest = malloc(VNAMELEN);
-    *dest = '\0';
-    strncat(dest, s, count);
-    iso_to_ascii(dest);
-    top_n_tail(dest);
-
-    dprintf("convert8to7: returning '%s'\n", dest);
+    dbprintf("convert8to7: returning '%s'\n", dest);
 
     return dest;
 }
@@ -237,7 +240,7 @@ static char *convert16to7 (const unsigned char *s, int count)
 	}
     }
 
-    dprintf("convert16to7: returning '%s'\n", dest);
+    dbprintf("convert16to7: returning '%s'\n", dest);
 
     return dest;    
 }
@@ -251,14 +254,14 @@ copy_unicode_string (unsigned char *src, int remlen,
     int this_skip = 3, skip_to_next = 3;
     int csize = (flags & 0x01)? 2 : 1;
 
-    dprintf("copy_unicode_string: count = %d, csize = %d\n",
+    dbprintf("copy_unicode_string: count = %d, csize = %d\n",
 	    count, csize);
 
     if (flags & 0x08) {
-	dprintf(" contains Rich-Text info\n");
+	dbprintf(" contains Rich-Text info\n");
     }
     if (flags & 0x04) {
-	dprintf(" contains Far-East info\n");
+	dbprintf(" contains Far-East info\n");
     }    
 
     skip_to_next += count * csize;
@@ -301,6 +304,7 @@ copy_unicode_string (unsigned char *src, int remlen,
 	/* let's not mess with excessive strings */
 	return g_strdup("bigstr");
     } else if (csize == 1) {
+	dbprintf("original string = '%s'\n", src + this_skip);
 	return convert8to7((char *) src + this_skip, count);
     } else { 
 	return convert16to7(src + this_skip, count);
@@ -341,30 +345,48 @@ static int row_col_err (int row, int col, PRN *prn)
     return err;
 }
 
+/* This function is called on LABELSST records: we check for possible
+   NAs and also for numeric values that may have strayed into the
+   string table, and that shouldn't really be treated as quoted
+   strings.  We don't just use strtod at the outset, because some
+   XLS files (put out by agencies that should know better!) contain
+   numerical values that are "nicely formatted" as text using spaces
+   or commas for thousands separators -- we try stripping this
+   junk out first before doing the numeric-value test.
+*/
+
 static int check_copy_string (struct sheetrow *prow, int row, int col, 
 			      int idx, const char *s)
 {
+    dbprintf("inspecting sst[%d] = '%s'\n", idx, s);
+    
     if (row > 0 && col > 0) {
+	const char *numok = "0123456789 -,.";
 	int i, len = strlen(s);
-	int commas = 0;
+	int commas = 0, digits = 0;
 	static int warned = 0;
 
 	if (len == 0) {
-	    dprintf("converting sst[%d] '%s' to NA\n", idx, s);
-	    prow->cells[col] = g_strdup("NA");
+	    dbprintf(" converting to NA\n");
+	    prow->cells[col] = g_strdup("-999");
 	    return 0;
-	}	    
+	}
 
 	for (i=0; i<len; i++) {
-	    if (!isdigit(s[i]) && s[i] != ' ' && s[i] != '-' &&
-		s[i] != ',' && s[i] != '.') {
+	    if (strchr(numok, s[i]) == NULL) {
+		/* does not look promising for numerical value */
 		len = 0;
 		break;
 	    }
-	    if (s[i] == ',') commas++;
+	    if (isdigit(s[i])) {
+		digits++;
+	    } else if (s[i] == ',') {
+		commas++;
+	    }
 	}
 
-	if (len > 0) {
+	if (len > 0 && digits > 0) {
+	    /* may be numerical? */
 	    char *p, *q = malloc(len + 1);
 
 	    if (q == NULL) return 1;
@@ -381,19 +403,27 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 			warned = 1;
 		    }
 		    if (len - i != 4) {
-			/* probably decimal? */
+			/* comma is probably decimal separator? */
 			*p++ = '.';
 		    }
 		}
 	    }
 	    *p = '\0';
-	    dprintf("converting sst[%d] '%s' to numeric as %s\n", idx, s, q);
-	    prow->cells[col] = q;
-	    return 0;
+
+	    /* If we don't do a rigorous check on q, as below, we
+	       may end up with zeros where there should be NAs.
+	    */ 
+	    if (numeric_string(q)) {
+		dbprintf(" taking '%s' to be numeric string: %s\n", s, q);
+		prow->cells[col] = q;
+		return 0;
+	    } else {
+		free(q);
+	    }
 	} 
     }
 
-    dprintf("copying sst[%d] '%s' into place\n", idx, s);
+    dbprintf(" copying '%s' into place as string\n", s);
     prow->cells[col] = g_strdup_printf("\"%s", s);
 
     return 0;
@@ -427,6 +457,67 @@ static int wbook_find_format (wbook *book, int xfref)
     return fmt;
 }
 
+static int func_is_date (guint8 *data, int version)
+{
+    /* check for built-in DATE function */
+    if (version < MS_BIFF_V4) {
+	return MS_OLE_GET_GUINT8(data) == 65;
+    } else {
+	return MS_OLE_GET_GUINT16(data) == 65;
+    }
+}
+
+#define t_func_size(v) ((v < MS_BIFF_V4)? 2 : 3)
+#define t_ref_size(v)  ((v < MS_BIFF_V8)? 4 : 5)
+
+/* Could be a date formula?  If so, it should have 3 cell reference fields
+   and a trailing function ID == 65 */
+
+static void check_for_date_formula (BiffQuery *q, wbook *book)
+{
+    int version = book->version;
+    int offset = (version < MS_BIFF_V5)? 16 : 20;
+    guint8 *fdata = q->data + offset;    
+    guint16 sz, targ;
+    guint8 u1;
+    int i;
+
+    targ = 3 * t_ref_size(version) + t_func_size(version);
+
+    if (version < MS_BIFF_V3) {
+	sz = MS_OLE_GET_GUINT8(fdata);
+	fdata += 1;
+    } else {
+	sz = MS_OLE_GET_GUINT16(fdata);
+	fdata += 2;
+    }
+
+    /* There's a one-byte ambiguity over the size of the
+       function ID field in the OpenOffice.org doc for
+       BIFF, so we'll allow sz to be one byte bigger than
+       targ.
+    */
+    if (sz != targ && sz != targ + 1) {
+	return;
+    }
+
+    for (i=0; i<3; i++) {
+	/* token ID */
+	u1 = MS_OLE_GET_GUINT8(fdata);
+	if (u1 != 0x44) { /* 0x44 = tRef */
+	    return;
+	}
+	fdata += t_ref_size(version);
+    }
+
+    u1 = MS_OLE_GET_GUINT8(fdata);
+
+    if (u1 == 0x41 && func_is_date(fdata + 1, version)) { /* 0x41 = tFunc */
+	fprintf(stderr, "Got DATE formula in first column\n");
+	book_set_numeric_dates(book);
+    }
+}
+
 #undef FORMAT_INFO
 
 static int process_item (BiffQuery *q, wbook *book, PRN *prn) 
@@ -444,7 +535,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	if (row_col_err(i, j, prn)) {
 	    return 1;
 	}
-	if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK) {
+	if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK || q->ls_op == BIFF_MULRK) {
 	    guint16 xfref = EX_GETXF(q);
 	    int fmt = wbook_find_format(book, xfref);
 
@@ -459,7 +550,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 			i, j, fmt);
 		book_set_numeric_dates(book);
 	    }
-	}
+	} 
     }
 
     switch (q->ls_op) {
@@ -480,8 +571,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    return 1;
 	}
 
-	dprintf("Got SST: allocated for %d strings (%d bytes), %p\n", 
-		sstsize, sstsize * sizeof *sst, (void *) sst);
+	dbprintf("Got SST: allocated for %d strings (%d bytes), %p\n", 
+		 sstsize, sstsize * sizeof *sst, (void *) sst);
 
 	for (k=oldsz; k<sstsize; k++) {
 	    /* careful: initialize all pointers to NULL */
@@ -492,8 +583,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
 	for (k=oldsz; k<sstsize; k++) {
 	    remlen = q->length - (ptr - q->data);
-	    dprintf("Working on sst[%d], data offset=%d, remlen=%d\n", 
-		    k, (int) (ptr - q->data), remlen);
+	    dbprintf("Working on sst[%d], data offset=%d, remlen=%d\n", 
+		     k, (int) (ptr - q->data), remlen);
 	    if (remlen <= 0) {
 		break;
 	    }
@@ -509,8 +600,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
     }	
 
     case BIFF_CONTINUE: 
-	dprintf("Got CONTINUE, sstnext = %d, len = %d\n", 
-		sstnext, (int) q->length);
+	dbprintf("Got CONTINUE, sstnext = %d, len = %d\n", 
+		 sstnext, (int) q->length);
 	if (sstnext > 0) {
 	    int k, skip, remlen;
 
@@ -519,8 +610,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		unsigned char flags = *ptr;
 		int csize = (flags & 0x01)? 2 : 1;
 
-		dprintf("BIFF_CONTINUE: slop = %d, csize = %d\n", (int) slop,
-			(int) csize);
+		dbprintf("BIFF_CONTINUE: slop = %d, csize = %d\n", (int) slop,
+			 (int) csize);
 		ptr += 1 + csize * slop;
 	    }
 	    for (k=sstnext; k<sstsize; k++) {
@@ -528,7 +619,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		if (remlen <= 0) {
 		    break;
 		}
-		dprintf("Working on sst[%d], remlen = %d\n", k, remlen);
+		dbprintf("Working on sst[%d], remlen = %d\n", k, remlen);
 		sst[k] = copy_unicode_string(ptr, remlen, &skip, &slop);
 		ptr += skip;
 	    }
@@ -539,7 +630,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	break;
 			   
     case BIFF_LABEL: 
-	dprintf("Got LABEL, row=%d, col=%d\n", i, j);
+	dbprintf("Got LABEL, row=%d, col=%d\n", i, j);
 	if (allocate_row_col(i, j, book)) {
 	    return 1;
 	} else {
@@ -553,7 +644,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	break;
   
     case BIFF_LABELSST:
-	dprintf("Got LABELSST, row=%d, col=%d\n", i, j);
+	dbprintf("Got LABELSST, row=%d, col=%d\n", i, j);
 	if (allocate_row_col(i, j, book)) {
 	    return 1;
 	} else {
@@ -566,7 +657,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    } else if (sst[sidx] != NULL) {
 		check_copy_string(prow, i, j, sidx, sst[sidx]);
 	    } else {
-		dprintf("sst[%d] seems to be NULL, leaving string blank\n", (int) sidx);
+		dbprintf("sst[%d] seems to be NULL, leaving string blank\n", (int) sidx);
 		prow->cells[j] = malloc(2);
 		if (prow->cells[j] != NULL) {
 		    prow->cells[j][0] = '\0';
@@ -582,7 +673,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    val = get_le_double(q->data + 6);
 	    prow = rows + i;
 	    prow->cells[j] = g_strdup_printf("%.15g", val);
-	    dprintf("Got NUMBER (%g), row=%d, col=%d\n", val, i, j);
+	    dbprintf("Got NUMBER (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
 
@@ -593,14 +684,14 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    val = biff_get_rk(q->data + 6);
 	    prow = rows + i;
 	    prow->cells[j] = g_strdup_printf("%.15g", val);
-	    dprintf("Got RK (%g), row=%d, col=%d\n", val, i, j);
+	    dbprintf("Got RK (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
 
     case BIFF_MULRK: {
 	int k, ncols = (q->length - 6) / 6;
 
-	dprintf("Got MULRK, row=%d, first_col=%d, ncols=%d\n", i, j, ncols);
+	dbprintf("Got MULRK, row=%d, first_col=%d, ncols=%d\n", i, j, ncols);
 	for (k=0; k<ncols; k++) {
 	    if (allocate_row_col(i, j, book)) {
 		return 1;
@@ -608,14 +699,14 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    val = biff_get_rk(q->data + 6 + 6 * k);
 	    prow = rows + i; /* might have moved */
 	    prow->cells[j] = g_strdup_printf("%.15g", val);
-	    dprintf(" MULRK[col=%d] = %g\n", j, val);
+	    dbprintf(" MULRK[col=%d] = %g\n", j, val);
 	    j++;
 	}
 	break;
     }
 
     case BIFF_FORMULA:  
-	dprintf("Got FORMULA, row=%d, col=%d\n", i, j);
+	dbprintf("Got FORMULA, row=%d, col=%d\n", i, j);
 	if (allocate_row_col(i, j, book)) {
 	    return 1;
 	} else {
@@ -633,21 +724,25 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		    prow->cells[j] = g_strdup((ptr[2])? "1" : "0");
 		} else if (fcode == 0x2 || fcode == 0x3) {
 		    /* error code or empty */
-		    prow->cells[j] = g_strdup("-999.0");
+		    prow->cells[j] = g_strdup("-999");
 		} else {
 		    fprintf(stderr, "Bad formula code 0x%u\n", 
 			    (unsigned) fcode);
-		    prow->cells[j] = g_strdup("-999.0");
+		    prow->cells[j] = g_strdup("-999");
 		}
 	    } else {
 		/* floating-point */
 		val = get_le_double(ptr);
-		dprintf(" floating-point value = %g\n", val);
+		dbprintf(" floating-point value = %g\n", val);
 		if (isnan(val)) {
 		    fprintf(stderr, "Got a NaN\n");
-		    prow->cells[j] = g_strdup("-999.0");
+		    prow->cells[j] = g_strdup("-999");
 		} else {
 		    prow->cells[j] = g_strdup_printf("%.15g", val);
+		    if (i == book->row_offset + 1 && j == book->col_offset) {
+			/* could be a date formula? */
+			check_for_date_formula(q, book);
+		    }
 		}
 	    }
 	}
@@ -655,12 +750,12 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
     case BIFF_STRING: 
 	if (string_targ == NULL) {
-	    dprintf("String record without preceding string formula\n");
+	    dbprintf("String record without preceding string formula\n");
 	} else {
 	    char *tmp = copy_unicode_string(q->data, 0, NULL, NULL);
 
 	    *string_targ = make_string(tmp);
-	    dprintf("Filled out string formula with '%s'\n", *string_targ);	
+	    dbprintf("Filled out string formula with '%s'\n", *string_targ);	
 	    string_targ = NULL;
 	}
 	break;
@@ -675,7 +770,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
 	    version = MS_OLE_GET_GUINT16(q->data + 0);
 	    boftype = MS_OLE_GET_GUINT16(q->data + 2);
-	    dprintf("Got BOF: version=%x, type=%x\n", version, boftype);
+	    dbprintf("Got BOF: version=%x, type=%x\n", version, boftype);
 	}
 	break;
 
@@ -769,11 +864,11 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
 	return 1;
     }
 
-    result = ms_ole_stream_open(&stream, file, "/", "workbook", 'r');
+    result = ms_ole_stream_open(&stream, file, "/", "workbook");
 
     if (result != MS_OLE_ERR_OK) {
 	ms_ole_stream_close(&stream);
-	result = ms_ole_stream_open(&stream, file, "/", "book", 'r');
+	result = ms_ole_stream_open(&stream, file, "/", "book");
 	if (result != MS_OLE_ERR_OK) {
 	    ms_ole_stream_close(&stream);
 	    fputs("No book or workbook streams found\n", stderr);
@@ -797,20 +892,20 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
     } 
 
     while (!err && ms_biff_query_next(q)) {
-	dprintf("At %lu: q->opcode=0x%02x\n", (unsigned long) q->streamPos, q->opcode);
+	dbprintf("At %lu: q->opcode=0x%02x\n", (unsigned long) q->streamPos, q->opcode);
 	if (q->opcode == BIFF_EOF) {
-	    dprintf("got MSEOF at %lu\n", (unsigned long) stream->position);
+	    dbprintf("got MSEOF at %lu\n", (unsigned long) ms_ole_stream_position(stream));
 	    eofcount++;
 
 	    if (eofcount == 1) {
-		if (stream->position < offset) {
+		if (ms_ole_stream_position(stream) < offset) {
 		    /* skip to the worksheet we want? */
 		    while (q->streamPos < offset && ms_biff_query_next(q)) ;
 		    fprintf(stderr, "skipped forward to %lu\n", 
 			    (unsigned long) q->streamPos);
 		} else {
 		    fprintf(stderr, "reading worksheet at %lu\n", 
-			    (unsigned long) stream->position);
+			    (unsigned long) ms_ole_stream_position(stream));
 		}
 	    }
 
@@ -824,11 +919,11 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
 	if (handled_record(q)) {
 	    err = process_item(q, book, prn);
 	} else if (q->ms_op == 0x02 && q->ls_op == BIFF_ROW) {
-	    dprintf("Got BIFF_ROW\n");
+	    dbprintf("Got BIFF_ROW\n");
 	} else if (q->opcode == BIFF_DBCELL) {
-	    dprintf("Got BIFF_DBCELL\n");
+	    dbprintf("Got BIFF_DBCELL\n");
 	} else {
-	    dprintf("skipping unhandled opcode 0x%02x\n", q->opcode);
+	    dbprintf("skipping unhandled opcode 0x%02x\n", q->opcode);
 	}
     }
 
@@ -860,7 +955,7 @@ static int allocate_row_col (int i, int j, wbook *book)
 
     started = 1;
 
-    dprintf("allocate: row=%d, col=%d, nrows=%d\n", i, j, nrows);
+    dbprintf("allocate: row=%d, col=%d, nrows=%d\n", i, j, nrows);
 
     if (i >= nrows) {
 	new_nrows = (i / 16 + 1) * 16;
@@ -874,18 +969,18 @@ static int allocate_row_col (int i, int j, wbook *book)
 	rows = myrows;
 
 	for (k=nrows; k<new_nrows; k++) {
-	    dprintf("allocate: initing rows[%d]\n", k);
+	    dbprintf("allocate: initing rows[%d]\n", k);
 	    row_init(&rows[k]);
-	    dprintf("rows[%d].end=%d\n", i, rows[k].end);
+	    dbprintf("rows[%d].end=%d\n", i, rows[k].end);
 	}
 	nrows = new_nrows;
     }
 
-    dprintf("allocate: col=%d and rows[%d].end = %d\n", j, i, rows[i].end);
+    dbprintf("allocate: col=%d and rows[%d].end = %d\n", j, i, rows[i].end);
 
     if (j >= rows[i].end) {
 	newcol = (j / 16 + 1) * 16;
-	dprintf("allocate: reallocing rows[%d].cells to size %d\n", i, newcol);
+	dbprintf("allocate: reallocing rows[%d].cells to size %d\n", i, newcol);
 	cells = realloc(rows[i].cells, newcol * sizeof *cells);
 
 	if (cells == NULL) {
@@ -911,7 +1006,7 @@ static void free_sheet (void)
 {
     int i, j;
 
-    dprintf("free_sheet(), nrows=%d\n", nrows);
+    dbprintf("free_sheet(), nrows=%d\n", nrows);
 
     /* free shared string table */
     if (sst != NULL) {
@@ -925,17 +1020,17 @@ static void free_sheet (void)
     if (rows != NULL) {
 	for (i=0; i<nrows; i++) {
 	    if (rows[i].cells == NULL) {
-		dprintf("rows[%d].cells = NULL, skipping free\n", i);
+		dbprintf("rows[%d].cells = NULL, skipping free\n", i);
 		continue;
 	    }
 	    for (j=0; j<rows[i].end; j++) {
 		if (rows[i].cells[j] != NULL) {
-		    dprintf("Freeing rows[%d].cells[%d] at %p\n",
-			    i, j, (void *) rows[i].cells[j]);
+		    dbprintf("Freeing rows[%d].cells[%d] at %p\n",
+			     i, j, (void *) rows[i].cells[j]);
 		    free(rows[i].cells[j]); 
 		}
 	    }
-	    dprintf("Freeing rows[%d].cells at %p\n", i, (void *) rows[i].cells);
+	    dbprintf("Freeing rows[%d].cells at %p\n", i, (void *) rows[i].cells);
 	    free(rows[i].cells);
 	}
 	free(rows);
@@ -957,17 +1052,20 @@ static int first_col_strings (wbook *book)
     int startrow = book->row_offset + 1;
     int ret = 1;
 
+    dbprintf("checking for first column strings...\n");
+
     for (i=startrow; i<nrows; i++) {
-	dprintf("book->row_offset=%d, i=%d\n", book->row_offset, i);
-	dprintf("rows = %p\n", (void *) rows);
+	dbprintf("book->row_offset=%d, i=%d\n", book->row_offset, i);
+	dbprintf("rows = %p\n", (void *) rows);
 	if (rows == NULL || rows[i].cells == NULL || 
 	    rows[i].cells[j] == NULL ||
 	    !IS_STRING(rows[i].cells[j])) {
+	    dbprintf("no: not a string at row %d\n", i);
 	    ret = 0;
 	    break;
 	}
-	dprintf("first_col_strings: rows[%d].cells[%d]: '%s'\n", i, j,
-		rows[i].cells[j]);
+	dbprintf("first_col_strings: rows[%d].cells[%d]: '%s'\n", i, j,
+		 rows[i].cells[j]);
     }
 
     if (ret) {
@@ -1006,7 +1104,7 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
     int vnames = 0;
     int ret = VARNAMES_NONE;
 
-    if (book_obs_labels(book)) {
+    if (book_obs_labels(book) || book_numeric_dates(book)) {
 	startcol++;
 	gotcols = 1;
     }
@@ -1026,14 +1124,14 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
 	}
 
 	if (rows[i].cells[j] == NULL) {
-	    dprintf("got_varnames: rows[%d].cells[%d] is NULL\n", i, j);
+	    dbprintf("got_varnames: rows[%d].cells[%d] is NULL\n", i, j);
 	    break;
 	}
 
 	gotcols++;
 
-	dprintf("got_varnames: rows[%d].cells[%d] is '%s'\n", i, j, 
-		rows[i].cells[j]);
+	dbprintf("got_varnames: rows[%d].cells[%d] is '%s'\n", i, j, 
+		 rows[i].cells[j]);
 
 	if (IS_STRING(rows[i].cells[j])) {
 	    /* skip beyond the quote */
@@ -1071,23 +1169,9 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
 
 static int missval_string (const char *s)
 {
-    if (s + 1 == 0) {
-	return 1;
-    } else {
-	char test[6] = {0};
+    s++;
 
-	strncat(test, s + 1, 4);
-	tailstrip(test);
-	lower(test);
-	if (!strcmp(test, "na") || 
-	    !strcmp(test, "n.a.") ||
-	    !strcmp(test, "..") ||
-	    !strcmp(test, "?")) {
-	    return 1;
-	}
-    }
-
-    return 0;
+    return (*s == '\0' || import_na_string(s));
 }
 
 struct string_err {
@@ -1106,7 +1190,7 @@ check_data_block (wbook *book, int totcols, const char *blank_col,
     int startrow = book->row_offset + 1;
     int j, i, ret = 0;
 
-    if (book_obs_labels(book)) {
+    if (book_obs_labels(book) || book_numeric_dates(book)) {
 	startcol++;
     }
 
@@ -1119,21 +1203,21 @@ check_data_block (wbook *book, int totcols, const char *blank_col,
 	    continue;
 	}
 	for (i=startrow; i<nrows; i++) {
-	    dprintf("data_block: looking at rows[%d], end = %d\n", i, rows[i].end);
+	    dbprintf("data_block: looking at rows[%d], end = %d\n", i, rows[i].end);
 	    if (rows[i].cells  == NULL) {
-		dprintf("data_block: rows[%d].cells = NULL\n", i);
+		dbprintf("data_block: rows[%d].cells = NULL\n", i);
 		ret = -1;
 	    } else if (j >= rows[i].end) {
-		dprintf("data_block: short row, fell off the end\n");
+		dbprintf("data_block: short row, fell off the end\n");
 		ret = -1;
 	    } else if (rows[i].cells[j] == NULL) {
-		dprintf("data_block: rows[%d].cells[%d] = NULL\n", i, j);
-		rows[i].cells[j] = g_strdup("-999.0");
+		dbprintf("data_block: rows[%d].cells[%d] = NULL\n", i, j);
+		rows[i].cells[j] = g_strdup("-999");
 		ret = -1;
 	    } else if (IS_STRING(rows[i].cells[j])) {
 		if (missval_string(rows[i].cells[j])) {
 		    free(rows[i].cells[j]);
-		    rows[i].cells[j] = g_strdup("-999.0");
+		    rows[i].cells[j] = g_strdup("-999");
 		    ret = -1;
 		} else {
 		    err->row = i + 1;
@@ -1161,6 +1245,7 @@ n_vars_from_col (wbook *book, int totcols, char *blank_col)
     int i, nv = 1;
 
     if (book_time_series(book) || book_obs_labels(book)) {
+	/* got a first non-data column */
 	offset++;
     }
 
@@ -1168,24 +1253,25 @@ n_vars_from_col (wbook *book, int totcols, char *blank_col)
 	if (!blank_col[i]) nv++;
     }
 
-    dprintf("n_vars_from_col: totcols=%d, nv=%d\n", totcols, nv);
+    dbprintf("n_vars_from_col: totcols=%d, nv=%d\n", totcols, nv);
 
     return nv;
 }
 
 static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo, 
-			    int totcols, char *blank_col)
+			    int totcols, char *blank_col, PRN *prn)
 {
     int startcol = book->col_offset;
     int roff = book->row_offset;
     int i, t, j = 1;
+    int err = 0;
 
-    if (book_time_series(book) || book_obs_labels(book)) {
+    if (book_obs_labels(book) || book_time_series(book)) {
 	startcol++;
     } 
 
-    for (i=startcol; i<=totcols; i++) {
-	int ts, missing = 0;
+    for (i=startcol; i<totcols; i++) {
+	int ts;
 
 	if (blank_col[i]) {
 	    continue;
@@ -1200,36 +1286,40 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 	    sprintf(pdinfo->varname[j], "v%d", j);
 	} else if (rows[roff].cells[i] == NULL) {
 	    sprintf(pdinfo->varname[j], "v%d", j);
+	} else if (i >= rows[roff].end) {
+	    sprintf(pdinfo->varname[j], "v%d", j);
 	} else {
 	    strncat(pdinfo->varname[j], rows[roff].cells[i] + 1, 
 		    VNAMELEN - 1);
-	    dprintf("accessing rows[%d].cells[%d] at %p\n",
-		    roff, i, (void *) rows[roff].cells[i]);
+	    dbprintf("accessing rows[%d].cells[%d] at %p\n",
+		     roff, i, (void *) rows[roff].cells[i]);
 	}
-	dprintf("set varname[%d] = '%s'\n", j, pdinfo->varname[j]);
+
+	/* remedial: replace space with underscore */
+	charsub(pdinfo->varname[j], ' ', '_');
+
+	err = check_varname(pdinfo->varname[j]);
+	if (err) {
+	    pprintf(prn, "%s\n", gretl_errmsg_get());
+	    break;
+	}
+
+	dbprintf("set varname[%d] = '%s'\n", j, pdinfo->varname[j]);
 
 	for (t=0; t<pdinfo->n; t++) {
-	    if (book->missmask != NULL) {
-		while (book->missmask[t]) {
-		    Z[j][t++] = NADBL;
-		    missing++;
-		}
-	    }
-
-	    ts = t + 1 + roff - missing;
-
+	    ts = t + 1 + roff;
 	    if (rows[ts].cells == NULL || i >= rows[ts].end ||
 		rows[ts].cells[i] == NULL) {
 		continue;
 	    }
 
-	    dprintf("accessing rows[%d].cells[%d] at %p\n", ts, i,
-		    (void *) rows[ts].cells[i]);
-	    dprintf("setting Z[%d][%d] = rows[%d].cells[%d] "
-		    "= '%s'\n", j, t, i, ts, rows[ts].cells[i]);
+	    dbprintf("accessing rows[%d].cells[%d] at %p\n", ts, i,
+		     (void *) rows[ts].cells[i]);
+	    dbprintf("setting Z[%d][%d] = rows[%d].cells[%d] "
+		     "= '%s'\n", j, t, i, ts, rows[ts].cells[i]);
 
 	    Z[j][t] = atof(rows[ts].cells[i]);
-	    if (Z[j][t] == -999.0) {
+	    if (Z[j][t] == -999 || Z[j][t] == -9999) {
 		Z[j][t] = NADBL;
 	    }
 	}
@@ -1237,12 +1327,12 @@ static int transcribe_data (wbook *book, double **Z, DATAINFO *pdinfo,
 	j++;
     }
 
-    return 0;
+    return err;
 }
 
 static int 
-get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
-		      PRN *prn)
+get_sheet_dimensions (wbook *book, int *totcols, int *datacols, 
+		      char **blank_col, PRN *prn)
 {
     char *blanks = NULL;
     int i, j;
@@ -1300,8 +1390,12 @@ get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
 	}
     }
 
-    printf("rows=%d, data cols=%d total cols=%d\n", nrows, 
-	   *datacols, *totcols);
+    if (book_numeric_dates(book)) {
+	*datacols -= 1;
+    }
+
+    printf("rows=%d, total cols=%d, data cols=%d\n", nrows, 
+	   *totcols, *datacols);
 
     if (*datacols < 1) {
 	pputs(prn, _("No data found.\n"));
@@ -1314,20 +1408,40 @@ get_sheet_dimensions (int *totcols, int *datacols, char **blank_col,
     return 0;
 }
 
-static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
+static int col0_is_numeric (int nrows, int row_offset, int col_offset)
 {
-    const char *s = rows[1 + book->row_offset].cells[book->col_offset];
+    int t, tstart = 1 + row_offset;
+    int nx = 0;
+    char *test;
 
-    if (book_numeric_dates(book)) {
-	int d0 = atoi(s);
+    fprintf(stderr, "testing for all numerical values in col %d\n", 
+	    col_offset);
 
-	MS_excel_date_string(newinfo->stobs, d0, pd, book_base_1904(book));
-    } else {
-	if (*s == '"' || *s == '\'') s++;
-	strcpy(newinfo->stobs, s);
-	colonize_obs(newinfo->stobs);
+    for (t=tstart; t<nrows; t++) {
+	test = rows[t].cells[col_offset];
+	if (!numeric_string(test)) {
+	    fprintf(stderr, " no: non-numeric cell at row %d\n", t + 1);
+	    return 0;
+	} else if (test != NULL && *test != '\0') {
+	    nx++;
+	}
     }
 
+    return nx > 0;
+}
+
+static int alpha_cell (const char *s)
+{
+    if (s != NULL) {
+	if (*s == '"' || *s == '\'') s++;
+	return isalpha(*s);
+    }
+
+    return 0;
+}
+
+static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
+{
     newinfo->pd = pd;
     newinfo->structure = TIME_SERIES;
 
@@ -1339,7 +1453,8 @@ static void book_time_series_setup (wbook *book, DATAINFO *newinfo, int pd)
     book_unset_obs_labels(book);
 }
 
-int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
+int xls_get_data (const char *fname, int *list, char *sheetname,
+		  double ***pZ, DATAINFO *pdinfo,
 		  gretlopt opt, PRN *prn)
 {
     int gui = (opt & OPT_G);
@@ -1350,6 +1465,10 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     int datacols, totcols;
     struct string_err strerr;
     char *blank_col = NULL;
+    int ts_markers = 0;
+    char **ts_S = NULL;
+    int r0, c0;
+    int merge = (*pZ != NULL);
     int i, t, pd = 0;
     int err = 0;
 
@@ -1359,9 +1478,13 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
+    if (sheetname != NULL) {
+	fprintf(stderr, "xls_get_data: sheetname='%s'\n", sheetname);
+    }
+
     gretl_push_c_numeric_locale();
 
-    wbook_init(book);
+    wbook_init(book, list, sheetname);
 
     if (excel_book_get_info(fname, book)) {
 	pputs(prn, _("Failed to get workbook info"));
@@ -1373,16 +1496,23 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	wbook_print_info(book);
     }
 
-    if (!err && gui) {
-	wsheet_menu(book, book->nsheets > 1);
-	if (book_debugging(book)) {
-	    debug_print = 1;
-	    print_version();
+    if (!err) {
+	if (gui) {
+	    wsheet_menu(book, book->nsheets > 1);
+	    if (book_debugging(book)) {
+		debug_print = 1;
+		print_version();
+	    }
+	} else {
+	    err = wbook_check_params(book);
+	    if (err) {
+		gretl_errmsg_set(_("Invalid argument for worksheet import"));
+	    }
 	}
     }
 
-    dprintf("sheet selected=%d; import offsets: col=%d, row=%d\n",
-	    book->selected, book->col_offset, book->row_offset);
+    dbprintf("sheet selected=%d; import offsets: col=%d, row=%d\n",
+	     book->selected, book->col_offset, book->row_offset);
 
     if (book->selected == -1) {
 	/* canceled */
@@ -1406,7 +1536,7 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* get sizes and locate any blank columns */
-    err = get_sheet_dimensions(&totcols, &datacols, &blank_col, prn);
+    err = get_sheet_dimensions(book, &totcols, &datacols, &blank_col, prn);
 
     if (err) goto getout;
 
@@ -1423,6 +1553,8 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 
     if (first_col_strings(book)) {
 	puts("found label strings in first imported column");
+    } else if (book_numeric_dates(book)) {
+	puts("found calendar dates in first imported column");
     } else {
 	puts("check for label strings in first imported column: not found");
     }
@@ -1457,45 +1589,50 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     } else if (err == -1) {
 	pputs(prn, _("Warning: there were missing values\n"));
 	err = 0;
-    }	    
+    }
 
-    /* do we have a first column containing dates? */
-    if (book_numeric_dates(book)) {
-	pd = pd_from_numeric_dates(nrows, book->row_offset, book->col_offset, 
-				   NULL, book);
-    } else if (!book_auto_varnames(book)) {
-	int r0 = book->row_offset;
-	int c0 = book->col_offset;
-	
-	if (obs_column_heading(rows[r0].cells[c0])) {
-	    pd = consistent_date_labels(nrows, r0, c0, NULL);
+    r0 = book->row_offset;
+    c0 = book->col_offset;
+    newinfo->n = nrows - 1 - book->row_offset;
+
+    if (book_numeric_dates(book) || 
+	(!book_auto_varnames(book) && import_obs_label(rows[r0].cells[c0]))) {
+	pd = importer_dates_check(book->row_offset + 1, book->col_offset, 
+				  &book->flags, NULL, newinfo, prn, &err);
+	if (pd > 0) {
+	    /* got time-series info from dates/labels */
+	    book_time_series_setup(book, newinfo, pd);
+	    ts_markers = newinfo->markers;
+	    ts_S = newinfo->S;
+	} else if (!book_numeric_dates(book) && alpha_cell(rows[r0].cells[c0]) && 
+	    col0_is_numeric(nrows, r0, c0)) {
+	    book_unset_obs_labels(book);
 	}
     }
 
-    if (pd) {
-	book_time_series_setup(book, newinfo, pd);
-    }    
-
-    /* number of variables and observations for import dataset */
+    /* dimensions of the dataset */
     newinfo->v = n_vars_from_col(book, totcols, blank_col);
-    newinfo->n = nrows - 1 - book->row_offset + book->totmiss;
     fprintf(stderr, "newinfo->v = %d, newinfo->n = %d\n",
 	    newinfo->v, newinfo->n);
 
     /* create import dataset */
-    err = start_new_Z(&newZ, newinfo, 0);
+    err = worksheet_start_dataset(&newZ, newinfo);
     if (err) {
 	goto getout;
     }
 
     if (book_time_series(book)) {
-	ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
+	newinfo->markers = ts_markers;
+	newinfo->S = ts_S;
     } else {
 	dataset_obs_info_default(newinfo);
     } 
 
     /* OK: actually populate the dataset */
-    transcribe_data(book, newZ, newinfo, totcols, blank_col);
+    err = transcribe_data(book, newZ, newinfo, totcols, blank_col, prn);
+    if (err) {
+	goto getout;
+    }
 
     if (fix_varname_duplicates(newinfo)) {
 	pputs(prn, _("warning: some variable names were duplicated\n"));
@@ -1513,8 +1650,20 @@ int xls_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+    if (book->flags & BOOK_DATA_REVERSED) {
+	reverse_data(newZ, newinfo, prn);
+    }
+
     err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
 
+    if (!err && !merge) {
+	dataset_add_import_info(pdinfo, fname, GRETL_XLS);
+    }
+
+    if (!err && gui) {
+	wbook_record_params(book, list);
+    }
+
  getout:
     
     free(blank_col);
