@@ -249,7 +249,7 @@ static void garch_iter_info (fcpinfo *f, int iter, double ll,
     pputs(prn, _("Parameters: "));
 
     for (i=0; i<f->npar; i++) {
-	x = f->theta[i];
+	x = f->parpre[i];
 	if (i < f->nc) {
 	    x *= f->scale;
 	} else if (i == f->nc) {
@@ -1214,7 +1214,6 @@ int garch_estimate (const double *y, const double **X,
 	fprintf(stderr, "*** Calling garch_info_matrix, round %d\n", it1);	    
 #endif
 	ll = garch_ll(f);
-	garch_iter_info(f, it1, ll, 0, prn);
 	for (i=0; i<npar; i++) {
 	    f->parpre[i] = f->theta[i];
 	}
@@ -1224,6 +1223,8 @@ int garch_estimate (const double *y, const double **X,
 	    goto garch_exit;
 	}
 
+	garch_iter_info(f, it1, ll, 0, prn);
+
 	if (converged(f, tol1)) {
 	    break;
 	}
