@@ -722,8 +722,16 @@ static void heckit_yhat_uhat (MODEL *hm, h_container *HC,
     double lnsig = log(HC->sigma);
     int kb = HC->kmain;
     int kg = HC->ksel;
+    double *llt;
     int i, t, v;
 
+    llt = malloc(pdinfo->n * sizeof *llt);
+    if (llt != NULL) {
+	for (i=0; i<pdinfo->n; i++) {
+	    llt[i] = NADBL;
+	}
+    }
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	xb = 0.0;
 	for (i=0; i<kb; i++) {
@@ -762,16 +770,26 @@ static void heckit_yhat_uhat (MODEL *hm, h_container *HC,
 	    /* censored */
 	    F = normal_cdf(-zg);
 	    hm->uhat[t] = -lam * invmills(zg);
-	    hm->llt[t] = log(F);
+	    if (llt != NULL) {
+		llt[t] = log(F);
+	    }
 	} else if (!na(xb) && Z[HC->selvar][t] == 1) {
 	    /* uncensored */
 	    hm->uhat[t] = u = Z[HC->depvar][t] - xb;
-	    u /= HC->sigma;
-	    x = (zg + rho*u) * rhofunc;
-	    hm->llt[t] = -LN_SQRT_2_PI - 0.5*u*u - lnsig + log(normal_cdf(x));
+	    if (llt != NULL) {
+		u /= HC->sigma;
+		x = (zg + rho*u) * rhofunc;
+		llt[t] = -LN_SQRT_2_PI - 0.5*u*u - lnsig + log(normal_cdf(x));
+	    }
 	}
     }
 
+    if (llt != NULL) {
+	gretl_model_set_data(hm, "llt", llt, 
+			     GRETL_TYPE_DOUBLE_ARRAY,
+			     pdinfo->n * sizeof *llt);
+    }
+
 #if 0
     /* R-squared based on correlation? */
     x = gretl_corr(hm->t1, hm->t2, Z[depvar], hm->yhat, NULL);
@@ -1060,7 +1078,7 @@ static gretl_matrix *heckit_init_H (double *theta,
 
 #endif
 
-int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
+int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
 {
     gretl_matrix *init_H = NULL;
     int maxit, fncount, grcount;
@@ -1133,9 +1151,24 @@ int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 	    }
 	}
 
+	if (opt & OPT_R) {
+	    gretl_matrix *GG = gretl_matrix_XTX_new(HC->score);
+	    gretl_matrix *tmp = gretl_matrix_alloc(np, np);
+
+	    if (GG == NULL || tmp == NULL) {
+		err = E_ALLOC;
+	    } else {
+		gretl_matrix_qform(HC->vcv, GRETL_MOD_NONE, GG, tmp, GRETL_MOD_NONE);
+		gretl_matrix_copy_values(HC->vcv, tmp);
+	    }
+	    gretl_matrix_free(tmp);
+	    gretl_matrix_free(GG);
+	}
+    }
+
+    if (!err) {
 	adjust_ml_vcv_hyperbolic(HC);
 	add_lambda_to_ml_vcv(HC);
-
 #if HDEBUG
 	for (i=0; i<np; i++) {
 	    hij = gretl_matrix_get(HC->vcv, i, i);
@@ -1151,7 +1184,7 @@ int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 }
 
 /*
-  This function just copies the VCV matrix for the ML estimator into
+  This function copies the VCV matrix for the ML estimator into
   the model struct.  Note that we don't transcribe the variances for
   sigma and rho into the model.
 */
@@ -1223,7 +1256,7 @@ static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
     }
 
     /* run initial auxiliary probit */
-    probmod = binary_probit(sellist, pZ, pdinfo, OPT_A, NULL);
+    probmod = binary_probit(sellist, *pZ, pdinfo, OPT_A, NULL);
     if (probmod.errcode) {
 	hm.errcode = probmod.errcode;
 	goto bailout;
@@ -1257,7 +1290,7 @@ static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
     gretl_list_append_term(&reglist, HC->millsvar);
 
     /* run OLS including Mills variable */
-    hm = lsq(reglist, pZ, pdinfo, OLS, OPT_A);
+    hm = lsq(reglist, *pZ, pdinfo, OLS, OPT_A);
     if (!hm.errcode) {
 	hm.ci = HECKIT;
 	gretl_model_set_int(&hm, "totobs", probmod.nobs); 
@@ -1329,7 +1362,7 @@ MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
 	err = heckit_2step_vcv(HC, &hm);
     } else {
 	/* use MLE */
-	err = heckit_ml(&hm, HC, vprn);
+	err = heckit_ml(&hm, HC, opt, vprn);
 	if (!err) {
 	    err = transcribe_ml_vcv(&hm, HC);
 	}
@@ -1351,6 +1384,11 @@ MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
 	hm.opt |= OPT_T;
     }
 
+    if (opt & OPT_R) {
+	gretl_model_set_vcv_info(&hm, VCV_ML, VCV_QML);
+	hm.opt |= OPT_R;
+    }
+
     if (pdinfo->v > oldv) {
 	dataset_drop_last_variables(1, pZ, pdinfo);
     }
