@@ -518,6 +518,7 @@ static void switcher_free (switcher *s)
 static int make_lsPi (Jwrap *J)
 {
     gretl_matrix *S11i;
+    int err = 0;
 
     if (J->lsPi != NULL) {
 	/* already done */
@@ -535,15 +536,19 @@ static int make_lsPi (Jwrap *J)
 	return E_ALLOC;
     }
 
-    gretl_invert_symmetric_matrix(S11i);
-    gretl_matrix_multiply_mod(S11i, GRETL_MOD_NONE,
-			      J->S01, GRETL_MOD_TRANSPOSE,
-			      J->lsPi, GRETL_MOD_NONE);
-    /* make into vec(\Pi'_{LS}) */
-    gretl_matrix_reuse(J->lsPi, J->p1 * J->p, 1);
+    err = gretl_invert_symmetric_matrix(S11i);
+
+    if (!err) {
+	gretl_matrix_multiply_mod(S11i, GRETL_MOD_NONE,
+				  J->S01, GRETL_MOD_TRANSPOSE,
+				  J->lsPi, GRETL_MOD_NONE);
+	/* make into vec(\Pi'_{LS}) */
+	gretl_matrix_reuse(J->lsPi, J->p1 * J->p, 1);
+    }
+
     gretl_matrix_free(S11i);
 
-    return 0;
+    return err;
 }
 
 static int switcher_init (switcher *s, Jwrap *J)
@@ -652,10 +657,12 @@ static int update_psi (Jwrap *J, switcher *s)
     gretl_matrix_multiply(s->TmpR1, J->lsPi, J->psi);
 
     /* combine */
-    gretl_cholesky_decomp_solve(J->I00, J->psi);
+    err = gretl_cholesky_decomp_solve(J->I00, J->psi);
 
     /* update alpha */
-    alpha_from_psi(J);
+    if (!err) {
+	alpha_from_psi(J);
+    }
 
     return err;
 }
@@ -724,23 +731,28 @@ static int update_phi (Jwrap *J, switcher *s)
     }
 
     /* combine first and second chunks */
-    gretl_cholesky_decomp_solve(J->I11, s->TmpL);
-
-    /* right-hand chunk */
-    gretl_matrix_copy_values(s->TmpR, J->lsPi);
-    if (J->h != NULL && !gretl_is_zero_matrix(J->h)) {
-	gretl_matrix_reuse(s->K2, J->p * J->p1, J->r * J->p1);
-	gretl_matrix_kronecker_I(J->alpha, J->p1, s->K2);
-	gretl_matrix_multiply_mod(s->K2, GRETL_MOD_NONE,
-				  J->h, GRETL_MOD_NONE,
-				  s->TmpR, GRETL_MOD_DECREMENT);
+    err = gretl_cholesky_decomp_solve(J->I11, s->TmpL);
+    if (err) {
+	fprintf(stderr, "cholesky decomp failed in update_phi\n");
     }
 
-    /* combine */
-    gretl_matrix_multiply(s->TmpL, s->TmpR, J->phi);
+    if (!err) {
+	/* right-hand chunk */
+	gretl_matrix_copy_values(s->TmpR, J->lsPi);
+	if (J->h != NULL && !gretl_is_zero_matrix(J->h)) {
+	    gretl_matrix_reuse(s->K2, J->p * J->p1, J->r * J->p1);
+	    gretl_matrix_kronecker_I(J->alpha, J->p1, s->K2);
+	    gretl_matrix_multiply_mod(s->K2, GRETL_MOD_NONE,
+				      J->h, GRETL_MOD_NONE,
+				      s->TmpR, GRETL_MOD_DECREMENT);
+	}
 
-    /* update beta */
-    beta_from_phi(J);
+	/* combine */
+	gretl_matrix_multiply(s->TmpL, s->TmpR, J->phi);
+
+	/* update beta */
+	beta_from_phi(J);
+    }
 
     return err;
 }
@@ -1233,14 +1245,21 @@ static int switchit (Jwrap *J, PRN *prn)
 	llbak = J->ll;
     }
 
-    pprintf(prn, _("Switching algorithm: %d iterations"), j);
-    if (uinit) {
-	pprintf(prn, " (%s)", _("user-supplied initial values"));
-    }
-    pputc(prn, '\n');
-    pprintf(prn, " -(T/2)log|Omega| = %.8g, lldiff = %g\n", J->ll, lldiff);
+    if (err) {
+	pputs(prn, _("Switching algorithm: failed"));
+	pputc(prn, '\n');
+    } else {
+	pprintf(prn, _("Switching algorithm: %d iterations"), j);
+
+	if (uinit) {
+	    pprintf(prn, " (%s)", _("user-supplied initial values"));
+	}
+	pputc(prn, '\n');
+
+	if (!na(J->ll) && !na(lldiff)) {
+	    pprintf(prn, " -(T/2)log|Omega| = %.8g, lldiff = %g\n", J->ll, lldiff);
+	}
 
-    if (!err) {
 	if (conv == 1) {
 	    pputs(prn, "Strong convergence\n"); 
 	} else if (conv == 2) {
@@ -2877,6 +2896,11 @@ static int simann (Jwrap *J, gretlopt opt, PRN *prn)
     int improved = 0;
     int err = 0;
 
+    if (b == NULL) {
+	fprintf(stderr, "simann: J->theta = NULL\n");
+	return E_DATA;
+    }
+
     b0 = gretl_matrix_copy(b);
     b1 = gretl_matrix_copy(b);
     bstar = gretl_matrix_copy(b);
@@ -3031,8 +3055,8 @@ static int make_theta (Jwrap *J)
     }
 
     if (k == 0) {
-	/* can't happen */
-	return 0;
+	fprintf(stderr, "jrestrict: make_theta: k = 0\n");
+	return E_DATA;
     }
 
     J->theta = gretl_column_vector_alloc(k);
@@ -3145,6 +3169,11 @@ int general_vecm_analysis (GRETL_VAR *jvar,
 
     if (!err) {
 	err = vecm_id_check(J, jvar, prn);
+	if (J->df == 0) {
+	    fprintf(stderr, "warning: test df = 0\n");
+	    J->flags &= ~J_USE_LBFGS;
+	    do_simann = 0;
+	}
     }
 
     if (!err && J->blen == 0 && J->G == NULL) {
