@@ -25,6 +25,7 @@
 #include "libgretl.h"
 #include "gretl_f2c.h"
 #include "usermat.h"
+#include "matrix_extra.h"
 #include "libset.h"
 
 #include <errno.h>
@@ -47,7 +48,8 @@ extern int rqfnb_ (integer *n,    /* number of observations */
 		   double *wn,     /* work array, length n */
 		   double *wp,     /* work array, length p */
 		   integer *nit,   /* iteration counts */
-		   integer *info); /* exit status */
+		   integer *info, /* exit status */
+		   void (*callback)(void));
 
 /* Modified simplex, a la Barrodale-Roberts: this variant lets us get
    confidence intervals using the rank-inversion method.
@@ -74,7 +76,8 @@ extern int rqbr_ (int n,         /* number of observations */
 		  double *tnmat, /* matrix of JGPK rank-test statistics */
 		  double big,    /* "large positive floating-point number" */
 		  int rmax,      /* max iterations (added for gretl) */
-		  int ci1);      /* doing confidence intervals? */
+		  int ci1,       /* doing confidence intervals? */
+		  void (*callback)(void));
 
 /* machine precision to the 2/3 */
 #define calc_eps23 (pow(2.22045e-16, 2/3.0))
@@ -100,6 +103,7 @@ struct br_info {
     int *s, *h;
     gretl_matrix *ci;
     gretl_matrix *tnmat;
+    void (*callback)();
 };
 
 /* wrapper struct for use with Frisch-Newton method */
@@ -117,6 +121,7 @@ struct fn_info {
     double *coeff;
     integer nit[3];
     integer info;
+    void (*callback)();
 };
 
 /* destructors for the above wrappers */
@@ -182,7 +187,9 @@ static double rq_loglik (MODEL *pmod, double tau)
     int t;
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	R += pmod->uhat[t] * (tau - (pmod->uhat[t] < 0));
+	if (!na(pmod->uhat[t])) {
+	    R += pmod->uhat[t] * (tau - (pmod->uhat[t] < 0));
+	}
     }
 
     return n * (log(tau * (1-tau)) - 1 - log(R/n));
@@ -190,7 +197,8 @@ static double rq_loglik (MODEL *pmod, double tau)
 
 enum {
     RQ_STAGE_1,
-    RQ_STAGE_2
+    RQ_STAGE_2,
+    RQ_LAD
 };
 
 /* Transcribe quantreg results into model struct.  Note: we have to be
@@ -208,8 +216,7 @@ static void rq_transcribe_results (MODEL *pmod,
 				   int stage)
 {
     double SAR = 0.0;
-    int n = y->rows;
-    int i, t;
+    int i, s, t;
 
     if (stage == RQ_STAGE_1) {
 	gretl_model_set_double(pmod, "tau", tau);
@@ -217,18 +224,22 @@ static void rq_transcribe_results (MODEL *pmod,
 
     for (i=0; i<pmod->ncoeff; i++) {
 	pmod->coeff[i] = b[i];
-	if (stage == RQ_STAGE_1) {
+	if (stage == RQ_STAGE_1 || stage == RQ_LAD) {
 	    pmod->sderr[i] = NADBL; /* these will be replaced later */
 	}
     }
 
     pmod->ess = 0.0;
 
-    for (i=0, t=pmod->t1; i<n; i++, t++) {
-	pmod->uhat[t] = u[i];
-	pmod->yhat[t] = y->val[i] - u[i];
-	SAR += fabs(u[i]);
-	pmod->ess += u[i] * u[i];
+    s = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->yhat[t])) {
+	    pmod->uhat[t] = u[s];
+	    pmod->yhat[t] = y->val[s] - u[s];
+	    SAR += fabs(u[s]);
+	    pmod->ess += u[s] * u[s];
+	    s++;
+	}
     }
 
     /* sum of absolute residuals */
@@ -598,11 +609,14 @@ static int br_info_alloc (struct br_info *rq, int n, int p,
 	return E_ALLOC;
     }
 
-    rq->ci = gretl_matrix_alloc(4, p);
-    rq->tnmat = gretl_matrix_alloc(4, p);
+    if (!(opt & OPT_L)) {
+	/* doing confidence intervals */
+	rq->ci = gretl_matrix_alloc(4, p);
+	rq->tnmat = gretl_matrix_alloc(4, p);
 
-    if (rq->ci == NULL || rq->tnmat == NULL) {
-	return E_ALLOC;
+	if (rq->ci == NULL || rq->tnmat == NULL) {
+	    return E_ALLOC;
+	}
     }
 
     /* real arrays */
@@ -626,13 +640,22 @@ static int br_info_alloc (struct br_info *rq, int n, int p,
     rq->big = NADBL / 100;
     rq->rmax = libset_get_int(RQ_MAXITER);
 
-    if (opt & OPT_N) {
-	/* no df correction */
+    if (opt & OPT_L) {
+	/* doing simple LAD */
+	rq->cut = 0;
+    } else if (opt & OPT_N) {
+	/* asymptotic: no df correction */
 	rq->cut = normal_cdf_inverse(1 - alpha/2);
     } else {
 	rq->cut = student_cdf_inverse(n - p, 1 - alpha/2);
     }
 
+    if (show_activity_func_installed()) {
+	rq->callback = show_activity_callback;
+    } else {
+	rq->callback = NULL;
+    }
+
     return 0;
 }
 
@@ -642,17 +665,24 @@ static int real_br_calc (gretl_matrix *y, gretl_matrix *X,
 			 double tau, struct br_info *rq,
 			 int calc_ci)
 {
+    double *ci_val, *tnmat_val;
     int ret, err = 0;
 
 #if QDEBUG
     fprintf(stderr, "real_br_calc: calling rqbr, calc_ci = %d\n", calc_ci);
 #endif
 
+    /* these inputs are not needed if we're not computing
+       confidence intervals */
+    ci_val = (rq->ci == NULL)? NULL : rq->ci->val; 
+    tnmat_val = (rq->tnmat == NULL)? NULL : rq->tnmat->val; 
+
     ret = rqbr_(rq->n, rq->p, X->val, y->val, tau, rq->tol, 
 		rq->coeff, rq->resid, rq->s, rq->wa, rq->wb, 
 		rq->sol, rq->dsol, rq->h, rq->qn, rq->cut, 
-		rq->ci->val, rq->tnmat->val, 
-		rq->big, rq->rmax, calc_ci);
+		ci_val, tnmat_val, 
+		rq->big, rq->rmax, calc_ci,
+		rq->callback);
 
 #if QDEBUG
     fprintf(stderr, "rqbr: ift = %d\n", ret);
@@ -709,6 +739,12 @@ static int fn_info_alloc (struct fn_info *rq, int n, int p,
     rq->beta = .99995;
     rq->eps = 1.0e-7;
 
+    if (show_activity_func_installed()) {
+	rq->callback = show_activity_callback;
+    } else {
+	rq->callback = NULL;
+    }
+
     return 0;
 }
 
@@ -752,7 +788,8 @@ static void rq_call_FN (integer *n, integer *p, gretl_matrix *XT,
 
     rqfnb_(n, p, XT->val, y->val, rq->rhs, 
 	   rq->d, rq->u, &rq->beta, &rq->eps, 
-	   rq->resid, rq->coeff, rq->nit, &rq->info);
+	   rq->resid, rq->coeff, rq->nit, &rq->info, 
+	   rq->callback);
 }
 
 static int rq_write_variance (const gretl_matrix *V,
@@ -1064,8 +1101,8 @@ write_tbeta_block_fn (gretl_matrix *tbeta, int nt, double *x,
     return 0;
 }
 
-/* sub-driver for Barrodale-Roberts estimation, with confidence
-   intervals
+/* Sub-driver for Barrodale-Roberts estimation, with confidence
+   intervals.
 */
 
 static int rq_fit_br (gretl_matrix *y, gretl_matrix *X, 
@@ -1076,13 +1113,12 @@ static int rq_fit_br (gretl_matrix *y, gretl_matrix *X,
     gretl_matrix *tbeta = NULL;
     integer n = y->rows;
     integer p = X->cols;
-    double tau, alpha;
+    double tau, alpha = 0;
     int i, ntau;
     int err = 0;
 
     err = get_ci_alpha(&alpha);
     if (err) {
-	/* return right away: don't call br_info_free! */
 	return err;
     }
 
@@ -1140,7 +1176,7 @@ static int rq_fit_br (gretl_matrix *y, gretl_matrix *X,
 		/* using multiple tau values */
 		err = write_tbeta_block_br(tbeta, ntau, rq.coeff, rq.ci, i);
 	    }
-	}
+	} 
     }
 
     if (!err && rq.warning) {
@@ -1249,21 +1285,24 @@ static int rq_fit_fn (gretl_matrix *y, gretl_matrix *XT,
 
 /* Write y and X from pmod into gretl matrices, respecting the sample
    range.  Note that for use with the Frisch-Newton version of the
-   underlying rq function we want the X matrix in transpose form.
+   underlying rq function we want the X matrix in transpose form,
+   which is signaled by @tr = 1. Also note that @pmod was first
+   estimated via OLS and its sample range may contain interior
+   NAs, in which case we have to avoid them when filling the
+   data matrices.
 */
 
 static int rq_make_matrices (MODEL *pmod,
 			     double **Z, DATAINFO *pdinfo,
 			     gretl_matrix **py,
 			     gretl_matrix **pX,
-			     gretlopt opt)
+			     int tr)
 {
     int n = pmod->nobs;
     int p = pmod->ncoeff;
     int yno = pmod->list[1];
     gretl_matrix *X = NULL;
     gretl_matrix *y = NULL;
-    int tr = !(opt & OPT_I);
     int i, s, t, v;
     int err = 0;
 
@@ -1283,17 +1322,21 @@ static int rq_make_matrices (MODEL *pmod,
 
     s = 0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	gretl_vector_set(y, s++, Z[yno][t]);
+	if (!na(pmod->uhat[t])) {
+	    gretl_vector_set(y, s++, Z[yno][t]);
+	}
     }
 
     for (i=0; i<p; i++) {
 	v = pmod->list[i+2];
 	s = 0;
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    if (tr) {
-		gretl_matrix_set(X, i, s++, Z[v][t]);
-	    } else {
-		gretl_matrix_set(X, s++, i, Z[v][t]);
+	    if (!na(pmod->uhat[t])) {
+		if (tr) {
+		    gretl_matrix_set(X, i, s++, Z[v][t]);
+		} else {
+		    gretl_matrix_set(X, s++, i, Z[v][t]);
+		}
 	    }
 	}
     }
@@ -1371,16 +1414,18 @@ int rq_driver (const gretl_matrix *tau, MODEL *pmod,
 	       using F-N so the model will be equipped with standard
 	       errors
 	    */
-	    err = rq_make_matrices(pmod, Z, pdinfo, &y, &X, OPT_NONE);
+	    err = rq_make_matrices(pmod, Z, pdinfo, &y, &X, 1);
 	    if (!err) {
 		err = rq_fit_fn(y, X, tau, opt, pmod);
 	    }
 	    if (!err) {
-		/* for F-N the X matrix is transposed */
+		/* flip the X matrix for use with B-R */
 		err = rq_transpose_X(&X);
 	    }
 	} else {
-	    err = rq_make_matrices(pmod, Z, pdinfo, &y, &X, opt);
+	    int tr = !(opt & OPT_I);
+
+	    err = rq_make_matrices(pmod, Z, pdinfo, &y, &X, tr);
 	}
     }
 
@@ -1416,3 +1461,226 @@ int rq_driver (const gretl_matrix *tau, MODEL *pmod,
 
     return err;
 }
+
+/* restock the y and X matrices with a bootstrap sample */
+
+static void rq_refill_matrices (MODEL *pmod,
+				double **Z, 
+				gretl_matrix *y,
+				gretl_matrix *X,
+				int *sample)
+{
+    int n = pmod->nobs;
+    int p = pmod->ncoeff;
+    int yno = pmod->list[1];
+    int i, j, t, v;
+
+    for (i=0; i<n; i++) {
+	t = sample[i];
+	gretl_vector_set(y, i, Z[yno][t]);
+    }
+
+    for (j=0; j<p; j++) {
+	v = pmod->list[j+2];
+	for (i=0; i<n; i++) {
+	    t = sample[i];
+	    gretl_matrix_set(X, i, j, Z[v][t]);
+	}
+    }
+}
+
+static int *good_observations_array (MODEL *pmod)
+{
+    int *g = malloc(pmod->nobs * sizeof *g);
+
+    if (g != NULL) {
+	int t, s = 0;
+
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (!na(pmod->uhat[t])) {
+		g[s++] = t;
+	    }
+	}
+    }
+
+    return g;
+}
+
+#define ITERS 500
+
+/* obtain bootstrap estimates of LAD covariance matrix */
+
+static int lad_bootstrap_vcv (MODEL *pmod, double **Z,
+			      gretl_matrix *y, gretl_matrix *X,
+			      struct br_info *rq)
+{
+    double **coeffs = NULL;
+    double *meanb = NULL;
+    int *sample = NULL;
+    int *goodobs = NULL;
+    double xi, xj;
+    int i, j, k;
+    int nc = pmod->ncoeff;
+    int nvcv, n = pmod->nobs;
+    int err = 0;
+
+    /* note: new_vcv sets all entries to zero */
+    err = gretl_model_new_vcv(pmod, &nvcv);
+    if (err) {
+	return err;
+    }
+
+    /* an array for each coefficient */
+    coeffs = doubles_array_new(nc, ITERS);
+
+    /* a scalar for each coefficient mean */
+    meanb = malloc(nc * sizeof *meanb);
+
+    /* resampling array of length pmod->nobs */
+    sample = malloc(n * sizeof *sample);
+
+    if (coeffs == NULL || meanb == NULL || sample == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }   
+
+    /* apparatus for handling interior NAs */
+    if (model_has_missing_obs(pmod)) {
+	goodobs = good_observations_array(pmod);
+	if (goodobs == NULL) {
+	    err = E_ALLOC;
+	    goto bailout;
+	} 
+    }
+
+    for (k=0; k<ITERS && !err; k++) {
+	/* create random sample index array */
+	for (i=0; i<n; i++) {
+	    j = gretl_rand_int_max(n);
+	    if (goodobs != NULL) {
+		sample[i] = goodobs[j];
+	    } else {
+		sample[i] = pmod->t1 + j;
+	    }
+	}
+
+	rq_refill_matrices(pmod, Z, y, X, sample);
+
+	/* re-estimate LAD model */
+	err = real_br_calc(y, X, 0.5, rq, 0);
+
+	if (!err) {
+	    for (i=0; i<nc; i++) {
+		coeffs[i][k] = rq->coeff[i];
+	    }
+	}
+    }
+
+    /* find means of coeff estimates */
+    for (i=0; i<nc && !err; i++) {
+	double bbar = 0.0;
+
+	for (k=0; k<ITERS; k++) {
+	   bbar += coeffs[i][k];
+	} 
+	meanb[i] = bbar / ITERS;
+    }    
+
+    /* find variances and covariances */
+    for (i=0; i<nc && !err; i++) {
+	double vi = 0.0;
+
+	for (k=0; k<ITERS; k++) {
+	    xi = coeffs[i][k] - meanb[i];
+	    vi += xi * xi;
+	    for (j=0; j<=i; j++) {
+		xj = coeffs[j][k] - meanb[j];
+		pmod->vcv[ijton(i, j, nc)] += xi * xj;
+	    }
+	}
+	pmod->sderr[i] = sqrt(vi / ITERS);
+    }
+
+    if (!err) {
+	for (i=0; i<nvcv; i++) {
+	    pmod->vcv[i] /= ITERS;
+	}
+    }
+
+ bailout:
+
+    free(sample);
+    free(meanb);
+    doubles_array_free(coeffs, nc);
+
+    if (goodobs != NULL) {
+	free(goodobs);
+    }
+
+    return err;
+}
+
+static int lad_fit_br (MODEL *pmod, double **Z,
+		       gretl_matrix *y, gretl_matrix *X)
+{
+    struct br_info rq;
+    integer n = y->rows;
+    integer p = X->cols;
+    int err = 0;
+
+    err = br_info_alloc(&rq, n, p, 0.5, 0.0, OPT_L);
+
+    if (!err) {
+	/* get the actual estimates */
+	err = real_br_calc(y, X, 0.5, &rq, 0);
+    }
+
+    if (!err) {
+	rq_transcribe_results(pmod, y, 0.5, rq.coeff, 
+			      rq.resid, RQ_LAD);
+	if (rq.warning) {
+	    gretl_model_set_int(pmod, "nonunique", 1);
+	}
+    }
+
+    if (!err) {
+	err = lad_bootstrap_vcv(pmod, Z, y, X, &rq);
+    }
+
+    br_info_free(&rq);
+
+    return err;
+}
+
+/* Support the "lad" command: estimate with tau = 0.5, using
+   the Barrodale-Roberts method, and add a bootstrapped
+   covariance matrix.
+*/
+
+int lad_driver (MODEL *pmod, double **Z, DATAINFO *pdinfo)
+{
+    gretl_matrix *y = NULL;
+    gretl_matrix *X = NULL;
+    int err;
+
+    err = rq_make_matrices(pmod, Z, pdinfo, &y, &X, 0);
+
+    if (!err) {
+	err = lad_fit_br(pmod, Z, y, X);
+    }
+
+    if (!err) {
+	gretl_model_add_y_median(pmod, Z[pmod->list[1]]);
+	pmod->ci = LAD;
+    }
+
+    gretl_matrix_free(y);
+    gretl_matrix_free(X);
+
+    if (err && pmod->errcode == 0) {
+	pmod->errcode = err;
+    }
+
+    return err;
+}
+
