@@ -3,10 +3,10 @@
 
 double *model_get_Xi (const MODEL *pmod, double **Z, int i);
 
-int fiml_driver (equation_system *sys, double ***pZ, 
+int fiml_driver (equation_system *sys, double **Z, 
 		 DATAINFO *pdinfo, gretlopt opt, PRN *prn);
 
-int liml_driver (equation_system *sys, double ***pZ, 
+int liml_driver (equation_system *sys, double **Z, 
 		 DATAINFO *pdinfo, PRN *prn);
 
 #endif /* SYSML_H_ */
