@@ -25,7 +25,7 @@
 #include <gtk/gtk.h>
 #include "tramo_x12a.h"
 
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 18)
 # include "gtk_compat.h"
 #endif
 
@@ -165,9 +165,127 @@ static int glib_spawn (const char *workdir, const char *fmt, ...)
 
 #endif /* !WIN32 */
 
+static void toggle_outliers (GtkToggleButton *b, tx_request *request)
+{
+    request->xopt.outliers = gtk_toggle_button_get_active(b);
+}
+
+static void toggle_trading_days (GtkToggleButton *b, tx_request *request)
+{
+    request->xopt.trdays = gtk_toggle_button_get_active(b);
+}
+
+static void set_logtrans (GtkButton *b, tx_request *request)
+{
+    gpointer p = g_object_get_data(G_OBJECT(b), "transval");
+
+    request->xopt.logtrans = GPOINTER_TO_INT(p);
+}
+
+static void show_x12a_options (tx_request *request, GtkBox *vbox)
+{
+    GtkWidget *tmp, *b[3];
+    GSList *group;
+
+    tmp = gtk_label_new(_("X-12-ARIMA options"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, TRUE, TRUE, 5);
+
+    tmp = gtk_check_button_new_with_label(_("Detect and correct for outliers"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), request->xopt.outliers);
+    g_signal_connect(GTK_TOGGLE_BUTTON(tmp), "toggled",
+		     G_CALLBACK(toggle_outliers), request);
+
+    tmp = gtk_check_button_new_with_label(_("Correct for trading days effect"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), request->xopt.trdays);
+    g_signal_connect(GTK_TOGGLE_BUTTON(tmp), "toggled",
+		     G_CALLBACK(toggle_trading_days), request);
+
+    tmp = gtk_hseparator_new();
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 5);
+
+    b[0] = gtk_radio_button_new_with_label(NULL, _("Log transformation"));
+    gtk_widget_show(b[0]);
+    gtk_box_pack_start(vbox, b[0], FALSE, FALSE, 0);
+    g_signal_connect(GTK_TOGGLE_BUTTON(b[0]), "toggled",
+		     G_CALLBACK(set_logtrans), request);
+    g_object_set_data(G_OBJECT(b[0]), "transval", GINT_TO_POINTER(1));
+
+    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b[0]));
+    b[1] = gtk_radio_button_new_with_label(group, _("No log transformation"));
+    gtk_widget_show(b[1]);
+    gtk_box_pack_start(vbox, b[1], FALSE, FALSE, 0);
+    g_signal_connect(GTK_TOGGLE_BUTTON(b[1]), "toggled",
+		     G_CALLBACK(set_logtrans), request);
+    g_object_set_data(G_OBJECT(b[1]), "transval", GINT_TO_POINTER(2));
+
+    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b[1]));
+    b[2] = gtk_radio_button_new_with_label(group, _("Automatic"));
+    gtk_widget_show(b[2]);
+    gtk_box_pack_start(vbox, b[2], FALSE, FALSE, 0);
+    g_signal_connect(GTK_TOGGLE_BUTTON(b[2]), "toggled",
+		     G_CALLBACK(set_logtrans), request);
+    g_object_set_data(G_OBJECT(b[2]), "transval", GINT_TO_POINTER(3));
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b[request->xopt.logtrans - 1]), 
+				 TRUE); 
+
+    tmp = gtk_hseparator_new();
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 5);
+
+    tmp = gtk_label_new(_("Save data"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+
+    tmp = gtk_check_button_new_with_label(_("Seasonally adjusted series"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    request->opts[TX_SA].check = tmp;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+				 (*request->popt & OPT_A)? TRUE : FALSE);
+
+    tmp = gtk_check_button_new_with_label(_("Trend/cycle"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    request->opts[TX_TR].check = tmp;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+				 (*request->popt & OPT_B)? TRUE : FALSE);
+
+    tmp = gtk_check_button_new_with_label(_("Irregular"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    request->opts[TX_IR].check = tmp;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+				 (*request->popt & OPT_C)? TRUE : FALSE);
+
+    tmp = gtk_hseparator_new();
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 5);
+    
+    tmp = gtk_check_button_new_with_label(_("Generate graph"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    request->opts[TRIGRAPH].check = tmp;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+				 (*request->popt & OPT_G)? TRUE : FALSE);
+
+    tmp = gtk_check_button_new_with_label(_("Show full output"));
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
+    request->opts[TEXTOUT].check = tmp;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp),
+				 (*request->popt & OPT_Q)? FALSE : TRUE);
+}
+
 static int tx_dialog (tx_request *request)
 {
-    GtkWidget *hbox, *vbox, *tmp;
+    GtkWidget *hbox, *vbox;
     gint i, ret = 0;
 
     for (i=0; i<TX_MAXOPT; i++) {
@@ -192,49 +310,7 @@ static int tx_dialog (tx_request *request)
 	gtk_dialog_set_has_separator(GTK_DIALOG(request->dialog), FALSE);
 	show_tramo_options(request, vbox);
     } else {
-	/* X-12-ARIMA */
-	tmp = gtk_label_new(_("Save data"));
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-
-	tmp = gtk_check_button_new_with_label(_("Seasonally adjusted series"));
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-	request->opts[TX_SA].check = tmp;
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
-				     (*request->popt & OPT_A)? TRUE : FALSE);
-
-	tmp = gtk_check_button_new_with_label(_("Trend/cycle"));
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-	request->opts[TX_TR].check = tmp;
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
-				     (*request->popt & OPT_B)? TRUE : FALSE);
-
-	tmp = gtk_check_button_new_with_label(_("Irregular"));
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-	request->opts[TX_IR].check = tmp;
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
-				     (*request->popt & OPT_C)? TRUE : FALSE);
-
-	tmp = gtk_hseparator_new();
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-    
-	tmp = gtk_check_button_new_with_label(_("Generate graph"));
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-	request->opts[TRIGRAPH].check = tmp;
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
-				     (*request->popt & OPT_G)? TRUE : FALSE);
-
-	tmp = gtk_check_button_new_with_label(_("Show full output"));
-	gtk_widget_show(tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
-	request->opts[TEXTOUT].check = tmp;
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp),
-				     (*request->popt & OPT_Q)? FALSE : TRUE);
+	show_x12a_options(request, GTK_BOX(vbox));
     }
 
     gtk_widget_show(vbox);
@@ -303,7 +379,7 @@ static int graph_series (const double **Z, const DATAINFO *pdinfo,
     char title[32];
     int t, err = 0;
 
-    obs = gretl_plotx(pdinfo);
+    obs = gretl_plotx(NULL, pdinfo);
     if (obs == NULL) {
 	return E_ALLOC;
     }
@@ -672,12 +748,16 @@ static int grab_deseasonal_series (double *y, const DATAINFO *pdinfo,
     return err;
 }
 
-static void request_opts_init (tx_request *request)
+static void request_opts_init (tx_request *request, const DATAINFO *pdinfo)
 {
     int i;
 
     request->savevars = 0;
 
+    request->xopt.logtrans = 3; /* x12a: automatic logs or not */
+    request->xopt.outliers = 1; /* x12a: detect outliers */
+    request->xopt.trdays = 0;   /* x12a: trading days correction */
+
     for (i=0; i<TX_MAXOPT; i++) {
 	request->opts[i].save = 0;
     }
@@ -687,15 +767,16 @@ static void request_opts_init (tx_request *request)
 
 static void set_opts (tx_request *request)
 {
+    GtkWidget *w;
     int i;
 
     request->savevars = 0;
 
-    *request->popt &= ~(OPT_A|OPT_B|OPT_C);
+    *request->popt &= ~(OPT_A | OPT_B | OPT_C | OPT_G);
 
     for (i=0; i<TX_MAXOPT; i++) {
-	if (request->opts[i].check != NULL && 
-	    GTK_TOGGLE_BUTTON(request->opts[i].check)->active) {
+	w = request->opts[i].check;
+	if (w != NULL && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
 	    request->opts[i].save = 1;
 	    if (i < TRIGRAPH) {
 		request->savevars++;
@@ -706,6 +787,8 @@ static void set_opts (tx_request *request)
 		} else if (i == 2) {
 		    *request->popt |= OPT_C;
 		}
+	    } else if (i == TRIGRAPH) {
+		*request->popt |= OPT_G;
 	    }
 	} else {
 	    request->opts[i].save = 0;
@@ -784,7 +867,8 @@ static int write_tramo_file (const char *fname,
 static int write_spc_file (const char *fname, const double *y,
 			   const char *vname,
 			   const DATAINFO *pdinfo, 
-			   const int *savelist) 
+			   const int *savelist,
+			   x12a_opts *xopt)
 {
     int startyr, startper;
     char *p, tmp[8];
@@ -811,12 +895,20 @@ static int write_spc_file (const char *fname, const double *y,
 
     fprintf(fp, "series{\n period=%d\n title=\"%s\"\n", pdinfo->pd, vname);
     fprintf(fp, " start=%d.%d\n", startyr, startper);
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (na(y[t])) {
+	    fputs(" missingcode=-99999\n", fp);
+	    break;
+	}
+    }
+
     fputs(" data=(\n", fp);
 
     i = 0;
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	if (na(y[t])) {
-	    fputs("-99999 ", fp); /* FIXME? */
+	    fputs("-99999 ", fp);
 	} else {
 	    fprintf(fp, "%g ", y[t]);
 	}
@@ -827,9 +919,25 @@ static int write_spc_file (const char *fname, const double *y,
     }
     fputs(" )\n}\n", fp);
 
-    /* FIXME: make these values configurable? */
+    if (xopt->logtrans == 1) {
+	fputs("transform{function=log}\n", fp);
+    } else if (xopt->logtrans == 2) {
+	fputs("transform{function=none}\n", fp);
+    } else {
+	fputs("transform{function=auto}\n", fp);
+    }
 
-    fputs("automdl{}\nx11{", fp);
+    if (xopt->trdays) {
+	fputs("regression{variables = td}\n", fp);
+    }
+
+    if (xopt->outliers) {
+	fputs("outlier{}\n", fp);
+    }
+
+    fputs("automdl{}\n", fp); 
+
+    fputs("x11{", fp);
 
     if (savelist[0] > 0) {
 	if (savelist[0] == 1) {
@@ -932,8 +1040,7 @@ static int helper_spawn (const char *path, const char *vname,
     if (cmd == NULL) {
 	err = E_ALLOC;
     } else {
-	err = winfork(cmd, workdir, SW_SHOWMINIMIZED, 
-		      CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS);
+	err = win_run_sync(cmd, workdir);
 	g_free(cmd);
     }
 
@@ -962,6 +1069,34 @@ static int helper_spawn (const char *path, const char *vname,
 
 #endif
 
+/* The 12a .err file is always produced, but often contains no
+   warning or error messages: in that case it contains 2 lines
+   of boiler-plate text followed by two blank lines. Here we
+   check if we got more than 4 lines of output.
+*/
+
+static int got_x12a_warning (const char *fname)
+{
+    FILE *fp = gretl_fopen(fname, "r");
+    int ret = 0;
+
+    if (fp != NULL) {
+	char line[128];
+	int n = 0;
+
+	while (fgets(line, sizeof line, fp)) {
+	    if (++n > 4 && !string_is_blank(line)) {
+		ret = 1;
+		break;
+	    }
+	}
+
+	fclose(fp);
+    }
+
+    return ret;
+}
+
 /* make a default x12a.mdl file if it doesn't already exist */
 
 static int check_x12a_model_file (const char *workdir, char *fname)
@@ -1035,7 +1170,7 @@ int write_tx_data (char *fname, int varnum,
 	workdir = gretl_x12_arima_dir();
     }	
 	
-    request_opts_init(&request);
+    request_opts_init(&request, pdinfo);
 
     err = check_sample_bound(request.prog, pdinfo, errmsg);
     if (err) {
@@ -1082,7 +1217,8 @@ int write_tx_data (char *fname, int varnum,
     if (request.prog == X12A) { 
 	/* write out the .spc file for x12a */
 	sprintf(fname, "%s%c%s.spc", workdir, SLASH, vname);
-	write_spc_file(fname, (*pZ)[varnum], vname, pdinfo, savelist);
+	write_spc_file(fname, (*pZ)[varnum], vname, pdinfo, savelist,
+		       &request.xopt);
     } else { 
 	/* TRAMO, possibly plus SEATS */
 	lower(vname);
@@ -1127,6 +1263,12 @@ int write_tx_data (char *fname, int varnum,
 	}
     } else {
 	if (request.prog == X12A) {
+	    /* first check the .err file for warnings */
+	    sprintf(fname, "%s%c%s.err", workdir, SLASH, vname);
+	    if (got_x12a_warning(fname)) {
+		*opt |= OPT_W;
+	    }
+	    /* then set the output filename */
 	    sprintf(fname, "%s%c%s.out", workdir, SLASH, vname); 
 	} else {
 	    sprintf(fname, "%s%coutput%c%s.out", workdir, SLASH, SLASH, vname);
@@ -1219,8 +1361,16 @@ int adjust_series (const double *x, double *y, const DATAINFO *pdinfo,
     } 
 
     if (prog == X12A) { 
+	x12a_opts xopt = { 2, /* auto log transformation */
+			   0, /* don't correct for outliers */
+			   0  /* trading days correction */
+	};
+
+	if (pdinfo->pd == 12) {
+	    xopt.trdays = 1;
+	}
 	sprintf(fname, "%s%c%s.spc", workdir, SLASH, vname);
-	write_spc_file(fname, x, vname, pdinfo, savelist);
+	write_spc_file(fname, x, vname, pdinfo, savelist, &xopt);
     } else { 
 	sprintf(fname, "%s%c%s", workdir, SLASH, vname);
 	write_tramo_file(fname, x, vname, pdinfo, NULL); 
